@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,16 +11,16 @@
  *******************************************************************************/
 package org.eclipse.ant.internal.ui.launchConfigurations;
 
-
 import java.io.File;
 import java.io.IOException;
 import java.net.URL;
-import java.text.MessageFormat;
+import com.ibm.icu.text.MessageFormat;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
+import org.apache.tools.ant.ProjectHelper;
 import org.eclipse.ant.core.AntCorePlugin;
 import org.eclipse.ant.core.AntCorePreferences;
 import org.eclipse.ant.core.AntRunner;
@@ -36,6 +36,7 @@ import org.eclipse.ant.internal.ui.debug.IAntDebugConstants;
 import org.eclipse.ant.internal.ui.debug.model.RemoteAntDebugBuildListener;
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.runtime.CoreException;
+import org.eclipse.core.runtime.FileLocator;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
@@ -43,6 +44,7 @@ import org.eclipse.core.runtime.Path;
 import org.eclipse.core.runtime.Platform;
 import org.eclipse.core.runtime.Status;
 import org.eclipse.core.runtime.SubProgressMonitor;
+import org.eclipse.core.variables.VariablesPlugin;
 import org.eclipse.debug.core.DebugEvent;
 import org.eclipse.debug.core.DebugPlugin;
 import org.eclipse.debug.core.IBreakpointManager;
@@ -65,6 +67,7 @@ import org.eclipse.jface.dialogs.MessageDialogWithToggle;
 import org.eclipse.jface.preference.IPreferenceStore;
 import org.eclipse.osgi.service.resolver.BundleDescription;
 import org.eclipse.ui.externaltools.internal.launchConfigurations.ExternalToolsUtil;
+import org.eclipse.ui.externaltools.internal.model.IExternalToolConstants;
 import org.eclipse.ui.externaltools.internal.program.launchConfigurations.BackgroundResourceRefresher;
 import org.osgi.framework.Bundle;
 
@@ -86,9 +89,16 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
 	
 	private String fMode;
     private boolean fUserSpecifiedLogger= false;
+    
+    private String getProgramArguments(ILaunchConfiguration configuration) throws CoreException {
+		String arguments = configuration.getAttribute(IExternalToolConstants.ATTR_TOOL_ARGUMENTS, ""); //$NON-NLS-1$
+		return VariablesPlugin.getDefault().getStringVariableManager().performStringSubstitution(arguments);
+	}
 	
 	/**
-	 * @see org.eclipse.debug.core.model.ILaunchConfigurationDelegate#launch(org.eclipse.debug.core.ILaunchConfiguration, java.lang.String, org.eclipse.debug.core.ILaunch, org.eclipse.core.runtime.IProgressMonitor)
+	 * @see org.eclipse.debug.core.model.ILaunchConfigurationDelegate#launch(org.eclipse.debug.core.ILaunchConfiguration,
+	 *      java.lang.String, org.eclipse.debug.core.ILaunch,
+	 *      org.eclipse.core.runtime.IProgressMonitor)
 	 */
 	public void launch(ILaunchConfiguration configuration, String mode, ILaunch launch, IProgressMonitor monitor) throws CoreException {
 		if (monitor.isCanceled()) {
@@ -103,9 +113,9 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
 		boolean isSeparateJRE= AntUtil.isSeparateJREAntBuild(configuration);
 		
 		if (CommonTab.isLaunchInBackground(configuration)) {
-			monitor.beginTask(MessageFormat.format(AntLaunchConfigurationMessages.AntLaunchDelegate_Launching__0__1, new String[] {configuration.getName()}), 10); //$NON-NLS-1$
+			monitor.beginTask(MessageFormat.format(AntLaunchConfigurationMessages.AntLaunchDelegate_Launching__0__1, new String[] {configuration.getName()}), 10);
 		} else {
-			monitor.beginTask(MessageFormat.format(AntLaunchConfigurationMessages.AntLaunchDelegate_Running__0__2, new String[] {configuration.getName()}), 100); //$NON-NLS-1$
+			monitor.beginTask(MessageFormat.format(AntLaunchConfigurationMessages.AntLaunchDelegate_Running__0__2, new String[] {configuration.getName()}), 100);
 		}
 		
 		// resolve location
@@ -117,7 +127,7 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
 		}
 		
 		if (!isSeparateJRE && AntRunner.isBuildRunning()) {
-			IStatus status= new Status(IStatus.ERROR, IAntUIConstants.PLUGIN_ID, 1, MessageFormat.format(AntLaunchConfigurationMessages.AntLaunchDelegate_Build_In_Progress, new String[]{location.toOSString()}), null); //$NON-NLS-1$
+			IStatus status= new Status(IStatus.ERROR, IAntUIConstants.PLUGIN_ID, 1, MessageFormat.format(AntLaunchConfigurationMessages.AntLaunchDelegate_Build_In_Progress, new String[]{location.toOSString()}), null);
 			throw new CoreException(status);
 		}		
 		
@@ -141,8 +151,13 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
 		idProperty.append('=');
 		idProperty.append(idStamp);
 		
-		// resolve arguments
-		String[] arguments = ExternalToolsUtil.getArguments(configuration);
+		// resolve arguments	
+		String[] arguments = null;
+		if (isSeparateJRE) {
+			arguments = new String[] {getProgramArguments(configuration)};
+        } else { 
+			arguments = ExternalToolsUtil.getArguments(configuration);
+        }
 		
 		Map userProperties= AntUtil.getProperties(configuration);
 		if (userProperties != null) {//create a copy so as to not affect the configuration with transient properties
@@ -190,7 +205,7 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
 		StringBuffer commandLine= generateCommandLine(location, arguments, userProperties, propertyFiles, targets, antHome, basedir, isSeparateJRE, captureOutput, setInputHandler);
 		
 		if (isSeparateJRE) {
-			monitor.beginTask(MessageFormat.format(AntLaunchConfigurationMessages.AntLaunchDelegate_Launching__0__1, new String[] {configuration.getName()}), 10); //$NON-NLS-1$
+			monitor.beginTask(MessageFormat.format(AntLaunchConfigurationMessages.AntLaunchDelegate_Launching__0__1, new String[] {configuration.getName()}), 10);
 			runInSeparateVM(configuration, launch, monitor, idStamp, antHome, port, requestPort, commandLine, captureOutput, setInputHandler);
 		} else {
 			runInSameVM(configuration, launch, monitor, location, idStamp, runner, commandLine, captureOutput);
@@ -214,7 +229,7 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
 					try {
 						finalRunner.run(process);
 					} catch (CoreException e) {
-						handleException(e, AntLaunchConfigurationMessages.AntLaunchDelegate_Failure); //$NON-NLS-1$
+						handleException(e, AntLaunchConfigurationMessages.AntLaunchDelegate_Failure);
 					}
 					process.terminated();
 				}
@@ -235,7 +250,7 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
 			} catch (CoreException e) {
 				process.terminated();
 				monitor.done();
-				handleException(e, AntLaunchConfigurationMessages.AntLaunchDelegate_23); //$NON-NLS-1$
+				handleException(e, AntLaunchConfigurationMessages.AntLaunchDelegate_23);
 				return;
 			}
 			process.terminated();
@@ -307,7 +322,7 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
 		if (store.getBoolean(IAntUIPreferenceConstants.ANT_ERROR_DIALOG)) {
 			AntUIPlugin.getStandardDisplay().asyncExec(new Runnable() {
 				public void run() {
-					MessageDialogWithToggle.openError(null, title, e.getMessage(), AntLaunchConfigurationMessages.AntLaunchDelegate_22, false, AntUIPlugin.getDefault().getPreferenceStore(), IAntUIPreferenceConstants.ANT_ERROR_DIALOG); //$NON-NLS-1$
+					MessageDialogWithToggle.openError(null, title, e.getMessage(), AntLaunchConfigurationMessages.AntLaunchDelegate_22, false, AntUIPlugin.getDefault().getPreferenceStore(), IAntUIPreferenceConstants.ANT_ERROR_DIALOG);
 				}
 			});
 		}
@@ -377,15 +392,18 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
 			properties= prefs.getRemoteAntProperties();
 		}
 		
-		String key;
 		//if we have user properties this means that the user has chosen to override the global properties
-		//if in a separate VM and have only two user properties these are really only Eclipse generated properties
+		//if in a separate VM and have only two (or three if debug) user properties these are really only Eclipse generated properties
 		//and the user is still using the global properties
-		boolean useGlobalProperties = userProperties == null || (separateVM && userProperties.size() == 2);
+		int numberOfEclipseProperties= 2;
+		if (userProperties != null && userProperties.get("eclipse.connect.request_port") != null){ //$NON-NLS-1$
+			numberOfEclipseProperties= 3; //debug mode
+		}
+		boolean useGlobalProperties = userProperties == null || (separateVM && userProperties.size() == numberOfEclipseProperties);
 		if (useGlobalProperties) {
 			for (Iterator iter = properties.iterator(); iter.hasNext();) {
 				Property property = (Property) iter.next();
-				key= property.getName();
+				String key= property.getName();
 				String value= property.getValue(false);
 				if (value != null) {
 					appendProperty(commandLine, key, value);
@@ -460,7 +478,8 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
 		while (itr.hasNext()) {
 			Task task = (Task) itr.next();
 			commandLine.append(" -eclipseTask "); //$NON-NLS-1$
-			commandLine.append(task.getTaskName());
+			String name= ProjectHelper.genComponentName(task.getURI(), task.getTaskName());
+			commandLine.append(name);
 			commandLine.append(',');
 			commandLine.append(task.getClassName());
 		}
@@ -470,7 +489,8 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
 		while (itr.hasNext()) {
 			Type type = (Type) itr.next();
 			commandLine.append(" -eclipseType "); //$NON-NLS-1$
-			commandLine.append(type.getTypeName());
+			String name= ProjectHelper.genComponentName(type.getURI(), type.getTypeName());
+			commandLine.append(name);
 			commandLine.append(',');
 			commandLine.append(type.getClassName());
 		}
@@ -479,8 +499,11 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
 	private void appendProperty(StringBuffer commandLine, String name, String value) {
 		commandLine.append(" \"-D"); //$NON-NLS-1$
 		commandLine.append(name);
-		commandLine.append('='); 
+		commandLine.append('=');
 		commandLine.append(value);
+        if (value.length() > 0 && value.charAt(value.length() - 1) == File.separatorChar) {
+            commandLine.append(File.separatorChar);
+        }
 		commandLine.append("\""); //$NON-NLS-1$
 	}
 	
@@ -501,6 +524,7 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
 		}
 		
 		ILaunchConfigurationWorkingCopy copy= configuration.getWorkingCopy();
+		setDefaultWorkingDirectory(copy);
 		copy.setAttribute(IJavaLaunchConfigurationConstants.ATTR_PROGRAM_ARGUMENTS, commandLine.toString());
 		StringBuffer vmArgs= generateVMArguments(copy, setInputHandler, antHome);
 		copy.setAttribute(IJavaLaunchConfigurationConstants.ATTR_VM_ARGUMENTS, vmArgs.toString());
@@ -513,11 +537,13 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
         }
         
         //set the ANT_HOME environment variable
-        Map vars = copy.getAttribute(ILaunchManager.ATTR_ENVIRONMENT_VARIABLES, new HashMap(1));
-		vars.put("ANT_HOME", antHome); //$NON-NLS-1$
-		copy.setAttribute(ILaunchManager.ATTR_ENVIRONMENT_VARIABLES, vars);
+        if (antHome != null) {
+            Map vars = copy.getAttribute(ILaunchManager.ATTR_ENVIRONMENT_VARIABLES, new HashMap(1));
+            vars.put("ANT_HOME", antHome); //$NON-NLS-1$
+            copy.setAttribute(ILaunchManager.ATTR_ENVIRONMENT_VARIABLES, vars);
+        }
 
-		//copy.setAttribute(IJavaLaunchConfigurationConstants.ATTR_VM_ARGUMENTS, "-Xdebug -Xnoagent -Djava.compiler=NONE -Xrunjdwp:transport=dt_socket,server=y,suspend=y,address=8000"); //$NON-NLS-1$
+		//copy.setAttribute(IJavaLaunchConfigurationConstants.ATTR_VM_ARGUMENTS, "-Xdebug -Xnoagent -Djava.compiler=NONE -Xrunjdwp:transport=dt_socket,server=y,suspend=y,address=8000");
 		IProgressMonitor subMonitor= new SubProgressMonitor(monitor, 10);
 		AntJavaLaunchDelegate delegate= new AntJavaLaunchDelegate();
 		delegate.preLaunchCheck(copy, ILaunchManager.RUN_MODE, subMonitor);
@@ -550,7 +576,7 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
 				}
 			};
 			DebugPlugin.getDefault().addDebugEventListener(listener);
-			monitor.subTask(AntLaunchConfigurationMessages.AntLaunchDelegate_28); //$NON-NLS-1$
+			monitor.subTask(AntLaunchConfigurationMessages.AntLaunchDelegate_28);
 			while (!monitor.isCanceled() && !terminated[0]) {
 				try {
 					Thread.sleep(50);
@@ -568,9 +594,10 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
 	private void setDefaultVM(ILaunchConfiguration configuration, ILaunchConfigurationWorkingCopy copy) {
 		try {
 			JavaRuntime.getJavaProject(configuration);
-			//remove the vm name and install type for the Java launching concept of default VM
+			//remove the vm name, install type and jre container path for the Java launching concept of default VM
 			copy.setAttribute(IJavaLaunchConfigurationConstants.ATTR_VM_INSTALL_NAME, (String)null);
 			copy.setAttribute(IJavaLaunchConfigurationConstants.ATTR_VM_INSTALL_TYPE, (String)null);
+			copy.setAttribute(IJavaLaunchConfigurationConstants.ATTR_JRE_CONTAINER_PATH, (String)null);
 		} catch (CoreException ce) {
 			//not in a Java project
 			IVMInstall defaultVMInstall= JavaRuntime.getDefaultVMInstall();
@@ -590,14 +617,16 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
 		} catch (CoreException e) {
 		}
 	
-		vmArgs.append("-Dant.home=\""); //$NON-NLS-1$
-		vmArgs.append(antHome);
-		vmArgs.append("\" "); //$NON-NLS-1$
-		File antLibDir= new File(antHome, "lib"); //$NON-NLS-1$
-		vmArgs.append("-Dant.library.dir=\""); //$NON-NLS-1$
-		vmArgs.append(antLibDir.getAbsolutePath());
-		vmArgs.append('\"');
-
+        if (antHome != null) {
+            vmArgs.append("-Dant.home=\""); //$NON-NLS-1$
+            vmArgs.append(antHome);
+            vmArgs.append("\" "); //$NON-NLS-1$
+        
+            File antLibDir= new File(antHome, "lib"); //$NON-NLS-1$
+            vmArgs.append("-Dant.library.dir=\""); //$NON-NLS-1$
+            vmArgs.append(antLibDir.getAbsolutePath());
+            vmArgs.append('\"');
+        }
 		if (setInputHandler) {
 			String swtLocation= getSWTLibraryLocation();
 			if (swtLocation != null) {
@@ -624,7 +653,7 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
             switch (c) {
                 case '"':
                     if (i != 0 && javaLibPath.charAt(i-1) == '\\') {
-                        buf.append(c); //$NON-NLS-1$
+                        buf.append(c);
                     }
                     break;
                 default:
@@ -664,9 +693,9 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
             if (fragments == null || fragments.length == 0) {
                 return null;
             }
-            Bundle fragBundle= Platform.getBundle(fragments[0].getSymbolicName()); //$NON-NLS-1$
+            Bundle fragBundle= Platform.getBundle(fragments[0].getSymbolicName());
             try {
-                URL url= Platform.asLocalURL(fragBundle.getEntry("/")); //$NON-NLS-1$
+                URL url= FileLocator.toFileURL(fragBundle.getEntry("/")); //$NON-NLS-1$
                 IPath path= new Path(url.getPath());
                 path= path.removeTrailingSeparator();
                 fgSWTLibraryLocation= path.toOSString();
@@ -687,4 +716,32 @@ public class AntLaunchDelegate extends LaunchConfigurationDelegate  {
          }
          return breakpointManager.getBreakpoints(IAntDebugConstants.ID_ANT_DEBUG_MODEL);
      }
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.core.model.LaunchConfigurationDelegate#saveBeforeLaunch(org.eclipse.debug.core.ILaunchConfiguration, java.lang.String, org.eclipse.core.runtime.IProgressMonitor)
+	 */
+	protected boolean saveBeforeLaunch(ILaunchConfiguration configuration, String mode, IProgressMonitor monitor) throws CoreException {
+		if (IExternalToolConstants.ID_EXTERNAL_TOOLS_BUILDER_LAUNCH_CATEGORY.equals(
+				configuration.getType().getCategory())) {
+					// don't prompt for builders
+					return true;
+		}
+		return super.saveBeforeLaunch(configuration, mode, monitor);
+	}
+	
+	/**
+	 * Sets the default working directory to be the parent folder of the buildfile if
+	 * the user has not explicitly set the working directory.
+	 */
+	private void setDefaultWorkingDirectory(ILaunchConfigurationWorkingCopy copy) {
+		try {
+			String wd = copy.getAttribute(IJavaLaunchConfigurationConstants.ATTR_WORKING_DIRECTORY, (String)null);
+			if (wd == null) {
+				wd= ExternalToolsUtil.getLocation(copy).removeLastSegments(1).toOSString();
+				copy.setAttribute(IJavaLaunchConfigurationConstants.ATTR_WORKING_DIRECTORY, wd);
+			}
+		} catch (CoreException e) {
+			AntUIPlugin.log(e.getStatus());
+		}
+	}
 }
