@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,17 +11,27 @@
 package org.eclipse.compare.internal;
 
 import java.io.*;
-import java.text.MessageFormat;
-import java.util.*;
+import com.ibm.icu.text.MessageFormat;
+import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.Iterator;
 import java.util.List;
+import java.util.Map;
+import java.util.MissingResourceException;
+import java.util.ResourceBundle;
 
 import org.eclipse.swt.widgets.*;
 
+import org.eclipse.core.runtime.ListenerList;
 import org.eclipse.core.resources.IEncodedStorage;
 import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.ResourceAttributes;
 import org.eclipse.core.resources.ResourcesPlugin;
 
+import org.eclipse.core.resources.mapping.ResourceMapping;
+import org.eclipse.core.resources.mapping.ResourceMappingContext;
+import org.eclipse.core.resources.mapping.ResourceTraversal;
+
 import org.eclipse.jface.action.IAction;
 import org.eclipse.jface.resource.ImageDescriptor;
 import org.eclipse.jface.util.*;
@@ -117,21 +127,31 @@ public class Utilities {
 	}
 	
 	private static ArrayList internalGetResources(ISelection selection, Class type) {
-		
 		ArrayList tmp= new ArrayList();
-
 		if (selection instanceof IStructuredSelection) {
-		
 			Object[] s= ((IStructuredSelection)selection).toArray();
 				
 			for (int i= 0; i < s.length; i++) {
-				
 				IResource resource= null;
-				
 				Object o= s[i];
 				if (type.isInstance(o)) {
 					resource= (IResource) o;
 						
+				} else if (o instanceof ResourceMapping) {
+					try {
+						ResourceTraversal[] travs= ((ResourceMapping)o).getTraversals(ResourceMappingContext.LOCAL_CONTEXT, null);
+						if (travs != null) {
+							for (int k= 0; k < travs.length; k++) {
+								IResource[] resources= travs[k].getResources();
+								for (int j= 0; j < resources.length; j++) {
+									if (type.isInstance(resources[j]) && resources[j].isAccessible())
+										tmp.add(resources[j]);
+								}
+							}
+						}
+					} catch (CoreException ex) {
+						CompareUIPlugin.log(ex);
+					}
 				} else if (o instanceof IAdaptable) {
 					IAdaptable a= (IAdaptable) o;
 					Object adapter= a.getAdapter(IResource.class);
@@ -143,9 +163,9 @@ public class Utilities {
 					tmp.add(resource);
 			}
 		}
-		
 		return tmp;
 	}
+
 	
 	/*
 	 * Convenience method: extract all accessible <code>IResources</code> from given selection.
@@ -273,7 +293,7 @@ public class Utilities {
 			try {
 				return bundle.getString(key);
 			} catch (MissingResourceException x) {
-				// NeedWork
+				// fall through
 			}
 		}
 		return dfltValue;
@@ -285,17 +305,17 @@ public class Utilities {
 			try {
 				return MessageFormat.format(bundle.getString(key), new String[] { arg });
 			} catch (MissingResourceException x) {
-				// NeedWork
+				CompareUIPlugin.log(x);
 			}
 		}
-		return "!" + key + "!";//$NON-NLS-2$ //$NON-NLS-1$
+		return "!" + key + "!";	//$NON-NLS-2$ //$NON-NLS-1$
 	}
 	
 	public static String getString(String key) {
 		try {
 			return CompareUI.getResourceBundle().getString(key);
 		} catch (MissingResourceException e) {
-			return "!" + key + "!";//$NON-NLS-2$ //$NON-NLS-1$
+			return "!" + key + "!";	//$NON-NLS-2$ //$NON-NLS-1$
 		}
 	}
 	
@@ -303,7 +323,7 @@ public class Utilities {
 		try {
 			return MessageFormat.format(CompareUI.getResourceBundle().getString(key), new String[] { arg });
 		} catch (MissingResourceException e) {
-			return "!" + key + "!";//$NON-NLS-2$ //$NON-NLS-1$
+			return "!" + key + "!";	//$NON-NLS-2$ //$NON-NLS-1$
 		}	
 	}
 
@@ -327,9 +347,9 @@ public class Utilities {
 				if (s != null)
 					return Integer.parseInt(s);
 			} catch (NumberFormatException x) {
-				// NeedWork
+				CompareUIPlugin.log(x);
 			} catch (MissingResourceException x) {
-				// NeedWork
+				// silently ignore Exception
 			}
 		}
 		return dfltValue;
@@ -447,7 +467,7 @@ public class Utilities {
 		IStatus status= ResourcesPlugin.getWorkspace().validateEdit(files, shell);
 		if (! status.isOK()) {
 			String message= getString("ValidateEdit.error.unable_to_perform"); //$NON-NLS-1$
-			ErrorDialog.openError(shell, title, message, status); //$NON-NLS-1$
+			ErrorDialog.openError(shell, title, message, status);
 			return false;
 		}
 			
@@ -538,7 +558,7 @@ public class Utilities {
 			return buffer.toString();
 			
 		} catch (IOException ex) {
-			// NeedWork
+			CompareUIPlugin.log(ex);
 		} finally {
 			if (reader != null) {
 				try {
@@ -556,7 +576,7 @@ public class Utilities {
 			try {
 				return ((IEncodedStorage)resource).getCharset();
 			} catch (CoreException ex) {
-				// fall  through
+				CompareUIPlugin.log(ex);
 			}
 		}
 		return ResourcesPlugin.getEncoding();
@@ -566,7 +586,7 @@ public class Utilities {
 		byte[] bytes= null;
 		if (s != null) {
 			try {
-				bytes= s.getBytes(encoding); //$NON-NLS-1$
+				bytes= s.getBytes(encoding);
 			} catch (UnsupportedEncodingException e) {
 				bytes= s.getBytes();
 			}
@@ -593,4 +613,11 @@ public class Utilities {
 			}
 		}
 	}
+
+	public static IResource getFirstResource(ISelection selection) {
+		IResource[] resources = getResources(selection);
+		if (resources.length > 0)
+			return resources[0];
+		return null;
+	}
 }
