@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -28,6 +28,7 @@ public class PlatformLogWriter implements ILogListener {
 	public PlatformLogWriter(FrameworkLog frameworkLog) {
 		this.frameworkLog = frameworkLog;
 	}
+
 	/**
 	 * @see ILogListener#logging(IStatus, String)
 	 */
@@ -36,10 +37,6 @@ public class PlatformLogWriter implements ILogListener {
 	}
 
 	protected FrameworkLogEntry getLog(IStatus status) {
-		StringBuffer entry = new StringBuffer();
-		entry.append(status.getPlugin()).append(" "); //$NON-NLS-1$
-		entry.append(Integer.toString(status.getSeverity())).append(" "); //$NON-NLS-1$
-		entry.append(Integer.toString(status.getCode()));
 		Throwable t = status.getException();
 		ArrayList childlist = new ArrayList();
 
@@ -61,6 +58,6 @@ public class PlatformLogWriter implements ILogListener {
 
 		FrameworkLogEntry[] children = (FrameworkLogEntry[]) (childlist.size() == 0 ? null : childlist.toArray(new FrameworkLogEntry[childlist.size()]));
 
-		return new FrameworkLogEntry(entry.toString(), status.getMessage(), stackCode, t, children);
+		return new FrameworkLogEntry(status.getPlugin(), status.getSeverity(), status.getCode(), status.getMessage(), stackCode, t, children);
 	}
 }
