@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -16,7 +16,7 @@ import java.io.File;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.UnsupportedEncodingException;
-import java.text.MessageFormat;
+import com.ibm.icu.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
@@ -26,7 +26,7 @@ import javax.xml.transform.TransformerException;
 
 import org.eclipse.core.resources.IContainer;
 import org.eclipse.core.resources.IFile;
-import org.eclipse.core.resources.IResourceRuleFactory;
+import org.eclipse.core.resources.IResource;
 import org.eclipse.core.resources.IWorkspaceRunnable;
 import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
@@ -34,8 +34,6 @@ import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Status;
-import org.eclipse.core.runtime.jobs.ISchedulingRule;
-import org.eclipse.core.runtime.jobs.MultiRule;
 import org.eclipse.debug.core.DebugException;
 import org.eclipse.debug.core.DebugPlugin;
 import org.eclipse.debug.core.ILaunchConfiguration;
@@ -84,7 +82,7 @@ public class LaunchConfigurationWorkingCopy extends LaunchConfiguration implemen
 	 * stored in when saved.
 	 */
 	private IContainer fContainer;
-
+	
 	/**
 	 * Constructs a working copy of the specified launch 
 	 * configuration.
@@ -168,7 +166,7 @@ public class LaunchConfigurationWorkingCopy extends LaunchConfiguration implemen
 					}
 				};
 				
-				ResourcesPlugin.getWorkspace().run(wr, getSchedulingRule(), 0, null);
+				ResourcesPlugin.getWorkspace().run(wr, null, 0, null);
 			} else {
 				//file is persisted in the metadata not the workspace
 				doSave0();
@@ -180,41 +178,6 @@ public class LaunchConfigurationWorkingCopy extends LaunchConfiguration implemen
 		return new LaunchConfiguration(getLocation());
 	}
 
-	/**
-	 * Returns the scheduling rule to be used when saving this launch configuration.
-	 * <code>null</code> is a valid scheduling rule.
-	 * 
-	 * @return the scheduling rule to be used when saving this launch configuration
-	 */
-	private ISchedulingRule getSchedulingRule() {
-		List rules= new ArrayList(2);
-		IResourceRuleFactory fac = ResourcesPlugin.getWorkspace().getRuleFactory();
-		if (!isLocal()) {
-			//working copy will be saved to a workspace location - create or modify
-			IFile[] files= ResourcesPlugin.getWorkspace().getRoot().findFilesForLocation(getLocation());
-			if (files.length > 0) {
-				IFile file = files[0];
-				ISchedulingRule rule = null;
-				if (file.exists()) {
-					rule = fac.modifyRule(file);
-				} else {
-					rule = fac.createRule(file);
-				}
-				rules.add(MultiRule.combine(rule, fac.validateEditRule(files)));
-			}
-		}
-		ILaunchConfiguration original = getOriginal();
-		if (!isNew() && isMoved() && !original.isLocal()) {
-			IFile[] files= ResourcesPlugin.getWorkspace().getRoot().findFilesForLocation(original.getLocation());
-			if (files.length > 0) {
-				rules.add(MultiRule.combine(fac.deleteRule(files[0]), fac.validateEditRule(files)));
-			}
-		}
-		if (rules.isEmpty()) {
-			return null;
-		}
-		return new MultiRule((ISchedulingRule[]) rules.toArray(new ISchedulingRule[rules.size()]));
-	}
 	
 	private void doSave0() throws CoreException {
 		// set up from/to information if this is a move
@@ -255,7 +218,7 @@ public class LaunchConfigurationWorkingCopy extends LaunchConfiguration implemen
 			throw new DebugException(
 				new Status(
 					IStatus.ERROR, DebugPlugin.getUniqueIdentifier(),
-					DebugException.REQUEST_FAILED, MessageFormat.format(DebugCoreMessages.LaunchConfigurationWorkingCopy__0__occurred_generating_launch_configuration_XML__1, new String[]{e.toString()}), null //$NON-NLS-1$
+					DebugException.REQUEST_FAILED, MessageFormat.format(DebugCoreMessages.LaunchConfigurationWorkingCopy__0__occurred_generating_launch_configuration_XML__1, new String[]{e.toString()}), null 
 					)
 				);		
 		}
@@ -283,7 +246,7 @@ public class LaunchConfigurationWorkingCopy extends LaunchConfiguration implemen
 				throw new DebugException(
 					new Status(
 					 IStatus.ERROR, DebugPlugin.getUniqueIdentifier(),
-					 DebugException.REQUEST_FAILED, MessageFormat.format(DebugCoreMessages.LaunchConfigurationWorkingCopy__0__occurred_generating_launch_configuration_XML__1, new String[]{ie.toString()}), null //$NON-NLS-1$
+					 DebugException.REQUEST_FAILED, MessageFormat.format(DebugCoreMessages.LaunchConfigurationWorkingCopy__0__occurred_generating_launch_configuration_XML__1, new String[]{ie.toString()}), null 
 					)
 				);				
 			}
@@ -295,7 +258,7 @@ public class LaunchConfigurationWorkingCopy extends LaunchConfiguration implemen
 				throw new DebugException(
 					new Status(
 					 IStatus.ERROR, DebugPlugin.getUniqueIdentifier(),
-					 DebugException.REQUEST_FAILED, DebugCoreMessages.LaunchConfigurationWorkingCopy_Specified_container_for_launch_configuration_does_not_exist_2, null //$NON-NLS-1$
+					 DebugException.REQUEST_FAILED, DebugCoreMessages.LaunchConfigurationWorkingCopy_Specified_container_for_launch_configuration_does_not_exist_2, null 
 					)
 				);				
 			}
@@ -306,7 +269,7 @@ public class LaunchConfigurationWorkingCopy extends LaunchConfiguration implemen
 				throw new DebugException(
 					new Status(
 						 IStatus.ERROR, DebugPlugin.getUniqueIdentifier(),
-						 DebugException.REQUEST_FAILED, DebugCoreMessages.LaunchConfigurationWorkingCopy_5, null //$NON-NLS-1$
+						 DebugException.REQUEST_FAILED, DebugCoreMessages.LaunchConfigurationWorkingCopy_5, null 
 					));
 			}
 			if (!file.exists()) {
@@ -344,7 +307,7 @@ public class LaunchConfigurationWorkingCopy extends LaunchConfiguration implemen
 	 * @see ILaunchConfigurationWorkingCopy#setAttribute(String, boolean)
 	 */
 	public void setAttribute(String attributeName, boolean value) {
-		getInfo().setAttribute(attributeName, new Boolean(value));
+		getInfo().setAttribute(attributeName, Boolean.valueOf(value));
 		setDirty();	
 	}
 
@@ -377,7 +340,7 @@ public class LaunchConfigurationWorkingCopy extends LaunchConfiguration implemen
 	 * working copy to the current values of the given
 	 * configuration.
 	 * 
-	 * @param originl the launch configuration this working
+	 * @param original the launch configuration this working
 	 *  copy is based on.
 	 * @exception CoreException if unable to initialize this
 	 *  working copy based on the original's current attribute
@@ -591,5 +554,26 @@ public class LaunchConfigurationWorkingCopy extends LaunchConfiguration implemen
 		getInfo().setAttributes(attributes);
 	}
 
-}
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.core.ILaunchConfigurationWorkingCopy#setResource(org.eclipse.core.resources.IResource)
+	 */
+	public void setMappedResources(IResource[] resources) {
+		ArrayList paths = null;
+		ArrayList types = null;
+		if(resources != null && resources.length > 0) {
+			paths = new ArrayList(resources.length);
+			types = new ArrayList(resources.length);
+			for (int i = 0; i < resources.length; i++) {
+				IResource resource = resources[i];
+				if(resource != null) {
+					paths.add(resource.getFullPath().toPortableString());
+					types.add(new Integer(resource.getType()).toString());
+				}
+			}
+		}
+		setAttribute(LaunchConfiguration.ATTR_MAPPED_RESOURCE_PATHS, paths);
+		setAttribute(LaunchConfiguration.ATTR_MAPPED_RESOURCE_TYPES, types);
+	}//end setResource
+
+}//end class
 
