@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2004, 2005 IBM Corporation and others.
+ * Copyright (c) 2004, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -30,7 +30,10 @@ import org.eclipse.debug.internal.ui.DebugUIPlugin;
 import org.eclipse.debug.internal.ui.IInternalDebugUIConstants;
 import org.eclipse.debug.internal.ui.memory.IMemoryRenderingUpdater;
 import org.eclipse.debug.internal.ui.preferences.IDebugPreferenceConstants;
+import org.eclipse.debug.internal.ui.views.memory.MemoryViewUtil;
+import org.eclipse.debug.ui.DebugUITools;
 import org.eclipse.debug.ui.IDebugUIConstants;
+import org.eclipse.debug.ui.memory.AbstractTableRendering;
 import org.eclipse.jface.viewers.StructuredViewer;
 import org.eclipse.jface.viewers.Viewer;
 
@@ -41,20 +44,26 @@ import org.eclipse.jface.viewers.Viewer;
  */
 public class TableRenderingContentProvider extends BasicDebugViewContentProvider {
 	
-	// cached information
+	// lines currently being displayed by the table rendering
 	protected Vector lineCache;
 	
-	// keeps track of all memory line ever retrieved
-	// allow us to compare and compute deltas
+	// Cache to allow the content provider to comppute change information
+	// Cache is taken by copying the lineCache after a suspend event
+	// or change event from the the memory block.
 	protected Hashtable contentCache;
 	
+	// cache in the form of MemoryByte
+	// needed for reorganizing cache when the row size changes
+	private MemoryByte[] fContentCacheInBytes;	
+	private String fContentCacheStartAddress;
+
 	private BigInteger fBufferTopAddress;
 	
 	private TableRenderingContentInput fInput;
 	private BigInteger fBufferEndAddress;
 	
 	private boolean fDynamicLoad;
-	
+
 	/**
 	 * @param memoryBlock
 	 * @param newTab
@@ -90,10 +99,10 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 					loadContentForSimpleMemoryBlock();
 				
 				// tell rendering to display table if the loading is successful
-				fInput.getMemoryRendering().displayTable();
+				getTableRendering(fInput).displayTable();
 			}
 		} catch (DebugException e) {
-			fInput.getMemoryRendering().displayError(e);
+			getTableRendering(fInput).displayError(e);
 		}
 	}
 
@@ -111,25 +120,46 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 		if (lineCache.isEmpty()) { 
 		
 			try {
-				IMemoryBlock memoryBlock = fInput.getMemoryBlock();
-				if (memoryBlock instanceof IMemoryBlockExtension)
-				{
-					loadContentForExtendedMemoryBlock();
-					fInput.getMemoryRendering().displayTable();
-				}
-				else
-				{
-					loadContentForSimpleMemoryBlock();
-					fInput.getMemoryRendering().displayTable();
-				}
+				getMemoryFromMemoryBlock();
 			} catch (DebugException e) {
 				DebugUIPlugin.log(e.getStatus());
-				fInput.getMemoryRendering().displayError(e);
+				getTableRendering(fInput).displayError(e);
+				return lineCache.toArray();
+			}
+		}
+		
+		// check to see if the row size has changed
+		TableRenderingLine line = (TableRenderingLine)lineCache.get(0);
+		int currentRowSize = line.getByteArray().length;
+		int renderingRowSize = getTableRendering(fInput).getBytesPerLine();
+		
+		if (renderingRowSize != currentRowSize)
+		{
+			try {
+				reorganizeContentCache(renderingRowSize);			
+				reorganizeLines(lineCache, renderingRowSize);
+			} catch (DebugException e) {
+				DebugUIPlugin.log(e.getStatus());
+				getTableRendering(fInput).displayError(e);
 				return lineCache.toArray();
 			}
 		}
 		return lineCache.toArray();
 	}
+	
+	private void getMemoryFromMemoryBlock() throws DebugException {
+		IMemoryBlock memoryBlock = fInput.getMemoryBlock();
+		if (memoryBlock instanceof IMemoryBlockExtension)
+		{
+			loadContentForExtendedMemoryBlock();
+			getTableRendering(fInput).displayTable();
+		}
+		else
+		{
+			loadContentForSimpleMemoryBlock();
+			getTableRendering(fInput).displayTable();
+		}
+	}
 
 	/**
 	 * @throws DebugException
@@ -142,7 +172,7 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 		long startAddress = fInput.getMemoryBlock().getStartAddress();
 		BigInteger address = BigInteger.valueOf(startAddress);
 		long length = fInput.getMemoryBlock().getLength();
-		long numLines = length / fInput.getMemoryRendering().getBytesPerLine();
+		long numLines = length / getTableRendering(fInput).getBytesPerLine();
 		getMemoryToFitTable(address, numLines, fInput.isUpdateDelta());
 	}
 
@@ -175,7 +205,7 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 			throw new DebugException(DebugUIPlugin.newErrorStatus(DebugUIMessages.TableRenderingContentProvider_0 + loadAddress.toString(16), null));
 		}
 		
-		int addressableUnitsPerLine = fInput.getMemoryRendering().getAddressableUnitPerLine();
+		int addressableUnitsPerLine = getTableRendering(fInput).getAddressableUnitPerLine();
 		BigInteger bufferStart = loadAddress.subtract(BigInteger.valueOf(fInput.getPreBuffer()*addressableUnitsPerLine));
 		BigInteger bufferEnd = loadAddress.add(BigInteger.valueOf(fInput.getPostBuffer()*addressableUnitsPerLine));
 		bufferEnd = bufferEnd.add(BigInteger.valueOf(fInput.getNumLines()*addressableUnitsPerLine));
@@ -256,15 +286,10 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 		
 		int addressLength = addressSize * IInternalDebugUIConstants.CHAR_PER_BYTE;
 
-		// align starting address with double word boundary
-		if (fInput.getMemoryBlock() instanceof IMemoryBlockExtension)
+		// align to the closest boundary based on addressable size per line
+		if ( getMemoryBlock() instanceof IMemoryBlockExtension)
 		{
-			if (!adjustedAddress.endsWith("0")) //$NON-NLS-1$
-			{
-				adjustedAddress = adjustedAddress.substring(0, adjustedAddress.length() - 1);
-				adjustedAddress += "0"; //$NON-NLS-1$
-				startingAddress = new BigInteger(adjustedAddress, 16);
-			}
+			startingAddress = MemoryViewUtil.alignToBoundary(startingAddress, getTableRendering(fInput).getAddressableUnitPerLine());
 		}
 
 		IMemoryBlockExtension extMemoryBlock = null;
@@ -277,11 +302,11 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 		{
 			if (fInput.getMemoryBlock() instanceof IMemoryBlockExtension)
 			{
-				reqNumBytes = fInput.getMemoryRendering().getBytesPerLine() * numberOfLines;
+				reqNumBytes = getTableRendering(fInput).getBytesPerLine() * numberOfLines;
 				// get memory from memory block
 				extMemoryBlock = (IMemoryBlockExtension) fInput.getMemoryBlock();
 				
-				long reqNumberOfUnits = fInput.getMemoryRendering().getAddressableUnitPerLine() * numberOfLines;
+				long reqNumberOfUnits = getTableRendering(fInput).getAddressableUnitPerLine() * numberOfLines;
 						
 				memoryBuffer =	extMemoryBlock.getBytesFromAddress(startingAddress,	reqNumberOfUnits);
 		
@@ -316,12 +341,12 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 				reqNumBytes = fInput.getMemoryBlock().getLength() + prefillNumBytes;
 				
 				// figure out number of dummy bytes to append
-				while (reqNumBytes % fInput.getMemoryRendering().getBytesPerLine() != 0)
+				while (reqNumBytes % getTableRendering(fInput).getBytesPerLine() != 0)
 				{
 					reqNumBytes ++;
 				}
 				
-				numberOfLines = reqNumBytes / fInput.getMemoryRendering().getBytesPerLine();
+				numberOfLines = reqNumBytes / getTableRendering(fInput).getBytesPerLine();
 				
 				// create memory byte for IMemoryBlock
 				memoryBuffer = new MemoryByte[(int)reqNumBytes];
@@ -403,7 +428,7 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 			
 		}
 		
-		// clear line cacheit'
+		// clear line cache
 		if (!lineCache.isEmpty())
 		{
 			lineCache.clear();
@@ -423,8 +448,17 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 		}
 			
 		// put memory information into MemoryViewLine
+		organizeLines(numberOfLines, updateDelta, addressLength, memoryBuffer, paddedString, address, manageDelta);
+		
+		if (error){
+			throw dbgEvt;
+		}
+	}
+
+	private void organizeLines(long numberOfLines, boolean updateDelta, int addressLength, MemoryByte[] memoryBuffer, String paddedString, String address, boolean manageDelta) 
+	{
 		for (int i = 0; i < numberOfLines; i++)
-		{ //chop the raw memory up 
+		{   //chop the raw memory up 
 			String tmpAddress = address.toUpperCase();
 			if (tmpAddress.length() < addressLength)
 			{
@@ -433,14 +467,15 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 					tmpAddress = "0" + tmpAddress; //$NON-NLS-1$
 				}
 			}
-			MemoryByte[] memory = new MemoryByte[fInput.getMemoryRendering().getBytesPerLine()];
+			int bytesPerLine = getTableRendering(fInput).getBytesPerLine();
+			MemoryByte[] memory = new MemoryByte[bytesPerLine];
 			boolean isMonitored = true;
 			
 			// counter for memory, starts from 0 to number of bytes per line
 			int k = 0;
 			// j is the counter for memArray, memory returned by debug adapter
-			for (int j = i * fInput.getMemoryRendering().getBytesPerLine();
-				j < i * fInput.getMemoryRendering().getBytesPerLine() + fInput.getMemoryRendering().getBytesPerLine();
+			for (int j = i * bytesPerLine;
+				j < i * bytesPerLine + bytesPerLine;
 				j++)
 			{
 				
@@ -491,7 +526,7 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 			}
 			
 			// calculate delta info for the memory view line
-			if (manageDelta && !fInput.getMemoryRendering().isDisplayingError())
+			if (manageDelta && !getTableRendering(fInput).isDisplayingError())
 			{
 				if (updateDelta)
 				{
@@ -517,7 +552,7 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 					}
 				}
 			}
-			else if (manageDelta && fInput.getMemoryRendering().isDisplayingError())
+			else if (manageDelta && getTableRendering(fInput).isDisplayingError())
 			{
 				// show as unmonitored if the view tab is previoulsy displaying error
 				newLine.isMonitored = false;
@@ -528,13 +563,9 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 			// increment row address
 			BigInteger bigInt = new BigInteger(address, 16);
 			fBufferEndAddress = bigInt;
-			int addressableUnit = fInput.getMemoryRendering().getBytesPerLine()/fInput.getMemoryRendering().getAddressableSize();
+			int addressableUnit = getTableRendering(fInput).getBytesPerLine()/getTableRendering(fInput).getAddressableSize();
 			address = bigInt.add(BigInteger.valueOf(addressableUnit)).toString(16);
 		}
-		
-		if (error){
-			throw dbgEvt;
-		}
 	}
 	
 	/**
@@ -546,7 +577,7 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 		// make up dummy memory, needed for recovery in case the debug adapter
 		// is capable of retrieving memory again
 
-		int numBytes = (int)(fInput.getMemoryRendering().getBytesPerLine() * numberOfLines);
+		int numBytes = (int)(getTableRendering(fInput).getBytesPerLine() * numberOfLines);
 		memoryBuffer = new MemoryByte[numBytes];
 		
 		for (int i=0; i<memoryBuffer.length; i++){
@@ -564,7 +595,7 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 	 */
 	protected void doHandleDebugEvent(DebugEvent event) {
 		
-		if (fInput.getMemoryRendering().isVisible())
+		if (getTableRendering(fInput).isVisible())
 		{
 			// only do this if it's visible
 			// still need to clear content cache if the rendering
@@ -588,7 +619,7 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 		if (event.getKind() == DebugEvent.CHANGE && event.getSource() == fInput.getMemoryBlock())
 		{
 			if (event.getDetail() == DebugEvent.STATE){
-				fInput.getMemoryRendering().updateLabels();
+				getTableRendering(fInput).updateLabels();
 			}
 			else
 			{	
@@ -624,10 +655,10 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 		takeContentSnapshot();
 		
 		//do not handle event if the rendering is not visible
-		if (!fInput.getMemoryRendering().isVisible())
+		if (!getTableRendering(fInput).isVisible())
 			 return;
 		
-		fInput.getMemoryRendering().refresh();
+		getTableRendering(fInput).refresh();
 		
 	}
 	
@@ -635,22 +666,25 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 	 *  Take a snapshot on the content, marking the lines as monitored
 	 */
 	public void takeContentSnapshot()
-	{
+	{	
 		// cache content before getting new ones
 		TableRenderingLine[] lines =(TableRenderingLine[]) lineCache.toArray(new TableRenderingLine[lineCache.size()]);
+		fContentCacheInBytes = convertLinesToBytes(lines);
+		fContentCacheStartAddress = lines[0].getAddress();
+		
 		if (contentCache != null)
 		{
 			contentCache.clear();
 		}
 		
 		//do not handle event if the rendering is not visible
-		if (!fInput.getMemoryRendering().isVisible())
+		if (!getTableRendering(fInput).isVisible())
 			 return;
 		
 		// use existing lines as cache is the rendering is not currently displaying
 		// error.  Otherwise, leave contentCache empty as we do not have updated
 		// content.
-		if (!fInput.getMemoryRendering().isDisplayingError())
+		if (!getTableRendering(fInput).isDisplayingError())
 		{
 			for (int i=0; i<lines.length; i++)
 			{
@@ -749,7 +783,7 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 			
 			BigInteger startAddress = new BigInteger(first.getAddress(), 16);
 			BigInteger lastAddress = new BigInteger(last.getAddress(), 16);
-			int addressableUnit = fInput.getMemoryRendering().getAddressableUnitPerLine();
+			int addressableUnit = getTableRendering(fInput).getAddressableUnitPerLine();
 			lastAddress = lastAddress.add(BigInteger.valueOf(addressableUnit)).subtract(BigInteger.valueOf(1));
 			
 			if (startAddress.compareTo(address) <= 0 &&
@@ -764,6 +798,8 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 	
 	public void clearContentCache()
 	{
+		fContentCacheInBytes = new MemoryByte[0];
+		fContentCacheStartAddress = null;
 		contentCache.clear();
 	}
 	
@@ -785,7 +821,7 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 		
 		// do not handle event if if the memory block wants to do its
 		// own update
-		if (managedMB != null && managedMB.supportsManagedUpdate(fInput.getMemoryRendering()))
+		if (managedMB != null && managedMB.supportsManagedUpdate(getTableRendering(fInput)))
 			return true;
 		
 		return false;
@@ -805,4 +841,118 @@ public class TableRenderingContentProvider extends BasicDebugViewContentProvider
 	{
 		fDynamicLoad = dynamicLoad;
 	}
+	
+	private void reorganizeLines(Vector lines, int numBytesPerLine) throws DebugException
+	{
+		if (lines == null || lines.isEmpty())
+			return;
+		
+		Object[] objs = lines.toArray();
+		
+		if (objs.length > 0)
+		{
+			TableRenderingLine[] renderingLines = (TableRenderingLine[])lines.toArray(new TableRenderingLine[lines.size()]);
+			MemoryByte[] buffer = convertLinesToBytes(renderingLines);
+			BigInteger lineAddress = new BigInteger(renderingLines[0].getAddress(), 16);
+			int numberOfLines = buffer.length / numBytesPerLine;
+			boolean updateDelta = false;
+			int addressLength = getAddressSize(lineAddress) * IInternalDebugUIConstants.CHAR_PER_BYTE;
+			MemoryByte[] memoryBuffer = buffer;
+			String address =renderingLines[0].getAddress();
+			String paddedString = DebugUITools.getPreferenceStore().getString(IDebugUIConstants.PREF_PADDED_STR);
+			
+			// set to false to preserve information delta information 
+			boolean manageDelta = true;
+			
+			// If change information is not managed by the memory block
+			// The view tab will manage it and calculate delta information
+			// for its content cache.
+			if (fInput.getMemoryBlock() instanceof IMemoryBlockExtension)
+			{
+				manageDelta = !((IMemoryBlockExtension)fInput.getMemoryBlock()).supportsChangeManagement();
+			}
+			lineCache.clear();
+			
+			organizeLines(numberOfLines, updateDelta, addressLength, memoryBuffer, paddedString, address, manageDelta);
+		}
+	}
+	
+	private void reorganizeContentCache(int bytesPerLine)
+	{
+		// if content cache is empty, do nothing
+		if (contentCache == null || contentCache.isEmpty()
+			|| fContentCacheInBytes.length == 0 || fContentCacheStartAddress == null)
+			return;
+		
+		MemoryByte[] bytes = fContentCacheInBytes;
+		TableRenderingLine[] convertedLines = convertBytesToLines(bytes, bytesPerLine, new BigInteger(fContentCacheStartAddress, 16));
+		
+		contentCache.clear();
+		for (int i=0; i<convertedLines.length; i++)
+		{
+			contentCache.put(convertedLines[i].getAddress(), convertedLines[i]);
+		}
+	}
+	
+	private MemoryByte[] convertLinesToBytes(TableRenderingLine[] lines)
+	{
+		// convert the lines back to a buffer of MemoryByte
+		TableRenderingLine temp = lines[0];
+		int lineLength = temp.getLength();
+
+		MemoryByte[] buffer = new MemoryByte[lines.length * lineLength];
+		for (int i=0; i<lines.length; i++)
+		{
+			TableRenderingLine line = lines[i];
+			MemoryByte[] bytes = line.getBytes();
+			System.arraycopy(bytes, 0, buffer, i*lineLength, lineLength);
+		}
+		return buffer;
+	}
+	
+	private TableRenderingLine[] convertBytesToLines(MemoryByte[] bytes, int bytesPerLine, BigInteger startAddress)
+	{
+		int numOfLines = bytes.length / bytesPerLine;
+		String address = startAddress.toString(16);
+		int addressLength;
+		try {
+			addressLength = getAddressSize(startAddress) * IInternalDebugUIConstants.CHAR_PER_BYTE;
+		} catch (DebugException e) {
+			DebugUIPlugin.log(e);
+			addressLength = 4 * IInternalDebugUIConstants.CHAR_PER_BYTE;
+		}
+		ArrayList lines = new ArrayList();
+		String paddedString = DebugUITools.getPreferenceStore().getString(IDebugUIConstants.PREF_PADDED_STR);
+		
+		for (int i=0; i<numOfLines; i++)
+		{
+			MemoryByte[] temp = new MemoryByte[bytesPerLine];
+			System.arraycopy(bytes, i*bytesPerLine, temp, 0, bytesPerLine);
+			
+			String tmpAddress = address.toUpperCase();
+			if (tmpAddress.length() < addressLength)
+			{
+				for (int j = 0; tmpAddress.length() < addressLength; j++)
+				{
+					tmpAddress = "0" + tmpAddress; //$NON-NLS-1$
+				}
+			}
+			
+			TableRenderingLine newLine = new TableRenderingLine(tmpAddress, temp, lines.size(), paddedString);
+			lines.add(newLine);
+			
+			// increment row address
+			BigInteger bigInt = new BigInteger(address, 16);
+			fBufferEndAddress = bigInt;
+			int addressableUnit = getTableRendering(fInput).getBytesPerLine()/getTableRendering(fInput).getAddressableSize();
+			address = bigInt.add(BigInteger.valueOf(addressableUnit)).toString(16);
+		}
+		
+		return (TableRenderingLine[])lines.toArray(new TableRenderingLine[lines.size()]);
+	}
+	
+	private AbstractTableRendering getTableRendering(TableRenderingContentInput input)
+	{
+		return (AbstractTableRendering)input.getAdapter(AbstractTableRendering.class);
+	}
 }
