@@ -1,17 +1,18 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.help.ui.internal;
 
-import java.text.*;
-import java.util.*;
+import com.ibm.icu.text.BreakIterator;
+import java.util.ArrayList;
+import java.util.Iterator;
 
 import org.eclipse.help.internal.context.*;
 import org.eclipse.swt.*;
