@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * Copyright (c) 2000, 2007 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -13,7 +13,7 @@ package org.eclipse.jdt.internal.core.builder;
 import org.eclipse.core.resources.*;
 import org.eclipse.core.runtime.*;
 
-import org.eclipse.jdt.core.JavaCore;
+import org.eclipse.jdt.core.*;
 import org.eclipse.jdt.core.compiler.*;
 import org.eclipse.jdt.internal.compiler.*;
 import org.eclipse.jdt.internal.compiler.classfmt.*;
@@ -77,28 +77,38 @@ public boolean build(SimpleLookupTable deltas) {
 	try {
 		resetCollections();
 
-		notifier.subTask(Messages.build_analyzingDeltas); 
-		IResourceDelta sourceDelta = (IResourceDelta) deltas.get(javaBuilder.currentProject);
-		if (sourceDelta != null)
-			if (!findSourceFiles(sourceDelta)) return false;
-		notifier.updateProgressDelta(0.10f);
+		notifier.subTask(Messages.build_analyzingDeltas);
+		if (javaBuilder.hasBuildpathErrors()) {
+			// if a mssing class file was detected in the last build, a build state was saved since its no longer fatal
+			// but we need to rebuild every source file since problems were not recorded
+			// AND to avoid the infinite build scenario if this project is involved in a cycle, see bug 160550
+			// we need to avoid unnecessary deltas caused by doing a full build in this case
+			javaBuilder.currentProject.deleteMarkers(IJavaModelMarker.JAVA_MODEL_PROBLEM_MARKER, false, IResource.DEPTH_ZERO);
+			addAllSourceFiles(sourceFiles);
+			notifier.updateProgressDelta(0.25f);
+		} else {
+			IResourceDelta sourceDelta = (IResourceDelta) deltas.get(javaBuilder.currentProject);
+			if (sourceDelta != null)
+				if (!findSourceFiles(sourceDelta)) return false;
+			notifier.updateProgressDelta(0.10f);
 
-		Object[] keyTable = deltas.keyTable;
-		Object[] valueTable = deltas.valueTable;
-		for (int i = 0, l = valueTable.length; i < l; i++) {
-			IResourceDelta delta = (IResourceDelta) valueTable[i];
-			if (delta != null) {
-				IProject p = (IProject) keyTable[i];
-				ClasspathLocation[] classFoldersAndJars = (ClasspathLocation[]) javaBuilder.binaryLocationsPerProject.get(p);
-				if (classFoldersAndJars != null)
-					if (!findAffectedSourceFiles(delta, classFoldersAndJars, p)) return false;
+			Object[] keyTable = deltas.keyTable;
+			Object[] valueTable = deltas.valueTable;
+			for (int i = 0, l = valueTable.length; i < l; i++) {
+				IResourceDelta delta = (IResourceDelta) valueTable[i];
+				if (delta != null) {
+					IProject p = (IProject) keyTable[i];
+					ClasspathLocation[] classFoldersAndJars = (ClasspathLocation[]) javaBuilder.binaryLocationsPerProject.get(p);
+					if (classFoldersAndJars != null)
+						if (!findAffectedSourceFiles(delta, classFoldersAndJars, p)) return false;
+				}
 			}
-		}
-		notifier.updateProgressDelta(0.10f);
+			notifier.updateProgressDelta(0.10f);
 
-		notifier.subTask(Messages.build_analyzingSources); 
-		addAffectedSourceFiles();
-		notifier.updateProgressDelta(0.05f);
+			notifier.subTask(Messages.build_analyzingSources); 
+			addAffectedSourceFiles();
+			notifier.updateProgressDelta(0.05f);
+		}
 
 		this.compileLoop = 0;
 		float increment = 0.40f;
@@ -421,6 +431,18 @@ protected boolean findSourceFiles(IResourceDelta delta) throws CoreException {
 	ArrayList visited = this.makeOutputFolderConsistent ? new ArrayList(sourceLocations.length) : null;
 	for (int i = 0, l = sourceLocations.length; i < l; i++) {
 		ClasspathMultiDirectory md = sourceLocations[i];
+		if (this.makeOutputFolderConsistent && md.hasIndependentOutputFolder && !visited.contains(md.binaryFolder)) {
+			// even a project which acts as its own source folder can have an independent/nested output folder
+			visited.add(md.binaryFolder);
+			IResourceDelta binaryDelta = delta.findMember(md.binaryFolder.getProjectRelativePath());
+			if (binaryDelta != null) {
+				int segmentCount = binaryDelta.getFullPath().segmentCount();
+				IResourceDelta[] children = binaryDelta.getAffectedChildren();
+				for (int j = 0, m = children.length; j < m; j++)
+					if (!checkForClassFileChanges(children[j], md, segmentCount))
+						return false;
+			}
+		}
 		if (md.sourceFolder.equals(javaBuilder.currentProject)) {
 			// skip nested source & output folders when the project is a source folder
 			int segmentCount = delta.getFullPath().segmentCount();
@@ -430,17 +452,6 @@ protected boolean findSourceFiles(IResourceDelta delta) throws CoreException {
 					if (!findSourceFiles(children[j], md, segmentCount))
 						return false;
 		} else {
-			if (this.makeOutputFolderConsistent && md.hasIndependentOutputFolder && !visited.contains(md.binaryFolder)) {
-				visited.add(md.binaryFolder);
-				IResourceDelta binaryDelta = delta.findMember(md.binaryFolder.getProjectRelativePath());
-				if (binaryDelta != null) {
-					int segmentCount = binaryDelta.getFullPath().segmentCount();
-					IResourceDelta[] children = binaryDelta.getAffectedChildren();
-					for (int j = 0, m = children.length; j < m; j++)
-						if (!checkForClassFileChanges(children[j], md, segmentCount))
-							return false;
-				}
-			}
 			IResourceDelta sourceDelta = delta.findMember(md.sourceFolder.getProjectRelativePath());
 			if (sourceDelta != null) {
 				if (sourceDelta.getKind() == IResourceDelta.REMOVED) {
