@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -14,23 +14,24 @@ import java.util.ArrayList;
 import java.util.Map;
 
 import org.eclipse.jdt.core.JavaCore;
-import org.eclipse.jdt.core.compiler.IProblem;
+import org.eclipse.jdt.core.compiler.CategorizedProblem;
+import org.eclipse.jdt.core.compiler.CharOperation;
 import org.eclipse.jdt.core.jdom.*;
 import org.eclipse.jdt.internal.compiler.DocumentElementParser;
 import org.eclipse.jdt.internal.compiler.IDocumentElementRequestor;
 import org.eclipse.jdt.internal.compiler.env.ICompilationUnit;
 import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
 import org.eclipse.jdt.internal.compiler.problem.DefaultProblemFactory;
-import org.eclipse.jdt.internal.core.util.CharArrayOps;
-
 /**
  * The DOMBuilder constructs each type of JDOM document fragment,
  * for the DOMFactory. The DOMBuilder has been separated from the
  * DOMFactory to hide the implmentation of node creation and the
  * public Requestor API methods.
  * 
+ * @deprecated The JDOM was made obsolete by the addition in 2.0 of the more
+ * powerful, fine-grained DOM/AST API found in the 
+ * org.eclipse.jdt.core.dom package.
  */
-
 public class DOMBuilder extends AbstractDOMBuilder implements IDocumentElementRequestor {
 	
 	/**
@@ -55,19 +56,21 @@ public class DOMBuilder extends AbstractDOMBuilder implements IDocumentElementRe
 /**
  * Creates a new DOMBuilder
  */
-public DOMBuilder() {}
+public DOMBuilder() {
+	// Creates a new DOMBuilder
+}
 /**
- * @see IDocumentElementRequestor#acceptImport(int declarationStart, int declarationEnd, int[] javaDocPositions, char[] name, int nameStartPosition, boolean onDemand)
+ * @see IDocumentElementRequestor#acceptImport(int, int, int[], char[], int, boolean, int)
  */
 public void acceptImport(int declarationStart, int declarationEnd, int[] javaDocPositions, char[] name, 
-	int nameStart, boolean onDemand) {
+	int nameStart, boolean onDemand, int modifiers) {
 	int[] sourceRange = {declarationStart, declarationEnd};
 	int[] nameRange = {nameStart, declarationEnd - 1};
 	
 	/* See 1FVII1P */
-	String importName = CharArrayOps.substring(fDocument, nameRange[0], nameRange[1] + 1 - nameRange[0]);
+	String importName = new String(fDocument, nameRange[0], nameRange[1] + 1 - nameRange[0]);
 
-	fNode= new DOMImport(fDocument, sourceRange, importName, nameRange, onDemand);
+	fNode= new DOMImport(fDocument, sourceRange, importName, nameRange, onDemand, modifiers);
 	addChild(fNode);
 	if (fBuildingSingleMember) {
 		fFinishedSingleMember= true;
@@ -87,7 +90,7 @@ public void acceptInitializer(int declarationStart, int declarationEnd, int[] ja
 	}
 
 	int[] modifiersRange = {-1, -1};
-	if (modifiersStart > declarationStart) {
+	if (modifiersStart >= declarationStart) {
 		modifiersRange[0] = modifiersStart;
 		modifiersRange[1] = bodyStart - 1;
 	}
@@ -105,7 +108,7 @@ public void acceptPackage(int declarationStart, int declarationEnd, int[] javaDo
 	int nameStartPosition) {
 	int[] sourceRange = {declarationStart, declarationEnd};
 	int[] nameRange = {nameStartPosition, declarationEnd - 1};
-	fNode= new DOMPackage(fDocument, sourceRange, CharArrayOps.charToString(name), nameRange);
+	fNode= new DOMPackage(fDocument, sourceRange, CharOperation.charToString(name), nameRange);
 	addChild(fNode);
 	if (fBuildingSingleMember) {
 		fFinishedSingleMember= true;
@@ -118,7 +121,7 @@ public void acceptPackage(int declarationStart, int declarationEnd, int[] javaDo
  *
  * @see IDocumentElementRequestor
  */
-public void acceptProblem(IProblem problem){
+public void acceptProblem(CategorizedProblem problem){
 	if (fBuildingSingleMember && fFinishedSingleMember) {
 		return;
 	}
@@ -317,7 +320,7 @@ protected void enterAbstractMethod(int declarationStart, int[] javaDocPositions,
 	int[] commentRange = {-1, -1};
 	if (javaDocPositions != null) {
 		int length = javaDocPositions.length;
-		commentRange[0] = javaDocPositions[0];
+		commentRange[0] = javaDocPositions[length - 2]; // get last javadoc comment (see bug 68772)
 		commentRange[1] = javaDocPositions[length - 1];
 	}
 	int[] modifiersRange = {-1, -1};
@@ -349,11 +352,11 @@ protected void enterAbstractMethod(int declarationStart, int[] javaDocPositions,
 	} else {
 		bodyRange = new int[] {parametersEnd + 1, -1};
 	}
-	fNode = new DOMMethod(fDocument, sourceRange, CharArrayOps.charToString(name), nameRange, commentRange, modifiers, 
-		modifiersRange, isConstructor, CharArrayOps.charToString(returnType), returnTypeRange,
-		CharArrayOps.charcharToString(parameterTypes),
-		CharArrayOps.charcharToString(parameterNames), 
-		parameterRange, CharArrayOps.charcharToString(exceptionTypes), exceptionRange, bodyRange);
+	fNode = new DOMMethod(fDocument, sourceRange, CharOperation.charToString(name), nameRange, commentRange, modifiers, 
+		modifiersRange, isConstructor, CharOperation.charToString(returnType), returnTypeRange,
+		CharOperation.charArrayToStringArray(parameterTypes),
+		CharOperation.charArrayToStringArray(parameterNames), 
+		parameterRange, CharOperation.charArrayToStringArray(exceptionTypes), exceptionRange, bodyRange);
 	addChild(fNode);
 	fStack.push(fNode);
 }
@@ -453,7 +456,7 @@ public void enterField(int declarationStart, int[] javaDocPositions, int modifie
 	int[] commentRange = {-1, -1};
 		if (javaDocPositions != null) {
 			int length = javaDocPositions.length;
-			commentRange[0] = javaDocPositions[0];
+			commentRange[0] = javaDocPositions[length - 2]; // get last javadoc comment (see bug 68772)
 			commentRange[1] = javaDocPositions[length - 1];
 		}
 	int[] modifiersRange = {-1, -1};
@@ -470,8 +473,8 @@ public void enterField(int declarationStart, int[] javaDocPositions, int modifie
 		if (field.fTypeRange[0] == typeStart)
 			isVariableDeclarator = true;
 	}	
-	fNode = new DOMField(fDocument, sourceRange, CharArrayOps.charToString(name), nameRange, commentRange, 
-		modifiers, modifiersRange, typeRange, CharArrayOps.charToString(type), hasInitializer, 
+	fNode = new DOMField(fDocument, sourceRange, CharOperation.charToString(name), nameRange, commentRange, 
+		modifiers, modifiersRange, typeRange, CharOperation.charToString(type), hasInitializer, 
 		initializerRange, isVariableDeclarator);
 	addChild(fNode);
 	fStack.push(fNode);
@@ -552,7 +555,7 @@ protected void enterType(int declarationStart, int[] javaDocPositions,
 		int[] commentRange = {-1, -1};
 		if (javaDocPositions != null) {
 			int length = javaDocPositions.length;
-			commentRange[0] = javaDocPositions[0];
+			commentRange[0] = javaDocPositions[length - 2];  // get last javadoc comment (see bug 68772)
 			commentRange[1] = javaDocPositions[length - 1];
 		}
 		int[] modifiersRange = {-1, -1};
@@ -596,7 +599,7 @@ protected void enterType(int declarationStart, int[] javaDocPositions,
 		int[] closeBodyRange = {-1, -1}; // will be fixed in exit		
 		fNode = new DOMType(fDocument, sourceRange, new String(name), nameRange, commentRange, 
 			modifiers, modifiersRange, typeKeywordRange, superclassRange, extendsKeywordRange, 
-			CharArrayOps.charcharToString(superinterfaces), interfacesRange,
+			CharOperation.charArrayToStringArray(superinterfaces), interfacesRange,
 			implementsKeywordRange, openBodyRange, 
 			closeBodyRange, isClass);
 		addChild(fNode);
