@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -16,26 +16,24 @@ import org.eclipse.jdt.core.ICompilationUnit;
 import org.eclipse.jdt.core.IJavaElement;
 import org.eclipse.jdt.core.IType;
 import org.eclipse.jdt.core.compiler.InvalidInputException;
-import org.eclipse.jdt.core.jdom.IDOMMethod;
-import org.eclipse.jdt.core.jdom.IDOMNode;
-import org.eclipse.jdt.core.jdom.IDOMType;
+import org.eclipse.jdt.core.jdom.*;
 import org.eclipse.jdt.internal.compiler.parser.Scanner;
 import org.eclipse.jdt.internal.compiler.parser.TerminalTokens;
-import org.eclipse.jdt.internal.core.Util;
 import org.eclipse.jdt.internal.core.util.CharArrayBuffer;
-import org.eclipse.jdt.internal.core.util.CharArrayOps;
-
+import org.eclipse.jdt.internal.core.util.Messages;
 /**
  * DOMType provides an implementation of IDOMType.
  *
  * @see IDOMType
  * @see DOMNode
- */
- 
+ * @deprecated The JDOM was made obsolete by the addition in 2.0 of the more
+ * powerful, fine-grained DOM/AST API found in the 
+ * org.eclipse.jdt.core.dom package.
+ */ 
+// TODO (jerome) - add implementation support for 1.5 features
 /* package */ class DOMType extends DOMMember implements IDOMType {
 
 	private static final String[] EMPTY_SUPERINTERFACES = new String[] {};
-
 	/**
 	 * The 'class' or 'interface' keyword if altered
 	 * from the documents contents, otherwise <code>null</code>.
@@ -118,6 +116,24 @@ import org.eclipse.jdt.internal.core.util.CharArrayOps;
 	 */
 	protected String[] fSuperInterfaces= new String[0];
 	
+	/**
+	 * The formal type parameters.
+	 * @since 3.0
+	 */
+	protected String[] fTypeParameters = new String[0];
+
+	/**
+	 * Indicates this type is an enum class.
+	 * @since 3.0
+	 */
+	protected boolean fIsEnum= false;
+	
+	/**
+	 * Indicates this type is an annotatation type (interface).
+	 * @since 3.0
+	 */
+	protected boolean fIsAnnotation= false;
+	
 	/**
 	 * This position is the position of the end of the last line separator before the closing brace starting
 	 * position of the receiver.
@@ -128,7 +144,7 @@ import org.eclipse.jdt.internal.core.util.CharArrayOps;
  * Constructs an empty type node.
  */
 DOMType() {
-
+	// Constructs an empty type node
 }
 /**
  * Creates a new detailed TYPE document fragment on the given range of the document.
@@ -240,7 +256,7 @@ DOMType(char[] document, int[] sourceRange, String name, int[] nameRange, int fl
  */
 public void addSuperInterface(String name) throws IllegalArgumentException {
 	if (name == null) {
-		throw new IllegalArgumentException(Util.bind("dom.addNullInterface")); //$NON-NLS-1$
+		throw new IllegalArgumentException(Messages.dom_addNullInterface); 
 	}
 	if (fSuperInterfaces == null) {
 		fSuperInterfaces= new String[1];
@@ -309,6 +325,9 @@ protected void appendMemberDeclarationContents(CharArrayBuffer  buffer) {
 		} else {
 			if (fSuperclassRange[0] < 0) {
 				buffer.append(' ');
+			} else if (fImplementsRange[0] > 0) {
+				buffer.append(fDocument, fSuperclassRange[1] + 1, fImplementsRange[0] - fSuperclassRange[1] - 1);
+				buffer.append(fDocument, fInterfacesRange[1] + 1, fOpenBodyRange[0] - fInterfacesRange[1] - 1);
 			} else {
 				buffer.append(fDocument, fSuperclassRange[1] + 1, fOpenBodyRange[0] - fSuperclassRange[1] - 1);
 			}
@@ -327,13 +346,17 @@ protected void appendMemberDeclarationContents(CharArrayBuffer  buffer) {
 				buffer.append(fDocument, fInterfacesRange[0], fInterfacesRange[1] + 1 - fInterfacesRange[0]);
 			}
 		} else {
-			buffer.append(fDocument, fNameRange[1] + 1, fOpenBodyRange[0] - fNameRange[1] - 1);
+			if (fImplementsRange[0] < 0) {
+				buffer.append(' ');
+			} else {
+				buffer.append(fDocument, fNameRange[1] + 1, fOpenBodyRange[0] - fNameRange[1] - 1);
+			}
 		}
 	}
 	
 }
 /**
- * @see DOMNode#appendSimpleContents(CharArrayBuffer)
+ * @see DOMMember#appendSimpleContents(CharArrayBuffer)
  */
 protected void appendSimpleContents(CharArrayBuffer buffer) {
 	// append eveything before my name
@@ -383,12 +406,14 @@ public int getInsertionPosition() {
  * @see IDOMNode#getJavaElement
  */
 public IJavaElement getJavaElement(IJavaElement parent) throws IllegalArgumentException {
-	if (parent.getElementType() == IJavaElement.TYPE) {
-		return ((IType)parent).getType(getName());
-	} else if (parent.getElementType() == IJavaElement.COMPILATION_UNIT) {
-		return ((ICompilationUnit)parent).getType(getName());
-	} else {
-		throw new IllegalArgumentException(Util.bind("element.illegalParent")); //$NON-NLS-1$
+	switch (parent.getElementType()) {
+		case IJavaElement.COMPILATION_UNIT:
+			return ((ICompilationUnit)parent).getType(getName());
+		case IJavaElement.TYPE:
+			return ((IType)parent).getType(getName());
+		// Note: creating local/anonymous type is not supported 
+		default:
+			throw new IllegalArgumentException(Messages.element_illegalParent); 
 	}
 }
 /**
@@ -418,7 +443,7 @@ public String getSuperclass() {
 		if (fSuperclass != null) {
 			return fSuperclass;
 		} else {
-			return CharArrayOps.substring(fDocument, fSuperclassRange[0], fSuperclassRange[1] + 1 - fSuperclassRange[0]);
+			return new String(fDocument, fSuperclassRange[0], fSuperclassRange[1] + 1 - fSuperclassRange[0]);
 		}
 	} else {
 		return null;
@@ -623,11 +648,11 @@ void setCloseBodyRangeStart(int start) {
  * as fragmented, since the names of the constructors must reflect the name
  * of this type.
  *
- * @see IDOMNode#setName(char[])
+ * @see IDOMNode#setName(String)
  */
 public void setName(String name) throws IllegalArgumentException {
 	if (name == null) {
-		throw new IllegalArgumentException(Util.bind("element.nullName")); //$NON-NLS-1$
+		throw new IllegalArgumentException(Messages.element_nullName); 
 	}
 	super.setName(name);
 	Enumeration children= getChildren();
@@ -651,7 +676,7 @@ void setOpenBodyRangeStart(int start) {
 	fOpenBodyRange[0] = start;
 }
 /**
- * @see IDOMType#setSuperclass(char[])
+ * @see IDOMType#setSuperclass(String)
  */
 public void setSuperclass(String superclassName) {
 	becomeDetailed();
@@ -665,7 +690,7 @@ public void setSuperclass(String superclassName) {
 public void setSuperInterfaces(String[] names) {
 	becomeDetailed();
 	if (names == null) {
-		throw new IllegalArgumentException(Util.bind("dom.nullInterfaces")); //$NON-NLS-1$
+		throw new IllegalArgumentException(Messages.dom_nullInterfaces); 
 	}
 	fragment();
 	fSuperInterfaces= names;
@@ -715,4 +740,63 @@ protected void shareContents(DOMNode node) {
 public String toString() {
 	return "TYPE: " + getName(); //$NON-NLS-1$
 }
+
+/**
+ * @see IDOMType#getTypeParameters()
+ * @since 3.0
+ */
+public String[] getTypeParameters() {
+	return this.fTypeParameters;
+}
+
+/**
+ * @see IDOMType#isEnum()
+ * @since 3.0
+ */
+public boolean isEnum() {
+	return this.fIsEnum;
+}
+
+/**
+ * @see IDOMType#isAnnotation()
+ * @since 3.0
+ */
+public boolean isAnnotation() {
+	return this.fIsAnnotation;
+}
+
+/**
+ * @see IDOMType#setEnum(boolean)
+ * @since 3.0
+ */
+public void setEnum(boolean b) {
+	this.fIsEnum = b;
+	if (this.fIsEnum) {
+		// enums are always classes with no superclass
+		setClass(true);
+		setSuperclass(null);
+	}
+}
+
+/**
+ * @see IDOMType#setAnnotation(boolean)
+ * @since 3.0
+ */
+public void setAnnotation(boolean b) {
+	this.fIsAnnotation= b;
+	if (this.fIsAnnotation) {
+		// annotation types are always interface with no superclass or superinterfaces
+		setClass(false);
+		setSuperclass(null);
+		setSuperInterfaces(new String[0]);
+	}
+}
+
+/**
+ * @see IDOMType#setTypeParameters(java.lang.String[])
+ * @since 3.0
+ */
+public void setTypeParameters(String[] typeParameters) {
+	this.fTypeParameters = typeParameters;
+}
 }
