@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -20,8 +20,8 @@ import org.eclipse.jdt.core.search.*;
 import org.eclipse.jdt.internal.compiler.ast.*;
 import org.eclipse.jdt.internal.compiler.env.IBinaryType;
 import org.eclipse.jdt.internal.compiler.lookup.*;
+import org.eclipse.jdt.internal.compiler.util.SimpleSet;
 import org.eclipse.jdt.internal.core.JavaElement;
-import org.eclipse.jdt.internal.core.util.SimpleSet;
 
 public class MethodLocator extends PatternLocator {
 
@@ -63,7 +63,7 @@ public void initializePolymorphicSearch(MatchLocator locator) {
 /*
  * Return whether a type name is in pattern all super declaring types names.
  */
-boolean isTypeInSuperDeclaringTypeNames(char[][] typeName) {
+private boolean isTypeInSuperDeclaringTypeNames(char[][] typeName) {
 	if (allSuperDeclaringTypeNames == null) return false;
 	int length = allSuperDeclaringTypeNames.length;
 	for (int i= 0; i<length; i++) {
@@ -116,7 +116,7 @@ public int match(MethodDeclaration node, MatchingNodeSet nodeSet) {
 		int argsLength = args == null ? 0 : args.length;
 		if (length != argsLength) return IMPOSSIBLE_MATCH;
 		for (int i = 0; i < argsLength; i++) {
-			if (!matchesTypeReference(this.pattern.parameterSimpleNames[i], ((Argument) args[i]).type)) {
+			if (args != null && !matchesTypeReference(this.pattern.parameterSimpleNames[i], ((Argument) args[i]).type)) {
 				// Do not return as impossible when source level is at least 1.5
 				if (this.mayBeGeneric) {
 					if (!((InternalSearchPattern)this.pattern).mustResolve) {
@@ -152,7 +152,7 @@ public int match(MessageSend node, MatchingNodeSet nodeSet) {
 	if (!this.pattern.findReferences) return IMPOSSIBLE_MATCH;
 
 	if (!matchesName(this.pattern.selector, node.selector)) return IMPOSSIBLE_MATCH;
-	if (this.pattern.parameterSimpleNames != null && (this.pattern.shouldCountParameter() || ((node.bits & ASTNode.InsideJavadoc) != 0))) {
+	if (this.pattern.parameterSimpleNames != null && (!this.pattern.varargs || ((node.bits & ASTNode.InsideJavadoc) != 0))) {
 		int length = this.pattern.parameterSimpleNames.length;
 		ASTNode[] args = node.arguments;
 		int argsLength = args == null ? 0 : args.length;
@@ -255,7 +255,7 @@ protected int matchMethod(MethodBinding method, boolean skipImpossibleArg) {
 	return level;
 }
 private boolean matchOverriddenMethod(ReferenceBinding type, MethodBinding method, MethodBinding matchMethod) {
-	if (type == null) return false;
+	if (type == null || this.pattern.selector == null) return false;
 
 	// matches superclass
 	if (!type.isInterface() && !CharOperation.equals(type.compoundName, TypeConstants.JAVA_LANG_OBJECT)) {
@@ -268,7 +268,7 @@ private boolean matchOverriddenMethod(ReferenceBinding type, MethodBinding metho
 					if (matchMethod == null) {
 						if (methodParametersEqualsPattern(methods[i].original())) return true;
 					} else {
-						if (methodsHaveSameParameters(methods[i].original(), matchMethod)) return true;
+						if (methods[i].original().areParametersEqual(matchMethod)) return true;
 					}
 				}
 			}
@@ -291,7 +291,7 @@ private boolean matchOverriddenMethod(ReferenceBinding type, MethodBinding metho
 					if (matchMethod == null) {
 						if (methodParametersEqualsPattern(methods[j].original())) return true;
 					} else {
-						if (methodsHaveSameParameters(methods[j].original(), matchMethod)) return true;
+						if (methods[j].original().areParametersEqual(matchMethod)) return true;
 					}
 				}
 			}
@@ -363,7 +363,7 @@ void matchReportReference(MessageSend messageSend, MatchLocator locator, MethodB
 		// Update match regarding declaring class type arguments
 		if (methodBinding.declaringClass.isParameterizedType() || methodBinding.declaringClass.isRawType()) {
 			ParameterizedTypeBinding parameterizedBinding = (ParameterizedTypeBinding)methodBinding.declaringClass;
-			if (!this.pattern.hasTypeArguments() && this.pattern.hasMethodArguments()) {
+			if (!this.pattern.hasTypeArguments() && this.pattern.hasMethodArguments() || parameterizedBinding.isParameterizedWithOwnVariables()) {
 				// special case for pattern which defines method arguments but not its declaring type
 				// in this case, we do not refine accuracy using declaring type arguments...!
 			} else {
@@ -387,7 +387,9 @@ void matchReportReference(MessageSend messageSend, MatchLocator locator, MethodB
 		isParameterized = true;
 		if (methodBinding.declaringClass.isParameterizedType() || methodBinding.declaringClass.isRawType()) {
 			ParameterizedTypeBinding parameterizedBinding = (ParameterizedTypeBinding)methodBinding.declaringClass;
-			updateMatch(parameterizedBinding, this.pattern.getTypeArguments(), this.pattern.hasTypeParameters(), 0, locator);
+			if (!parameterizedBinding.isParameterizedWithOwnVariables()) {
+				updateMatch(parameterizedBinding, this.pattern.getTypeArguments(), this.pattern.hasTypeParameters(), 0, locator);
+			}
 		} else if (this.pattern.hasTypeArguments()) {
 			match.setRule(SearchPattern.R_ERASURE_MATCH);
 		}
@@ -427,9 +429,8 @@ void matchReportReference(MessageSend messageSend, MatchLocator locator, MethodB
 private boolean methodParametersEqualsPattern(MethodBinding method) {
 	TypeBinding[] methodParameters = method.parameters;
 
-	int length = methodParameters==null ? 0 : methodParameters.length;
-	int patternLength = this.pattern.parameterSimpleNames==null ? 0 : this.pattern.parameterSimpleNames.length;
-	if (length != patternLength) return false;
+	int length = methodParameters.length;
+	if (length != this.pattern.parameterSimpleNames.length) return false;
 
 	for (int i = 0; i < length; i++) {
 		char[] paramQualifiedName = qualifiedPattern(this.pattern.parameterSimpleNames[i], this.pattern.parameterQualifications[i]);
@@ -439,21 +440,6 @@ private boolean methodParametersEqualsPattern(MethodBinding method) {
 	}
 	return true;
 }
-private boolean methodsHaveSameParameters(MethodBinding method, MethodBinding matchMethod) {
-	TypeBinding[] methodParameters = method.parameters;
-	TypeBinding[] matchMethodParameters = matchMethod.parameters;
-
-	int length = methodParameters==null ? 0 : methodParameters.length;
-	int matchLength = matchMethodParameters==null ? 0 : matchMethodParameters.length;
-	if (length != matchLength) return false;
-
-	for (int i = 0; i < length; i++) {
-		if (!CharOperation.equals(methodParameters[i].readableName(), matchMethodParameters[i].readableName(), this.isCaseSensitive)) {
-			return false;
-		}
-	}
-	return true;
-}
 public SearchMatch newDeclarationMatch(ASTNode reference, IJavaElement element, Binding elementBinding, int accuracy, int length, MatchLocator locator) {
 	if (elementBinding != null) {
 		MethodBinding methodBinding = (MethodBinding) elementBinding;
@@ -467,18 +453,20 @@ public SearchMatch newDeclarationMatch(ASTNode reference, IJavaElement element,
 				}
 				return null;
 			}
-			// If method binding override a method in super hierarchy which original match pattern then report match
 			if (matchOverriddenMethod(methodBinding.declaringClass, methodBinding, null)) {
 				this.methodDeclarationsWithInvalidParam.put(reference, Boolean.TRUE);
 				return super.newDeclarationMatch(reference, element, elementBinding, accuracy, length, locator);
 			}
-			// If pattern binding override a method in super hierarchy which original match method binding then report match
-			MethodBinding patternBinding = locator.getMethodBinding(this.pattern);
-			if (patternBinding != null) {
-				if (matchOverriddenMethod(patternBinding.declaringClass, patternBinding, methodBinding)) {
-					this.methodDeclarationsWithInvalidParam.put(reference, Boolean.TRUE);
-					return super.newDeclarationMatch(reference, element, elementBinding, accuracy, length, locator);
+			if (isTypeInSuperDeclaringTypeNames(methodBinding.declaringClass.compoundName)) {
+				MethodBinding patternBinding = locator.getMethodBinding(this.pattern);
+				if (patternBinding != null) {
+					if (!matchOverriddenMethod(patternBinding.declaringClass, patternBinding, methodBinding)) {
+						this.methodDeclarationsWithInvalidParam.put(reference, Boolean.FALSE);
+						return null;
+					}
 				}
+				this.methodDeclarationsWithInvalidParam.put(reference, Boolean.TRUE);
+				return super.newDeclarationMatch(reference, element, elementBinding, accuracy, length, locator);
 			}
 			this.methodDeclarationsWithInvalidParam.put(reference, Boolean.FALSE);
 			return null;
@@ -495,22 +483,35 @@ protected void reportDeclaration(MethodBinding methodBinding, MatchLocator locat
 	if (type == null) return; // case of a secondary type
 
 	char[] bindingSelector = methodBinding.selector;
+	boolean isBinary = type.isBinary();
+	IMethod method = null;
 	TypeBinding[] parameters = methodBinding.original().parameters;
 	int parameterLength = parameters.length;
-	String[] parameterTypes = new String[parameterLength];
-	for (int i = 0; i  < parameterLength; i++) {
-		char[] typeName = parameters[i].shortReadableName();
-		if (parameters[i].isMemberType()) {
-			typeName = CharOperation.subarray(typeName, CharOperation.indexOf('.', typeName)+1, typeName.length);
+	if (isBinary) {
+		char[][] parameterTypes = new char[parameterLength][];
+		for (int i = 0; i<parameterLength; i++) {
+			char[] typeName = parameters[i].qualifiedSourceName();
+			for (int j=0, dim=parameters[i].dimensions(); j<dim; j++) {
+				typeName = CharOperation.concat(typeName, new char[] {'[', ']'});
+			}
+			parameterTypes[i] = typeName;
 		}
-		parameterTypes[i] = Signature.createTypeSignature(typeName, false);
+		method = locator.createBinaryMethodHandle(type, methodBinding.selector, parameterTypes);
+	} else {
+		String[] parameterTypes = new String[parameterLength];
+		for (int i = 0; i  < parameterLength; i++) {
+			char[] typeName = parameters[i].shortReadableName();
+			if (parameters[i].isMemberType()) {
+				typeName = CharOperation.subarray(typeName, CharOperation.indexOf('.', typeName)+1, typeName.length);
+			}
+			parameterTypes[i] = Signature.createTypeSignature(typeName, false);
+		}
+		method = type.getMethod(new String(bindingSelector), parameterTypes);
 	}
-	IMethod method = type.getMethod(new String(bindingSelector), parameterTypes);
-	if (knownMethods.includes(method)) return;
+	if (method == null || knownMethods.includes(method)) return;
 
 	knownMethods.add(method);
 	IResource resource = type.getResource();
-	boolean isBinary = type.isBinary();
 	IBinaryType info = null;
 	if (isBinary) {
 		if (resource == null)
@@ -594,7 +595,9 @@ public int resolveLevel(Binding binding) {
 }
 protected int resolveLevel(MessageSend messageSend) {
 	MethodBinding method = messageSend.binding;
-	if (method == null) return INACCURATE_MATCH;
+	if (method == null) {
+		return INACCURATE_MATCH;
+	}
 	if (messageSend.resolvedType == null) {
 		// Closest match may have different argument numbers when ProblemReason is NotFound
 		// see MessageSend#resolveType(BlockScope)
