@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -22,10 +22,10 @@ import java.util.Set;
 import org.eclipse.core.resources.IWorkspaceRoot;
 import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
+import org.eclipse.core.runtime.IAdapterFactory;
 import org.eclipse.core.runtime.IAdapterManager;
 import org.eclipse.core.runtime.IConfigurationElement;
 import org.eclipse.core.runtime.IExtensionPoint;
-import org.eclipse.core.runtime.IPluginDescriptor;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Platform;
@@ -33,58 +33,83 @@ import org.eclipse.core.runtime.Status;
 import org.eclipse.debug.ui.DebugUITools;
 import org.eclipse.debug.ui.IDebugModelPresentation;
 import org.eclipse.debug.ui.ILaunchConfigurationTab;
-import org.eclipse.jdt.core.IElementChangedListener;
 import org.eclipse.jdt.core.IJavaModel;
 import org.eclipse.jdt.core.IJavaProject;
-import org.eclipse.jdt.core.IMethod;
+import org.eclipse.jdt.core.IMember;
 import org.eclipse.jdt.core.IPackageFragment;
 import org.eclipse.jdt.core.JavaCore;
 import org.eclipse.jdt.core.JavaModelException;
+import org.eclipse.jdt.debug.core.IJavaBreakpoint;
 import org.eclipse.jdt.debug.core.IJavaDebugTarget;
 import org.eclipse.jdt.debug.core.IJavaHotCodeReplaceListener;
 import org.eclipse.jdt.debug.core.IJavaStackFrame;
 import org.eclipse.jdt.debug.core.IJavaThread;
+import org.eclipse.jdt.debug.core.IJavaThreadGroup;
 import org.eclipse.jdt.debug.core.IJavaVariable;
 import org.eclipse.jdt.debug.core.JDIDebugModel;
-import org.eclipse.jdt.debug.eval.IAstEvaluationEngine;
 import org.eclipse.jdt.debug.ui.IJavaDebugUIConstants;
+import org.eclipse.jdt.internal.debug.ui.breakpoints.ExceptionInspector;
+import org.eclipse.jdt.internal.debug.ui.breakpoints.JavaBreakpointTypeAdapterFactory;
+import org.eclipse.jdt.internal.debug.ui.classpath.ClasspathEntryAdapterFactory;
 import org.eclipse.jdt.internal.debug.ui.display.JavaInspectExpression;
+import org.eclipse.jdt.internal.debug.ui.monitors.JavaContendedMonitor;
+import org.eclipse.jdt.internal.debug.ui.monitors.JavaDebugElementAdapterFactory;
+import org.eclipse.jdt.internal.debug.ui.monitors.JavaOwnedMonitor;
+import org.eclipse.jdt.internal.debug.ui.monitors.JavaOwningThread;
+import org.eclipse.jdt.internal.debug.ui.monitors.JavaWaitingThread;
 import org.eclipse.jdt.internal.debug.ui.snippeteditor.SnippetFileDocumentProvider;
-import org.eclipse.jdt.launching.JavaRuntime;
+import org.eclipse.jdt.internal.debug.ui.sourcelookup.JavaDebugShowInAdapterFactory;
+import org.eclipse.jdt.internal.debug.ui.threadgroups.JavaDebugElementLabelAdapterFactory;
+import org.eclipse.jdt.internal.debug.ui.variables.ColumnPresentationAdapterFactory;
+import org.eclipse.jdt.internal.launching.DefaultProjectClasspathEntry;
 import org.eclipse.jdt.launching.sourcelookup.IJavaSourceLocation;
 import org.eclipse.jdt.ui.JavaElementLabelProvider;
+import org.eclipse.jdt.ui.PreferenceConstants;
+import org.eclipse.jdt.ui.text.JavaTextTools;
 import org.eclipse.jface.dialogs.ErrorDialog;
-import org.eclipse.jface.dialogs.ProgressMonitorDialog;
 import org.eclipse.jface.operation.IRunnableWithProgress;
-import org.eclipse.jface.preference.IPreferenceStore;
+import org.eclipse.jface.preference.IPreferenceNode;
+import org.eclipse.jface.preference.IPreferencePage;
+import org.eclipse.jface.preference.PreferenceDialog;
+import org.eclipse.jface.preference.PreferenceManager;
+import org.eclipse.jface.preference.PreferenceNode;
 import org.eclipse.jface.resource.ImageRegistry;
+import org.eclipse.jface.window.Window;
+import org.eclipse.swt.custom.BusyIndicator;
 import org.eclipse.swt.widgets.Display;
 import org.eclipse.swt.widgets.Shell;
 import org.eclipse.ui.IWorkbenchPage;
 import org.eclipse.ui.IWorkbenchWindow;
+import org.eclipse.ui.PlatformUI;
 import org.eclipse.ui.dialogs.ElementListSelectionDialog;
-import org.eclipse.ui.editors.text.FileDocumentProvider;
 import org.eclipse.ui.plugin.AbstractUIPlugin;
 import org.eclipse.ui.texteditor.IDocumentProvider;
+import org.osgi.framework.Bundle;
+import org.osgi.framework.BundleContext;
 
 /**
  * Plug-in class for the org.eclipse.jdt.debug.ui plug-in.
  */
 public class JDIDebugUIPlugin extends AbstractUIPlugin {
 
+	/**
+	 * Unique identifier constant (value <code>"org.eclipse.jdt.debug.ui"</code>)
+	 * for the JDI Debug plug-in.
+	 */
+	private static final String PI_JDI_DEBUG = "org.eclipse.jdt.debug.ui"; //$NON-NLS-1$
+	
 	/**
 	 * Java Debug UI plug-in instance
 	 */
 	private static JDIDebugUIPlugin fgPlugin;
 	
-	private FileDocumentProvider fSnippetDocumentProvider;
+	private IDocumentProvider fSnippetDocumentProvider;
 	
 	private ImageDescriptorRegistry fImageDescriptorRegistry;
 	
-	private JavaEvaluationEngineManager fEvaluationEngineManager;
-	
 	private ActionFilterAdapterFactory fActionFilterAdapterFactory;
 	private JavaSourceLocationWorkbenchAdapterFactory fSourceLocationAdapterFactory;
+	private JavaBreakpointWorkbenchAdapterFactory fBreakpointAdapterFactory;
 	
 	private IDebugModelPresentation fUtilPresentation;
 	
@@ -92,7 +117,6 @@ public class JDIDebugUIPlugin extends AbstractUIPlugin {
 	 * Java Debug UI listeners
 	 */
 	private IJavaHotCodeReplaceListener fHCRListener;
-	private IElementChangedListener fJavaModelListener;
 	
 	// Map of VMInstallTypeIDs to IConfigurationElements
 	protected Map fVmInstallTypePageMap;
@@ -104,10 +128,15 @@ public class JDIDebugUIPlugin extends AbstractUIPlugin {
 	private boolean fShuttingDown= false;
 
 	/**
-	 * @see Plugin(IPluginDescriptor)
+	 * Singleton text tools for debug plug-in.
 	 */
-	public JDIDebugUIPlugin(IPluginDescriptor descriptor) {
-		super(descriptor);
+	private JavaTextTools fTextTools = null;
+	
+	/**
+	 * @see Plugin()
+	 */
+	public JDIDebugUIPlugin() {
+		super();
 		setDefault(this);
 	}
 	
@@ -133,13 +162,7 @@ public class JDIDebugUIPlugin extends AbstractUIPlugin {
 	 * Convenience method which returns the unique identifier of this plugin.
 	 */
 	public static String getUniqueIdentifier() {
-		if (getDefault() == null) {
-			// If the default instance is not yet initialized,
-			// return a static identifier. This identifier must
-			// match the plugin id defined in plugin.xml
-			return "org.eclipse.jdt.debug.ui"; //$NON-NLS-1$
-		}
-		return getDefault().getDescriptor().getUniqueIdentifier();
+		return PI_JDI_DEBUG;
 	}
 	
 	/**
@@ -166,7 +189,7 @@ public class JDIDebugUIPlugin extends AbstractUIPlugin {
 	 * @param e the exception to be logged
 	 */	
 	public static void log(Throwable e) {
-		log(new Status(IStatus.ERROR, getUniqueIdentifier(), IJavaDebugUIConstants.INTERNAL_ERROR, DebugUIMessages.getString("JDIDebugUIPlugin.Internal_Error_1"), e));  //$NON-NLS-1$
+		log(new Status(IStatus.ERROR, getUniqueIdentifier(), IJavaDebugUIConstants.INTERNAL_ERROR, DebugUIMessages.JDIDebugUIPlugin_Internal_Error_1, e));  
 	}
 	
 	/**
@@ -200,37 +223,25 @@ public class JDIDebugUIPlugin extends AbstractUIPlugin {
 		return null;
 	}	
 	
-	/* (non - Javadoc)
-	 * Method declared in AbstractUIPlugin
+	/* (non-Javadoc)
+	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#createImageRegistry()
 	 */
 	protected ImageRegistry createImageRegistry() {
 		return JavaDebugImages.getImageRegistry();
 	}	
 	
 	public IDocumentProvider getSnippetDocumentProvider() {
-		if (fSnippetDocumentProvider == null)
+		if (fSnippetDocumentProvider == null) {
 			fSnippetDocumentProvider= new SnippetFileDocumentProvider();
-		return fSnippetDocumentProvider;
-	}	
-	
-	/**
-	 * Logs the given message if in debug mode.
-	 * 
-	 * @param String message to log
-	 */
-	public static void logDebugMessage(String message) {
-		if (getDefault().isDebugging()) {
-			// this message is intentionally not internationalized, as an exception may
-			// be due to the resource bundle itself
-			log(new Status(IStatus.ERROR, getUniqueIdentifier(), IJavaDebugUIConstants.INTERNAL_ERROR, "Internal message logged from JDT Debug UI: " + message, null)); //$NON-NLS-1$
 		}
+		return fSnippetDocumentProvider;
 	}
 	
 	public static void errorDialog(String message, IStatus status) {
 		log(status);
 		Shell shell = getActiveWorkbenchShell();
 		if (shell != null) {
-			ErrorDialog.openError(shell, DebugUIMessages.getString("JDIDebugUIPlugin.Error_1"), message, status); //$NON-NLS-1$
+			ErrorDialog.openError(shell, DebugUIMessages.JDIDebugUIPlugin_Error_1, message, status); 
 		}
 	}
 	
@@ -242,85 +253,117 @@ public class JDIDebugUIPlugin extends AbstractUIPlugin {
 		Shell shell = getActiveWorkbenchShell();
 		if (shell != null) {
 			IStatus status= new Status(IStatus.ERROR, getUniqueIdentifier(), IJavaDebugUIConstants.INTERNAL_ERROR, "Error logged from JDT Debug UI: ", t); //$NON-NLS-1$	
-			ErrorDialog.openError(shell, DebugUIMessages.getString("JDIDebugUIPlugin.Error_1"), message, status); //$NON-NLS-1$
+			ErrorDialog.openError(shell, DebugUIMessages.JDIDebugUIPlugin_Error_1, message, status); 
 		}
 	}
 	
 	/**
-	 * @see AbstractUIPlugin#initializeDefaultPreferences
+	 * Creates an extension.  If the extension plugin has not
+	 * been loaded a busy cursor will be activated during the duration of
+	 * the load.
+	 *
+	 * @param element the config element defining the extension
+	 * @param classAttribute the name of the attribute carrying the class
+	 * @return the extension object
 	 */
-	protected void initializeDefaultPreferences(IPreferenceStore store) {
-		if(isShuttingDown()) {
-			return;
+	public static Object createExtension(final IConfigurationElement element, final String classAttribute) throws CoreException {
+		// If plugin has been loaded create extension.
+		// Otherwise, show busy cursor then create extension.
+		Bundle bundle = Platform.getBundle(element.getContributor().getName());
+		if (bundle.getState() == Bundle.ACTIVE) {
+			return element.createExecutableExtension(classAttribute);
 		}
-		// JavaDebugPreferencePage
-		store.setDefault(IJDIPreferencesConstants.PREF_SUSPEND_ON_COMPILATION_ERRORS, true);
-		store.setDefault(IJDIPreferencesConstants.PREF_SUSPEND_ON_UNCAUGHT_EXCEPTIONS, true);
-		store.setDefault(IJDIPreferencesConstants.PREF_ALERT_HCR_FAILED, true);
-		store.setDefault(IJDIPreferencesConstants.PREF_ALERT_HCR_NOT_SUPPORTED, true);
-		store.setDefault(IJDIPreferencesConstants.PREF_ALERT_OBSOLETE_METHODS, true);
-
-		store.setDefault(IJDIPreferencesConstants.PREF_SHOW_QUALIFIED_NAMES, false);
-		store.setDefault(JDIDebugModel.PREF_HCR_WITH_COMPILATION_ERRORS, true);
-
-		store.setDefault(JDIDebugModel.PREF_REQUEST_TIMEOUT, JDIDebugModel.DEF_REQUEST_TIMEOUT);
-		store.setDefault(JavaRuntime.PREF_CONNECT_TIMEOUT, JavaRuntime.DEF_CONNECT_TIMEOUT);
 		
-		// JavaStepFilterPreferencePage
-		store.setDefault(IJDIPreferencesConstants.PREF_ACTIVE_FILTERS_LIST, "java.lang.ClassLoader"); //$NON-NLS-1$
-		store.setDefault(IJDIPreferencesConstants.PREF_INACTIVE_FILTERS_LIST, "com.ibm.*,com.sun.*,java.*,javax.*,org.omg.*,sun.*,sunw.*"); //$NON-NLS-1$
-				
-		store.setDefault(IJDIPreferencesConstants.PREF_SHOW_CONSTANTS, false);
-		store.setDefault(IJDIPreferencesConstants.PREF_SHOW_STATIC_VARIALBES, false);
-		store.setDefault(IJDIPreferencesConstants.PREF_SHOW_CHAR, false);
-		store.setDefault(IJDIPreferencesConstants.PREF_SHOW_HEX, false);
-		store.setDefault(IJDIPreferencesConstants.PREF_SHOW_UNSIGNED, false);
-	}
+		final Object [] ret = new Object[1];
+		final CoreException [] exc = new CoreException[1];
+		BusyIndicator.showWhile(null, new Runnable() {
+			public void run() {
+				try {
+					ret[0] = element.createExecutableExtension(classAttribute);
+				} catch (CoreException e) {
+					exc[0] = e;
+				}
+			}
+		});
+		if (exc[0] != null) {
+			throw exc[0];
+		}
+		return ret[0];
+	}	
 	
-	/**
-	 * @see AbstractUIPlugin#startup()
+	/* (non-Javadoc)
+	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
 	 */
-	public void startup() throws CoreException {
-		super.startup();
+	public void start(BundleContext context) throws Exception {
+		super.start(context);
 		JavaDebugOptionsManager.getDefault().startup();
 		
 		IAdapterManager manager= Platform.getAdapterManager();
 		fActionFilterAdapterFactory= new ActionFilterAdapterFactory();
-		manager.registerAdapters(fActionFilterAdapterFactory, IMethod.class);
+		manager.registerAdapters(fActionFilterAdapterFactory, IMember.class);
 		manager.registerAdapters(fActionFilterAdapterFactory, IJavaVariable.class);
 		manager.registerAdapters(fActionFilterAdapterFactory, IJavaStackFrame.class);
 		manager.registerAdapters(fActionFilterAdapterFactory, IJavaThread.class);
 		manager.registerAdapters(fActionFilterAdapterFactory, JavaInspectExpression.class);
-		manager.registerAdapters(fActionFilterAdapterFactory, JavaWatchExpression.class);
 		fSourceLocationAdapterFactory = new JavaSourceLocationWorkbenchAdapterFactory();
 		manager.registerAdapters(fSourceLocationAdapterFactory, IJavaSourceLocation.class);
+		fBreakpointAdapterFactory= new JavaBreakpointWorkbenchAdapterFactory();
+		manager.registerAdapters(fBreakpointAdapterFactory, IJavaBreakpoint.class);
+        IAdapterFactory typeFactory = new JavaBreakpointTypeAdapterFactory();
+        manager.registerAdapters(typeFactory, IJavaBreakpoint.class);
+        
+        IAdapterFactory monitorFactory = new JavaDebugElementAdapterFactory();
+        manager.registerAdapters(monitorFactory, IJavaThread.class);
+        manager.registerAdapters(monitorFactory, JavaContendedMonitor.class);
+        manager.registerAdapters(monitorFactory, JavaOwnedMonitor.class);
+        manager.registerAdapters(monitorFactory, JavaOwningThread.class);
+        manager.registerAdapters(monitorFactory, JavaWaitingThread.class);
+        IAdapterFactory labelFactory = new JavaDebugElementLabelAdapterFactory();
+        manager.registerAdapters(labelFactory, IJavaThreadGroup.class);
+        manager.registerAdapters(labelFactory, IJavaDebugTarget.class);
+        
+        IAdapterFactory showInFactory = new JavaDebugShowInAdapterFactory();
+        manager.registerAdapters(showInFactory, IJavaStackFrame.class);
+        
+        IAdapterFactory columnFactory = new ColumnPresentationAdapterFactory();
+        manager.registerAdapters(columnFactory, IJavaVariable.class);
+        manager.registerAdapters(columnFactory, IJavaStackFrame.class);
+        
+        IAdapterFactory entryFactory = new ClasspathEntryAdapterFactory();
+        manager.registerAdapters(entryFactory, DefaultProjectClasspathEntry.class);
 		
-		fEvaluationEngineManager= new JavaEvaluationEngineManager();
-		fJavaModelListener= new JavaModelListener();
-		JavaCore.addElementChangedListener(fJavaModelListener);
 		fHCRListener= new JavaHotCodeReplaceListener();
 		JDIDebugModel.addHotCodeReplaceListener(fHCRListener);
+		
+		
+		// initialize exception inspector handler
+		new ExceptionInspector();
 	}
 	
-	/**
-	 * @see AbstractUIPlugin#shutdown()
+	/* (non-Javadoc)
+	 * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
 	 */
-	public void shutdown() throws CoreException {
-		setShuttingDown(true);
-		JDIDebugModel.removeHotCodeReplaceListener(fHCRListener);
-		JavaCore.removeElementChangedListener(fJavaModelListener);
-		JavaDebugOptionsManager.getDefault().shutdown();
-		if (fImageDescriptorRegistry != null) {
-			fImageDescriptorRegistry.dispose();
+	public void stop(BundleContext context) throws Exception {
+		try {
+			setShuttingDown(true);
+			JDIDebugModel.removeHotCodeReplaceListener(fHCRListener);
+			JavaDebugOptionsManager.getDefault().shutdown();
+			if (fImageDescriptorRegistry != null) {
+				fImageDescriptorRegistry.dispose();
+			}
+			IAdapterManager manager= Platform.getAdapterManager();
+			manager.unregisterAdapters(fActionFilterAdapterFactory);
+			manager.unregisterAdapters(fSourceLocationAdapterFactory);
+			manager.unregisterAdapters(fBreakpointAdapterFactory);
+			if (fUtilPresentation != null) {
+				fUtilPresentation.dispose();
+			} 
+			if (fTextTools != null) {
+				fTextTools.dispose();
+			}
+		} finally {
+			super.stop(context);
 		}
-		fEvaluationEngineManager.dispose();
-		IAdapterManager manager= Platform.getAdapterManager();
-		manager.unregisterAdapters(fActionFilterAdapterFactory);
-		manager.unregisterAdapters(fSourceLocationAdapterFactory);
-		if (fUtilPresentation != null) {
-			fUtilPresentation.dispose();
-		}
-		super.shutdown();
 	}
 	
 	/**
@@ -368,19 +411,6 @@ public class JDIDebugUIPlugin extends AbstractUIPlugin {
 		return display;		
 	}
 	
-	/**
-	 * Returns an evaluation engine for the given project in the given debug target.
-	 * 
-	 * @see JavaEvaluationEngineManager#getEvaluationEngine(IJavaProject, IJavaDebugTarget)
-	 * 
-	 * @param project java project
-	 * @param target the debug target
-	 * @return evalaution engine
-	 */
-	public IAstEvaluationEngine getEvaluationEngine(IJavaProject project, IJavaDebugTarget target) {
-		return fEvaluationEngineManager.getEvaluationEngine(project, target);
-	}
-	
 	/**
 	 * Utility method to create and return a selection dialog that allows
 	 * selection of a specific Java package.  Empty packages are not returned.
@@ -397,15 +427,19 @@ public class JDIDebugUIPlugin extends AbstractUIPlugin {
 		}
 		final IJavaProject[] projects= originals;
 		final JavaModelException[] exception= new JavaModelException[1];
-		ProgressMonitorDialog monitor= new ProgressMonitorDialog(shell);
+		final boolean[] monitorCanceled = new boolean[] {false};
 		IRunnableWithProgress r= new IRunnableWithProgress() {
-			public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
+			public void run(IProgressMonitor monitor) {
 				try {
 					Set packageNameSet= new HashSet();
-					monitor.beginTask(DebugUIMessages.getString("JDIDebugUIPlugin.Searching_1"), projects.length); //$NON-NLS-1$
+					monitor.beginTask(DebugUIMessages.JDIDebugUIPlugin_Searching_1, projects.length); 
 					for (int i = 0; i < projects.length; i++) {						
 						IPackageFragment[] pkgs= projects[i].getPackageFragments();	
 						for (int j = 0; j < pkgs.length; j++) {
+							if (monitor.isCanceled()) {
+								monitorCanceled[0] = true;
+								return;
+							}
 							IPackageFragment pkg = pkgs[j];
 							if (!pkg.hasChildren() && (pkg.getNonJavaResources().length > 0)) {
 								continue;
@@ -427,7 +461,7 @@ public class JDIDebugUIPlugin extends AbstractUIPlugin {
 			}
 		};
 		try {
-			monitor.run(false, false, r);	
+			PlatformUI.getWorkbench().getProgressService().busyCursorWhile(r);
 		} catch (InvocationTargetException e) {
 			JDIDebugUIPlugin.log(e);
 		} catch (InterruptedException e) {
@@ -436,8 +470,12 @@ public class JDIDebugUIPlugin extends AbstractUIPlugin {
 		if (exception[0] != null) {
 			throw exception[0];
 		}
+		if (monitorCanceled[0]) {
+			return null;
+		}
+		
 		int flags= JavaElementLabelProvider.SHOW_DEFAULT;
-		ElementListSelectionDialog dialog= new ElementListSelectionDialog(shell, new JavaElementLabelProvider(flags));
+		PackageSelectionDialog dialog= new PackageSelectionDialog(shell, new JavaElementLabelProvider(flags));
 		dialog.setIgnoreCase(false);
 		dialog.setElements(packageList.toArray()); // XXX inefficient
 		return dialog;
@@ -457,7 +495,7 @@ public class JDIDebugUIPlugin extends AbstractUIPlugin {
 			try {
 				tab = (ILaunchConfigurationTab) configElement.createExecutableExtension("class"); //$NON-NLS-1$
 			} catch(CoreException ce) {			 
-				log(new Status(Status.ERROR, getUniqueIdentifier(), IJavaDebugUIConstants.INTERNAL_ERROR, DebugUIMessages.getString("JDIDebugUIPlugin.An_error_occurred_retrieving_a_VMInstallType_page_1"), ce)); //$NON-NLS-1$
+				log(new Status(IStatus.ERROR, getUniqueIdentifier(), IJavaDebugUIConstants.INTERNAL_ERROR, DebugUIMessages.JDIDebugUIPlugin_An_error_occurred_retrieving_a_VMInstallType_page_1, ce)); 
 			} 
 		}
 		return tab;
@@ -466,8 +504,7 @@ public class JDIDebugUIPlugin extends AbstractUIPlugin {
 	protected void initializeVMInstallTypePageMap() {
 		fVmInstallTypePageMap = new HashMap(10);
 
-		IPluginDescriptor descriptor= JDIDebugUIPlugin.getDefault().getDescriptor();
-		IExtensionPoint extensionPoint= descriptor.getExtensionPoint(IJavaDebugUIConstants.EXTENSION_POINT_VM_INSTALL_TYPE_PAGE);
+		IExtensionPoint extensionPoint= Platform.getExtensionRegistry().getExtensionPoint(getUniqueIdentifier(), IJavaDebugUIConstants.EXTENSION_POINT_VM_INSTALL_TYPE_PAGE);
 		IConfigurationElement[] infos= extensionPoint.getConfigurationElements();
 		for (int i = 0; i < infos.length; i++) {
 			String id = infos[i].getAttribute("vmInstallTypeID"); //$NON-NLS-1$
@@ -487,5 +524,39 @@ public class JDIDebugUIPlugin extends AbstractUIPlugin {
 		}
 		return fUtilPresentation;
 	}
+	
+	/**
+	 * Displays the given preference page.
+	 * 
+	 * @param id pref page id
+	 * @param page pref page
+	 */
+	public static void showPreferencePage(String id, IPreferencePage page) {
+		final IPreferenceNode targetNode = new PreferenceNode(id, page);
+		
+		PreferenceManager manager = new PreferenceManager();
+		manager.addToRoot(targetNode);
+		final PreferenceDialog dialog = new PreferenceDialog(JDIDebugUIPlugin.getActiveWorkbenchShell(), manager);
+		final boolean [] result = new boolean[] { false };
+		BusyIndicator.showWhile(JDIDebugUIPlugin.getStandardDisplay(), new Runnable() {
+			public void run() {
+				dialog.create();
+				dialog.setMessage(targetNode.getLabelText());
+				result[0]= (dialog.open() == Window.OK);
+			}
+		});		
+	}	
+	
+	/**
+	 * Returns the text tools used by this plug-in
+	 * 
+	 * @return
+	 */
+	public JavaTextTools getJavaTextTools() {
+		if (fTextTools == null) {
+			fTextTools = new JavaTextTools(PreferenceConstants.getPreferenceStore());
+		}
+		return fTextTools;
+	}
 }
 
