@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -13,7 +13,7 @@ package org.eclipse.jdt.internal.debug.ui.actions;
 
 
 import java.lang.reflect.InvocationTargetException;
-import java.text.MessageFormat;
+import com.ibm.icu.text.MessageFormat;
 import java.util.Iterator;
 
 import org.eclipse.core.resources.IResource;
@@ -34,7 +34,6 @@ import org.eclipse.debug.ui.IDebugView;
 import org.eclipse.jdt.core.IJavaElement;
 import org.eclipse.jdt.core.IJavaProject;
 import org.eclipse.jdt.core.JavaCore;
-import org.eclipse.jdt.debug.core.IJavaArray;
 import org.eclipse.jdt.debug.core.IJavaDebugTarget;
 import org.eclipse.jdt.debug.core.IJavaObject;
 import org.eclipse.jdt.debug.core.IJavaStackFrame;
@@ -61,10 +60,14 @@ import org.eclipse.jface.text.BadLocationException;
 import org.eclipse.jface.text.IDocument;
 import org.eclipse.jface.text.IRegion;
 import org.eclipse.jface.text.ITextSelection;
+import org.eclipse.jface.text.ITextViewer;
 import org.eclipse.jface.text.Region;
 import org.eclipse.jface.viewers.ISelection;
 import org.eclipse.jface.viewers.ISelectionProvider;
 import org.eclipse.jface.viewers.IStructuredSelection;
+import org.eclipse.swt.custom.StyledText;
+import org.eclipse.swt.graphics.GC;
+import org.eclipse.swt.graphics.Point;
 import org.eclipse.swt.widgets.Display;
 import org.eclipse.swt.widgets.Shell;
 import org.eclipse.ui.IEditorActionDelegate;
@@ -146,7 +149,7 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 									try {
 										if (!var.getName().equals("this")) { //$NON-NLS-1$
 											IValue value= var.getValue();
-											if (value instanceof IJavaObject && !(value instanceof IJavaArray)) {
+											if (value instanceof IJavaObject) {
 												return (IJavaObject)value;
 											}
 										} 
@@ -155,7 +158,7 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 									}
 								} else if (selection instanceof JavaInspectExpression) {
 									IValue value= ((JavaInspectExpression)selection).getValue();
-									if (value instanceof IJavaObject && !(value instanceof IJavaArray)) {
+									if (value instanceof IJavaObject) {
 										return (IJavaObject)value;
 									}
 								}
@@ -216,14 +219,14 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 		final IJavaObject object = getObjectContext();		
 		final IJavaStackFrame stackFrame= getStackFrameContext();
 		if (stackFrame == null) {
-			reportError(ActionMessages.Evaluate_error_message_stack_frame_context); //$NON-NLS-1$
+			reportError(ActionMessages.Evaluate_error_message_stack_frame_context); 
 			return;
 		}
 		
 		// check for nested evaluation
 		IJavaThread thread = (IJavaThread)stackFrame.getThread();
 		if (thread.isPerformingEvaluation()) {
-			reportError(ActionMessages.EvaluateAction_Cannot_perform_nested_evaluations__1); //$NON-NLS-1$
+			reportError(ActionMessages.EvaluateAction_Cannot_perform_nested_evaluations__1); 
 			return;
 		}
 		
@@ -256,10 +259,10 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
                             throw new InvocationTargetException(e, getExceptionMessage(e));
                         }
                     }
-                    throw new InvocationTargetException(null, ActionMessages.Evaluate_error_message_src_context); //$NON-NLS-1$
+                    throw new InvocationTargetException(null, ActionMessages.Evaluate_error_message_src_context); 
                 }
                 // thread not suspended
-                throw new InvocationTargetException(null, ActionMessages.EvaluateAction_Thread_not_suspended___unable_to_perform_evaluation__1); //$NON-NLS-1$
+                throw new InvocationTargetException(null, ActionMessages.EvaluateAction_Thread_not_suspended___unable_to_perform_evaluation__1); 
             }
         };
         
@@ -439,7 +442,7 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 					try {
 						view= page.showView(IJavaDebugUIConstants.ID_DISPLAY_VIEW);
 					} catch (PartInitException e) {
-						JDIDebugUIPlugin.errorDialog(ActionMessages.EvaluateAction_Cannot_open_Display_view, e); //$NON-NLS-1$
+						JDIDebugUIPlugin.errorDialog(ActionMessages.EvaluateAction_Cannot_open_Display_view, e); 
 					} finally {
 						page.activate(activePart);
 					}
@@ -510,13 +513,13 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 		IDataDisplay dataDisplay= getDirectDataDisplay();
 		if (dataDisplay != null) {
 			if (message.length() != 0) {
-				dataDisplay.displayExpressionValue(MessageFormat.format(ActionMessages.EvaluateAction__evaluation_failed__Reason, new String[] {format(message)})); //$NON-NLS-1$
+				dataDisplay.displayExpressionValue(MessageFormat.format(ActionMessages.EvaluateAction__evaluation_failed__Reason, new String[] {format(message)})); 
 			} else {
-				dataDisplay.displayExpressionValue(ActionMessages.EvaluateAction__evaluation_failed__1); //$NON-NLS-1$
+				dataDisplay.displayExpressionValue(ActionMessages.EvaluateAction__evaluation_failed__1); 
 			}
 		} else {
 			Status status= new Status(IStatus.ERROR, JDIDebugUIPlugin.getUniqueIdentifier(), IStatus.ERROR, message, null);
-			ErrorDialog.openError(getShell(), ActionMessages.Evaluate_error_title_eval_problems, null, status); //$NON-NLS-1$
+			ErrorDialog.openError(getShell(), ActionMessages.Evaluate_error_title_eval_problems, null, status); 
 		}
 	}
 	
@@ -544,9 +547,9 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 			}
 			return ce.getStatus().getMessage();
 		}
-		String message= MessageFormat.format(ActionMessages.Evaluate_error_message_direct_exception, new Object[] { exception.getClass() }); //$NON-NLS-1$
+		String message= MessageFormat.format(ActionMessages.Evaluate_error_message_direct_exception, new Object[] { exception.getClass() }); 
 		if (exception.getMessage() != null) {
-			message= MessageFormat.format(ActionMessages.Evaluate_error_message_exception_pattern, new Object[] { message, exception.getMessage() }); //$NON-NLS-1$
+			message= MessageFormat.format(ActionMessages.Evaluate_error_message_exception_pattern, new Object[] { message, exception.getMessage() }); 
 		}
 		return message;
 	}
@@ -557,7 +560,7 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 	protected static String getInvocationExceptionMessage(com.sun.jdi.InvocationException exception) {
 			InvocationException ie= exception;
 			ObjectReference ref= ie.exception();
-			return MessageFormat.format(ActionMessages.Evaluate_error_message_wrapped_exception, new Object[] { ref.referenceType().name() }); //$NON-NLS-1$
+			return MessageFormat.format(ActionMessages.Evaluate_error_message_wrapped_exception, new Object[] { ref.referenceType().name() }); 
 	}
 	
 	protected String getErrorMessage(IEvaluationResult result) {
@@ -575,7 +578,7 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 			if (i == 0) {
 				message= msg;
 			} else {
-				message= MessageFormat.format(ActionMessages.Evaluate_error_problem_append_pattern, new Object[] { message, msg }); //$NON-NLS-1$
+				message= MessageFormat.format(ActionMessages.Evaluate_error_problem_append_pattern, new Object[] { message, msg }); 
 			}
 		}
 		return message;
@@ -754,6 +757,7 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 	public void snippetStateChanged(JavaSnippetEditor editor) {
 		if (editor != null && !editor.isEvaluating() && editor.getFile() != null) {
 			update();
+			getAction().setEnabled(getSelectedObject() != null);
 		} else {
 			getAction().setEnabled(false);
 		}
@@ -783,4 +787,25 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 	protected IRegion getRegion() {
 		return fRegion;
 	}
+	
+	/**
+	 * Computes an anchor point for a popup dialog on top of a text viewer.
+	 * 
+	 * @param viewer
+	 * @return desired anchor point
+	 */
+	public static Point getPopupAnchor(ITextViewer viewer) {
+		StyledText textWidget = viewer.getTextWidget();
+        Point docRange = textWidget.getSelectionRange();
+        int midOffset = docRange.x + (docRange.y / 2);
+        Point point = textWidget.getLocationAtOffset(midOffset);
+        point = textWidget.toDisplay(point);
+
+        GC gc = new GC(textWidget);
+        gc.setFont(textWidget.getFont());
+        int height = gc.getFontMetrics().getHeight();
+        gc.dispose();
+        point.y += height;
+        return point;
+	}
 }
