@@ -1,12 +1,13 @@
-/**********************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others. All rights reserved.   This
- * program and the accompanying materials are made available under the terms of
- * the Eclipse Public License v1.0 which accompanies this distribution, and is
- * available at http://www.eclipse.org/legal/epl-v10.html
- * 
- * Contributors: 
+/*******************************************************************************
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors:
  * IBM - Initial API and implementation
- **********************************************************************/
+ *******************************************************************************/
 package org.eclipse.jdt.internal.debug.ui.breakpoints;
 
 import org.eclipse.osgi.util.NLS;
@@ -16,20 +17,22 @@ public class BreakpointMessages extends NLS {
 
 	public static String AddExceptionAction_0;
 	public static String AddExceptionAction_1;
+	public static String AddExceptionAction_2;
 
 	public static String AddExceptionDialog_15;
 	public static String AddExceptionDialog_16;
-	public static String AddExceptionDialog_17;
-	public static String AddExceptionDialog_21;
-	public static String AddExceptionDialog_0;
+	public static String AddExceptionDialogExtension_0;
 
 	public static String AddClassPrepareBreakpointAction_0;
 	public static String AddClassPrepareBreakpointAction_1;
 	public static String AddClassPrepareBreakpointAction_2;
+	public static String AddClassPrepareBreakpointAction_3;
+	
 	public static String JavaBreakpointTypeAdapterFactory_0;
 
 	static {
 		// load message values from bundle file
 		NLS.initializeMessages(BUNDLE_NAME, BreakpointMessages.class);
 	}
-}
\ No newline at end of file
+	
+}
