@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -13,7 +13,7 @@ package org.eclipse.jdt.internal.debug.ui.jres;
 import java.io.File;
 import java.io.IOException;
 import java.lang.reflect.InvocationTargetException;
-import java.text.MessageFormat;
+import com.ibm.icu.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.HashSet;
 import java.util.Iterator;
@@ -22,6 +22,7 @@ import java.util.Set;
 
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
+import org.eclipse.core.runtime.ListenerList;
 import org.eclipse.debug.internal.ui.SWTUtil;
 import org.eclipse.jdt.internal.debug.ui.JDIDebugUIPlugin;
 import org.eclipse.jdt.launching.AbstractVMInstallType;
@@ -35,7 +36,6 @@ import org.eclipse.jface.dialogs.IDialogSettings;
 import org.eclipse.jface.dialogs.MessageDialog;
 import org.eclipse.jface.dialogs.ProgressMonitorDialog;
 import org.eclipse.jface.operation.IRunnableWithProgress;
-import org.eclipse.jface.util.ListenerList;
 import org.eclipse.jface.viewers.CheckStateChangedEvent;
 import org.eclipse.jface.viewers.CheckboxTableViewer;
 import org.eclipse.jface.viewers.DoubleClickEvent;
@@ -50,21 +50,16 @@ import org.eclipse.jface.viewers.ITableLabelProvider;
 import org.eclipse.jface.viewers.LabelProvider;
 import org.eclipse.jface.viewers.SelectionChangedEvent;
 import org.eclipse.jface.viewers.StructuredSelection;
-import org.eclipse.jface.viewers.TableLayout;
 import org.eclipse.jface.viewers.Viewer;
 import org.eclipse.jface.viewers.ViewerSorter;
 import org.eclipse.jface.window.Window;
 import org.eclipse.swt.SWT;
-import org.eclipse.swt.events.ControlAdapter;
-import org.eclipse.swt.events.ControlEvent;
 import org.eclipse.swt.events.KeyAdapter;
 import org.eclipse.swt.events.KeyEvent;
 import org.eclipse.swt.events.SelectionAdapter;
 import org.eclipse.swt.events.SelectionEvent;
 import org.eclipse.swt.graphics.Font;
 import org.eclipse.swt.graphics.Image;
-import org.eclipse.swt.graphics.Point;
-import org.eclipse.swt.graphics.Rectangle;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
 import org.eclipse.swt.widgets.Button;
@@ -108,15 +103,9 @@ public class InstalledJREsBlock implements IAddVMDialogRequestor, ISelectionProv
 	private Button fAddButton;
 	private Button fRemoveButton;
 	private Button fEditButton;
+	private Button fCopyButton;
 	private Button fSearchButton;	
-	
-	// column weights
-	private float fWeight1 = 1/3F;
-	private float fWeight2 = 1/3F;
-	
-	// ignore column re-sizing when the table is being resized
-	private boolean fResizingTable = false; 
-	
+    
 	// index of column used for sorting
 	private int fSortColumn = 0;
 	
@@ -129,6 +118,8 @@ public class InstalledJREsBlock implements IAddVMDialogRequestor, ISelectionProv
 	 * Previous selection
 	 */
 	private ISelection fPrevSelection = new StructuredSelection();
+
+    private Table fTable;
 			
 	// Make sure that VMStandin ids are unique if multiple calls to System.currentTimeMillis()
 	// happen very quickly
@@ -137,18 +128,14 @@ public class InstalledJREsBlock implements IAddVMDialogRequestor, ISelectionProv
 	/** 
 	 * Content provider to show a list of JREs
 	 */ 
-	class JREsContentProvider implements IStructuredContentProvider {	
-	
+	class JREsContentProvider implements IStructuredContentProvider {		
 		public Object[] getElements(Object input) {
 			return fVMs.toArray();
 		}
-
 		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
 		}
-
 		public void dispose() {
 		}
-	
 	}
 	
 	/**
@@ -164,6 +151,9 @@ public class InstalledJREsBlock implements IAddVMDialogRequestor, ISelectionProv
 				IVMInstall vm= (IVMInstall)element;
 				switch(columnIndex) {
 					case 0:
+						if (isContributed(vm)) {
+							return MessageFormat.format(JREMessages.InstalledJREsBlock_19, new String[]{vm.getName()});
+						}
 						return vm.getName();
 					case 1:
 						return vm.getInstallLocation().getAbsolutePath();
@@ -249,49 +239,47 @@ public class InstalledJREsBlock implements IAddVMDialogRequestor, ISelectionProv
 		GridData data;
 				
 		Label tableLabel = new Label(parent, SWT.NONE);
-		tableLabel.setText(JREMessages.InstalledJREsBlock_15); //$NON-NLS-1$
+		tableLabel.setText(JREMessages.InstalledJREsBlock_15); 
 		data = new GridData();
 		data.horizontalSpan = 2;
 		tableLabel.setLayoutData(data);
 		tableLabel.setFont(font);
 				
-		Table table= new Table(parent, SWT.CHECK | SWT.BORDER | SWT.MULTI | SWT.FULL_SELECTION);
+		fTable= new Table(parent, SWT.CHECK | SWT.BORDER | SWT.MULTI | SWT.FULL_SELECTION);
 		
 		data= new GridData(GridData.FILL_BOTH);
-		table.setLayoutData(data);
-		table.setFont(font);
+        data.widthHint = 450;
+		fTable.setLayoutData(data);
+		fTable.setFont(font);
 				
-		table.setHeaderVisible(true);
-		table.setLinesVisible(true);		
+		fTable.setHeaderVisible(true);
+		fTable.setLinesVisible(true);		
 
-		TableLayout tableLayout= new TableLayout();
-		table.setLayout(tableLayout);
-
-		TableColumn column1= new TableColumn(table, SWT.NULL);
-		column1.setText(JREMessages.InstalledJREsBlock_0); //$NON-NLS-1$
+		TableColumn column1= new TableColumn(fTable, SWT.NULL);
+		column1.setText(JREMessages.InstalledJREsBlock_0); 
 		column1.addSelectionListener(new SelectionAdapter() {
 			public void widgetSelected(SelectionEvent e) {
 				sortByName();
 			}
 		});
 	
-		TableColumn column2= new TableColumn(table, SWT.NULL);
-		column2.setText(JREMessages.InstalledJREsBlock_1); //$NON-NLS-1$
+		TableColumn column2= new TableColumn(fTable, SWT.NULL);
+		column2.setText(JREMessages.InstalledJREsBlock_1); 
 		column2.addSelectionListener(new SelectionAdapter() {
 			public void widgetSelected(SelectionEvent e) {
 				sortByLocation();
 			}
 		});
-		
-		TableColumn column3= new TableColumn(table, SWT.NULL);
-		column3.setText(JREMessages.InstalledJREsBlock_2); //$NON-NLS-1$
+        
+		TableColumn column3= new TableColumn(fTable, SWT.NULL);
+		column3.setText(JREMessages.InstalledJREsBlock_2); 
 		column3.addSelectionListener(new SelectionAdapter() {
 			public void widgetSelected(SelectionEvent e) {
 				sortByType();
 			}
 		});
-		
-		fVMList= new CheckboxTableViewer(table);			
+
+		fVMList= new CheckboxTableViewer(fTable);			
 		fVMList.setLabelProvider(new VMLabelProvider());
 		fVMList.setContentProvider(new JREsContentProvider());
 		// by default, sort by name
@@ -320,7 +308,7 @@ public class InstalledJREsBlock implements IAddVMDialogRequestor, ISelectionProv
 				}
 			}
 		});
-		table.addKeyListener(new KeyAdapter() {
+		fTable.addKeyListener(new KeyAdapter() {
 			public void keyPressed(KeyEvent event) {
 				if (event.character == SWT.DEL && event.stateMask == 0) {
 					removeVMs();
@@ -336,21 +324,28 @@ public class InstalledJREsBlock implements IAddVMDialogRequestor, ISelectionProv
 		buttons.setLayout(layout);
 		buttons.setFont(font);
 		
-		fAddButton = createPushButton(buttons, JREMessages.InstalledJREsBlock_3); //$NON-NLS-1$
+		fAddButton = createPushButton(buttons, JREMessages.InstalledJREsBlock_3); 
 		fAddButton.addListener(SWT.Selection, new Listener() {
 			public void handleEvent(Event evt) {
 				addVM();
 			}
 		});
 		
-		fEditButton= createPushButton(buttons, JREMessages.InstalledJREsBlock_4); //$NON-NLS-1$
+		fEditButton= createPushButton(buttons, JREMessages.InstalledJREsBlock_4); 
 		fEditButton.addListener(SWT.Selection, new Listener() {
 			public void handleEvent(Event evt) {
 				editVM();
 			}
 		});
 		
-		fRemoveButton= createPushButton(buttons, JREMessages.InstalledJREsBlock_5); //$NON-NLS-1$
+		fCopyButton = createPushButton(buttons, JREMessages.InstalledJREsBlock_16); 
+		fCopyButton.addListener(SWT.Selection, new Listener() {
+			public void handleEvent(Event evt) {
+				copyVM();
+			}
+		});
+		
+		fRemoveButton= createPushButton(buttons, JREMessages.InstalledJREsBlock_5); 
 		fRemoveButton.addListener(SWT.Selection, new Listener() {
 			public void handleEvent(Event evt) {
 				removeVMs();
@@ -366,20 +361,69 @@ public class InstalledJREsBlock implements IAddVMDialogRequestor, ISelectionProv
 		gd.heightHint= 4;
 		separator.setLayoutData(gd);
 		
-		fSearchButton = createPushButton(buttons, JREMessages.InstalledJREsBlock_6); //$NON-NLS-1$
+		fSearchButton = createPushButton(buttons, JREMessages.InstalledJREsBlock_6); 
 		fSearchButton.addListener(SWT.Selection, new Listener() {
 			public void handleEvent(Event evt) {
 				search();
 			}
 		});		
 		
-		configureTableResizing(parent, buttons, table, column1, column2, column3);
-		
 		fillWithWorkspaceJREs();
 		enableButtons();
 		fAddButton.setEnabled(JavaRuntime.getVMInstallTypes().length > 0);
 	}
 	
+	/**
+	 * Adds a duplicate of the selected VM to the block 
+	 * @since 3.2
+	 */
+	protected void copyVM() {
+        IStructuredSelection selection = (IStructuredSelection) fVMList.getSelection();
+        Iterator it = selection.iterator();
+
+        ArrayList newEntries = new ArrayList();
+        while (it.hasNext()) {
+            IVMInstall selectedVM = (IVMInstall) it.next();
+
+            // duplicate & add vm
+            VMStandin standin = new VMStandin(selectedVM, createUniqueId(selectedVM.getVMInstallType()));
+            standin.setName(generateName(selectedVM.getName()));
+            AddVMDialog dialog = new AddVMDialog(this, getShell(), JavaRuntime.getVMInstallTypes(), standin);
+            dialog.setTitle(JREMessages.InstalledJREsBlock_18);
+            if (dialog.open() != Window.OK) {
+                return;
+            }
+            newEntries.add(standin);
+            fVMs.add(standin);
+        }
+        fVMList.refresh();
+        fVMList.setSelection(new StructuredSelection(newEntries.toArray()));
+    }
+
+	/**
+	 * Compares the given name against current names and adds the appropriate numerical 
+	 * suffix to ensure that it is unique.
+	 * @param name the name with which to ensure uniqueness 
+	 * @return the unique version of the given name
+	 * @since 3.2
+	 */
+	public String generateName(String name){
+            if (!isDuplicateName(name)) {
+                return name;
+            }
+            
+            if (name.matches(".*\\(\\d*\\)")) { //$NON-NLS-1$
+                int start = name.lastIndexOf('(');
+                int end = name.lastIndexOf(')');
+                String stringInt = name.substring(start+1, end);
+                int numericValue = Integer.parseInt(stringInt);
+                String newName = name.substring(0, start+1) + (numericValue+1) + ")"; //$NON-NLS-1$
+                return generateName(newName);
+            } else {
+                return generateName(name + " (1)"); //$NON-NLS-1$
+            }
+        }
+	
 	/**
 	 * Fire current selection
 	 */
@@ -462,88 +506,32 @@ public class InstalledJREsBlock implements IAddVMDialogRequestor, ISelectionProv
 	}
 		
 	private void enableButtons() {
-		int selectionCount= ((IStructuredSelection)fVMList.getSelection()).size();
+		IStructuredSelection selection = (IStructuredSelection) fVMList.getSelection();
+		int selectionCount= selection.size();
 		fEditButton.setEnabled(selectionCount == 1);
-		fRemoveButton.setEnabled(selectionCount > 0 && selectionCount < fVMList.getTable().getItemCount());
+		fCopyButton.setEnabled(selectionCount > 0);
+		if (selectionCount > 0 && selectionCount < fVMList.getTable().getItemCount()) {
+			Iterator iterator = selection.iterator();
+			while (iterator.hasNext()) {
+				IVMInstall install = (IVMInstall)iterator.next();
+				if (isContributed(install)) {
+					fRemoveButton.setEnabled(false);
+					return;
+				}
+			}
+			fRemoveButton.setEnabled(true);
+		} else {
+			fRemoveButton.setEnabled(false);
+		}
 	}	
 	
+	private boolean isContributed(IVMInstall install) {
+		return JavaRuntime.isContributedVMInstall(install.getId());
+	}
+	
 	protected Button createPushButton(Composite parent, String label) {
 		return SWTUtil.createPushButton(parent, label, null);
-	}
-	
-	/**
-	 * Correctly resizes the table so no phantom columns appear
-	 */
-	protected void configureTableResizing(final Composite parent, final Composite buttons, final Table table, final TableColumn column1, final TableColumn column2, final TableColumn column3) {
-		parent.addControlListener(new ControlAdapter() {
-			public void controlResized(ControlEvent e) {
-				resizeTable(parent, buttons, table, column1, column2, column3);
-			}
-		}); 
-		table.addListener(SWT.Paint, new Listener() {
-			public void handleEvent(Event event) {
-				table.removeListener(SWT.Paint, this);
-				resizeTable(parent, buttons, table, column1, column2, column3);
-			}
-		});
-		column1.addControlListener(new ControlAdapter() {
-			public void controlResized(ControlEvent e) {
-				if (column1.getWidth() > 0 && !fResizingTable) {
-					fWeight1 = getColumnWeight(0);
-				}
-			}
-		});
-		column2.addControlListener(new ControlAdapter() {
-			public void controlResized(ControlEvent e) {
-				if (column2.getWidth() > 0 && !fResizingTable) {
-					fWeight2 = getColumnWeight(1);
-				}
-			}
-		});
 	}	
-
-	private void resizeTable(Composite parent, Composite buttons, Table table, TableColumn column1, TableColumn column2, TableColumn column3) {
-		fResizingTable = true;
-		int parentWidth = -1;
-		int parentHeight = -1;
-		if (parent.isVisible()) {
-			Rectangle area = parent.getClientArea();
-			parentWidth = area.width;
-			parentHeight = area.height;
-		} else {
-			Point parentSize = parent.computeSize(SWT.DEFAULT, SWT.DEFAULT);
-			parentWidth = parentSize.x;
-			parentHeight = parentSize.y;
-		}
-		Point preferredSize = table.computeSize(SWT.DEFAULT, SWT.DEFAULT);
-		int width = parentWidth - 2 * table.getBorderWidth();
-		if (preferredSize.y > parentHeight) {
-			// Subtract the scrollbar width from the total column width
-			// if a vertical scrollbar will be required
-			Point vBarSize = table.getVerticalBar().getSize();
-			width -= vBarSize.x;
-		}
-		width-= buttons.getSize().x;
-		Point oldSize = table.getSize();
-		if (oldSize.x > width) {
-			// table is getting smaller so make the columns
-			// smaller first and then resize the table to
-			// match the client area width
-			column1.setWidth(Math.round(width * fWeight1));
-			column2.setWidth(Math.round(width * fWeight2));
-			column3.setWidth(width - (column1.getWidth() + column2.getWidth()));
-			table.setSize(width, parentHeight);
-		} else {
-			// table is getting bigger so make the table
-			// bigger first and then make the columns wider
-			// to match the client area width
-			table.setSize(width, parentHeight);
-			column1.setWidth(Math.round(width * fWeight1));
-			column2.setWidth(Math.round(width * fWeight2));
-			column3.setWidth(width - (column1.getWidth() + column2.getWidth()));
-		 }
-		 fResizingTable = false;		
-	}
 	
 	/**
 	 * Returns this block's control
@@ -582,7 +570,7 @@ public class InstalledJREsBlock implements IAddVMDialogRequestor, ISelectionProv
 	 */
 	private void addVM() {
 		AddVMDialog dialog= new AddVMDialog(this, getShell(), JavaRuntime.getVMInstallTypes(), null);
-		dialog.setTitle(JREMessages.InstalledJREsBlock_7); //$NON-NLS-1$
+		dialog.setTitle(JREMessages.InstalledJREsBlock_7); 
 		if (dialog.open() != Window.OK) {
 			return;
 		}
@@ -616,12 +604,17 @@ public class InstalledJREsBlock implements IAddVMDialogRequestor, ISelectionProv
 		if (vm == null) {
 			return;
 		}
-		AddVMDialog dialog= new AddVMDialog(this, getShell(), JavaRuntime.getVMInstallTypes(), vm);
-		dialog.setTitle(JREMessages.InstalledJREsBlock_8); //$NON-NLS-1$
-		if (dialog.open() != Window.OK) {
-			return;
+		if (isContributed(vm)) {
+			VMDetailsDialog dialog= new VMDetailsDialog(getShell(), vm);
+			dialog.open();
+		} else {
+			AddVMDialog dialog= new AddVMDialog(this, getShell(), JavaRuntime.getVMInstallTypes(), vm);
+			dialog.setTitle(JREMessages.InstalledJREsBlock_8); 
+			if (dialog.open() != Window.OK) {
+				return;
+			}
+			fVMList.refresh(vm);
 		}
-		fVMList.refresh(vm);
 	}
 	
 	private void removeVMs() {
@@ -666,8 +659,8 @@ public class InstalledJREsBlock implements IAddVMDialogRequestor, ISelectionProv
 		
 		// choose a root directory for the search 
 		DirectoryDialog dialog = new DirectoryDialog(getShell());
-		dialog.setMessage(JREMessages.InstalledJREsBlock_9); //$NON-NLS-1$
-		dialog.setText(JREMessages.InstalledJREsBlock_10); //$NON-NLS-1$
+		dialog.setMessage(JREMessages.InstalledJREsBlock_9); 
+		dialog.setText(JREMessages.InstalledJREsBlock_10); 
 		String path = dialog.open();
 		if (path == null) {
 			return;
@@ -687,7 +680,7 @@ public class InstalledJREsBlock implements IAddVMDialogRequestor, ISelectionProv
 
 		IRunnableWithProgress r = new IRunnableWithProgress() {
 			public void run(IProgressMonitor monitor) {
-				monitor.beginTask(JREMessages.InstalledJREsBlock_11, IProgressMonitor.UNKNOWN); //$NON-NLS-1$
+				monitor.beginTask(JREMessages.InstalledJREsBlock_11, IProgressMonitor.UNKNOWN); 
 				search(rootDir, locations, types, exstingLocations, monitor);
 				monitor.done();
 			}
@@ -704,7 +697,7 @@ public class InstalledJREsBlock implements IAddVMDialogRequestor, ISelectionProv
 		}
 		
 		if (locations.isEmpty()) {
-			MessageDialog.openInformation(getShell(), JREMessages.InstalledJREsBlock_12, MessageFormat.format(JREMessages.InstalledJREsBlock_13, new String[]{path})); //$NON-NLS-1$ //$NON-NLS-2$
+			MessageDialog.openInformation(getShell(), JREMessages.InstalledJREsBlock_12, MessageFormat.format(JREMessages.InstalledJREsBlock_13, new String[]{path})); // 
 		} else {
 			iter = locations.iterator();
 			Iterator iter2 = types.iterator();
@@ -774,7 +767,7 @@ public class InstalledJREsBlock implements IAddVMDialogRequestor, ISelectionProv
 			}
 			File file = new File(directory, names[i]);
 			try {
-				monitor.subTask(MessageFormat.format(JREMessages.InstalledJREsBlock_14, new String[]{Integer.toString(found.size()), file.getCanonicalPath()})); //$NON-NLS-1$
+				monitor.subTask(MessageFormat.format(JREMessages.InstalledJREsBlock_14, new String[]{Integer.toString(found.size()), file.getCanonicalPath()})); 
 			} catch (IOException e) {
 			}		
 			IVMInstallType[] vmTypes = JavaRuntime.getVMInstallTypes();	
@@ -848,23 +841,13 @@ public class InstalledJREsBlock implements IAddVMDialogRequestor, ISelectionProv
 	 * @param qualifier key qualifier
 	 */
 	public void saveColumnSettings(IDialogSettings settings, String qualifier) {
-		for (int i = 0; i < 2; i++) {
-			//persist the first 2 column weights
-			settings.put(qualifier + ".column" + i, getColumnWeight(i));	 //$NON-NLS-1$
+        int columnCount = fTable.getColumnCount();
+		for (int i = 0; i < columnCount; i++) {
+			settings.put(qualifier + ".columnWidth" + i, fTable.getColumn(i).getWidth());	 //$NON-NLS-1$
 		}
 		settings.put(qualifier + ".sortColumn", fSortColumn); //$NON-NLS-1$
 	}
 	
-	private float getColumnWeight(int col) {
-		Table table = fVMList.getTable();
-		int tableWidth = table.getSize().x;
-		int columnWidth= table.getColumn(col).getWidth();
-		if (tableWidth > columnWidth) {
-			return ((float)columnWidth) / tableWidth;
-		}
-		return 1/3F;
-	}
-	
 	/**
 	 * Restore table settings from the given dialog store using the
 	 * given key.
@@ -873,9 +856,8 @@ public class InstalledJREsBlock implements IAddVMDialogRequestor, ISelectionProv
 	 * @param qualifier key to restore settings from
 	 */
 	public void restoreColumnSettings(IDialogSettings settings, String qualifier) {
-		fWeight1 = restoreColumnWeight(settings, qualifier, 0);
-		fWeight2 = restoreColumnWeight(settings, qualifier, 1);
 		fVMList.getTable().layout(true);
+        restoreColumnWidths(settings, qualifier);
 		try {
 			fSortColumn = settings.getInt(qualifier + ".sortColumn"); //$NON-NLS-1$
 		} catch (NumberFormatException e) {
@@ -894,13 +876,21 @@ public class InstalledJREsBlock implements IAddVMDialogRequestor, ISelectionProv
 		}
 	}
 	
-	private float restoreColumnWeight(IDialogSettings settings, String qualifier, int col) {		
-		try {
-			return settings.getFloat(qualifier + ".column" + col); //$NON-NLS-1$
-		} catch (NumberFormatException e) {
-			return 1/3F;
-		}
-
+	private void restoreColumnWidths(IDialogSettings settings, String qualifier) {
+        int columnCount = fTable.getColumnCount();
+        for (int i = 0; i < columnCount; i++) {
+            int width = -1;
+            
+            try {
+                width = settings.getInt(qualifier + ".columnWidth" + i); //$NON-NLS-1$
+            } catch (NumberFormatException e) {}
+            
+            if (width <= 0) {
+                fTable.getColumn(i).pack();
+            } else {
+                fTable.getColumn(i).setWidth(width);
+            }
+        }
 	}
 	
 	/**
