@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -12,7 +12,7 @@ package org.eclipse.jdt.internal.debug.ui.launcher;
 
 
 import java.lang.reflect.InvocationTargetException;
-import java.text.MessageFormat;
+import com.ibm.icu.text.MessageFormat;
 import org.eclipse.core.resources.IMarker;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
@@ -53,14 +53,14 @@ public class SelectSystemLibraryQuickFix extends JREResolution {
 		try {
 			handleContainerResolutionError(fUnboundPath, fProject);
 		} catch (CoreException e) {
-			JDIDebugUIPlugin.errorDialog(LauncherMessages.JREContainerResolution_Unable_to_update_classpath_1, e.getStatus());  //$NON-NLS-1$
+			JDIDebugUIPlugin.errorDialog(LauncherMessages.JREContainerResolution_Unable_to_update_classpath_1, e.getStatus());  
 		}
 	}
 	
 	protected void handleContainerResolutionError(final IPath unboundPath, final IJavaProject project) throws CoreException {			
 		
-		String title = LauncherMessages.JREResolution_Select_System_Library_1; //$NON-NLS-1$
-		String message = MessageFormat.format(LauncherMessages.JREResolution_Select_a_system_library_to_use_when_building__0__2, new String[]{project.getElementName()}); //$NON-NLS-1$
+		String title = LauncherMessages.JREResolution_Select_System_Library_1; 
+		String message = MessageFormat.format(LauncherMessages.JREResolution_Select_a_system_library_to_use_when_building__0__2, new String[]{project.getElementName()}); 
 		
 		final IVMInstall vm = chooseVMInstall(title, message);
 		if (vm == null) {
@@ -112,7 +112,7 @@ public class SelectSystemLibraryQuickFix extends JREResolution {
 			throw new CoreException(new Status(IStatus.ERROR,
 				JDIDebugUIPlugin.getUniqueIdentifier(),
 				IJavaDebugUIConstants.INTERNAL_ERROR,
-				LauncherMessages.JREContainerResolution_An_exception_occurred_while_updating_the_classpath__1, e.getTargetException())); //$NON-NLS-1$
+				LauncherMessages.JREContainerResolution_An_exception_occurred_while_updating_the_classpath__1, e.getTargetException())); 
 		} catch (InterruptedException e) {
 			// cancelled
 		}
@@ -121,7 +121,7 @@ public class SelectSystemLibraryQuickFix extends JREResolution {
 	 * @see org.eclipse.ui.IMarkerResolution#getLabel()
 	 */
 	public String getLabel() {
-		return MessageFormat.format(LauncherMessages.JREContainerResolution_Select_a_system_library_to_use_when_building__0__2, new String[]{fProject.getElementName()}); //$NON-NLS-1$
+		return MessageFormat.format(LauncherMessages.JREContainerResolution_Select_a_system_library_to_use_when_building__0__2, new String[]{fProject.getElementName()}); 
 	}
 
 }
