@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -15,6 +15,7 @@ import org.eclipse.core.runtime.CoreException;
 import org.eclipse.debug.core.ILaunchConfiguration;
 import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
 import org.eclipse.debug.ui.StringVariableSelectionDialog;
+import org.eclipse.jdt.debug.ui.launchConfigurations.JavaLaunchTab;
 import org.eclipse.jdt.internal.debug.ui.JDIDebugUIPlugin;
 import org.eclipse.jdt.internal.debug.ui.actions.ControlAccessibleListener;
 import org.eclipse.jdt.launching.IJavaLaunchConfigurationConstants;
@@ -34,7 +35,7 @@ import org.eclipse.swt.widgets.Text;
 /**
  * Editor for VM arguments of a Java launch configuration.
  */
-public class VMArgumentsBlock extends JavaLaunchConfigurationTab {
+public class VMArgumentsBlock extends JavaLaunchTab {
 
 	// VM arguments widgets
 	protected Text fVMArgumentsText;
@@ -53,7 +54,7 @@ public class VMArgumentsBlock extends JavaLaunchConfigurationTab {
 		GridData gd = new GridData(GridData.FILL_BOTH);
 		group.setLayoutData(gd);
 		group.setFont(font);
-		group.setText(LauncherMessages.JavaArgumentsTab_VM_ar_guments__6); //$NON-NLS-1$
+		group.setText(LauncherMessages.JavaArgumentsTab_VM_ar_guments__6); 
 		
 		fVMArgumentsText = new Text(group, SWT.MULTI | SWT.WRAP| SWT.BORDER | SWT.V_SCROLL);
 		gd = new GridData(GridData.FILL_BOTH);
@@ -100,7 +101,7 @@ public class VMArgumentsBlock extends JavaLaunchConfigurationTab {
 		try {
 			fVMArgumentsText.setText(configuration.getAttribute(IJavaLaunchConfigurationConstants.ATTR_VM_ARGUMENTS, "")); //$NON-NLS-1$
 		} catch (CoreException e) {
-			setErrorMessage(LauncherMessages.JavaArgumentsTab_Exception_occurred_reading_configuration___15 + e.getStatus().getMessage()); //$NON-NLS-1$
+			setErrorMessage(LauncherMessages.JavaArgumentsTab_Exception_occurred_reading_configuration___15 + e.getStatus().getMessage()); 
 			JDIDebugUIPlugin.log(e);			
 		}
 	}
@@ -116,7 +117,7 @@ public class VMArgumentsBlock extends JavaLaunchConfigurationTab {
 	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#getName()
 	 */
 	public String getName() {
-		return LauncherMessages.VMArgumentsBlock_VM_Arguments; //$NON-NLS-1$
+		return LauncherMessages.VMArgumentsBlock_VM_Arguments; 
 	}
 	
 	/**
