@@ -1,17 +1,17 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.jdt.internal.debug.eval.ast.instructions;
 
 
-import java.text.MessageFormat;
+import com.ibm.icu.text.MessageFormat;
 
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.debug.core.model.IVariable;
@@ -78,7 +78,7 @@ public class LocalVariableCreation extends CompoundInstruction {
 			JDIDebugTarget debugTarget= (JDIDebugTarget)getVM();
 			VirtualMachine vm = debugTarget.getVM();
 			if (vm == null) {
-				debugTarget.requestFailed(InstructionsEvaluationMessages.LocalVariableCreation_Execution_failed___VM_disconnected__1, null); //$NON-NLS-1$
+				debugTarget.requestFailed(InstructionsEvaluationMessages.LocalVariableCreation_Execution_failed___VM_disconnected__1, null); 
 			}			
 			type= JDIType.createType(debugTarget, PrimitiveTypeImpl.create((VirtualMachineImpl)vm, fTypeSignature));
 		} else if (fDimension == 0) {
@@ -93,6 +93,6 @@ public class LocalVariableCreation extends CompoundInstruction {
 	}
 
 	public String toString() {
-		return MessageFormat.format(InstructionsEvaluationMessages.LocalVariableCreation_create_local_variable__0___1___1, new String[]{fName, fTypeSignature}); //$NON-NLS-1$
+		return MessageFormat.format(InstructionsEvaluationMessages.LocalVariableCreation_create_local_variable__0___1___1, new String[]{fName, fTypeSignature}); 
 	}
 }
