@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -135,7 +135,7 @@ public abstract class EventRequestImpl extends MirrorImpl implements EventReques
 	 * @return Returns string representation.
 	 */
 	public String toString() {
-		return super.toString() + (fRequestID == null ? RequestMessages.EventRequestImpl___not_enabled__1 : RequestMessages.EventRequestImpl____2 + fRequestID); //$NON-NLS-1$ //$NON-NLS-2$
+		return super.toString() + (fRequestID == null ? RequestMessages.EventRequestImpl___not_enabled__1 : RequestMessages.EventRequestImpl____2 + fRequestID); // 
 	}
 	
 	/**
@@ -196,6 +196,8 @@ public abstract class EventRequestImpl extends MirrorImpl implements EventReques
 					throw new InvalidRequestStateException();
 			}
 			defaultReplyErrorHandler(replyPacket.errorCode());
+			
+			virtualMachineImpl().eventRequestManagerImpl().removeRequestIDMapping(this);
 			fRequestID = null;
 		} catch (IOException e) {
 			defaultIOExceptionHandler(e);
@@ -442,7 +444,7 @@ public abstract class EventRequestImpl extends MirrorImpl implements EventReques
 			case SUSPEND_ALL:
 				return SUSPENDPOL_ALL_JDWP;
 			default:
-				throw new InternalException(RequestMessages.EventRequestImpl_Invalid_suspend_policy_encountered___3 + fSuspendPolicy); //$NON-NLS-1$
+				throw new InternalException(RequestMessages.EventRequestImpl_Invalid_suspend_policy_encountered___3 + fSuspendPolicy); 
 		}
 	}
 
@@ -456,7 +458,7 @@ public abstract class EventRequestImpl extends MirrorImpl implements EventReques
 			case StepRequest.STEP_LINE:
 				return STEP_SIZE_LINE_JDWP;
 			default:
-				throw new InternalException(RequestMessages.EventRequestImpl_Invalid_step_size_encountered___4 + threadStepSize); //$NON-NLS-1$
+				throw new InternalException(RequestMessages.EventRequestImpl_Invalid_step_size_encountered___4 + threadStepSize); 
 		}
 	}
 	
@@ -472,7 +474,7 @@ public abstract class EventRequestImpl extends MirrorImpl implements EventReques
 			case StepRequest.STEP_OUT:
 				return STEP_DEPTH_OUT_JDWP;
 			default:
-				throw new InternalException(RequestMessages.EventRequestImpl_Invalid_step_depth_encountered___5 + threadStepDepth); //$NON-NLS-1$
+				throw new InternalException(RequestMessages.EventRequestImpl_Invalid_step_depth_encountered___5 + threadStepDepth); 
 		}
 	}
 
