@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -358,4 +358,23 @@ public interface IJavaDebugTarget extends IDebugTarget, IStepFilters {
 	 * @since 3.0
 	 */
 	public String getDefaultStratum();
+	
+	/**
+	 * Returns the top level thread groups in this target. Top level thread groups
+	 * do not have a parent.
+	 * 
+	 * @return top level thread groups
+	 * @throws DebugException
+	 * @since 3.2
+	 */
+	public IJavaThreadGroup[] getRootThreadGroups() throws DebugException;
+	
+	/**
+	 * Returns all thread groups in this target.
+	 * 
+	 * @return all thread groups in this target
+	 * @throws DebugException
+	 * @since 3.2
+	 */
+	public IJavaThreadGroup[] getAllThreadGroups() throws DebugException;
 }
