@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,13 +12,13 @@ package org.eclipse.jdt.internal.launching;
 
 
 import java.io.ByteArrayInputStream;
+import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.net.URL;
-import java.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Hashtable;
@@ -30,13 +30,22 @@ import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.parsers.FactoryConfigurationError;
 import javax.xml.parsers.ParserConfigurationException;
+import javax.xml.transform.OutputKeys;
+import javax.xml.transform.Transformer;
 import javax.xml.transform.TransformerException;
+import javax.xml.transform.TransformerFactory;
+import javax.xml.transform.dom.DOMSource;
+import javax.xml.transform.stream.StreamResult;
 
+import org.eclipse.core.resources.IMarker;
+import org.eclipse.core.resources.IResource;
 import org.eclipse.core.resources.IResourceChangeEvent;
 import org.eclipse.core.resources.IResourceChangeListener;
+import org.eclipse.core.resources.IResourceDelta;
 import org.eclipse.core.resources.IWorkspaceRunnable;
 import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
+import org.eclipse.core.runtime.FileLocator;
 import org.eclipse.core.runtime.IConfigurationElement;
 import org.eclipse.core.runtime.IExtensionPoint;
 import org.eclipse.core.runtime.IPath;
@@ -67,6 +76,8 @@ import org.eclipse.jdt.launching.IVMInstall;
 import org.eclipse.jdt.launching.IVMInstallChangedListener;
 import org.eclipse.jdt.launching.JavaRuntime;
 import org.eclipse.jdt.launching.VMStandin;
+import org.eclipse.jdt.launching.environments.IExecutionEnvironment;
+import org.eclipse.jdt.launching.environments.IExecutionEnvironmentsManager;
 import org.eclipse.jdt.launching.sourcelookup.ArchiveSourceLocation;
 import org.osgi.framework.BundleContext;
 import org.w3c.dom.Document;
@@ -77,6 +88,8 @@ import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
 import org.xml.sax.helpers.DefaultHandler;
 
+import com.ibm.icu.text.MessageFormat;
+
 public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChangeListener, IVMInstallChangedListener, IResourceChangeListener, ILaunchesListener, IDebugEventSetListener {
 	
 	/**
@@ -92,7 +105,14 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 	/**
 	 * Identifier for 'runtimeClasspathEntries' extension point
 	 */
-	public static final String ID_EXTENSION_POINT_RUNTIME_CLASSPATH_ENTRIES = "runtimeClasspathEntries"; //$NON-NLS-1$	
+	public static final String ID_EXTENSION_POINT_RUNTIME_CLASSPATH_ENTRIES = "runtimeClasspathEntries"; //$NON-NLS-1$
+	
+	/**
+	 * Marker type for JRE container problems.
+	 * 
+	 * @since 3.2
+	 */
+	public static final String ID_JRE_CONTAINER_MARKER = ID_PLUGIN + ".jreContainerMarker"; //$NON-NLS-1$
 	
 	private static LaunchingPlugin fgLaunchingPlugin;
 	
@@ -208,7 +228,7 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 			IWorkspaceRunnable runnable = new IWorkspaceRunnable() {
 				public void run(IProgressMonitor monitor1) throws CoreException {
 					IJavaProject[] projects = JavaCore.create(ResourcesPlugin.getWorkspace().getRoot()).getJavaProjects();
-					monitor1.beginTask(LaunchingMessages.LaunchingPlugin_0, projects.length + 1); //$NON-NLS-1$
+					monitor1.beginTask(LaunchingMessages.LaunchingPlugin_0, projects.length + 1); 
 					rebind(monitor1, projects);
 					monitor1.done();
 				}
@@ -283,7 +303,7 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 		private VMChanges fChanges;
 		
 		public JREUpdateJob(VMChanges changes) {
-			super(LaunchingMessages.LaunchingPlugin_1); //$NON-NLS-1$
+			super(LaunchingMessages.LaunchingPlugin_1); 
 			fChanges = changes;
 			setSystem(true);
 		}
@@ -326,7 +346,7 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 	 * path.
 	 * 
 	 * @param javaInstallPath home location for a JRE
-	 * @param info the libary information, or <code>null</code> to remove
+	 * @param info the library information, or <code>null</code> to remove
 	 */
 	public static void setLibraryInfo(String javaInstallPath, LibraryInfo info) {
 		if (fgLibraryInfoMap == null) {
@@ -348,7 +368,7 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 		try {
 			URL installURL =
 				new URL(getDefault().getBundle().getEntry("/"), path.toString()); //$NON-NLS-1$
-			URL localURL = Platform.asLocalURL(installURL);
+			URL localURL = FileLocator.toFileURL(installURL);
 			return new File(localURL.getFile());
 		} catch (IOException ioe) {
 			return null;
@@ -415,7 +435,7 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 			filters= launchFilter;
 			modified = true;
 		} else if (filters.indexOf(launchFilter) == -1) {
-			filters= filters + ',' + launchFilter; //$NON-NLS-1$
+			filters= filters + ',' + launchFilter; 
 			modified = true;
 		}
 
@@ -429,7 +449,7 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 		getPluginPreferences().addPropertyChangeListener(this);
 
 		JavaRuntime.addVMInstallChangedListener(this);
-		ResourcesPlugin.getWorkspace().addResourceChangeListener(this, IResourceChangeEvent.PRE_DELETE | IResourceChangeEvent.PRE_CLOSE);
+		ResourcesPlugin.getWorkspace().addResourceChangeListener(this, IResourceChangeEvent.PRE_DELETE | IResourceChangeEvent.PRE_CLOSE | IResourceChangeEvent.PRE_BUILD);
 		DebugPlugin.getDefault().getLaunchManager().addLaunchListener(this);
 		DebugPlugin.getDefault().addDebugEventListener(this);
 	}
@@ -466,7 +486,7 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 	private void initializeVMConnectors() {
 		IExtensionPoint extensionPoint= Platform.getExtensionRegistry().getExtensionPoint(ID_PLUGIN, ID_EXTENSION_POINT_VM_CONNECTORS);
 		IConfigurationElement[] configs= extensionPoint.getConfigurationElements(); 
-		MultiStatus status= new MultiStatus(getUniqueIdentifier(), IStatus.OK, LaunchingMessages.LaunchingPlugin_Exception_occurred_reading_vmConnectors_extensions_1, null); //$NON-NLS-1$
+		MultiStatus status= new MultiStatus(getUniqueIdentifier(), IStatus.OK, LaunchingMessages.LaunchingPlugin_Exception_occurred_reading_vmConnectors_extensions_1, null); 
 		fVMConnectors = new HashMap(configs.length);
 		for (int i= 0; i < configs.length; i++) {
 			try {
@@ -494,7 +514,7 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 		}
 		IConfigurationElement config = (IConfigurationElement) fClasspathEntryExtensions.get(id);
 		if (config == null) {
-			abort(MessageFormat.format(LaunchingMessages.LaunchingPlugin_32, new String[]{id}), null); //$NON-NLS-1$
+			abort(MessageFormat.format(LaunchingMessages.LaunchingPlugin_32, new String[]{id}), null); 
 		}
 		return (IRuntimeClasspathEntry2) config.createExecutableExtension("class"); //$NON-NLS-1$
 	}
@@ -633,10 +653,9 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 	 * @return VMDefinitionsContainer
 	 */
 	private VMDefinitionsContainer getVMDefinitions(String xml) {
-		byte[] bytes = xml.getBytes();
-		if (bytes.length > 0) {
-			ByteArrayInputStream stream = new ByteArrayInputStream(bytes);
+		if (xml.length() > 0) {
 			try {
+				ByteArrayInputStream stream = new ByteArrayInputStream(xml.getBytes("UTF8")); //$NON-NLS-1$
 				return VMDefinitionsContainer.parseXMLIntoContainer(stream);
 			} catch (IOException e) {
 				LaunchingPlugin.log(e);
@@ -698,11 +717,41 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 
 	/**
 	 * Clear the archive cache when a project is about to be deleted.
+	 * Warn when a buildpath changes and references an execution environment
+	 * that does not have a perfect match.
 	 * 
 	 * @see IResourceChangeListener#resourceChanged(IResourceChangeEvent)
 	 */
 	public void resourceChanged(IResourceChangeEvent event) {
 		ArchiveSourceLocation.closeArchives();
+		if (event.getType() == IResourceChangeEvent.PRE_BUILD) {
+			IResourceDelta delta = event.getDelta();
+			IResourceDelta[] projectDeltas = delta.getAffectedChildren();
+			for (int i = 0, length = projectDeltas.length; i < length; i++) {
+				IResourceDelta projectDelta = projectDeltas[i];
+				IResourceDelta classpathDelta = projectDelta.findMember(new Path(".classpath")); //$NON-NLS-1$
+				if (classpathDelta != null || (projectDelta.getFlags() & IResourceDelta.DESCRIPTION) > 0) {
+					IJavaProject project = (IJavaProject) JavaCore.create(projectDelta.getResource());
+					if (project != null && project.exists()) {
+						try {
+							IClasspathEntry[] rawClasspath = project.getRawClasspath();
+							for (int j = 0; j < rawClasspath.length; j++) {
+								IClasspathEntry entry = rawClasspath[j];
+								if (entry.getEntryKind() == IClasspathEntry.CPE_CONTAINER) {
+									IPath path = entry.getPath();
+									if (JavaRuntime.JRE_CONTAINER.equals(path.segment(0))) {
+										IVMInstall vm = JREContainerInitializer.resolveVM(path);
+										validateEnvironment(path, project, vm);
+									}
+								}
+							}
+						} catch (CoreException e) {
+							LaunchingPlugin.log(e);
+						}
+					}
+				}
+			}
+		}
 	}
 
 	public void setIgnoreVMDefPropertyChangeEvents(boolean ignore) {
@@ -741,7 +790,7 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 		}
 		
 		// Serialize the Document and return the resulting String
-		return JavaLaunchConfigurationUtils.serializeDocument(doc);
+		return LaunchingPlugin.serializeDocument(doc);
 	}	
 	
 	/**
@@ -930,7 +979,7 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 	}
 	
 	/**
-	 * When a debug target or process terminates, close source arhives.
+	 * When a debug target or process terminates, close source archives.
 	 * Prevents file sharing violations.
 	 * 
 	 * @see IDebugEventSetListener#handleDebugEvents(DebugEvent[])
@@ -947,6 +996,28 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 		}
 	}
 	
+	/**
+	 * Serializes a XML document into a string - encoded in UTF8 format,
+	 * with platform line separators.
+	 * 
+	 * @param doc document to serialize
+	 * @return the document as a string
+	 */
+	public static String serializeDocument(Document doc) throws IOException, TransformerException {
+		ByteArrayOutputStream s= new ByteArrayOutputStream();
+		
+		TransformerFactory factory= TransformerFactory.newInstance();
+		Transformer transformer= factory.newTransformer();
+		transformer.setOutputProperty(OutputKeys.METHOD, "xml"); //$NON-NLS-1$
+		transformer.setOutputProperty(OutputKeys.INDENT, "yes"); //$NON-NLS-1$
+		
+		DOMSource source= new DOMSource(doc);
+		StreamResult outputTarget= new StreamResult(s);
+		transformer.transform(source, outputTarget);
+		
+		return s.toString("UTF8"); //$NON-NLS-1$			
+	}
+
 	/**
 	 * Returns a shared XML parser.
 	 * 
@@ -960,9 +1031,9 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 				fgXMLParser = DocumentBuilderFactory.newInstance().newDocumentBuilder();
 				fgXMLParser.setErrorHandler(new DefaultHandler());
 			} catch (ParserConfigurationException e) {
-				abort(LaunchingMessages.LaunchingPlugin_33, e); //$NON-NLS-1$
+				abort(LaunchingMessages.LaunchingPlugin_33, e); 
 			} catch (FactoryConfigurationError e) {
-				abort(LaunchingMessages.LaunchingPlugin_34, e); //$NON-NLS-1$
+				abort(LaunchingMessages.LaunchingPlugin_34, e); 
 			}
 		}
 		return fgXMLParser;
@@ -979,6 +1050,57 @@ public class LaunchingPlugin extends Plugin implements Preferences.IPropertyChan
 		IStatus status = new Status(IStatus.ERROR, LaunchingPlugin.getUniqueIdentifier(), 0, message, exception);
 		throw new CoreException(status);
 	}	
+	
+	private void validateEnvironment(IPath containerPath, final IJavaProject project, IVMInstall vm) {
+		try {
+			project.getProject().deleteMarkers(ID_JRE_CONTAINER_MARKER, false, IResource.DEPTH_ZERO);
+		} catch (CoreException e) {
+			LaunchingPlugin.log(e);
+		}
+		String id = JREContainerInitializer.getExecutionEnvironmentId(containerPath);
+		if (id != null) {
+			IExecutionEnvironmentsManager manager = JavaRuntime.getExecutionEnvironmentsManager();
+			final IExecutionEnvironment environment = manager.getEnvironment(id);
+			if (environment != null) {
+				if (!environment.isStrictlyCompatible(vm)) {
+					// warn that VM does not match EE
+					// first determine if there is a strictly compatible JRE available
+					IVMInstall[] compatibleVMs = environment.getCompatibleVMs();
+					int exact = 0;
+					for (int i = 0; i < compatibleVMs.length; i++) {
+						if (environment.isStrictlyCompatible(compatibleVMs[i])) {
+							exact++;
+						}
+					}
+					String message = null;
+					if (exact == 0) {
+						message = MessageFormat.format(
+							LaunchingMessages.LaunchingPlugin_35,
+							new String[]{environment.getId()});
+					} else {
+						message = MessageFormat.format(
+								LaunchingMessages.LaunchingPlugin_36,
+								new String[]{environment.getId()});
+					}
+					try {
+						IMarker marker = project.getProject().createMarker(ID_JRE_CONTAINER_MARKER);
+						marker.setAttributes(
+							new String[] { 
+									IMarker.MESSAGE, 
+									IMarker.SEVERITY, 
+									IMarker.LOCATION								},
+								new Object[] {
+									message,
+									new Integer(IMarker.SEVERITY_WARNING), 
+									LaunchingMessages.LaunchingPlugin_37
+								});
+					} catch (CoreException e) {
+						return;
+					}
+				}
+			}
+		}
+	}	
 }
 
  
