@@ -1,16 +1,16 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.jdt.launching;
 import java.io.File;
-import java.text.MessageFormat;
+import com.ibm.icu.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
@@ -38,7 +38,6 @@ import org.eclipse.debug.core.ILaunchManager;
 import org.eclipse.debug.core.model.IBreakpoint;
 import org.eclipse.debug.core.model.LaunchConfigurationDelegate;
 import org.eclipse.debug.core.sourcelookup.ISourceLookupDirector;
-import org.eclipse.jdt.core.IClasspathEntry;
 import org.eclipse.jdt.core.IJavaModelMarker;
 import org.eclipse.jdt.core.IJavaProject;
 import org.eclipse.jdt.core.JavaCore;
@@ -64,7 +63,7 @@ public abstract class AbstractJavaLaunchConfigurationDelegate
 		extends
 			LaunchConfigurationDelegate implements IDebugEventSetListener {
 	/**
-	 * A list of prequisite projects ordered by their build order.
+	 * A list of prerequisite projects ordered by their build order.
 	 */
 	private IProject[] fOrderedProjects;
 	/**
@@ -182,29 +181,29 @@ public abstract class AbstractJavaLaunchConfigurationDelegate
 		IVMInstall vm = getVMInstall(configuration);
 		if (vm == null) {
 			abort(
-					LaunchingMessages.AbstractJavaLaunchConfigurationDelegate_The_specified_JRE_installation_does_not_exist_4, //$NON-NLS-1$
+					LaunchingMessages.AbstractJavaLaunchConfigurationDelegate_The_specified_JRE_installation_does_not_exist_4, 
 					null,
-					IJavaLaunchConfigurationConstants.ERR_VM_INSTALL_DOES_NOT_EXIST); //$NON-NLS-1$
+					IJavaLaunchConfigurationConstants.ERR_VM_INSTALL_DOES_NOT_EXIST); 
 		}
 		File location = vm.getInstallLocation();
 		if (location == null) {
 			abort(
 					MessageFormat
 							.format(
-									LaunchingMessages.AbstractJavaLaunchConfigurationDelegate_JRE_home_directory_not_specified_for__0__5, //$NON-NLS-1$
+									LaunchingMessages.AbstractJavaLaunchConfigurationDelegate_JRE_home_directory_not_specified_for__0__5, 
 									new String[]{vm.getName()}),
 					null,
-					IJavaLaunchConfigurationConstants.ERR_VM_INSTALL_DOES_NOT_EXIST); //$NON-NLS-1$
+					IJavaLaunchConfigurationConstants.ERR_VM_INSTALL_DOES_NOT_EXIST); 
 		}
 		if (!location.exists()) {
 			abort(
 					MessageFormat
 							.format(
-									LaunchingMessages.AbstractJavaLaunchConfigurationDelegate_JRE_home_directory_for__0__does_not_exist___1__6, //$NON-NLS-1$
+									LaunchingMessages.AbstractJavaLaunchConfigurationDelegate_JRE_home_directory_for__0__does_not_exist___1__6, 
 									new String[]{vm.getName(),
 											location.getAbsolutePath()}),
 					null,
-					IJavaLaunchConfigurationConstants.ERR_VM_INSTALL_DOES_NOT_EXIST); //$NON-NLS-1$
+					IJavaLaunchConfigurationConstants.ERR_VM_INSTALL_DOES_NOT_EXIST); 
 		}
 		return vm;
 	}
@@ -235,8 +234,8 @@ public abstract class AbstractJavaLaunchConfigurationDelegate
 	 * @param configuration
 	 *            launch configuration
 	 * @return the bootpath specified by the given launch configuration. An
-	 *         empty bootpath is specfied by an empty array, and
-	 *         <code>null</code> represents a default boothpath.
+	 *         empty bootpath is specified by an empty array, and
+	 *         <code>null</code> represents a default bootpath.
 	 * @exception CoreException
 	 *                if unable to retrieve the attribute
 	 */
@@ -300,18 +299,13 @@ public abstract class AbstractJavaLaunchConfigurationDelegate
 				.computeUnresolvedRuntimeClasspath(configuration);
 		List bootEntriesPrepend = new ArrayList();
 		int index = 0;
-		boolean jreContainerFound = false;
-		while (!jreContainerFound && index < entries.length) {
+		IRuntimeClasspathEntry jreEntry = null;
+		while (jreEntry == null && index < entries.length) {
 			IRuntimeClasspathEntry entry = entries[index++];
 			if (entry.getClasspathProperty() == IRuntimeClasspathEntry.BOOTSTRAP_CLASSES
 					|| entry.getClasspathProperty() == IRuntimeClasspathEntry.STANDARD_CLASSES) {
-				int entryKind = entry.getClasspathEntry().getEntryKind();
-				String segment0 = entry.getPath().segment(0);
-				if (entryKind == IClasspathEntry.CPE_CONTAINER
-						&& JavaRuntime.JRE_CONTAINER.equals(segment0)
-						|| entryKind == IClasspathEntry.CPE_VARIABLE
-						&& JavaRuntime.JRELIB_VARIABLE.equals(segment0)) {
-					jreContainerFound = true;
+				if (JavaRuntime.isVMInstallReference(entry)) {
+					jreEntry = entry;
 				} else {
 					bootEntriesPrepend.add(entry);
 				}
@@ -329,7 +323,7 @@ public abstract class AbstractJavaLaunchConfigurationDelegate
 				entriesPrep[i] = bootEntriesPrep[i].getLocation();
 			}
 		}
-		if (jreContainerFound) {
+		if (jreEntry != null) {
 			List bootEntriesAppend = new ArrayList();
 			for (; index < entries.length; index++) {
 				IRuntimeClasspathEntry entry = entries[index];
@@ -353,18 +347,34 @@ public abstract class AbstractJavaLaunchConfigurationDelegate
 			LibraryLocation[] libraryLocations = install.getLibraryLocations();
 			if (libraryLocations != null) {
 				// determine if explicit bootpath should be used
+				// TODO: this test does not tell us if the bootpath entries are different (could still be
+				// the same, as a non-bootpath entry on the JRE may have been removed/added)
+				// We really need a way to ask a VM type for its default bootpath library locations and
+				// compare that to the resolved entries for the "jreEntry" to see if they 
+				// are different (requires explicit bootpath)
 				if (!JRERuntimeClasspathEntryResolver.isSameArchives(libraryLocations, install.getVMInstallType().getDefaultLibraryLocations(install.getInstallLocation()))) {
-					// non-default JRE libaries - use explicit bootpath only
+					// resolve bootpath entries in JRE entry
+					IRuntimeClasspathEntry[] bootEntries = null; 
+					if (jreEntry.getType() == IRuntimeClasspathEntry.CONTAINER) {
+						IRuntimeClasspathEntry bootEntry = JavaRuntime.newRuntimeContainerClasspathEntry(
+								jreEntry.getPath(),
+								IRuntimeClasspathEntry.BOOTSTRAP_CLASSES,
+								getJavaProject(configuration));
+						bootEntries = JavaRuntime.resolveRuntimeClasspathEntry(bootEntry, configuration);
+					} else {
+						bootEntries = JavaRuntime.resolveRuntimeClasspathEntry(jreEntry, configuration);
+					}
+					
+					// non-default JRE libraries - use explicit bootpath only
 					String[] bootpath = new String[bootEntriesPrep.length
-							+ libraryLocations.length + bootEntriesApp.length];
+							+ bootEntries.length + bootEntriesApp.length];
 					if (bootEntriesPrep.length > 0) {
 						System.arraycopy(bootpathInfo[0], 0, bootpath, 0,
 								bootEntriesPrep.length);
 					}
 					int dest = bootEntriesPrep.length;
-					for (int i = 0; i < libraryLocations.length; i++) {
-						bootpath[dest] = libraryLocations[i].getSystemLibraryPath()
-								.toOSString();
+					for (int i = 0; i < bootEntries.length; i++) {
+						bootpath[dest] = bootEntries[i].getLocation();
 						dest++;
 					}
 					if (bootEntriesApp.length > 0) {
@@ -515,7 +525,7 @@ public abstract class AbstractJavaLaunchConfigurationDelegate
 		String args = VariablesPlugin.getDefault().getStringVariableManager().performStringSubstitution(arguments);
 		int libraryPath = args.indexOf("-Djava.library.path"); //$NON-NLS-1$
 		if (libraryPath < 0) {
-			// if a library path is alread specified, do not override
+			// if a library path is already specified, do not override
 			String[] javaLibraryPath = getJavaLibraryPath(configuration);
 			if (javaLibraryPath != null && javaLibraryPath.length > 0) {
 				StringBuffer path = new StringBuffer(args);
@@ -627,16 +637,16 @@ public abstract class AbstractJavaLaunchConfigurationDelegate
 		String name = getJavaProjectName(configuration);
 		if (name == null) {
 			abort(
-					LaunchingMessages.AbstractJavaLaunchConfigurationDelegate_Java_project_not_specified_9, //$NON-NLS-1$
+					LaunchingMessages.AbstractJavaLaunchConfigurationDelegate_Java_project_not_specified_9, 
 					null,
-					IJavaLaunchConfigurationConstants.ERR_UNSPECIFIED_PROJECT); //$NON-NLS-1$
+					IJavaLaunchConfigurationConstants.ERR_UNSPECIFIED_PROJECT); 
 		}
 		IJavaProject project = getJavaProject(configuration);
 		if (project == null) {
 			abort(
-					LaunchingMessages.AbstractJavaLaunchConfigurationDelegate_Project_does_not_exist_or_is_not_a_Java_project_10, //$NON-NLS-1$
+					LaunchingMessages.AbstractJavaLaunchConfigurationDelegate_Project_does_not_exist_or_is_not_a_Java_project_10, 
 					null,
-					IJavaLaunchConfigurationConstants.ERR_NOT_A_JAVA_PROJECT); //$NON-NLS-1$
+					IJavaLaunchConfigurationConstants.ERR_NOT_A_JAVA_PROJECT); 
 		}
 		return project;
 	}
@@ -656,9 +666,9 @@ public abstract class AbstractJavaLaunchConfigurationDelegate
 		String name = getMainTypeName(configuration);
 		if (name == null) {
 			abort(
-					LaunchingMessages.AbstractJavaLaunchConfigurationDelegate_Main_type_not_specified_11, //$NON-NLS-1$
+					LaunchingMessages.AbstractJavaLaunchConfigurationDelegate_Main_type_not_specified_11, 
 					null,
-					IJavaLaunchConfigurationConstants.ERR_UNSPECIFIED_MAIN_TYPE); //$NON-NLS-1$
+					IJavaLaunchConfigurationConstants.ERR_UNSPECIFIED_MAIN_TYPE); 
 		}
 		return name;
 	}
@@ -678,11 +688,17 @@ public abstract class AbstractJavaLaunchConfigurationDelegate
 			throws CoreException {
 		IPath path = getWorkingDirectoryPath(configuration);
 		if (path == null) {
-			// default working dir is the project if this config has a project
-			IJavaProject jp = getJavaProject(configuration);
-			if (jp != null) {
-				IProject p = jp.getProject();
-				return p.getLocation().toFile();
+			File dir = getDefaultWorkingDirectory(configuration);
+			if (dir != null) {
+				if (!dir.isDirectory()) {
+					abort(
+							MessageFormat.format(
+									LaunchingMessages.AbstractJavaLaunchConfigurationDelegate_Working_directory_does_not_exist___0__12, 
+									new String[]{dir.toString()}),
+									null,
+									IJavaLaunchConfigurationConstants.ERR_WORKING_DIRECTORY_DOES_NOT_EXIST); 
+				}
+				return dir;
 			}
 		} else {
 			if (path.isAbsolute()) {
@@ -700,10 +716,10 @@ public abstract class AbstractJavaLaunchConfigurationDelegate
 				abort(
 					MessageFormat
 							.format(
-									LaunchingMessages.AbstractJavaLaunchConfigurationDelegate_Working_directory_does_not_exist___0__12, //$NON-NLS-1$
+									LaunchingMessages.AbstractJavaLaunchConfigurationDelegate_Working_directory_does_not_exist___0__12, 
 									new String[]{path.toString()}),
 					null,
-					IJavaLaunchConfigurationConstants.ERR_WORKING_DIRECTORY_DOES_NOT_EXIST); //$NON-NLS-1$
+					IJavaLaunchConfigurationConstants.ERR_WORKING_DIRECTORY_DOES_NOT_EXIST); 
 			} else {
 				IResource res = ResourcesPlugin.getWorkspace().getRoot()
 						.findMember(path);
@@ -713,10 +729,10 @@ public abstract class AbstractJavaLaunchConfigurationDelegate
 				abort(
 					MessageFormat
 							.format(
-									LaunchingMessages.AbstractJavaLaunchConfigurationDelegate_Working_directory_does_not_exist___0__12, //$NON-NLS-1$
+									LaunchingMessages.AbstractJavaLaunchConfigurationDelegate_Working_directory_does_not_exist___0__12, 
 									new String[]{path.toString()}),
 					null,
-					IJavaLaunchConfigurationConstants.ERR_WORKING_DIRECTORY_DOES_NOT_EXIST); //$NON-NLS-1$
+					IJavaLaunchConfigurationConstants.ERR_WORKING_DIRECTORY_DOES_NOT_EXIST); 
 			}
 		}
 		return null;
@@ -833,7 +849,7 @@ public abstract class AbstractJavaLaunchConfigurationDelegate
 													mainType, "main", //$NON-NLS-1$
 													"([Ljava/lang/String;)V", //$NON-NLS-1$
 													true, false, false, -1, -1,
-													-1, 1, false, map); //$NON-NLS-1$
+													-1, 1, false, map); 
 									bp.setPersisted(false);
 									target.breakpointAdded(bp);
 									DebugPlugin.getDefault()
@@ -887,7 +903,7 @@ public abstract class AbstractJavaLaunchConfigurationDelegate
 			String mode, IProgressMonitor monitor) throws CoreException {
 		// build project list
 		if (monitor != null) {
-			monitor.subTask(LaunchingMessages.AbstractJavaLaunchConfigurationDelegate_20); //$NON-NLS-1$
+			monitor.subTask(LaunchingMessages.AbstractJavaLaunchConfigurationDelegate_20); 
 		}
 		fOrderedProjects = null;
 		IJavaProject javaProject = JavaRuntime.getJavaProject(configuration);
@@ -925,7 +941,7 @@ public abstract class AbstractJavaLaunchConfigurationDelegate
 		IVMInstall vm = verifyVMInstall(configuration);
 		IVMRunner runner = vm.getVMRunner(mode);
 		if (runner == null) {
-			abort(MessageFormat.format(LaunchingMessages.JavaLocalApplicationLaunchConfigurationDelegate_0, new String[]{vm.getName(), mode}), null, IJavaLaunchConfigurationConstants.ERR_VM_RUNNER_DOES_NOT_EXIST); //$NON-NLS-1$
+			abort(MessageFormat.format(LaunchingMessages.JavaLocalApplicationLaunchConfigurationDelegate_0, new String[]{vm.getName(), mode}), null, IJavaLaunchConfigurationConstants.ERR_VM_RUNNER_DOES_NOT_EXIST); 
 		}
 		return runner;
 	}
@@ -963,4 +979,24 @@ public abstract class AbstractJavaLaunchConfigurationDelegate
 		}
 		return null;
 	}
+	
+	/**
+	 * Returns the default working directory for the given launch configuration,
+	 * or <code>null</code> if none. Subclasses may override as necessary.
+	 * 
+	 * @param configuration
+	 * @return default working directory or <code>null</code> if none
+	 * @throws CoreException if an exception occurs computing the default working
+	 * 	 directory
+	 * @since 3.2
+	 */
+	protected File getDefaultWorkingDirectory(ILaunchConfiguration configuration) throws CoreException {
+		// default working directory is the project if this config has a project
+		IJavaProject jp = getJavaProject(configuration);
+		if (jp != null) {
+			IProject p = jp.getProject();
+			return p.getLocation().toFile();
+		}
+		return null;
+	}
 }
