@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -101,11 +101,27 @@ public interface IJavaLaunchConfigurationConstants {
 	 */
 	public static final String ATTR_WORKING_DIRECTORY = LaunchingPlugin.getUniqueIdentifier() + ".WORKING_DIRECTORY";	 //$NON-NLS-1$
 	
+	/**
+	 * Launch configuration attribute key. The value is a path identifying the JRE used
+	 * when launching a local VM. The path is a classpath container corresponding
+	 * to the <code>JavaRuntime.JRE_CONTAINER</code> classpath container.
+	 * <p>
+	 * When unspecified the default JRE for a launch configuration is used (which is the
+	 * JRE associated with the project being launched, or the workspace default JRE when
+	 * no project is associated with a configuration). The default JRE classpath container
+	 * refers explicitly to the workspace default JRE.
+	 * </p>
+	 * @since 3.2
+	 */
+	public static final String ATTR_JRE_CONTAINER_PATH = JavaRuntime.JRE_CONTAINER;
+	
 	/**
 	 * Launch configuration attribute key. The value is a name of a VM install
 	 * to use when launching a local VM. This attribute must be qualified
 	 * by a VM install type, via the <code>ATTR_VM_INSTALL_TYPE</code>
 	 * attribute. When unspecified, the default VM is used.
+	 * 
+	 * @deprecated use <code>ATTR_JRE_CONTAINER_PATH</code>
 	 */
 	public static final String ATTR_VM_INSTALL_NAME = LaunchingPlugin.getUniqueIdentifier() + ".VM_INSTALL_NAME"; //$NON-NLS-1$
 		
@@ -115,6 +131,8 @@ public interface IJavaLaunchConfigurationConstants {
 	 * specify the VM to use when launching a local Java application.
 	 * The associated VM install name is specified via the attribute
 	 * <code>ATTR_VM_INSTALL_NAME</code>.
+	 * 
+	 * @deprecated use <code>ATTR_JRE_CONTAINER_PATH</code>
 	 */
 	public static final String ATTR_VM_INSTALL_TYPE = LaunchingPlugin.getUniqueIdentifier() + ".VM_INSTALL_TYPE_ID"; //$NON-NLS-1$
 	
@@ -143,7 +161,7 @@ public interface IJavaLaunchConfigurationConstants {
 	public static final String ATTR_CLASSPATH = LaunchingPlugin.getUniqueIdentifier() + ".CLASSPATH";	 //$NON-NLS-1$
 	
 	/**
-	 * Launch configuration attribute key. The value is a boolean specfying
+	 * Launch configuration attribute key. The value is a boolean specifying
 	 * whether a default classpath should be used when launching a local
 	 * Java application. When <code>false</code>, a classpath must be specified
 	 * via the <code>ATTR_CLASSPATH</code> attribute. When <code>true</code> or
@@ -196,14 +214,15 @@ public interface IJavaLaunchConfigurationConstants {
 	public static final String ATTR_ALLOW_TERMINATE = LaunchingPlugin.getUniqueIdentifier() + ".ALLOW_TERMINATE";	 //$NON-NLS-1$
 	
 	/**
-	 * Launch configuration attribute key. The value is a String, indicating
-	 * the String to use to invoke the JRE.
+	 * Attribute key for VM specific attributes found in the
+	 * <code>ATTR_VM_INSTALL_TYPE_SPECIFIC_ATTRS_MAP</code>. The value is a String,
+	 * indicating the String to use to invoke the JRE.
 	 */
 	public static final String ATTR_JAVA_COMMAND = LaunchingPlugin.getUniqueIdentifier() + ".JAVA_COMMAND";	 //$NON-NLS-1$
 
 	/**
 	 * Launch configuration attribute key. The value is 
-	 * a map. Keys in the map correpsond to arguments names
+	 * a map. Keys in the map correspond to arguments names
 	 * returned by <code>IVMConnector#getDefaultArguments()</code>.
 	 * Values are strings corresponding to the values to use when
 	 * establishing a connection to a remote VM.
@@ -245,11 +264,47 @@ public interface IJavaLaunchConfigurationConstants {
 
 	/**
 	 * Launch configuration attribute key. The value is a String indicating the
-	 * fully qualified name of the appletviewer utility class to use.
+	 * fully qualified name of the applet viewer utility class to use.
 	 * 
 	 * @since 2.1
 	 */
 	public static final String ATTR_APPLET_APPLETVIEWER_CLASS = LaunchingPlugin.getUniqueIdentifier() + ".APPLET_APPLETVIEWER_CLASS";	 //$NON-NLS-1$
+	
+	/**
+	 * Attribute key for a VM specific argument. Value is an array
+	 * of strings describing paths in the local file system that
+	 * should be prepended to the bootpath, or <code>null</code>
+	 * if none. The value is computed dynamically at launch time
+	 * and placed in the VM specific arguments map by the Java 
+	 * application launch delegate.
+	 * 
+	 * @since 3.0
+	 */
+	public static final String ATTR_BOOTPATH_PREPEND = LaunchingPlugin.getUniqueIdentifier() + ".-Xbootclasspath/p:";	 //$NON-NLS-1$
+
+	/**
+	 * Attribute key for a VM specific argument. Value is an array
+	 * of strings describing paths in the local file system that
+	 * should be placed on the bootpath explicitly, or <code>null</code>
+	 * if none. The value is computed dynamically at launch time
+	 * and placed in the VM specific arguments map by the Java 
+	 * application launch delegate.
+	 * 
+	 * @since 3.0
+	 */
+	public static final String ATTR_BOOTPATH = LaunchingPlugin.getUniqueIdentifier() + ".-Xbootclasspath:";	 //$NON-NLS-1$
+	
+	/**
+	 * Attribute key for a VM specific argument. Value is an array
+	 * of strings describing paths in the local file system that
+	 * should be appended to the bootpath, or <code>null</code>
+	 * if none. The value is computed dynamically at launch time
+	 * and placed in the VM specific arguments map by the Java 
+	 * application launch delegate.
+	 * 
+	 * @since 3.0
+	 */	
+	public static final String ATTR_BOOTPATH_APPEND = LaunchingPlugin.getUniqueIdentifier() + ".-Xbootclasspath/a:";	 //$NON-NLS-1$
 
 	/**
 	 * Status code indicating a launch configuration does not
@@ -412,6 +467,14 @@ public interface IJavaLaunchConfigurationConstants {
 	 * @since 2.1
 	 */
 	public static final int ERR_COULD_NOT_BUILD_HTML = 123;		
+	
+	/**
+	 * Status code indicating that the project referenced by a launch configuration
+	 * is closed.
+	 * 
+	 * @since 3.0
+	 */
+	public static final int ERR_PROJECT_CLOSED = 124;			
 
 	/**
 	 * Status code indicating an unexpected internal error.
