@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,14 +7,21 @@
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Matt Chapman, mpchapman@gmail.com - 89977 Make JDT .java agnostic
  *******************************************************************************/
 package org.eclipse.jdt.internal.corext.buildpath;
 
+import java.net.URI;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.HashSet;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Set;
 
+import org.eclipse.core.filesystem.EFS;
+import org.eclipse.core.filesystem.IFileStore;
+
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
@@ -56,7 +63,7 @@ import org.eclipse.jdt.internal.ui.wizards.buildpaths.BuildPathsBlock.IRemoveOld
 import org.eclipse.jdt.internal.ui.wizards.buildpaths.newsourcepage.ClasspathModifierQueries;
 import org.eclipse.jdt.internal.ui.wizards.buildpaths.newsourcepage.ClasspathModifierQueries.IAddArchivesQuery;
 import org.eclipse.jdt.internal.ui.wizards.buildpaths.newsourcepage.ClasspathModifierQueries.IAddLibrariesQuery;
-import org.eclipse.jdt.internal.ui.wizards.buildpaths.newsourcepage.ClasspathModifierQueries.IFolderCreationQuery;
+import org.eclipse.jdt.internal.ui.wizards.buildpaths.newsourcepage.ClasspathModifierQueries.ICreateFolderQuery;
 import org.eclipse.jdt.internal.ui.wizards.buildpaths.newsourcepage.ClasspathModifierQueries.IInclusionExclusionQuery;
 import org.eclipse.jdt.internal.ui.wizards.buildpaths.newsourcepage.ClasspathModifierQueries.ILinkToQuery;
 import org.eclipse.jdt.internal.ui.wizards.buildpaths.newsourcepage.ClasspathModifierQueries.IOutputLocationQuery;
@@ -88,7 +95,7 @@ public class ClasspathModifier {
 
 	private IClasspathModifierListener fListener;
 
-	protected ClasspathModifier() {
+	public ClasspathModifier() {
 		this(null);
 	}
 
@@ -138,10 +145,10 @@ public class ClasspathModifier {
 	 * (e.g. the operation was cancelled).
 	 * @throws CoreException 
 	 * @throws OperationCanceledException 
-	 * @see ClasspathModifierQueries.IFolderCreationQuery
+	 * @see ClasspathModifierQueries.ICreateFolderQuery
 	 * @see ClasspathModifierQueries.OutputFolderQuery
 	 */
-	protected List createFolder(IFolderCreationQuery folderQuery, OutputFolderQuery outputQuery, IJavaProject project, IProgressMonitor monitor) throws OperationCanceledException, CoreException {
+	protected List createFolder(ICreateFolderQuery folderQuery, OutputFolderQuery outputQuery, IJavaProject project, IProgressMonitor monitor) throws OperationCanceledException, CoreException {
 		if (folderQuery.doQuery()) {
 			IFolder folder= folderQuery.getCreatedFolder();
 			if (folder != null) {
@@ -224,7 +231,7 @@ public class ClasspathModifier {
 				}
 				
 				Set modifiedSourceEntries= new HashSet();
-				BuildPathBasePage.fixNestingConflicts(newEntries, existingEntries, modifiedSourceEntries);
+				BuildPathBasePage.fixNestingConflicts((CPListElement[])newEntries.toArray(new CPListElement[newEntries.size()]), (CPListElement[])existingEntries.toArray(new CPListElement[existingEntries.size()]), modifiedSourceEntries);
 
 				setNewEntry(existingEntries, newEntries, project, new SubProgressMonitor(monitor, 1));
 
@@ -1106,7 +1113,7 @@ public class ClasspathModifier {
 	 * is a source folder 
 	 * 
 	 * @param project the project to test
-	 * @return <code>true</code> if is theproject is a source folder
+	 * @return <code>true</code> if <code>project</code> is a source folder
 	 * <code>false</code> otherwise.
 	 */
 	public static boolean isSourceFolder(IJavaProject project) throws JavaModelException {
@@ -1181,7 +1188,7 @@ public class ClasspathModifier {
 	 * @throws CoreException 
 	 * @throws OperationCanceledException 
 	 */
-	private CPListElement addToClasspath(IResource resource, List existingEntries, List newEntries, IJavaProject project, IProgressMonitor monitor) throws OperationCanceledException, CoreException {
+	public static CPListElement addToClasspath(IResource resource, List existingEntries, List newEntries, IJavaProject project, IProgressMonitor monitor) throws OperationCanceledException, CoreException {
 		if (monitor == null)
 			monitor= new NullProgressMonitor();
 		try {
@@ -1221,7 +1228,7 @@ public class ClasspathModifier {
 	 * @throws CoreException 
 	 * @throws OperationCanceledException 
 	 */
-	private CPListElement addToClasspath(IJavaElement javaElement, List existingEntries, List newEntries, IJavaProject project, IProgressMonitor monitor) throws OperationCanceledException, CoreException {
+	public static CPListElement addToClasspath(IJavaElement javaElement, List existingEntries, List newEntries, IJavaProject project, IProgressMonitor monitor) throws OperationCanceledException, CoreException {
 		if (monitor == null)
 			monitor= new NullProgressMonitor();
 		try {
@@ -1243,7 +1250,7 @@ public class ClasspathModifier {
 	 * @return returns the Java project
 	 * @throws CoreException
 	 */
-	private IJavaProject removeFromClasspath(IJavaProject project, List existingEntries, IProgressMonitor monitor) throws CoreException {
+	public static IJavaProject removeFromClasspath(IJavaProject project, List existingEntries, IProgressMonitor monitor) throws CoreException {
 		CPListElement elem= getListElement(project.getPath(), existingEntries);
 		if (elem != null) {
 			existingEntries.remove(elem);
@@ -1262,7 +1269,7 @@ public class ClasspathModifier {
 	 * @return returns the <code>IResource</code> that has been removed from the build path; 
 	 * is of type <code>IFile</code> if the root was an archive, otherwise <code>IFolder</code> or <code>null<code> for external archives.
 	 */
-	private IResource removeFromClasspath(IPackageFragmentRoot root, List existingEntries, IJavaProject project, IProgressMonitor monitor) throws CoreException {
+	public static IResource removeFromClasspath(IPackageFragmentRoot root, List existingEntries, IJavaProject project, IProgressMonitor monitor) throws CoreException {
 		if (monitor == null)
 			monitor= new NullProgressMonitor();
 		try {
@@ -1270,11 +1277,66 @@ public class ClasspathModifier {
 			IClasspathEntry entry= root.getRawClasspathEntry();
 			CPListElement elem= CPListElement.createFromExisting(entry, project);
 			existingEntries.remove(elem);
+			removeFilters(elem.getPath(), project, existingEntries);
 			return elem.getResource();
 		} finally {
 			monitor.done();
 		}
 	}
+	
+	/**
+	 * Remove <code>path</code> from inclusion/exlusion filters in all <code>existingEntries</code>
+	 * 
+	 * @param path the path to remove
+	 * @param project the Java project
+	 * @param existingEntries a list of <code>CPListElement</code> representing the build path
+	 * entries of the project.
+	 * @return returns a <code>List</code> of <code>CPListElement</code> of modified elements, not null.
+	 */
+	public static List removeFilters(IPath path, IJavaProject project, List existingEntries) {
+		if (path == null)
+			return Collections.EMPTY_LIST;
+		
+		IPath projPath= project.getPath();
+		if (projPath.isPrefixOf(path)) {
+			path= path.removeFirstSegments(projPath.segmentCount()).addTrailingSeparator();
+		}
+		
+		List result= new ArrayList();
+		for (Iterator iter= existingEntries.iterator(); iter.hasNext();) {
+			CPListElement element= (CPListElement)iter.next();
+			boolean hasChange= false;
+			IPath[] exlusions= (IPath[])element.getAttribute(CPListElement.EXCLUSION);
+			if (exlusions != null) {
+				List exlusionList= new ArrayList(exlusions.length);
+				for (int i= 0; i < exlusions.length; i++) {
+					if (!exlusions[i].equals(path)) {
+						exlusionList.add(exlusions[i]);
+					} else {
+						hasChange= true;
+					}
+				}
+				element.setAttribute(CPListElement.EXCLUSION, exlusionList.toArray(new IPath[exlusionList.size()]));
+			}
+			
+			IPath[] inclusion= (IPath[])element.getAttribute(CPListElement.INCLUSION);
+			if (inclusion != null) {
+				List inclusionList= new ArrayList(inclusion.length);
+				for (int i= 0; i < inclusion.length; i++) {
+					if (!inclusion[i].equals(path)) {
+						inclusionList.add(inclusion[i]);
+					} else {
+						hasChange= true;
+					}
+				}
+				element.setAttribute(CPListElement.INCLUSION, inclusionList.toArray(new IPath[inclusionList.size()]));
+			}
+			if (hasChange) {
+				result.add(element);
+			}
+		}
+		return result;
+	}
 
 	/**
 	 * Include the given <code>IResource</code>. This means that the inclusion filter for the
@@ -1329,7 +1391,7 @@ public class ClasspathModifier {
 	 * @return a <code>IResource</code> corresponding to the excluded element
 	 * @throws JavaModelException 
 	 */
-	private IResource exclude(String name, IPath fullPath, CPListElement entry, IJavaProject project, IProgressMonitor monitor) throws JavaModelException {
+	private static IResource exclude(String name, IPath fullPath, CPListElement entry, IJavaProject project, IProgressMonitor monitor) throws JavaModelException {
 		if (monitor == null)
 			monitor= new NullProgressMonitor();
 		IResource result;
@@ -1371,7 +1433,7 @@ public class ClasspathModifier {
 	 * @param project the Java project
 	 * @param monitor progress monitor, can be <code>null</code>
 	 */
-	private void exclude(IPath path, List existingEntries, List newEntries, IJavaProject project, IProgressMonitor monitor) throws JavaModelException {
+	public static void exclude(IPath path, List existingEntries, List newEntries, IJavaProject project, IProgressMonitor monitor) throws JavaModelException {
 		if (monitor == null)
 			monitor= new NullProgressMonitor();
 		try {
@@ -1392,7 +1454,7 @@ public class ClasspathModifier {
 			if (elem == null) {
 				elem= existingElem;
 			}
-			exclude(path.removeFirstSegments(path.segmentCount() - i).toString(), null, elem, project, new SubProgressMonitor(monitor, 1)); //$NON-NLS-1$
+			exclude(path.removeFirstSegments(path.segmentCount() - i).toString(), null, elem, project, new SubProgressMonitor(monitor, 1)); 
 		} finally {
 			monitor.done();
 		}
@@ -1415,7 +1477,7 @@ public class ClasspathModifier {
 	 * @return the resulting <code>IResource<code>
 	 * @throws JavaModelException
 	 */
-	private IResource exclude(IJavaElement javaElement, CPListElement entry, IJavaProject project, IProgressMonitor monitor) throws JavaModelException {
+	public static IResource exclude(IJavaElement javaElement, CPListElement entry, IJavaProject project, IProgressMonitor monitor) throws JavaModelException {
 		if (monitor == null)
 			monitor= new NullProgressMonitor();
 		try {
@@ -1477,7 +1539,7 @@ public class ClasspathModifier {
 	 * 
 	 * @see #exclude(List, IJavaProject, IProgressMonitor)
 	 */
-	private void unExclude(IResource resource, CPListElement entry, IJavaProject project, IProgressMonitor monitor) throws JavaModelException {
+	public static void unExclude(IResource resource, CPListElement entry, IJavaProject project, IProgressMonitor monitor) throws JavaModelException {
 		if (monitor == null)
 			monitor= new NullProgressMonitor();
 		try {
@@ -1532,7 +1594,7 @@ public class ClasspathModifier {
 	 * @return an attribute representing the modified output folder
 	 * @throws JavaModelException 
 	 */
-	private CPListElementAttribute resetOutputFolder(CPListElement entry, IJavaProject project) throws JavaModelException {
+	public static CPListElementAttribute resetOutputFolder(CPListElement entry, IJavaProject project) throws JavaModelException {
 		entry.setAttribute(CPListElement.OUTPUT, null);
 		CPListElementAttribute outputFolder= new CPListElementAttribute(entry, CPListElement.OUTPUT, entry.getAttribute(CPListElement.OUTPUT), true);
 		return outputFolder;
@@ -1550,7 +1612,7 @@ public class ClasspathModifier {
 	 * the second <code>CPListElement</code> parameter itself if there is no match.
 	 * @throws JavaModelException
 	 */
-	private CPListElement getClasspathEntry(List elements, CPListElement cpElement) throws JavaModelException {
+	public static CPListElement getClasspathEntry(List elements, CPListElement cpElement) throws JavaModelException {
 		for (int i= 0; i < elements.size(); i++) {
 			if (((CPListElement) elements.get(i)).getPath().equals(cpElement.getPath()))
 				return (CPListElement) elements.get(i);
@@ -1567,7 +1629,7 @@ public class ClasspathModifier {
 	 * @return the mathed <code>CPListElement</code> or <code>null</code> if 
 	 * no match could be found
 	 */
-	private CPListElement getListElement(IPath path, List elements) {
+	private static CPListElement getListElement(IPath path, List elements) {
 		for (int i= 0; i < elements.size(); i++) {
 			CPListElement element= (CPListElement) elements.get(i);
 			if (element.getEntryKind() == IClasspathEntry.CPE_SOURCE && element.getPath().equals(path)) {
@@ -1606,6 +1668,23 @@ public class ClasspathModifier {
 			monitor.done();
 		}
 	}
+	
+	public static void commitClassPath(List newEntries, IJavaProject project, IProgressMonitor monitor) throws JavaModelException {
+		if (monitor == null)
+			monitor= new NullProgressMonitor();
+		try {
+			IClasspathEntry[] entries= convert(newEntries);
+			IPath outputLocation= project.getOutputLocation();
+
+			IJavaModelStatus status= JavaConventions.validateClasspath(project, entries, outputLocation);
+			if (!status.isOK())
+				throw new JavaModelException(status);
+
+			project.setRawClasspath(entries, outputLocation, new SubProgressMonitor(monitor, 2));
+		} finally {
+			monitor.done();
+		}
+	}
 
 	/**
 	 * For a given list of entries, find out what representation they 
@@ -1619,7 +1698,7 @@ public class ClasspathModifier {
 	 * @param project the Java project
 	 * @return a list of elements corresponding to the passed entries.
 	 */
-	private List getCorrespondingElements(List entries, IJavaProject project) {
+	public static List getCorrespondingElements(List entries, IJavaProject project) {
 		List result= new ArrayList();
 		for (int i= 0; i < entries.size(); i++) {
 			Object element= entries.get(i);
@@ -1651,7 +1730,7 @@ public class ClasspathModifier {
 	 * @return the resource matching to the path. Can be
 	 * either an <code>IFile</code> or an <code>IFolder</code>.
 	 */
-	private IResource getResource(IPath path, IJavaProject project) {
+	private static IResource getResource(IPath path, IJavaProject project) {
 		return project.getProject().getWorkspace().getRoot().findMember(path);
 	}
 
@@ -1673,7 +1752,7 @@ public class ClasspathModifier {
 		try {
 			monitor.beginTask(NewWizardMessages.ClasspathModifier_Monitor_ComparePaths, paths.length); 
 			if (path.getFileExtension() == null)
-				path= new Path(completeName(path.toString())); //$NON-NLS-1$
+				path= new Path(completeName(path.toString())); 
 			for (int i= 0; i < paths.length; i++) {
 				if (paths[i].equals(path))
 					return true;
@@ -1687,14 +1766,14 @@ public class ClasspathModifier {
 
 	/**
 	 * Add a '/' at the end of the name if
-	 * it does not end with '.java'.
+	 * it does not end with '.java', or other Java-like extension.
 	 * 
 	 * @param name append '/' at the end if
 	 * necessary
 	 * @return modified string
 	 */
 	private static String completeName(String name) {
-		if (!name.endsWith(".java")) { //$NON-NLS-1$
+		if (!JavaCore.isJavaLikeFileName(name)) {
 			name= name + "/"; //$NON-NLS-1$
 			name= name.replace('.', '/');
 			return name;
@@ -1714,7 +1793,7 @@ public class ClasspathModifier {
 	 * @param monitor progress monitor, can be <code>null</code>
 	 * @return array which does not contain <code>path</code>
 	 */
-	private IPath[] remove(IPath path, IPath[] paths, IProgressMonitor monitor) {
+	private static IPath[] remove(IPath path, IPath[] paths, IProgressMonitor monitor) {
 		if (monitor == null)
 			monitor= new NullProgressMonitor();
 		try {
@@ -1793,7 +1872,7 @@ public class ClasspathModifier {
 	 * @param monitor a progress monitor, can be <code>null</code>
 	 * @throws CoreException in case that validation on one of the new entries fails
 	 */
-	private void setNewEntry(List existingEntries, List newEntries, IJavaProject project, IProgressMonitor monitor) throws CoreException {
+	public static void setNewEntry(List existingEntries, List newEntries, IJavaProject project, IProgressMonitor monitor) throws CoreException {
 		try {
 			monitor.beginTask(NewWizardMessages.ClasspathModifier_Monitor_SetNewEntry, existingEntries.size()); 
 			for (int i= 0; i < newEntries.size(); i++) {
@@ -1814,7 +1893,7 @@ public class ClasspathModifier {
 	 * @return an array containing build path entries 
 	 * corresponding to the list
 	 */
-	private IClasspathEntry[] convert(List list) {
+	private static IClasspathEntry[] convert(List list) {
 		IClasspathEntry[] entries= new IClasspathEntry[list.size()];
 		for (int i= 0; i < list.size(); i++) {
 			CPListElement element= (CPListElement) list.get(i);
@@ -1834,7 +1913,7 @@ public class ClasspathModifier {
 	 * @param project the Java project
 	 * @throws CoreException in case that validation fails
 	 */
-	private void validateAndAddEntry(CPListElement entry, List existingEntries, IJavaProject project) throws CoreException {
+	private static void validateAndAddEntry(CPListElement entry, List existingEntries, IJavaProject project) throws CoreException {
 		IPath path= entry.getPath();
 		IPath projPath= project.getProject().getFullPath();
 		IWorkspaceRoot workspaceRoot= ResourcesPlugin.getWorkspace().getRoot();
@@ -1854,10 +1933,13 @@ public class ClasspathModifier {
 						throw new CoreException(rootStatus);
 					}
 				} else {
-					IPath projLocation= project.getProject().getLocation();
-					if (projLocation != null && path.toFile().exists()) {
-						rootStatus.setError(NewWizardMessages.NewSourceFolderWizardPage_error_AlreadyExistingDifferentCase); 
-						throw new CoreException(rootStatus);
+					URI projLocation= project.getProject().getLocationURI();
+					if (projLocation != null) {
+						IFileStore store= EFS.getStore(projLocation).getChild(path);
+						if (store.fetchInfo().exists()) {
+							rootStatus.setError(NewWizardMessages.NewSourceFolderWizardPage_error_AlreadyExistingDifferentCase); 
+							throw new CoreException(rootStatus);
+						}
 					}
 				}
 			}
@@ -1876,7 +1958,7 @@ public class ClasspathModifier {
 				exclude(entry.getPath(), existingEntries, new ArrayList(), project, null);
 
 			IPath outputLocation= project.getOutputLocation();
-			existingEntries.add(entry);
+			insertAtEndOfCategory(entry, existingEntries);
 
 			IClasspathEntry[] entries= convert(existingEntries);
 
@@ -1907,7 +1989,37 @@ public class ClasspathModifier {
 		}
 	}
 
-	private boolean isExternalArchiveOrLibrary(CPListElement entry, IJavaProject project) {
+	private static void insertAtEndOfCategory(CPListElement entry, List existingEntries) {
+		int length= existingEntries.size();
+		CPListElement[] elements= (CPListElement[])existingEntries.toArray(new CPListElement[length]);
+		int i= 0;
+		while (i < length && elements[i].getClasspathEntry().getEntryKind() != entry.getClasspathEntry().getEntryKind()) {
+			i++;
+		}
+		if (i < length) {
+			i++;
+			while (i < length && elements[i].getClasspathEntry().getEntryKind() == entry.getClasspathEntry().getEntryKind()) {
+				i++;
+			}
+			existingEntries.add(i, entry);
+			return;
+		}
+		
+		switch (entry.getClasspathEntry().getEntryKind()) {
+		case IClasspathEntry.CPE_SOURCE:
+			existingEntries.add(0, entry);
+			break;
+		case IClasspathEntry.CPE_CONTAINER:
+		case IClasspathEntry.CPE_LIBRARY:
+		case IClasspathEntry.CPE_PROJECT:
+		case IClasspathEntry.CPE_VARIABLE:
+		default:
+			existingEntries.add(entry);
+			break;
+		}
+	}
+
+	private static boolean isExternalArchiveOrLibrary(CPListElement entry, IJavaProject project) {
 		if (entry.getEntryKind() == IClasspathEntry.CPE_LIBRARY || entry.getEntryKind() == IClasspathEntry.CPE_CONTAINER) {
 			if (entry.getResource() instanceof IFolder) {
 				return false;
@@ -1954,7 +2066,7 @@ public class ClasspathModifier {
 		};
 	}
 
-	private OutputFolderValidator getValidator(final List newElements, final IJavaProject project) throws JavaModelException {
+	public static OutputFolderValidator getValidator(final List newElements, final IJavaProject project) throws JavaModelException {
 		return new OutputFolderValidator(newElements, project) {
 
 			public boolean validate(IPath outputLocation) {
