@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,6 +10,8 @@
  *******************************************************************************/
 package org.eclipse.jdt.internal.corext.util;
 
+import org.eclipse.core.runtime.Assert;
+
 import org.eclipse.jdt.core.Flags;
 import org.eclipse.jdt.core.IJavaElement;
 import org.eclipse.jdt.core.IMember;
@@ -26,7 +28,6 @@ import org.eclipse.jdt.core.dom.IVariableBinding;
 import org.eclipse.jdt.core.dom.Modifier;
 import org.eclipse.jdt.core.dom.TypeDeclaration;
 
-import org.eclipse.jdt.internal.corext.Assert;
 
 public class JdtFlags {
 	private JdtFlags(){
@@ -274,7 +275,20 @@ public class JdtFlags {
 			return VISIBILITY_STRING_PRIVATE;
 		return VISIBILITY_STRING_PACKAGE;
 	}
-	
+
+	public static int getVisibilityCode(String visibilityString) {
+		Assert.isNotNull(visibilityString);
+		if (VISIBILITY_STRING_PACKAGE.equals(visibilityString))
+			return 0;
+		else if (VISIBILITY_STRING_PRIVATE.equals(visibilityString))
+			return Modifier.PRIVATE;
+		else if (VISIBILITY_STRING_PROTECTED.equals(visibilityString))
+			return Modifier.PROTECTED;
+		else if (VISIBILITY_STRING_PUBLIC.equals(visibilityString))
+			return Modifier.PUBLIC;
+		return VISIBILITY_CODE_INVALID;
+	}
+
 	public static void assertVisibility(int visibility){
 		Assert.isTrue(	visibility == Modifier.PUBLIC ||
 		            	visibility == Modifier.PROTECTED ||
