@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -13,6 +13,8 @@
  *******************************************************************************/
 package org.eclipse.jdt.internal.corext.refactoring.code.flow;
 
+import org.eclipse.core.runtime.Assert;
+
 import org.eclipse.jface.text.IRegion;
 
 import org.eclipse.jdt.core.dom.ASTNode;
@@ -29,7 +31,6 @@ import org.eclipse.jdt.core.dom.Statement;
 import org.eclipse.jdt.core.dom.SwitchStatement;
 import org.eclipse.jdt.core.dom.WhileStatement;
 
-import org.eclipse.jdt.internal.corext.Assert;
 import org.eclipse.jdt.internal.corext.dom.Selection;
 
 public class InputFlowAnalyzer extends FlowAnalyzer {
@@ -53,9 +54,12 @@ public class InputFlowAnalyzer extends FlowAnalyzer {
 			return fLoopNode;
 		}
 		public void process(ASTNode node) {
-			fFlowContext.setLoopReentranceMode(true);	
-			node.accept(this);
-			fFlowContext.setLoopReentranceMode(false);
+			try {
+				fFlowContext.setLoopReentranceMode(true);	
+				node.accept(this);
+			} finally {
+				fFlowContext.setLoopReentranceMode(false);
+			}
 		}
 		public void endVisit(DoStatement node) {
 			if (skipNode(node))
@@ -253,7 +257,7 @@ public class InputFlowAnalyzer extends FlowAnalyzer {
 	}
 	
 	private void handleLoopReentrance(ASTNode node) {
-		if (!fSelection.enclosedBy(node) || fLoopReentranceVisitor == null || fLoopReentranceVisitor.getLoopNode() != node)
+		if (!fSelection.coveredBy(node) || fLoopReentranceVisitor == null || fLoopReentranceVisitor.getLoopNode() != node)
 			return;
 		
 		fLoopReentranceVisitor.process(node);
