@@ -15,7 +15,7 @@ import org.eclipse.core.runtime.CoreException;
 import org.eclipse.ltk.core.refactoring.RefactoringStatus;
 
 /**
- * Represents processors in the JDT space the rename elements.
+ * Represents processors in the JDT space that rename elements.
  */
 public interface INameUpdating {
 	
@@ -34,6 +34,14 @@ public interface INameUpdating {
 	 */
 	public String getCurrentElementName();
 	
+	/**
+	 * Gets the original elements. Since an <code>INameUpdating</code> only renames one element,
+	 * this method must return an array containing exactly one element.
+	 * @return an array containing exactly one element
+	 * @see org.eclipse.ltk.core.refactoring.participants.RefactoringProcessor#getElements()
+	 */
+	public Object[] getElements();
+	
 	/**
 	 * Gets the element after renaming.
 	 */	
