@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -98,7 +98,7 @@ public class RefactoringExecutionHelper {
 		fNeedsSavedEditors= needsSavedEditors;
 	}
 	
-	public void perform() throws InterruptedException, InvocationTargetException {
+	public void perform(boolean cancelable) throws InterruptedException, InvocationTargetException {
 		Assert.isTrue(Display.getCurrent() != null);
 		final IJobManager manager=  Platform.getJobManager();
 		final IWorkspaceRoot rule= ResourcesPlugin.getWorkspace().getRoot();
@@ -128,12 +128,12 @@ public class RefactoringExecutionHelper {
 			Operation op= new Operation();
 			fRefactoring.setValidationContext(fParent);
 			try{
-				fExecContext.run(false, false, new OperationRunner(op, rule));
+				fExecContext.run(false, cancelable, new OperationRunner(op, rule));
 				RefactoringStatus validationStatus= op.fPerformChangeOperation.getValidationStatus();
 				if (validationStatus != null && validationStatus.hasFatalError()) {
 					MessageDialog.openError(fParent, fRefactoring.getName(), 
 						Messages.format(
-							RefactoringMessages.RefactoringExecutionHelper_cannot_execute, //$NON-NLS-1$
+							RefactoringMessages.RefactoringExecutionHelper_cannot_execute, 
 							validationStatus.getMessageMatchingSeverity(RefactoringStatus.FATAL)));
 					return;
 				}
@@ -152,6 +152,8 @@ public class RefactoringExecutionHelper {
 				} else {
 					throw e;
 				}
+			}catch (OperationCanceledException e) {
+				throw new InterruptedException(e.getMessage());
 			} finally {
 				saveHelper.triggerBuild();
 			}
