@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -13,7 +13,7 @@ package org.eclipse.jdt.internal.ui.packageview;
 import org.eclipse.jdt.internal.ui.IJavaHelpContextIds;
 import org.eclipse.jdt.internal.ui.JavaPluginImages;
 import org.eclipse.jface.action.Action;
-import org.eclipse.ui.help.WorkbenchHelp;
+import org.eclipse.ui.PlatformUI;
 
 /**
  * Collapse all nodes.
@@ -23,13 +23,13 @@ class CollapseAllAction extends Action {
 	private PackageExplorerPart fPackageExplorer;
 	
 	CollapseAllAction(PackageExplorerPart part) {
-		super(PackagesMessages.getString("CollapseAllAction.label")); //$NON-NLS-1$
-		setDescription(PackagesMessages.getString("CollapseAllAction.description")); //$NON-NLS-1$
-		setToolTipText(PackagesMessages.getString("CollapseAllAction.tooltip")); //$NON-NLS-1$
+		super(PackagesMessages.CollapseAllAction_label); 
+		setDescription(PackagesMessages.CollapseAllAction_description); 
+		setToolTipText(PackagesMessages.CollapseAllAction_tooltip); 
 		JavaPluginImages.setLocalImageDescriptors(this, "collapseall.gif"); //$NON-NLS-1$
 		
 		fPackageExplorer= part;
-		WorkbenchHelp.setHelp(this, IJavaHelpContextIds.COLLAPSE_ALL_ACTION);
+		PlatformUI.getWorkbench().getHelpSystem().setHelp(this, IJavaHelpContextIds.COLLAPSE_ALL_ACTION);
 	}
  
 	public void run() { 
