@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -21,6 +21,8 @@ import org.eclipse.swt.widgets.Shell;
 
 import org.eclipse.jface.preference.IPreferenceStore;
 import org.eclipse.jface.preference.PreferenceConverter;
+import org.eclipse.jface.util.Assert;
+
 import org.eclipse.jface.text.BadLocationException;
 import org.eclipse.jface.text.DefaultInformationControl;
 import org.eclipse.jface.text.IDocument;
@@ -29,25 +31,30 @@ import org.eclipse.jface.text.IInformationControlCreator;
 import org.eclipse.jface.text.IRegion;
 import org.eclipse.jface.text.ITextViewer;
 import org.eclipse.jface.text.Position;
-import org.eclipse.jface.text.contentassist.ContentAssistant;
+import org.eclipse.jface.text.quickassist.IQuickAssistAssistant;
+import org.eclipse.jface.text.quickassist.QuickAssistAssistant;
 import org.eclipse.jface.text.source.Annotation;
 import org.eclipse.jface.text.source.IAnnotationModel;
-import org.eclipse.jface.util.Assert;
+import org.eclipse.jface.text.source.ISourceViewer;
 
 import org.eclipse.ui.IEditorPart;
 import org.eclipse.ui.texteditor.IDocumentProvider;
 import org.eclipse.ui.texteditor.ITextEditor;
 
+import org.eclipse.jdt.core.ICompilationUnit;
+import org.eclipse.jdt.core.IJavaElement;
+
 import org.eclipse.jdt.ui.JavaUI;
 import org.eclipse.jdt.ui.PreferenceConstants;
 import org.eclipse.jdt.ui.text.IColorManager;
 import org.eclipse.jdt.ui.text.JavaTextTools;
 
 import org.eclipse.jdt.internal.ui.JavaPlugin;
+import org.eclipse.jdt.internal.ui.javaeditor.ASTProvider;
 import org.eclipse.jdt.internal.ui.text.HTMLTextPresenter;
 
 
-public class JavaCorrectionAssistant extends ContentAssistant {
+public class JavaCorrectionAssistant extends QuickAssistAssistant {
 
 	private ITextViewer fViewer;
 	private ITextEditor fEditor;
@@ -67,12 +74,8 @@ public class JavaCorrectionAssistant extends ContentAssistant {
 
 		JavaCorrectionProcessor processor= new JavaCorrectionProcessor(this);
 
-		setContentAssistProcessor(processor, IDocument.DEFAULT_CONTENT_TYPE);
+		setQuickAssistProcessor(processor);
 
-		enableAutoActivation(false);
-		enableAutoInsert(false);
-
-		setContextInformationPopupOrientation(CONTEXT_INFO_ABOVE);
 		setInformationControlCreator(getInformationControlCreator());
 
 		JavaTextTools textTools= JavaPlugin.getDefault().getJavaTextTools();
@@ -108,11 +111,11 @@ public class JavaCorrectionAssistant extends ContentAssistant {
 	/* (non-Javadoc)
 	 * @see org.eclipse.jface.text.contentassist.IContentAssistant#install(org.eclipse.jface.text.ITextViewer)
 	 */
-	public void install(ITextViewer textViewer) {
-		super.install(textViewer);
-		fViewer= textViewer;
+	public void install(ISourceViewer sourceViewer) {
+		super.install(sourceViewer);
+		fViewer= sourceViewer;
 
-		fLightBulbUpdater= new QuickAssistLightBulbUpdater(fEditor, textViewer);
+		fLightBulbUpdater= new QuickAssistLightBulbUpdater(fEditor, sourceViewer);
 		fLightBulbUpdater.install();
 	}
 
@@ -129,6 +132,11 @@ public class JavaCorrectionAssistant extends ContentAssistant {
 		super.uninstall();
 	}
 
+	/*
+	 * @see org.eclipse.jface.text.quickassist.QuickAssistAssistant#showPossibleQuickAssists()
+	 * @since 3.2
+	 */
+
 	/**
 	 * Show completions at caret position. If current
 	 * position does not contain quick fixes look for
@@ -136,15 +144,15 @@ public class JavaCorrectionAssistant extends ContentAssistant {
 	 * to right and restarting at end of line if the
 	 * beginning of the line is reached.
 	 *
-	 * @see org.eclipse.jface.text.contentassist.IContentAssistant#showPossibleCompletions()
+	 * @see IQuickAssistAssistant#showPossibleQuickAssists()
 	 */
-	public String showPossibleCompletions() {
+	public String showPossibleQuickAssists() {
 		fPosition= null;
 		fCurrentAnnotations= null;
 		
 		if (fViewer == null || fViewer.getDocument() == null)
 			// Let superclass deal with this
-			return super.showPossibleCompletions();
+			return super.showPossibleQuickAssists();
 
 
 		ArrayList resultingAnnotations= new ArrayList(20);
@@ -165,7 +173,7 @@ public class JavaCorrectionAssistant extends ContentAssistant {
 		}
 		fCurrentAnnotations= (Annotation[]) resultingAnnotations.toArray(new Annotation[resultingAnnotations.size()]);
 
-		return super.showPossibleCompletions();
+		return super.showPossibleQuickAssists();
 	}
 	
 	
@@ -186,6 +194,9 @@ public class JavaCorrectionAssistant extends ContentAssistant {
 		if (model == null) {
 			return invocationLocation;
 		}
+		
+		ensureUpdatedAnnotations(editor);
+		
 		Iterator iter= model.getAnnotationIterator();
 		if (goToClosest) {
 			IRegion lineInfo= getRegionOfInterest(editor, invocationLocation);
@@ -233,6 +244,13 @@ public class JavaCorrectionAssistant extends ContentAssistant {
 		}
 	}
 
+	private static void ensureUpdatedAnnotations(ITextEditor editor) {
+		Object inputElement= editor.getEditorInput().getAdapter(IJavaElement.class);
+		if (inputElement instanceof ICompilationUnit) {
+			JavaPlugin.getDefault().getASTProvider().getAST((ICompilationUnit) inputElement, ASTProvider.WAIT_ACTIVE_ONLY, null);
+		}
+	}
+
 	private static int processAnnotation(Annotation annot, Position pos, int invocationLocation, int bestOffset) {
 		int posBegin= pos.offset;
 		int posEnd= posBegin + pos.length;
@@ -315,5 +333,4 @@ public class JavaCorrectionAssistant extends ContentAssistant {
 	public Annotation[] getAnnotationsAtOffset() {
 		return fCurrentAnnotations;
 	}
-	
 }
