@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -14,6 +14,7 @@ import java.io.File;
 import java.lang.reflect.InvocationTargetException;
 import java.util.List;
 
+import org.eclipse.core.runtime.Assert;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
@@ -25,7 +26,6 @@ import org.eclipse.core.resources.IWorkspaceRoot;
 import org.eclipse.core.resources.ResourcesPlugin;
 
 import org.eclipse.swt.SWT;
-import org.eclipse.swt.custom.CLabel;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
 import org.eclipse.swt.widgets.Composite;
@@ -50,7 +50,6 @@ import org.eclipse.jdt.core.IClasspathEntry;
 import org.eclipse.jdt.core.IJavaProject;
 import org.eclipse.jdt.core.JavaCore;
 
-import org.eclipse.jdt.internal.corext.Assert;
 import org.eclipse.jdt.internal.corext.util.Messages;
 
 import org.eclipse.jdt.internal.ui.IJavaHelpContextIds;
@@ -91,7 +90,7 @@ public class SourceAttachmentBlock {
 	private IWorkspaceRoot fWorkspaceRoot;
 	
 	private Control fSWTWidget;
-	private CLabel fFullPathResolvedLabel;
+	private Label fFullPathResolvedLabel;
 	
 	private IJavaProject fProject;
 	private IClasspathEntry fEntry;
@@ -211,39 +210,36 @@ public class SourceAttachmentBlock {
 		if (isVariableEntry()) {
 			int widthHint= converter.convertWidthInCharsToPixels(50);
 			
-			GridData gd= new GridData(GridData.HORIZONTAL_ALIGN_FILL);
-			gd.horizontalSpan= 4;
-			gd.widthHint= converter.convertWidthInCharsToPixels(50);
-
-			Label message= new Label(composite, SWT.LEFT + SWT.WRAP);
+			Label message= new Label(composite, SWT.WRAP);
+			GridData gd= new GridData(GridData.FILL, GridData.BEGINNING, false, false, 4, 1);
+			gd.widthHint= widthHint;
 			message.setLayoutData(gd);
 			message.setText(Messages.format(NewWizardMessages.SourceAttachmentBlock_message, fEntry.getPath().lastSegment())); 
 			
-			DialogField.createEmptySpace(composite, 1);
-			gd= new GridData(GridData.HORIZONTAL_ALIGN_FILL);
+			//DialogField.createEmptySpace(composite, 1);
+
+			Label desc= new Label(composite, SWT.WRAP);
+			gd= new GridData(GridData.FILL, GridData.BEGINNING, false, false, 4, 1);
 			gd.widthHint= widthHint;
-			gd.horizontalSpan= 2;
-			Label desc= new Label(composite, SWT.LEFT + SWT.WRAP);
-			desc.setText(NewWizardMessages.SourceAttachmentBlock_filename_description); 
 			desc.setLayoutData(gd);
-			DialogField.createEmptySpace(composite, 1);
+			desc.setText(NewWizardMessages.SourceAttachmentBlock_filename_description);
 			
 			fFileNameField.doFillIntoGrid(composite, 4);
 			LayoutUtil.setWidthHint(fFileNameField.getTextControl(null), widthHint);
 
 			// label that shows the resolved path for variable jars
-			DialogField.createEmptySpace(composite, 1);	
-			fFullPathResolvedLabel= new CLabel(composite, SWT.LEFT);
+			//DialogField.createEmptySpace(composite, 1);	
+			fFullPathResolvedLabel= new Label(composite, SWT.WRAP);
 			fFullPathResolvedLabel.setText(getResolvedLabelString());
-			fFullPathResolvedLabel.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
-			DialogField.createEmptySpace(composite, 2);
+			gd= new GridData(GridData.FILL, GridData.BEGINNING, false, false, 4, 1);
+			gd.widthHint= widthHint;
+			fFullPathResolvedLabel.setLayoutData(gd);
 			
 			LayoutUtil.setHorizontalGrabbing(fFileNameField.getTextControl(null));
 		} else {
 			int widthHint= converter.convertWidthInCharsToPixels(60);
 			
-			GridData gd= new GridData(GridData.HORIZONTAL_ALIGN_FILL);
-			gd.horizontalSpan= 3;
+			GridData gd= new GridData(GridData.FILL, GridData.BEGINNING, false, false, 3, 1);
 			gd.widthHint= converter.convertWidthInCharsToPixels(50);
 
 			Label message= new Label(composite, SWT.LEFT + SWT.WRAP);
@@ -410,6 +406,8 @@ public class SourceAttachmentBlock {
 				}
 				
 			} else {
+				// JDT/Core only supports source attachments in archives on the
+				// local file system. So using getLocation is save here. 
 				File file= filePath.toFile();
 				IResource res= fWorkspaceRoot.findMember(filePath);
 				if (res != null && res.getLocation() != null) {
@@ -439,7 +437,7 @@ public class SourceAttachmentBlock {
 	
 	private IPath chooseExtension() {
 		IPath currPath= getFilePath();
-		if (currPath.isEmpty()) {
+		if (currPath.segmentCount() == 0) {
 			currPath= fEntry.getPath();
 		}		
 	
@@ -465,7 +463,7 @@ public class SourceAttachmentBlock {
 	 */
 	private IPath chooseExtJarFile() {
 		IPath currPath= getFilePath();
-		if (currPath.isEmpty()) {
+		if (currPath.segmentCount() == 0) {
 			currPath= fEntry.getPath();
 		}		
 			
@@ -486,7 +484,7 @@ public class SourceAttachmentBlock {
 	
 	private IPath chooseExtFolder() {
 		IPath currPath= getFilePath();
-		if (currPath.isEmpty()) {
+		if (currPath.segmentCount() == 0) {
 			currPath= fEntry.getPath();
 		}
 		if (ArchiveFileFilter.isArchivePath(currPath)) {
@@ -577,7 +575,8 @@ public class SourceAttachmentBlock {
 		return new IRunnableWithProgress() {
 			public void run(IProgressMonitor monitor) throws InvocationTargetException {				
 				try {
-					BuildPathSupport.modifyClasspathEntry(shell, newEntry, jproject, containerPath, monitor);
+					String[] changedAttributes= { CPListElement.SOURCEATTACHMENT };
+					BuildPathSupport.modifyClasspathEntry(shell, newEntry, changedAttributes, jproject, containerPath, monitor);
 				} catch (CoreException e) {
 					throw new InvocationTargetException(e);
 				}
