@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,51 +11,41 @@
 package org.eclipse.jdt.internal.ui.wizards.buildpaths;
 
 import java.util.ArrayList;
-import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
-import java.util.Set;
-
-import org.eclipse.core.resources.IContainer;
-import org.eclipse.core.resources.IFolder;
-import org.eclipse.core.resources.IProject;
-import org.eclipse.core.resources.IResource;
-import org.eclipse.core.resources.IWorkspaceRoot;
-import org.eclipse.core.resources.ResourcesPlugin;
 
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.Path;
 
+import org.eclipse.core.resources.IContainer;
+import org.eclipse.core.resources.IProject;
+import org.eclipse.core.resources.IResource;
+
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.KeyEvent;
 import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Control;
 import org.eclipse.swt.widgets.Shell;
 
-import org.eclipse.jface.dialogs.MessageDialog;
-import org.eclipse.jface.util.Assert;
-import org.eclipse.jface.viewers.ILabelProvider;
-import org.eclipse.jface.viewers.ITreeContentProvider;
+import org.eclipse.jface.action.IAction;
+import org.eclipse.jface.util.IPropertyChangeListener;
+import org.eclipse.jface.util.PropertyChangeEvent;
 import org.eclipse.jface.viewers.StructuredSelection;
-import org.eclipse.jface.viewers.ViewerFilter;
 import org.eclipse.jface.window.Window;
 
-import org.eclipse.ui.dialogs.NewFolderDialog;
-import org.eclipse.ui.model.BaseWorkbenchContentProvider;
-import org.eclipse.ui.model.WorkbenchLabelProvider;
+import org.eclipse.ui.INewWizard;
 
 import org.eclipse.jdt.core.IClasspathEntry;
 import org.eclipse.jdt.core.IJavaProject;
 
-import org.eclipse.jdt.internal.corext.util.Messages;
+import org.eclipse.jdt.internal.corext.buildpath.ClasspathModifier;
 
-import org.eclipse.jdt.ui.PreferenceConstants;
+import org.eclipse.jdt.ui.actions.AbstractOpenWizardAction;
 
 import org.eclipse.jdt.internal.ui.JavaPlugin;
 import org.eclipse.jdt.internal.ui.util.PixelConverter;
 import org.eclipse.jdt.internal.ui.wizards.NewWizardMessages;
-import org.eclipse.jdt.internal.ui.wizards.TypedViewerFilter;
 import org.eclipse.jdt.internal.ui.wizards.dialogfields.DialogField;
 import org.eclipse.jdt.internal.ui.wizards.dialogfields.IDialogFieldListener;
 import org.eclipse.jdt.internal.ui.wizards.dialogfields.ITreeListAdapter;
@@ -66,15 +56,74 @@ import org.eclipse.jdt.internal.ui.wizards.dialogfields.StringDialogField;
 import org.eclipse.jdt.internal.ui.wizards.dialogfields.TreeListDialogField;
 
 public class SourceContainerWorkbookPage extends BuildPathBasePage {
+	
+	private class OpenBuildPathWizardAction extends AbstractOpenWizardAction implements IPropertyChangeListener {
+		
+		private final BuildPathWizard fWizard;
+		private final List fSelectedElements;
+		
+		public OpenBuildPathWizardAction(BuildPathWizard wizard) {
+			fWizard= wizard;
+			addPropertyChangeListener(this);
+			fSelectedElements= fFoldersList.getSelectedElements();
+		}
+		
+		/**
+		 * {@inheritDoc}
+		 */
+		protected INewWizard createWizard() throws CoreException {
+			return fWizard;
+		}
+
+		/**
+		 * {@inheritDoc}
+		 */
+		public void propertyChange(PropertyChangeEvent event) {
+			if (event.getProperty().equals(IAction.RESULT)) {
+				if (event.getNewValue().equals(Boolean.TRUE)) {
+					finishWizard();
+				} else {
+					fWizard.cancel();
+				}
+			}
+		}
+		
+		protected void finishWizard() {
+			List insertedElements= fWizard.getInsertedElements();
+			refresh(insertedElements, fWizard.getRemovedElements(), fWizard.getModifiedElements(), fWizard.getOutputLocation());
+			
+			if (insertedElements.isEmpty()) {
+				fFoldersList.postSetSelection(new StructuredSelection(fSelectedElements));
+			}
+		}
+
+	}
+	
+	private static AddSourceFolderWizard newSourceFolderWizard(CPListElement element, List/*<CPListElement>*/ existingElements, String outputLocation, boolean newFolder) {
+		CPListElement[] existing= (CPListElement[])existingElements.toArray(new CPListElement[existingElements.size()]);
+		AddSourceFolderWizard wizard= new AddSourceFolderWizard(existing, element, new Path(outputLocation).makeAbsolute(), false, newFolder, newFolder, newFolder?CPListElement.isProjectSourceFolder(existing, element.getJavaProject()):false, newFolder);
+		wizard.setDoFlushChange(false);
+		return wizard;
+	}
+	
+	private static AddSourceFolderWizard newLinkedSourceFolderWizard(CPListElement element, List/*<CPListElement>*/ existingElements, String outputLocation, boolean newFolder) {
+		CPListElement[] existing= (CPListElement[])existingElements.toArray(new CPListElement[existingElements.size()]);
+		AddSourceFolderWizard wizard= new AddSourceFolderWizard(existing, element, new Path(outputLocation).makeAbsolute(), true, newFolder, newFolder, newFolder?CPListElement.isProjectSourceFolder(existing, element.getJavaProject()):false, newFolder);
+		wizard.setDoFlushChange(false);
+		return wizard;
+	}
+	
+	private static EditFilterWizard newEditFilterWizard(CPListElement element, List/*<CPListElement>*/ existingElements, String outputLocation) {
+		CPListElement[] existing= (CPListElement[])existingElements.toArray(new CPListElement[existingElements.size()]);
+		EditFilterWizard result = new EditFilterWizard(existing, element, new Path(outputLocation).makeAbsolute());
+		result.setDoFlushChange(false);
+		return result;
+	}
 
 	private ListDialogField fClassPathList;
 	private IJavaProject fCurrJProject;
-	private IPath fProjPath;
 	
 	private Control fSWTControl;
-	
-	private IWorkspaceRoot fWorkspaceRoot;
-	
 	private TreeListDialogField fFoldersList;
 	
 	private StringDialogField fOutputLocationField;
@@ -82,11 +131,11 @@ public class SourceContainerWorkbookPage extends BuildPathBasePage {
 	private SelectionButtonDialogField fUseFolderOutputs;
 	
 	private final int IDX_ADD= 0;
-	private final int IDX_EDIT= 2;
-	private final int IDX_REMOVE= 3;	
+	private final int IDX_ADD_LINK= 1;
+	private final int IDX_EDIT= 3;
+	private final int IDX_REMOVE= 4;	
 
 	public SourceContainerWorkbookPage(ListDialogField classPathList, StringDialogField outputLocationField) {
-		fWorkspaceRoot= ResourcesPlugin.getWorkspace().getRoot();
 		fClassPathList= classPathList;
 	
 		fOutputLocationField= outputLocationField;
@@ -99,6 +148,7 @@ public class SourceContainerWorkbookPage extends BuildPathBasePage {
 
 		buttonLabels= new String[] { 
 			NewWizardMessages.SourceContainerWorkbookPage_folders_add_button, 
+			NewWizardMessages.SourceContainerWorkbookPage_folders_link_source_button,
 			/* 1 */ null,
 			NewWizardMessages.SourceContainerWorkbookPage_folders_edit_button, 
 			NewWizardMessages.SourceContainerWorkbookPage_folders_remove_button
@@ -118,8 +168,7 @@ public class SourceContainerWorkbookPage extends BuildPathBasePage {
 	}
 	
 	public void init(IJavaProject jproject) {
-		fCurrJProject= jproject;
-		fProjPath= fCurrJProject.getProject().getFullPath();	
+		fCurrJProject= jproject;	
 		updateFoldersList();
 	}
 	
@@ -144,9 +193,10 @@ public class SourceContainerWorkbookPage extends BuildPathBasePage {
 		
 		for (int i= 0; i < folders.size(); i++) {
 			CPListElement cpe= (CPListElement) folders.get(i);
-			IPath[] patterns= (IPath[]) cpe.getAttribute(CPListElement.EXCLUSION);
+			IPath[] ePatterns= (IPath[]) cpe.getAttribute(CPListElement.EXCLUSION);
+			IPath[] iPatterns= (IPath[])cpe.getAttribute(CPListElement.INCLUSION);
 			boolean hasOutputFolder= (cpe.getAttribute(CPListElement.OUTPUT) != null);
-			if (patterns.length > 0 || hasOutputFolder) {
+			if (ePatterns.length > 0 || iPatterns.length > 0 || hasOutputFolder) {
 				fFoldersList.expandElement(cpe, 3);
 			}				
 		}
@@ -156,7 +206,7 @@ public class SourceContainerWorkbookPage extends BuildPathBasePage {
 		PixelConverter converter= new PixelConverter(parent);
 		Composite composite= new Composite(parent, SWT.NONE);
 		
-		LayoutUtil.doDefaultLayout(composite, new DialogField[] { fFoldersList, fUseFolderOutputs }, true, SWT.DEFAULT, SWT.DEFAULT);
+		LayoutUtil.doDefaultLayout(composite, new DialogField[] { fFoldersList, fUseFolderOutputs , fOutputLocationField}, true, SWT.DEFAULT, SWT.DEFAULT);
 		LayoutUtil.setHorizontalGrabbing(fFoldersList.getTreeControl(null));
 		
 		int buttonBarWidth= converter.convertWidthInCharsToPixels(24);
@@ -251,8 +301,39 @@ public class SourceContainerWorkbookPage extends BuildPathBasePage {
 			}
 		}
 	}
+
+	protected void sourcePageCustomButtonPressed(DialogField field, int index) {
+		if (field == fFoldersList) {
+			if (index == IDX_ADD) {
+				IProject project= fCurrJProject.getProject();
+				if (project.exists() && hasFolders(project)) {
+					List existingElements= fFoldersList.getElements();
+					CPListElement[] existing= (CPListElement[])existingElements.toArray(new CPListElement[existingElements.size()]);
+					CreateMultipleSourceFoldersDialog dialog= new CreateMultipleSourceFoldersDialog(fCurrJProject, existing, fOutputLocationField.getText(), getShell());
+					if (dialog.open() == Window.OK) {
+						refresh(dialog.getInsertedElements(), dialog.getRemovedElements(), dialog.getModifiedElements(), dialog.getOutputLocation());
+					}
+				} else {
+					CPListElement newElement= new CPListElement(fCurrJProject, IClasspathEntry.CPE_SOURCE);
+					AddSourceFolderWizard wizard= newSourceFolderWizard(newElement, fFoldersList.getElements(), fOutputLocationField.getText(), true);
+					OpenBuildPathWizardAction action= new OpenBuildPathWizardAction(wizard);
+					action.run();
+				}
+			} else if (index == IDX_ADD_LINK) {
+				CPListElement newElement= new CPListElement(fCurrJProject, IClasspathEntry.CPE_SOURCE);
+				AddSourceFolderWizard wizard= newLinkedSourceFolderWizard(newElement, fFoldersList.getElements(), fOutputLocationField.getText(), true);
+				OpenBuildPathWizardAction action= new OpenBuildPathWizardAction(wizard);
+				action.run();
+			} else if (index == IDX_EDIT) {
+				editEntry();
+			} else if (index == IDX_REMOVE) {
+				removeEntry();
+			}
+		}
+	}
 	
 	private boolean hasFolders(IContainer container) {
+		
 		try {
 			IResource[] members= container.members();
 			for (int i= 0; i < members.length; i++) {
@@ -263,63 +344,19 @@ public class SourceContainerWorkbookPage extends BuildPathBasePage {
 		} catch (CoreException e) {
 			// ignore
 		}
-		return false;
-	}
-	
-	
-	protected void sourcePageCustomButtonPressed(DialogField field, int index) {
-		if (field == fFoldersList) {
-			if (index == IDX_ADD) {
-				List elementsToAdd= new ArrayList(10);
-				IProject project= fCurrJProject.getProject();
-				if (project.exists()) {
-					if (hasFolders(project)) {
-						CPListElement[] srcentries= openSourceContainerDialog(null);
-						if (srcentries != null) {
-							for (int i= 0; i < srcentries.length; i++) {
-								elementsToAdd.add(srcentries[i]);
-							}
-						}
-					} else {
-						CPListElement entry= openNewSourceContainerDialog(null, true);
-						if (entry != null) {
-							elementsToAdd.add(entry);
-						}	
-					}
-				} else {
-					CPListElement entry= openNewSourceContainerDialog(null, false);
-					if (entry != null) {
-						elementsToAdd.add(entry);
-					}
-				}
-				if (!elementsToAdd.isEmpty()) {
-					if (fFoldersList.getSize() == 1) {
-						CPListElement existing= (CPListElement) fFoldersList.getElement(0);
-						if (existing.getResource() instanceof IProject) {
-							askForChangingBuildPathDialog(existing);
-						}
-					}
-					HashSet modifiedElements= new HashSet();
-					askForAddingExclusionPatternsDialog(elementsToAdd, modifiedElements);
-					
-					fFoldersList.addElements(elementsToAdd);
-					fFoldersList.postSetSelection(new StructuredSelection(elementsToAdd));
-					
-					if (!modifiedElements.isEmpty()) {
-						for (Iterator iter= modifiedElements.iterator(); iter.hasNext();) {
-							Object elem= iter.next();
-							fFoldersList.refresh(elem);
-							fFoldersList.expandElement(elem, 3);
-						}
-					}
-
-				}				
-			} else if (index == IDX_EDIT) {
-				editEntry();
-			} else if (index == IDX_REMOVE) {
-				removeEntry();
-			}
-		}
+		
+		List elements= fFoldersList.getElements();
+		if (elements.size() > 1)
+			return true;
+		
+		if (elements.size() == 0)
+			return false;
+		
+		CPListElement single= (CPListElement)elements.get(0);
+		if (single.getPath().equals(fCurrJProject.getPath()))
+			return false;
+		
+		return true;
 	}
 
 	private void editEntry() {
@@ -336,12 +373,14 @@ public class SourceContainerWorkbookPage extends BuildPathBasePage {
 	}
 
 	private void editElementEntry(CPListElement elem) {
-		CPListElement res= null;
-		
-		res= openNewSourceContainerDialog(elem, true);
-		
-		if (res != null) {
-			fFoldersList.replaceElement(elem, res);
+		if (elem.getLinkTarget() != null) {
+			AddSourceFolderWizard wizard= newLinkedSourceFolderWizard(elem, fFoldersList.getElements(), fOutputLocationField.getText(), false);
+			OpenBuildPathWizardAction action= new OpenBuildPathWizardAction(wizard);
+			action.run();
+		} else {
+			AddSourceFolderWizard wizard= newSourceFolderWizard(elem, fFoldersList.getElements(), fOutputLocationField.getText(), false);
+			OpenBuildPathWizardAction action= new OpenBuildPathWizardAction(wizard);
+			action.run();
 		}
 	}
 
@@ -355,20 +394,18 @@ public class SourceContainerWorkbookPage extends BuildPathBasePage {
 				fFoldersList.refresh();
 				fClassPathList.dialogFieldChanged(); // validate
 			}
-		} else if (key.equals(CPListElement.EXCLUSION)) {
-			showExclusionInclusionDialog(elem.getParent(), true);		
-		} else if (key.equals(CPListElement.INCLUSION)) {
-			showExclusionInclusionDialog(elem.getParent(), false);
-		}
-	}
-
-	private void showExclusionInclusionDialog(CPListElement selElement, boolean focusOnExclusion) {
-		ExclusionInclusionDialog dialog= new ExclusionInclusionDialog(getShell(), selElement, focusOnExclusion);
-		if (dialog.open() == Window.OK) {
-			selElement.setAttribute(CPListElement.INCLUSION, dialog.getInclusionPattern());
-			selElement.setAttribute(CPListElement.EXCLUSION, dialog.getExclusionPattern());
-			fFoldersList.refresh();
-			fClassPathList.dialogFieldChanged(); // validate
+		} else if (key.equals(CPListElement.EXCLUSION) || key.equals(CPListElement.INCLUSION)) {
+			EditFilterWizard wizard= newEditFilterWizard(elem.getParent(), fFoldersList.getElements(), fOutputLocationField.getText());
+			OpenBuildPathWizardAction action= new OpenBuildPathWizardAction(wizard);
+			action.run();
+		} else if (key.equals(CPListElement.NATIVE_LIB_PATH)) {
+			CPListElement selElement=  elem.getParent();
+			NativeLibrariesDialog dialog= new NativeLibrariesDialog(getShell(), selElement);
+			if (dialog.open() == Window.OK) {
+				selElement.setAttribute(CPListElement.NATIVE_LIB_PATH, dialog.getNativeLibraryPath());
+				fFoldersList.refresh();
+				fClassPathList.dialogFieldChanged(); // validate
+			}
 		}
 	}
 
@@ -399,6 +436,17 @@ public class SourceContainerWorkbookPage extends BuildPathBasePage {
 			fFoldersList.refresh();
 			fClassPathList.dialogFieldChanged(); // validate
 		} else {
+			for (Iterator iter= selElements.iterator(); iter.hasNext();) {
+				CPListElement element= (CPListElement)iter.next();
+				if (element.getEntryKind() == IClasspathEntry.CPE_SOURCE) {
+					List list= ClasspathModifier.removeFilters(element.getPath(), fCurrJProject, fFoldersList.getElements());
+					for (Iterator iterator= list.iterator(); iterator.hasNext();) {
+						CPListElement modified= (CPListElement)iterator.next();
+						fFoldersList.refresh(modified);
+						fFoldersList.expandElement(modified, 3);
+					}
+				}
+			}
 			fFoldersList.removeElements(selElements);
 		}
 	}
@@ -439,7 +487,11 @@ public class SourceContainerWorkbookPage extends BuildPathBasePage {
 		}
 		Object elem= selElements.get(0);
 		if (elem instanceof CPListElement) {
-			return false;
+			CPListElement cp= ((CPListElement)elem);
+			if (cp.getPath().equals(cp.getJavaProject().getPath()))
+				return false;
+			
+			return true;
 		}
 		if (elem instanceof CPListElementAttribute) {
 			return true;
@@ -498,128 +550,6 @@ public class SourceContainerWorkbookPage extends BuildPathBasePage {
 			fClassPathList.setElements(cpelements);
 		}
 	}
-		
-	private CPListElement openNewSourceContainerDialog(CPListElement existing, boolean includeLinked) {	
-		if (includeLinked) {
-			NewFolderDialog dialog= new NewFolderDialog(getShell(), fCurrJProject.getProject());
-			dialog.setTitle(NewWizardMessages.SourceContainerWorkbookPage_NewSourceFolderDialog_new_title); 
-			if (dialog.open() == Window.OK) {
-				IResource createdFolder= (IResource) dialog.getResult()[0];
-				return newCPSourceElement(createdFolder);
-			}
-			return null;
-		} else {
-			String title= (existing == null) ? NewWizardMessages.SourceContainerWorkbookPage_NewSourceFolderDialog_new_title : NewWizardMessages.SourceContainerWorkbookPage_NewSourceFolderDialog_edit_title; 
-	
-			IProject proj= fCurrJProject.getProject();
-			NewSourceFolderDialog dialog= new NewSourceFolderDialog(getShell(), title, proj, getExistingContainers(existing), existing);
-			dialog.setMessage(Messages.format(NewWizardMessages.SourceContainerWorkbookPage_NewSourceFolderDialog_description, fProjPath.toString())); 
-			if (dialog.open() == Window.OK) {
-				IResource folder= dialog.getSourceFolder();
-				return newCPSourceElement(folder);
-			}
-			return null;
-		}
-	}
-	
-	
-	
-	/**
-	 * Asks to change the output folder to 'proj/bin' when no source folders were existing
-	 */ 
-	private void askForChangingBuildPathDialog(CPListElement existing) {
-		IPath outputFolder= new Path(fOutputLocationField.getText());
-		
-		IPath newOutputFolder= null;
-		String message;
-		if (outputFolder.segmentCount() == 1) {
-			String outputFolderName= PreferenceConstants.getPreferenceStore().getString(PreferenceConstants.SRCBIN_BINNAME);
-			newOutputFolder= outputFolder.append(outputFolderName);
-			message= Messages.format(NewWizardMessages.SourceContainerWorkbookPage_ChangeOutputLocationDialog_project_and_output_message, newOutputFolder); 
-		} else {
-			message= NewWizardMessages.SourceContainerWorkbookPage_ChangeOutputLocationDialog_project_message; 
-		}
-		String title= NewWizardMessages.SourceContainerWorkbookPage_ChangeOutputLocationDialog_title; 
-		if (MessageDialog.openQuestion(getShell(), title, message)) {
-			fFoldersList.removeElement(existing);
-			if (newOutputFolder != null) {
-				fOutputLocationField.setText(newOutputFolder.toString());
-			}
-		}			
-	}
-	
-	private void askForAddingExclusionPatternsDialog(List newEntries, Set modifiedEntries) {
-		fixNestingConflicts(newEntries, fFoldersList.getElements(), modifiedEntries);
-		if (!modifiedEntries.isEmpty()) {
-			String title= NewWizardMessages.SourceContainerWorkbookPage_exclusion_added_title; 
-			String message= NewWizardMessages.SourceContainerWorkbookPage_exclusion_added_message; 
-			MessageDialog.openInformation(getShell(), title, message);
-		}
-	}
-	
-	private CPListElement[] openSourceContainerDialog(CPListElement existing) {
-		
-		Class[] acceptedClasses= new Class[] { IProject.class, IFolder.class };
-		List existingContainers= getExistingContainers(null);
-		
-		IProject[] allProjects= fWorkspaceRoot.getProjects();
-		ArrayList rejectedElements= new ArrayList(allProjects.length);
-		IProject currProject= fCurrJProject.getProject();
-		for (int i= 0; i < allProjects.length; i++) {
-			if (!allProjects[i].equals(currProject)) {
-				rejectedElements.add(allProjects[i]);
-			}
-		}
-		ViewerFilter filter= new TypedViewerFilter(acceptedClasses, rejectedElements.toArray());
-		
-		ILabelProvider lp= new WorkbenchLabelProvider();
-		ITreeContentProvider cp= new BaseWorkbenchContentProvider();
-
-		String title= (existing == null) ? NewWizardMessages.SourceContainerWorkbookPage_ExistingSourceFolderDialog_new_title : NewWizardMessages.SourceContainerWorkbookPage_ExistingSourceFolderDialog_edit_title; 
-		String message= (existing == null) ? NewWizardMessages.SourceContainerWorkbookPage_ExistingSourceFolderDialog_new_description : NewWizardMessages.SourceContainerWorkbookPage_ExistingSourceFolderDialog_edit_description; 
-
-		MultipleFolderSelectionDialog dialog= new MultipleFolderSelectionDialog(getShell(), lp, cp);
-		dialog.setExisting(existingContainers.toArray());
-		dialog.setTitle(title);
-		dialog.setMessage(message);
-		dialog.addFilter(filter);
-		dialog.setInput(fCurrJProject.getProject().getParent());
-		if (existing == null) {
-			dialog.setInitialFocus(fCurrJProject.getProject());
-		} else {
-			dialog.setInitialFocus(existing.getResource());
-		}		
-		if (dialog.open() == Window.OK) {
-			Object[] elements= dialog.getResult();	
-			CPListElement[] res= new CPListElement[elements.length];
-			for (int i= 0; i < res.length; i++) {
-				IResource elem= (IResource)elements[i];
-				res[i]= newCPSourceElement(elem);
-			}
-			return res;
-		}
-		return null;
-	}
-	
-	private List getExistingContainers(CPListElement existing) {
-		List res= new ArrayList();
-		List cplist= fFoldersList.getElements();
-		for (int i= 0; i < cplist.size(); i++) {
-			CPListElement elem= (CPListElement)cplist.get(i);
-			if (elem != existing) {
-				IResource resource= elem.getResource();
-				if (resource instanceof IContainer) { // defensive code
-					res.add(resource);	
-				}
-			}
-		}
-		return res;
-	}
-	
-	private CPListElement newCPSourceElement(IResource res) {
-		Assert.isNotNull(res);
-		return new CPListElement(fCurrJProject, IClasspathEntry.CPE_SOURCE, res.getFullPath(), res);
-	}
 	
 	/*
 	 * @see BuildPathBasePage#getSelection
@@ -645,6 +575,29 @@ public class SourceContainerWorkbookPage extends BuildPathBasePage {
 	 */
 	public boolean isEntryKind(int kind) {
 		return kind == IClasspathEntry.CPE_SOURCE;
+	}
+
+	private void refresh(List insertedElements, List removedElements, List modifiedElements, IPath outputLocation) {
+		fFoldersList.addElements(insertedElements);
+		for (Iterator iter= insertedElements.iterator(); iter.hasNext();) {
+			CPListElement element= (CPListElement)iter.next();
+			fFoldersList.expandElement(element, 3);
+		}
+		
+		fFoldersList.removeElements(removedElements);
+		
+		for (Iterator iter= modifiedElements.iterator(); iter.hasNext();) {
+			CPListElement element= (CPListElement)iter.next();
+			fFoldersList.refresh(element);
+			fFoldersList.expandElement(element, 3);
+		}
+
+		fFoldersList.refresh(); //does enforce the order of the entries.
+		if (!insertedElements.isEmpty()) {
+			fFoldersList.postSetSelection(new StructuredSelection(insertedElements));
+		}
+
+		fOutputLocationField.setText(outputLocation.makeRelative().toOSString());
 	}	
 
 }
