@@ -1,17 +1,17 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.jdt.internal.ui.wizards.dialogfields;
 
-
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Iterator;
 import java.util.List;
 
@@ -54,7 +54,7 @@ import org.eclipse.jdt.internal.ui.util.TableLayoutComposite;
 /**
  * A list with a button bar.
  * Typical buttons are 'Add', 'Remove', 'Up' and 'Down'.
- * List model is independend of widget creation.
+ * List model is independent of widget creation.
  * DialogFields controls are: Label, List and Composite containing buttons.
  */
 public class ListDialogField extends DialogField {
@@ -88,6 +88,7 @@ public class ListDialogField extends DialogField {
 	}
 	
 	protected TableViewer fTable;
+	protected Control fTableControl;
 	protected ILabelProvider fLabelProvider;
 	protected ListViewerAdapter fListViewerAdapter;
 	protected List fElements;
@@ -104,7 +105,7 @@ public class ListDialogField extends DialogField {
 	
 	private Label fLastSeparator;
 	
-	private Control fTableControl;
+
 	private Composite fButtonsControl;
 	private ISelection fSelectionWhenEnabled;
 	
@@ -154,8 +155,8 @@ public class ListDialogField extends DialogField {
 		
 	/**
 	 * Sets the index of the 'remove' button in the button label array passed in the constructor.
-	 * The behaviour of the button marked as the 'remove' button will then be handled internally.
-	 * (enable state, button invocation behaviour)
+	 * The behavior of the button marked as the 'remove' button will then be handled internally.
+	 * (enable state, button invocation behavior)
 	 */	
 	public void setRemoveButtonIndex(int removeButtonIndex) {
 		Assert.isTrue(removeButtonIndex < fButtonLabels.length);
@@ -164,8 +165,8 @@ public class ListDialogField extends DialogField {
 
 	/**
 	 * Sets the index of the 'up' button in the button label array passed in the constructor.
-	 * The behaviour of the button marked as the 'up' button will then be handled internally.
-	 * (enable state, button invocation behaviour)
+	 * The behavior of the button marked as the 'up' button will then be handled internally.
+	 * (enable state, button invocation behavior)
 	 */	
 	public void setUpButtonIndex(int upButtonIndex) {
 		Assert.isTrue(upButtonIndex < fButtonLabels.length);
@@ -174,8 +175,8 @@ public class ListDialogField extends DialogField {
 
 	/**
 	 * Sets the index of the 'down' button in the button label array passed in the constructor.
-	 * The behaviour of the button marked as the 'down' button will then be handled internally.
-	 * (enable state, button invocation behaviour)
+	 * The behavior of the button marked as the 'down' button will then be handled internally.
+	 * (enable state, button invocation behavior)
 	 */	
 	public void setDownButtonIndex(int downButtonIndex) {
 		Assert.isTrue(downButtonIndex < fButtonLabels.length);
@@ -213,8 +214,14 @@ public class ListDialogField extends DialogField {
 			remove();
 		} else if (index == fUpButtonIndex) {
 			up();
+			if (!fButtonControls[index].isEnabled() && fDownButtonIndex != -1) {
+				fButtonControls[fDownButtonIndex].setFocus();
+			}
 		} else if (index == fDownButtonIndex) {
 			down();
+			if (!fButtonControls[index].isEnabled() && fUpButtonIndex != -1) {
+				fButtonControls[fUpButtonIndex].setFocus();
+			}
 		} else {
 			return false;
 		}
@@ -278,11 +285,11 @@ public class ListDialogField extends DialogField {
 	}
 	
 	
-	// ------ ui creation
+	// ------ UI creation
 	
 	/**
 	 * Returns the list control. When called the first time, the control will be created.
-	 * @param The parent composite when called the first time, or <code>null</code>
+	 * @param parent The parent composite when called the first time, or <code>null</code>
 	 * after.
 	 */
 	public Control getListControl(Composite parent) {
@@ -297,6 +304,7 @@ public class ListDialogField extends DialogField {
 				tableControl.setLayout(new TableLayout());
 			} else {
 				TableLayoutComposite composite= new TableLayoutComposite(parent, SWT.NONE);
+				composite.setFont(parent.getFont());
 				fTableControl= composite;
 				
 				fTable= createTableViewer(composite);
@@ -362,31 +370,34 @@ public class ListDialogField extends DialogField {
 	
 	protected TableViewer createTableViewer(Composite parent) {
 		Table table= new Table(parent, getListStyle());
+		table.setFont(parent.getFont());
 		return new TableViewer(table);
 	}	
 	
 	protected Button createButton(Composite parent, String label, SelectionListener listener) {
 		Button button= new Button(parent, SWT.PUSH);
+		button.setFont(parent.getFont());
 		button.setText(label);
 		button.addSelectionListener(listener);
 		GridData gd= new GridData();
 		gd.horizontalAlignment= GridData.FILL;
 		gd.grabExcessHorizontalSpace= true;
 		gd.verticalAlignment= GridData.BEGINNING;
-		gd.heightHint = SWTUtil.getButtonHeigthHint(button);
 		gd.widthHint = SWTUtil.getButtonWidthHint(button);
 	
 		button.setLayoutData(gd);
+		
 		return button;
 	}
 	
 	private Label createSeparator(Composite parent) {
-		Label separator= new Label(parent, SWT.NONE);
+		Label separator= new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL);
+		separator.setFont(parent.getFont());
 		separator.setVisible(false);
 		GridData gd= new GridData();
 		gd.horizontalAlignment= GridData.FILL;
 		gd.verticalAlignment= GridData.BEGINNING;
-		gd.heightHint= 4;
+		gd.verticalIndent= 4;
 		separator.setLayoutData(gd);
 		return separator;
 	}			
@@ -394,7 +405,7 @@ public class ListDialogField extends DialogField {
 	/**
 	 * Returns the composite containing the buttons. When called the first time, the control
 	 * will be created.
-	 * @param The parent composite when called the first time, or <code>null</code>
+	 * @param parent The parent composite when called the first time, or <code>null</code>
 	 * after.
 	 */	
 	public Composite getButtonBox(Composite parent) {
@@ -410,7 +421,8 @@ public class ListDialogField extends DialogField {
 				}
 			};
 			
-			Composite contents= new Composite(parent, SWT.NULL);
+			Composite contents= new Composite(parent, SWT.NONE);
+			contents.setFont(parent.getFont());
 			GridLayout layout= new GridLayout();
 			layout.marginWidth= 0;
 			layout.marginHeight= 0;
@@ -476,7 +488,7 @@ public class ListDialogField extends DialogField {
 	 * Updates the enable state of the all buttons
 	 */ 
 	protected void updateButtonState() {
-		if (fButtonControls != null) {
+		if (fButtonControls != null && isOkToUse(fTableControl) && fTableControl.isEnabled()) {
 			ISelection sel= fTable.getSelection();
 			for (int i= 0; i < fButtonControls.length; i++) {
 				Button button= fButtonControls[i];
@@ -539,9 +551,9 @@ public class ListDialogField extends DialogField {
 	/**
 	 * Sets the elements shown in the list.
 	 */
-	public void setElements(List elements) {
+	public void setElements(Collection elements) {
 		fElements= new ArrayList(elements);
-		if (fTable != null) {
+		if (isOkToUse(fTableControl)) {
 			fTable.refresh();
 		}
 		dialogFieldChanged();
@@ -570,13 +582,13 @@ public class ListDialogField extends DialogField {
 	}	
 
 	/**
-	 * Replace an element.
+	 * Replaces an element.
 	 */		
 	public void replaceElement(Object oldElement, Object newElement) throws IllegalArgumentException { 
 		int idx= fElements.indexOf(oldElement);
 		if (idx != -1) {
 			fElements.set(idx, newElement);
-			if (fTable != null) {
+			if (isOkToUse(fTableControl)) {
 				List selected= getSelectedElements();
 				if (selected.remove(oldElement)) {
 					selected.add(newElement);
@@ -588,26 +600,50 @@ public class ListDialogField extends DialogField {
 		} else {
 			throw new IllegalArgumentException();
 		}
-	}	
+	}
+	
+	/**
+	 * Notifies clients that the element has changed.
+	 */
+	public void elementChanged(Object element) throws IllegalArgumentException {
+		if (fElements.contains(element)) {
+			if (isOkToUse(fTableControl)) {
+				fTable.update(element, null);
+			}
+			dialogFieldChanged();
+		} else {
+			throw new IllegalArgumentException();
+		}
+	}
 
 	/**
 	 * Adds an element at the end of the list.
 	 */		
-	public void addElement(Object element) {		
-		if (fElements.contains(element)) {
-			return;
-		}
-		fElements.add(element);
-		if (fTable != null) {
-			fTable.add(element);
-		}
-		dialogFieldChanged();
+	public boolean addElement(Object element) {
+		return addElement(element, fElements.size());
 	}
+	
+	/**
+	 * Adds an element at a position.
+	 */		
+	public boolean addElement(Object element, int index) {
+		if (fElements.contains(element)) {
+			return false;
+		}
+		fElements.add(index, element);
+		if (isOkToUse(fTableControl)) {
+			fTable.refresh();
+			fTable.setSelection(new StructuredSelection(element));
+		}
+		
+		dialogFieldChanged();
+		return true;
+	}	
 
 	/**
 	 * Adds elements at the end of the list.
 	 */	
-	public void addElements(List elements) {
+	public boolean addElements(List elements) {
 		int nElements= elements.size();
 		
 		if (nElements > 0) {
@@ -620,28 +656,19 @@ public class ListDialogField extends DialogField {
 					elementsToAdd.add(elem);
 				}	
 			}
-			fElements.addAll(elementsToAdd);
-			if (fTable != null) {
-				fTable.add(elementsToAdd.toArray());
+			if (!elementsToAdd.isEmpty()) {
+				fElements.addAll(elementsToAdd);
+				if (isOkToUse(fTableControl)) {
+					fTable.add(elementsToAdd.toArray());
+					fTable.setSelection(new StructuredSelection(elementsToAdd));
+				}
+				dialogFieldChanged();
+				return true;
 			}
-			dialogFieldChanged();
 		}
+		return false;
 	}	
 
-	/**
-	 * Adds an element at a position.
-	 */		
-	public void insertElementAt(Object element, int index) {
-		if (fElements.contains(element)) {
-			return;
-		}
-		fElements.add(index, element);
-		if (fTable != null) {
-			fTable.add(element);
-		}
-		
-		dialogFieldChanged();
-	}	
 
 
 	/**
@@ -650,7 +677,7 @@ public class ListDialogField extends DialogField {
 	public void removeAllElements() {
 		if (fElements.size() > 0) {
 			fElements.clear();
-			if (fTable != null) {
+			if (isOkToUse(fTableControl)) {
 				fTable.refresh();
 			}
 			dialogFieldChanged();
@@ -662,7 +689,7 @@ public class ListDialogField extends DialogField {
 	 */		
 	public void removeElement(Object element) throws IllegalArgumentException {
 		if (fElements.remove(element)) {
-			if (fTable != null) {
+			if (isOkToUse(fTableControl)) {
 				fTable.remove(element);
 			}
 			dialogFieldChanged();
@@ -677,7 +704,7 @@ public class ListDialogField extends DialogField {
 	public void removeElements(List elements) {
 		if (elements.size() > 0) {
 			fElements.removeAll(elements);
-			if (fTable != null) {
+			if (isOkToUse(fTableControl)) {
 				fTable.remove(elements.toArray());
 			}
 			dialogFieldChanged();
@@ -694,7 +721,7 @@ public class ListDialogField extends DialogField {
 
 	public void selectElements(ISelection selection) {
 		fSelectionWhenEnabled= selection;
-		if (fTable != null) {
+		if (isOkToUse(fTableControl)) {
 			fTable.setSelection(selection, true);
 		}
 	}
@@ -717,6 +744,13 @@ public class ListDialogField extends DialogField {
 		}
 	}
 	
+	public void editElement(Object element) {
+		if (isOkToUse(fTableControl)) {
+			fTable.refresh(element);
+			fTable.editElement(element, 0);
+		}
+	}
+	
 		
 	public void postSetSelection(final ISelection selection) {
 		if (isOkToUse(fTableControl)) {
@@ -735,7 +769,8 @@ public class ListDialogField extends DialogField {
 	 * Refreshes the table.
 	 */
 	public void refresh() {
-		if (fTable != null) {
+		super.refresh();
+		if (isOkToUse(fTableControl)) {
 			fTable.refresh();
 		}
 	}
@@ -828,7 +863,7 @@ public class ListDialogField extends DialogField {
 	 */
 	public List getSelectedElements() {
 		List result= new ArrayList();
-		if (fTable != null) {
+		if (isOkToUse(fTableControl)) {
 			ISelection selection= fTable.getSelection();
 			if (selection instanceof IStructuredSelection) {
 				Iterator iter= ((IStructuredSelection)selection).iterator();
