@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -54,6 +54,13 @@ public class JdtActionConstants {
 	 */
 	public static final String OPEN_TYPE_HIERARCHY= "org.eclipse.jdt.ui.actions.OpenTypeHierarchy"; //$NON-NLS-1$
 
+    /**
+     * Navigate menu: name of standard Open Call Hierarchy global action
+     * (value <code>"org.eclipse.jdt.ui.actions.OpenCallHierarchy"</code>).
+     * @since 3.0
+     */
+    public static final String OPEN_CALL_HIERARCHY= "org.eclipse.jdt.ui.actions.OpenCallHierarchy"; //$NON-NLS-1$
+
 	/**
 	 * Navigate menu: name of standard Open External Javadoc global action
 	 * (value <code>"org.eclipse.jdt.ui.actions.OpenExternalJavaDoc"</code>).
@@ -101,7 +108,38 @@ public class JdtActionConstants {
 	public static final String UNCOMMENT= "org.eclipse.jdt.ui.actions.Uncomment"; //$NON-NLS-1$
 	
 	/**
-	 * Source menu: name of standard Shift Rightl action
+	 * Source menu: name of standard ToggleComment global action
+	 * (value <code>"org.eclipse.jdt.ui.actions.ToggleComment"</code>).
+	 * @since 3.0
+	 */
+	public static final String TOGGLE_COMMENT= "org.eclipse.jdt.ui.actions.ToggleComment"; //$NON-NLS-1$
+	
+	/**
+	 * Source menu: name of standard Block Comment global action
+	 * (value <code>"org.eclipse.jdt.ui.actions.AddBlockComment"</code>).
+	 * 
+	 * @since 3.0
+	 */
+	public static final String ADD_BLOCK_COMMENT= "org.eclipse.jdt.ui.actions.AddBlockComment"; //$NON-NLS-1$
+	
+	/**
+	 * Source menu: name of standard Block Uncomment global action
+	 * (value <code>"org.eclipse.jdt.ui.actions.RemoveBlockComment"</code>).
+	 * 
+	 * @since 3.0
+	 */
+	public static final String REMOVE_BLOCK_COMMENT= "org.eclipse.jdt.ui.actions.RemoveBlockComment"; //$NON-NLS-1$
+	
+	/**
+	 * Source menu: name of standard Indent global action
+	 * (value <code>"org.eclipse.jdt.ui.actions.Indent"</code>).
+	 * 
+	 * @since 3.0
+	 */
+	public static final String INDENT= "org.eclipse.jdt.ui.actions.Indent"; //$NON-NLS-1$
+	
+	/**
+	 * Source menu: name of standard Shift Right action
 	 * (value <code>"org.eclipse.jdt.ui.actions.ShiftRight"</code>).
 	 */
 	public static final String SHIFT_RIGHT= "org.eclipse.jdt.ui.actions.ShiftRight"; //$NON-NLS-1$
@@ -118,6 +156,13 @@ public class JdtActionConstants {
 	 */
 	public static final String FORMAT= "org.eclipse.jdt.ui.actions.Format"; //$NON-NLS-1$
 	
+	/**
+	 * Source menu: name of standard Format Element global action
+	 * (value <code>"org.eclipse.jdt.ui.actions.FormatElement"</code>).
+	 * @since 3.0
+	 */
+	public static final String FORMAT_ELEMENT= "org.eclipse.jdt.ui.actions.FormatElement"; //$NON-NLS-1$
+	
 	/**
 	 * Source menu: name of standard Add Import global action
 	 * (value <code>"org.eclipse.jdt.ui.actions.AddImport"</code>).
@@ -156,7 +201,7 @@ public class JdtActionConstants {
 	public static final String GENERATE_GETTER_SETTER= "org.eclipse.jdt.ui.actions.GenerateGetterSetter"; //$NON-NLS-1$
 
 	/**
-	 * Source menu: name of standard delegate methdos global action (value
+	 * Source menu: name of standard delegate methods global action (value
 	 * <code>"org.eclipse.jdt.ui.actions.GenerateDelegateMethods"</code>).
 	 * @since 2.1
 	 */
@@ -168,6 +213,19 @@ public class JdtActionConstants {
 	 */
 	public static final String ADD_CONSTRUCTOR_FROM_SUPERCLASS= "org.eclipse.jdt.ui.actions.AddConstructorFromSuperclass"; //$NON-NLS-1$
 	
+	/**
+	 * Source menu: name of standard Generate Constructor using Fields global action
+	 * (value <code>"org.eclipse.jdt.ui.actions.GenerateConstructorUsingFields"</code>).
+	 */
+	public static final String GENERATE_CONSTRUCTOR_USING_FIELDS= "org.eclipse.jdt.ui.actions.GenerateConstructorUsingFields"; //$NON-NLS-1$
+	
+	/**
+	 * Source menu: name of standard Generate hashCode() and equals() global action
+	 * (value <code>"org.eclipse.jdt.ui.actions.GenerateHashCodeEquals"</code>).
+	 * @since 3.2
+	 */
+	public static final String GENERATE_HASHCODE_EQUALS= "org.eclipse.jdt.ui.actions.GenerateHashCodeEquals"; //$NON-NLS-1$
+
 	/**
 	 * Source menu: name of standard Add Javadoc Comment global action
 	 * (value <code>"org.eclipse.jdt.ui.actions.AddJavaDocComment"</code>).
@@ -190,19 +248,27 @@ public class JdtActionConstants {
 	 * Source menu: name of standard Convert Line Delimiters To Windows global action
 	 * (value <code>"org.eclipse.jdt.ui.actions.ConvertLineDelimitersToWindows"</code>).
 	 */
-	public static String CONVERT_LINE_DELIMITERS_TO_WINDOWS= "org.eclipse.jdt.ui.actions.ConvertLineDelimitersToWindows"; //$NON-NLS-1$
+	public static final String CONVERT_LINE_DELIMITERS_TO_WINDOWS= "org.eclipse.jdt.ui.actions.ConvertLineDelimitersToWindows"; //$NON-NLS-1$
 
 	/**
 	 * Source menu: name of standard Convert Line Delimiters To UNIX global action
 	 * (value <code>"org.eclipse.jdt.ui.actions.ConvertLineDelimitersToUNIX"</code>).
 	 */
-	public static String CONVERT_LINE_DELIMITERS_TO_UNIX= "org.eclipse.jdt.ui.actions.ConvertLineDelimitersToUNIX"; //$NON-NLS-1$
+	public static final String CONVERT_LINE_DELIMITERS_TO_UNIX= "org.eclipse.jdt.ui.actions.ConvertLineDelimitersToUNIX"; //$NON-NLS-1$
 
 	/**
-	 * Source menu: name of standardConvert Line Delimiters ToMac global action
+	 * Source menu: name of standardConvert Line Delimiters To Mac global action
 	 * (value <code>"org.eclipse.jdt.ui.actions.ConvertLineDelimitersToMac"</code>).
 	 */
-	public static String CONVERT_LINE_DELIMITERS_TO_MAC= "org.eclipse.jdt.ui.actions.ConvertLineDelimitersToMac"; //$NON-NLS-1$
+	public static final String CONVERT_LINE_DELIMITERS_TO_MAC= "org.eclipse.jdt.ui.actions.ConvertLineDelimitersToMac"; //$NON-NLS-1$
+	
+	/**
+	 * Source menu: name of standard Clean up global action 
+	 * (value <code>"org.eclipse.jdt.ui.actions.CleanUp"</code>).
+	 * 
+	 * @since 3.2
+	 */
+	public static final String CLEAN_UP= "org.eclipse.jdt.ui.actions.CleanUp"; //$NON-NLS-1$
 
 	// Refactor menu
 	
@@ -265,11 +331,43 @@ public class JdtActionConstants {
 	 */
 	public static final String EXTRACT_CONSTANT= "org.eclipse.jdt.ui.actions.ExtractConstant"; //$NON-NLS-1$
 
+	/**
+	 * Refactor menu: name of standard Introduce Parameter global action
+	 * (value <code>"org.eclipse.jdt.ui.actions.IntroduceParameter"</code>).
+	 * 
+	 * @since 3.0
+	 */
+	public static final String INTRODUCE_PARAMETER= "org.eclipse.jdt.ui.actions.IntroduceParameter"; //$NON-NLS-1$
+
+	/**
+	 * Refactor menu: name of standard Introduce Factory global action
+	 * (value <code>"org.eclipse.jdt.ui.actions.IntroduceFactory"</code>).
+	 * 
+	 * @since 3.0
+	 */
+	public static final String INTRODUCE_FACTORY= "org.eclipse.jdt.ui.actions.IntroduceFactory"; //$NON-NLS-1$
+
 	/**
 	 * Refactor menu: name of standard Extract Method global action
 	 * (value <code>"org.eclipse.jdt.ui.actions.ExtractMethod"</code>).
 	 */
 	public static final String EXTRACT_METHOD= "org.eclipse.jdt.ui.actions.ExtractMethod"; //$NON-NLS-1$
+	
+	/**
+	 * Refactor menu: name of standard Replace Invocations global action
+	 * (value <code>"org.eclipse.jdt.ui.actions.ReplaceInvocations"</code>).
+	 * 
+	 * @since 3.2
+	 */
+	public static final String REPLACE_INVOCATIONS="org.eclipse.jdt.ui.actions.ReplaceInvocations"; //$NON-NLS-1$
+	
+	/**
+	 * Refactor menu: name of standard Introduce Indirection global action
+	 * (value <code>"org.eclipse.jdt.ui.actions.IntroduceIndirection"</code>).
+	 * 
+	 * @since 3.2
+	 */
+	public static final String INTRODUCE_INDIRECTION= "org.eclipse.jdt.ui.actions.IntroduceIndirection"; //$NON-NLS-1$
 
 	/**
 	 * Refactor menu: name of standard Inline global action 
@@ -287,6 +385,14 @@ public class JdtActionConstants {
 	 */
 	public static final String EXTRACT_INTERFACE= "org.eclipse.jdt.ui.actions.ExtractInterface"; //$NON-NLS-1$
 
+	/**
+	 * Refactor menu: name of standard Generalize Declared Type global action
+	 * (value <code>"org.eclipse.jdt.ui.actions.ChangeType"</code>).
+	 * 
+	 * @since 3.0
+	 */
+	public static final String CHANGE_TYPE= "org.eclipse.jdt.ui.actions.ChangeType"; //$NON-NLS-1$
+
 	/**
 	 * Refactor menu: name of standard global action to convert a nested type to a top level type
 	 * (value <code>"org.eclipse.jdt.ui.actions.MoveInnerToTop"</code>).
@@ -303,6 +409,14 @@ public class JdtActionConstants {
 	 */
 	public static final String USE_SUPERTYPE= "org.eclipse.jdt.ui.actions.UseSupertype"; //$NON-NLS-1$
 
+	/**
+	 * Refactor menu: name of standard Infer Generic Type Arguments global action
+	 * (value <code>"org.eclipse.jdt.ui.actions.InferTypeArguments"</code>).
+	 * 
+	 * @since 3.1
+	 */
+	public static final String INFER_TYPE_ARGUMENTS= "org.eclipse.jdt.ui.actions.InferTypeArguments"; //$NON-NLS-1$
+
 	/**
 	 * Refactor menu: name of standard global action to convert a local
 	 * variable to a field (value <code>"org.eclipse.jdt.ui.actions.ConvertLocalToField"</code>).
@@ -327,6 +441,12 @@ public class JdtActionConstants {
 	 */
 	public static final String FIND_REFERENCES_IN_WORKSPACE= "org.eclipse.jdt.ui.actions.ReferencesInWorkspace"; //$NON-NLS-1$
 
+	/**
+	 * Search menu: name of standard Find References in Project global action
+	 * (value <code>"org.eclipse.jdt.ui.actions.ReferencesInProject"</code>).
+	 */
+	public static final String FIND_REFERENCES_IN_PROJECT= "org.eclipse.jdt.ui.actions.ReferencesInProject"; //$NON-NLS-1$
+
 	/**
 	 * Search menu: name of standard Find References in Hierarchy global action
 	 * (value <code>"org.eclipse.jdt.ui.actions.ReferencesInHierarchy"</code>).
@@ -347,6 +467,12 @@ public class JdtActionConstants {
 	 */
 	public static final String FIND_DECLARATIONS_IN_WORKSPACE= "org.eclipse.jdt.ui.actions.DeclarationsInWorkspace"; //$NON-NLS-1$
 
+	/**
+	 * Search menu: name of standard Find Declarations in Project global action
+	 * (value <code>"org.eclipse.jdt.ui.actions.DeclarationsInProject"</code>).
+	 */
+	public static final String FIND_DECLARATIONS_IN_PROJECT= "org.eclipse.jdt.ui.actions.DeclarationsInProject"; //$NON-NLS-1$
+
 	/**
 	 * Search menu: name of standard Find Declarations in Hierarchy global action
 	 * (value <code>"org.eclipse.jdt.ui.actions.DeclarationsInHierarchy"</code>).
@@ -365,6 +491,12 @@ public class JdtActionConstants {
 	 */
 	public static final String FIND_IMPLEMENTORS_IN_WORKSPACE= "org.eclipse.jdt.ui.actions.ImplementorsInWorkspace"; //$NON-NLS-1$
 
+	/**
+	 * Search menu: name of standard Find Implementors in Project global action
+	 * (value <code>"org.eclipse.jdt.ui.actions.ImplementorsInProject"</code>).
+	 */
+	public static final String FIND_IMPLEMENTORS_IN_PROJECT= "org.eclipse.jdt.ui.actions.ImplementorsInProject"; //$NON-NLS-1$
+
 	/**
 	 * Search menu: name of standard Find Implementors in Working Set global action
 	 * (value <code>"org.eclipse.jdt.ui.actions.ImplementorsInWorkingSet"</code>).
@@ -377,6 +509,12 @@ public class JdtActionConstants {
 	 */
 	public static final String FIND_READ_ACCESS_IN_WORKSPACE= "org.eclipse.jdt.ui.actions.ReadAccessInWorkspace"; //$NON-NLS-1$
 
+	/**
+	 * Search menu: name of standard Find Read Access in Project global action
+	 * (value <code>"org.eclipse.jdt.ui.actions.ReadAccessInProject"</code>).
+	 */
+	public static final String FIND_READ_ACCESS_IN_PROJECT= "org.eclipse.jdt.ui.actions.ReadAccessInProject"; //$NON-NLS-1$
+
 	/**
 	 * Search menu: name of standard Find Read Access in Hierarchy global action
 	 * (value <code>"org.eclipse.jdt.ui.actions.ReadAccessInHierarchy"</code>).
@@ -395,6 +533,12 @@ public class JdtActionConstants {
 	 */
 	public static final String FIND_WRITE_ACCESS_IN_WORKSPACE= "org.eclipse.jdt.ui.actions.WriteAccessInWorkspace"; //$NON-NLS-1$
 
+	/**
+	 * Search menu: name of standard Find Write Access in Project global action
+	 * (value <code>"org.eclipse.jdt.ui.actions.WriteAccessInProject"</code>).
+	 */
+	public static final String FIND_WRITE_ACCESS_IN_PROJECT= "org.eclipse.jdt.ui.actions.WriteAccessInProject"; //$NON-NLS-1$
+
 	/**
 	 * Search menu: name of standard Find Read Access in Hierarchy global action
 	 * (value <code>"org.eclipse.jdt.ui.actions.WriteAccessInHierarchy"</code>).
@@ -414,4 +558,22 @@ public class JdtActionConstants {
 	 * @since 2.1
 	 */
 	public static final String FIND_OCCURRENCES_IN_FILE= "org.eclipse.jdt.ui.actions.OccurrencesInFile"; //$NON-NLS-1$
+	
+	/**
+	 * Search menu: name of standard Find exception occurrences global action (value
+	 * <code>"org.eclipse.jdt.ui.actions.ExceptionOccurrences"</code>).
+	 * 
+	 * @since 3.0
+	 */
+	public static final String FIND_EXCEPTION_OCCURRENCES= "org.eclipse.jdt.ui.actions.ExceptionOccurrences"; //$NON-NLS-1$
+	
+	/**
+	 * Search menu: name of standard Find implement occurrences global action (value
+	 * <code>"org.eclipse.jdt.ui.actions.ImplementOccurrences"</code>).
+	 * 
+	 * @since 3.1
+	 */
+	public static final String FIND_IMPLEMENT_OCCURRENCES= "org.eclipse.jdt.ui.actions.ImplementOccurrences"; //$NON-NLS-1$		
+
+
 }
