@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -434,6 +434,10 @@ public class TextPresentation {
 	 */
 	private void applyStyle(StyleRange template, StyleRange target, boolean merge) {
 		if (merge) {
+			if (template.font != null)
+				target.font= template.font;
+			if (template.metrics != null)
+				target.metrics= template.metrics;
 			if (template.foreground != null)
 				target.foreground= template.foreground;
 			if (template.background != null)
@@ -447,6 +451,8 @@ public class TextPresentation {
 			target.fontStyle= template.fontStyle;
 			target.strikeout= template.strikeout;
 			target.underline= template.underline;
+			target.font= template.font;
+			target.metrics= template.metrics;
 		}
 	}
 
