@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2004, 2005 IBM Corporation and others.
+ * Copyright (c) 2004, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,10 +12,14 @@
 package org.eclipse.jface.bindings.keys;
 
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Collections;
+import java.util.Iterator;
 import java.util.List;
 import java.util.TreeSet;
 
+import org.eclipse.jface.util.IPropertyChangeListener;
+import org.eclipse.jface.util.PropertyChangeEvent;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.DisposeEvent;
 import org.eclipse.swt.events.DisposeListener;
@@ -361,7 +365,7 @@ public final class KeySequenceText {
 					break;
 				}
 
-			// fall through -- either no modifiers, or just shift.
+				// fall through -- either no modifiers, or just shift.
 
 			case SWT.TRAVERSE_ARROW_NEXT:
 			case SWT.TRAVERSE_ARROW_PREVIOUS:
@@ -467,6 +471,14 @@ public final class KeySequenceText {
 	 */
 	public static final int INFINITE = -1;
 
+	/**
+	 * The name of the property representing the current key sequence in this
+	 * key sequence widget.
+	 * 
+	 * @since 3.2
+	 */
+	public static final String P_KEY_SEQUENCE = "org.eclipse.jface.bindings.keys.KeySequenceText.KeySequence"; //$NON-NLS-1$
+
 	/**
 	 * The keys trapped by this widget. This list is guaranteed to be roughly
 	 * accurate. Perfection is not possible, as SWT does not export traversal
@@ -484,6 +496,12 @@ public final class KeySequenceText {
 	 */
 	private KeySequence keySequence = KeySequence.getInstance();
 
+	/**
+	 * Those listening to changes to the key sequence in this widget. This value
+	 * may be <code>null</code> if there are no listeners.
+	 */
+	private Collection listeners = null;
+
 	/** The maximum number of key strokes permitted in the sequence. */
 	private int maxStrokes = INFINITE;
 
@@ -532,12 +550,36 @@ public final class KeySequenceText {
 		text.addModifyListener(updateSequenceListener);
 	}
 
+	/**
+	 * Adds a property change listener to this key sequence widget. It will be
+	 * notified when the key sequence changes.
+	 * 
+	 * @param listener
+	 *            The listener to be notified when changes occur; must not be
+	 *            <code>null</code>.
+	 * @since 3.2
+	 */
+	public final void addPropertyChangeListener(
+			final IPropertyChangeListener listener) {
+		if (listener == null) {
+			return;
+		}
+
+		if (listeners == null) {
+			listeners = new ArrayList(1);
+		}
+
+		listeners.add(listener);
+	}
+
 	/**
 	 * Clears the text field and resets all the internal values.
 	 */
 	public void clear() {
+		final KeySequence oldKeySequence = keySequence;
 		keySequence = KeySequence.getInstance();
 		text.setText(EMPTY_STRING);
+		firePropertyChangeEvent(oldKeySequence);
 	}
 
 	/**
@@ -644,6 +686,27 @@ public final class KeySequenceText {
 		return startStrokeIndex;
 	}
 
+	/**
+	 * Fires a property change event to all of the listeners.
+	 * 
+	 * @param oldKeySequence
+	 *            The old key sequence; must not be <code>null</code>.
+	 * @since 3.2
+	 */
+	protected final void firePropertyChangeEvent(
+			final KeySequence oldKeySequence) {
+		if (listeners != null) {
+			final Iterator listenerItr = listeners.iterator();
+			final PropertyChangeEvent event = new PropertyChangeEvent(this,
+					P_KEY_SEQUENCE, oldKeySequence, getKeySequence());
+			while (listenerItr.hasNext()) {
+				final IPropertyChangeListener listener = (IPropertyChangeListener) listenerItr
+						.next();
+				listener.propertyChange(event);
+			}
+		}
+	}
+
 	/**
 	 * An accessor for the <code>KeySequence</code> that corresponds to the
 	 * current state of the text field. This includes incomplete strokes.
@@ -709,14 +772,12 @@ public final class KeySequenceText {
 		}
 
 		KeyStroke[][] deletedKeyStrokes = new KeyStroke[1][];
-		int index = deleteSelection(newKeyStrokes, false,
-				deletedKeyStrokes);
+		int index = deleteSelection(newKeyStrokes, false, deletedKeyStrokes);
 		if (index == -1) {
 			index = 0;
 		}
-		final KeyStroke[] strokes = (deletedKeyStrokes[0] == null) ? new KeyStroke[0]
-				: deletedKeyStrokes[0];
-		final KeyStroke[] keyStrokes = insertStrokeAt(strokes, stroke, index);
+
+		final KeyStroke[] keyStrokes = insertStrokeAt(newKeyStrokes, stroke, index);
 		keyFilter.clearInsertionIndex();
 		setKeySequence(KeySequence.getInstance(keyStrokes));
 	}
@@ -755,7 +816,7 @@ public final class KeySequenceText {
 		newKeyStrokes[index] = stroke;
 		if (index < keyStrokesLength) {
 			System.arraycopy(keyStrokes, index, newKeyStrokes, index + 1,
-					keyStrokesLength);
+					keyStrokesLength-index);
 		}
 		return newKeyStrokes;
 	}
@@ -771,6 +832,22 @@ public final class KeySequenceText {
 		return (text.getSelection().y >= getText().length());
 	}
 
+	/**
+	 * Removes the given listener from this key sequence widget.
+	 * 
+	 * @param listener
+	 *            The listener to be removed; must not be <code>null</code>.
+	 * @since 3.2
+	 */
+	public final void removePropertyChangeListener(
+			final IPropertyChangeListener listener) {
+		if ((listener == null) || (listeners == null)) {
+			return;
+		}
+
+		listeners.remove(listener);
+	}
+
 	/**
 	 * <p>
 	 * A mutator for the key sequence stored within this widget. The text and
@@ -787,6 +864,7 @@ public final class KeySequenceText {
 	 *            if none.
 	 */
 	public void setKeySequence(KeySequence newKeySequence) {
+		final KeySequence oldKeySequence = keySequence;
 		keySequence = newKeySequence;
 
 		// Trim any extra strokes.
@@ -811,6 +889,8 @@ public final class KeySequenceText {
 			text.addModifyListener(updateSequenceListener);
 			text.setSelection(getText().length());
 		}
+
+		firePropertyChangeEvent(oldKeySequence);
 	}
 
 	/**
@@ -833,10 +913,11 @@ public final class KeySequenceText {
 	 *            <code>INFINITE</code>.
 	 */
 	public void setKeyStrokeLimit(int keyStrokeLimit) {
-		if (keyStrokeLimit > 0 || keyStrokeLimit == INFINITE)
+		if (keyStrokeLimit > 0 || keyStrokeLimit == INFINITE) {
 			this.maxStrokes = keyStrokeLimit;
-		else
+		} else {
 			throw new IllegalArgumentException();
+		}
 
 		// Make sure we are obeying the new limit.
 		setKeySequence(getKeySequence());
