@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -69,14 +69,13 @@ public class RefactoringWizardDialog2 extends Dialog implements IWizardContainer
 	private int fPreviewWidth;
 	private int fPreviewHeight;
 	private IDialogSettings fSettings;
+	private boolean fHasAdditionalPages;
+	private Rectangle fInitialSize;
+	
 	private static final String DIALOG_SETTINGS= "RefactoringWizard.preview"; //$NON-NLS-1$
 	private static final String WIDTH= "width"; //$NON-NLS-1$
 	private static final String HEIGHT= "height"; //$NON-NLS-1$
 	
-	private static final Image INFO= RefactoringPluginImages.get(RefactoringPluginImages.IMG_OBJS_REFACTORING_INFO);
-	private static final Image WARNING= RefactoringPluginImages.get(RefactoringPluginImages.IMG_OBJS_REFACTORING_WARNING);
-	private static final Image ERROR= RefactoringPluginImages.get(RefactoringPluginImages.IMG_OBJS_REFACTORING_ERROR);
-	
 	private static class MessageBox extends Composite {
 		private Label fImage;
 		private Label fText;
@@ -86,7 +85,7 @@ public class RefactoringWizardDialog2 extends Dialog implements IWizardContainer
 			layout.numColumns= 2;
 			setLayout(layout);
 			fImage= new Label(this, SWT.NONE);
-			fImage.setImage(INFO);
+			fImage.setImage(RefactoringPluginImages.get(RefactoringPluginImages.IMG_OBJS_REFACTORING_INFO));
 			Point size= fImage.computeSize(SWT.DEFAULT, SWT.DEFAULT);
 			GridData gd= new GridData();
 			gd.verticalAlignment= SWT.TOP;			
@@ -114,20 +113,20 @@ public class RefactoringWizardDialog2 extends Dialog implements IWizardContainer
 			Image image= null;
 			switch (type) {
 				case IMessageProvider.INFORMATION:
-					image= INFO;
+					image= RefactoringPluginImages.get(RefactoringPluginImages.IMG_OBJS_REFACTORING_INFO);
 					break;
 				case IMessageProvider.WARNING:
-					image= WARNING;
+					image= RefactoringPluginImages.get(RefactoringPluginImages.IMG_OBJS_REFACTORING_WARNING);
 					break;
 				case IMessageProvider.ERROR:
-					image= ERROR;
+					image= RefactoringPluginImages.get(RefactoringPluginImages.IMG_OBJS_REFACTORING_ERROR);
 					break;
 			}
 			if (msg == null)
 				msg= ""; //$NON-NLS-1$
 			fText.setText(escapeAmpersands(msg));
 			if (image == null && msg.length() > 0)
-				image= INFO;
+				image= RefactoringPluginImages.get(RefactoringPluginImages.IMG_OBJS_REFACTORING_INFO);
 			fImage.setImage(image);
 		}
 		private String escapeAmpersands(String message) {
@@ -169,6 +168,7 @@ public class RefactoringWizardDialog2 extends Dialog implements IWizardContainer
 		fWizard.setContainer(this);
 		fWizard.addPages();
 		initSize();
+		fHasAdditionalPages= wizard.getPageCount() > 3;
 	}
 	
 	private void initSize() {
@@ -233,6 +233,17 @@ public class RefactoringWizardDialog2 extends Dialog implements IWizardContainer
 			if (previewButton.isEnabled())
 				defaultButton= previewButton;
 		}
+		Button nextButton= getButton(IDialogConstants.NEXT_ID);
+		if (nextButton != null && !nextButton.isDisposed()) {
+			nextButton.setEnabled(!previewPage);
+			if (!previewPage)
+				nextButton.setEnabled(canFlip);
+			if (nextButton.isEnabled())
+				defaultButton= nextButton;
+		}
+		Button backButton= getButton(IDialogConstants.BACK_ID);
+		if (backButton != null && !backButton.isDisposed())
+			backButton.setEnabled(!isFirstPage());
 		Button okButton= getButton(IDialogConstants.OK_ID);
 		if (okButton != null && !okButton.isDisposed()) {
 			okButton.setEnabled(ok);
@@ -340,6 +351,8 @@ public class RefactoringWizardDialog2 extends Dialog implements IWizardContainer
 		Map savedState= new HashMap(10);
 		saveEnableStateAndSet(getButton(PREVIEW_ID), savedState, "preview", false); //$NON-NLS-1$
 		saveEnableStateAndSet(getButton(IDialogConstants.OK_ID), savedState, "ok", false); //$NON-NLS-1$
+		saveEnableStateAndSet(getButton(IDialogConstants.BACK_ID), savedState, "back", false); //$NON-NLS-1$
+		saveEnableStateAndSet(getButton(IDialogConstants.NEXT_ID), savedState, "next", false); //$NON-NLS-1$
 		saveEnableStateAndSet(getButton(IDialogConstants.CANCEL_ID), savedState, "cancel", keepCancelEnabled); //$NON-NLS-1$
 		savedState.put("page", ControlEnableState.disable(fVisiblePage.getControl())); //$NON-NLS-1$
 		return savedState;
@@ -347,7 +360,7 @@ public class RefactoringWizardDialog2 extends Dialog implements IWizardContainer
 	
 	private void saveEnableStateAndSet(Control w, Map h, String key, boolean enabled) {
 		if (w != null) {
-			h.put(key, new Boolean(w.getEnabled()));
+			h.put(key, Boolean.valueOf(w.getEnabled()));
 			w.setEnabled(enabled);
 		}
 	}
@@ -385,6 +398,8 @@ public class RefactoringWizardDialog2 extends Dialog implements IWizardContainer
 	private void restoreUIState(Map state) {
 		restoreEnableState(getButton(PREVIEW_ID), state, "preview");//$NON-NLS-1$
 		restoreEnableState(getButton(IDialogConstants.OK_ID), state, "ok");//$NON-NLS-1$
+		restoreEnableState(getButton(IDialogConstants.BACK_ID), state, "back"); //$NON-NLS-1$
+		restoreEnableState(getButton(IDialogConstants.NEXT_ID), state, "next"); //$NON-NLS-1$
 		restoreEnableState(getButton(IDialogConstants.CANCEL_ID), state, "cancel");//$NON-NLS-1$
 		ControlEnableState pageState = (ControlEnableState) state.get("page");//$NON-NLS-1$
 		pageState.restore();
@@ -414,6 +429,7 @@ public class RefactoringWizardDialog2 extends Dialog implements IWizardContainer
 
 	protected void okPressed() {
 		IWizardPage current= fCurrentPage;
+		saveInitialSize();
 		if (fWizard.performFinish()) {
 			saveSize();
 			super.okPressed();
@@ -421,14 +437,25 @@ public class RefactoringWizardDialog2 extends Dialog implements IWizardContainer
 		}
 		if (fCurrentPage == current)
 			return;
-		Assert.isTrue(ErrorWizardPage.PAGE_NAME.equals(fCurrentPage.getName()));
-		if (showErrorDialog((ErrorWizardPage)fCurrentPage)) {
+		Assert.isTrue(IErrorWizardPage.PAGE_NAME.equals(fCurrentPage.getName()));
+		if (fHasAdditionalPages) {
+			// Show error page as a normal page
+			showCurrentPage();
+		} else if (showErrorDialog((ErrorWizardPage) fCurrentPage)) {
+			// Show error page as a dialog
 			if (fWizard.performFinish()) {
 				super.okPressed();
 				return;
 			}
 		}
-		fCurrentPage= current;
+	}
+
+	private void showCurrentPage() {
+		if (fCurrentPage.getControl() == null)
+			fCurrentPage.createControl(fPageContainer);
+		resize();
+		makeVisible(fCurrentPage);
+		updateButtons();
 	}
 	
 	protected void handleShellCloseEvent() {
@@ -442,33 +469,45 @@ public class RefactoringWizardDialog2 extends Dialog implements IWizardContainer
 		return IPreviewWizardPage.PAGE_NAME.equals(fCurrentPage.getName());
 	}
 	
-	private void previewPressed() {
+	private void nextOrPreviewPressed() {
 		IWizardPage current= fCurrentPage;
+		saveInitialSize();
 		fCurrentPage= fCurrentPage.getNextPage();
 		if (current == fCurrentPage)
 			return;
-		String pageName= fCurrentPage.getName();
-		if (ErrorWizardPage.PAGE_NAME.equals(pageName)) {
+		if (!fHasAdditionalPages && IErrorWizardPage.PAGE_NAME.equals(fCurrentPage.getName())) {
 			if (showErrorDialog((ErrorWizardPage)fCurrentPage)) {
 				fCurrentPage= fCurrentPage.getNextPage();
-				pageName= fCurrentPage.getName();
 			} else {
 				return;
 			}
 		}
-		if (IPreviewWizardPage.PAGE_NAME.equals(pageName)) {
-			fCurrentPage.createControl(fPageContainer);
-			makeVisible(fCurrentPage);
-			updateButtons();
-			if (((PreviewWizardPage)fCurrentPage).hasChanges())
-				resize();
-			else
-				getButton(IDialogConstants.OK_ID).setEnabled(false);
-		} else {
-			fCurrentPage= current;
+		
+		showCurrentPage();
+	}
+
+	private void saveInitialSize() {
+		if (isFirstPage()) {
+			// Moving away from initial page;
+			// store size (may have changed any time)
+			fInitialSize= getShell().getBounds();
 		}
 	}
 	
+	private boolean isFirstPage() {
+		IWizardPage[] pages= fWizard.getPages();
+		return (fCurrentPage.equals(pages[0]));
+	}
+
+	private void backPressed() {
+		IWizardPage current= fCurrentPage;
+		fCurrentPage= fCurrentPage.getPreviousPage();
+		if (current == fCurrentPage)
+			return;
+
+		showCurrentPage();
+	}
+	
 	private boolean showErrorDialog(ErrorWizardPage page) {
 		RefactoringStatusDialog dialog= new RefactoringStatusDialog(getShell(), page, 
 			fWizard.internalShowBackButtonOnStatusDialog(InternalAPI.INSTANCE));
@@ -485,6 +524,12 @@ public class RefactoringWizardDialog2 extends Dialog implements IWizardContainer
 	}
 	
 	private void resize() {
+		
+		if (isFirstPage()) {
+			getShell().setBounds(fInitialSize);
+			return;
+		}
+		
 		Control control= fPageContainer.getTopPage();
 		Point size= control.getSize();
 		int dw= Math.max(0, fPreviewWidth - size.x);
@@ -493,22 +538,25 @@ public class RefactoringWizardDialog2 extends Dialog implements IWizardContainer
 		int dy= dh / 2;
 		Shell shell= getShell();
 		Rectangle rect= shell.getBounds();
-		Rectangle displayRect= shell.getDisplay().getClientArea();
-		Rectangle monitorRect= shell.getMonitor().getClientArea();
-		rect.x= Math.max(0, rect.x - dx);
-		rect.y= Math.max(0, rect.y - dy);
-		// limit with and hight to monitor
-		rect.width= Math.min(rect.width + dw, monitorRect.width);
-		rect.height= Math.min(rect.height + dh, monitorRect.height);
-		int xe= rect.x + rect.width;
-		// position the dialog on the display
-		if (xe > displayRect.width) {
-			rect.x-= xe - displayRect.width; 
+		Rectangle clientRect= shell.getMonitor().getClientArea();
+		rect.x= Math.max(clientRect.x, rect.x - dx);
+		rect.y= Math.max(clientRect.y, rect.y - dy);
+		
+		// limit with and height to monitor
+		rect.width= Math.min(rect.width + dw, clientRect.width);
+		rect.height= Math.min(rect.height + dh, clientRect.height);
+
+		// Reposition the dialog so that it will be fully visible.
+		// Normalize x and y relative to the client area.
+		int xe= (rect.x - clientRect.x) + rect.width;
+		if (xe > clientRect.width) {
+			rect.x-= xe - clientRect.width; 
 		}
-		int ye= rect.y + rect.height;
-		if (ye > displayRect.height) {
-			rect.y-= ye - displayRect.height; 
+		int ye= (rect.y - clientRect.y) + rect.height;
+		if (ye > clientRect.height) {
+			rect.y-= ye - clientRect.height; 
 		}
+		
 		shell.setBounds(rect);
 	}
 	
@@ -574,19 +622,13 @@ public class RefactoringWizardDialog2 extends Dialog implements IWizardContainer
 	}
 	
 	protected void createButtonsForButtonBar(Composite parent) {
-		if (! (fCurrentPage instanceof PreviewWizardPage) && fWizard.internalHasPreviewPage(InternalAPI.INSTANCE)) {
-			Button preview= createButton(parent, PREVIEW_ID, RefactoringUIMessages.RefactoringWizardDialog2_buttons_preview_label, false); 
-			if (fMakeNextButtonDefault) {
-				preview.getShell().setDefaultButton(preview);
-			}
-			preview.addSelectionListener(new SelectionAdapter() {
-				public void widgetSelected(SelectionEvent e) {
-					previewPressed();
-				}
-			});
-		}
 		
-		String OK_LABEL= IDialogConstants.OK_LABEL;
+		if (fHasAdditionalPages)
+			createPreviousAndNextButtons(parent);
+		else
+			createPreviewButton(parent);
+
+		String OK_LABEL= (fHasAdditionalPages) ? IDialogConstants.FINISH_LABEL : IDialogConstants.OK_LABEL;
 		String CANCEL_LABEL= IDialogConstants.CANCEL_LABEL;
 		if (fWizard.internalIsYesNoStyle(InternalAPI.INSTANCE)) {
 			OK_LABEL= IDialogConstants.YES_LABEL;
@@ -606,6 +648,64 @@ public class RefactoringWizardDialog2 extends Dialog implements IWizardContainer
 		okButton.setFocus();
 	}
 	
+    private void createPreviewButton(Composite parent) {
+		if (! (fCurrentPage instanceof PreviewWizardPage) && fWizard.internalHasPreviewPage(InternalAPI.INSTANCE)) {
+			Button preview= createButton(parent, PREVIEW_ID, RefactoringUIMessages.RefactoringWizardDialog2_buttons_preview_label, false);
+			if (fMakeNextButtonDefault) {
+				preview.getShell().setDefaultButton(preview);
+			}
+			preview.addSelectionListener(new SelectionAdapter() {
+
+				public void widgetSelected(SelectionEvent e) {
+					nextOrPreviewPressed();
+				}
+			});
+		}
+	}
+
+	private Composite createPreviousAndNextButtons(Composite parent) {
+		// Copied from Wizard Dialog.
+		
+		// increment the number of columns in the button bar
+		((GridLayout) parent.getLayout()).numColumns+= 2; // parent is assumed to have a GridLayout (see javadoc of Dialog#createButton(..))
+		Composite composite= new Composite(parent, SWT.NONE);
+		// create a layout with spacing and margins appropriate for the font
+		// size.
+		GridLayout layout= new GridLayout();
+		layout.numColumns= 0; // will be incremented by createButton
+		layout.marginWidth= 0;
+		layout.marginHeight= 0;
+		layout.horizontalSpacing= 0;
+		layout.verticalSpacing= 0;
+		composite.setLayout(layout);
+		composite.setFont(parent.getFont());
+		Button backButton= createButton(composite, IDialogConstants.BACK_ID, IDialogConstants.BACK_LABEL, false);
+		backButton.addSelectionListener(new SelectionAdapter() {
+
+			public void widgetSelected(SelectionEvent e) {
+				backPressed();
+			}
+		});
+		Button nextButton= createButton(composite, IDialogConstants.NEXT_ID, IDialogConstants.NEXT_LABEL, false);
+		nextButton.addSelectionListener(new SelectionAdapter() {
+
+			public void widgetSelected(SelectionEvent e) {
+				nextOrPreviewPressed();
+			}
+		});
+
+		GridData data= new GridData();
+		int widthHint= convertHorizontalDLUsToPixels(IDialogConstants.BUTTON_WIDTH);
+		Point minSize1= backButton.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
+		Point minSize2= nextButton.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
+		data.widthHint= Math.max(widthHint * 2, minSize1.x + minSize2.x);
+		data.horizontalAlignment= SWT.END;
+		data.horizontalSpan= 2;
+		composite.setLayoutData(data);
+
+		return composite;
+	}
+	
 	private void makeVisible(IWizardPage page) {
 		if (fVisiblePage == page)
 			return;
