@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2004, 2005 IBM Corporation and others.
+ * Copyright (c) 2004, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,149 +11,117 @@
 package org.eclipse.osgi.internal.module;
 
 import java.util.*;
+import org.eclipse.osgi.internal.resolver.ExportPackageDescriptionImpl;
 import org.eclipse.osgi.service.resolver.*;
 import org.osgi.framework.Constants;
-import org.osgi.framework.Version;
 
-public class ResolverBundle implements VersionSupplier {
+/*
+ * A companion to BundleDescription from the state used while resolving.
+ */
+public class ResolverBundle extends VersionSupplier {
 	public static final int UNRESOLVED = 0;
 	public static final int RESOLVING = 1;
 	public static final int RESOLVED = 2;
 
-	private BundleDescription bundle;
 	private Long bundleID;
 	private BundleConstraint host;
 	private ResolverImport[] imports;
 	private ResolverExport[] exports;
 	private BundleConstraint[] requires;
+	private GenericCapability[] capabilities;
+	private GenericConstraint[] genericReqiures;
 	// Fragment support
 	private ArrayList fragments;
 	private HashMap fragmentExports;
 	private HashMap fragmentImports;
 	private HashMap fragmentRequires;
+	private HashMap fragmentGenericRequires;
 	// Flag specifying whether this bundle is resolvable
 	private boolean resolvable = true;
 	// Internal resolver state for this bundle
 	private int state = UNRESOLVED;
-	// Store for RESOLVING modules that this module is dependent on (cyclic dependencies)
-	private ArrayList cyclicDependencies = new ArrayList();
 
 	private ResolverImpl resolver;
 	private boolean newFragmentExports;
+	private ArrayList refs;
 
 	ResolverBundle(BundleDescription bundle, ResolverImpl resolver) {
-		this.bundle = bundle;
+		super(bundle);
 		this.bundleID = new Long(bundle.getBundleId());
 		this.resolver = resolver;
 		initialize(bundle.isResolved());
 	}
 
 	void initialize(boolean useSelectedExports) {
-		if (bundle.getHost() != null) {
-			host = new BundleConstraint(this, bundle.getHost());
+		if (getBundle().isSingleton())
+			refs = new ArrayList();
+		// always add generic capabilities
+		GenericDescription[] actualCapabilities = getBundle().getGenericCapabilities();
+		capabilities = new GenericCapability[actualCapabilities.length];
+		for (int i = 0; i < capabilities.length; i++)
+			capabilities[i] = new GenericCapability(this, actualCapabilities[i]);
+		if (getBundle().getHost() != null) {
+			host = new BundleConstraint(this, getBundle().getHost());
 			exports = new ResolverExport[0];
 			imports = new ResolverImport[0];
 			requires = new BundleConstraint[0];
+			genericReqiures = new GenericConstraint[0];
 			return;
 		}
 
-		ImportPackageSpecification[] actualImports = bundle.getImportPackages();
+		ImportPackageSpecification[] actualImports = getBundle().getImportPackages();
 		// Reorder imports so that optionals are at the end so that we wire statics before optionals
 		ArrayList importList = new ArrayList(actualImports.length);
-		for (int i = actualImports.length - 1; i >= 0; i--) {
-			if (ImportPackageSpecification.RESOLUTION_OPTIONAL.equals(actualImports[i].getDirective(Constants.RESOLUTION_DIRECTIVE))) {
+		for (int i = actualImports.length - 1; i >= 0; i--)
+			if (ImportPackageSpecification.RESOLUTION_OPTIONAL.equals(actualImports[i].getDirective(Constants.RESOLUTION_DIRECTIVE)))
 				importList.add(new ResolverImport(this, actualImports[i]));
-			} else {
+			else
 				importList.add(0, new ResolverImport(this, actualImports[i]));
-			}
-		}
 		imports = (ResolverImport[]) importList.toArray(new ResolverImport[importList.size()]);
 
-		ExportPackageDescription[] actualExports;
-		if (useSelectedExports) {
-			actualExports = bundle.getSelectedExports();
-		} else {
-			actualExports = bundle.getExportPackages();
-		}
+		ExportPackageDescription[] actualExports = useSelectedExports ? getBundle().getSelectedExports() : getBundle().getExportPackages();
 		exports = new ResolverExport[actualExports.length];
-		for (int i = 0; i < actualExports.length; i++) {
+		for (int i = 0; i < actualExports.length; i++)
 			exports[i] = new ResolverExport(this, actualExports[i]);
-		}
 
-		BundleSpecification[] actualRequires = bundle.getRequiredBundles();
+		BundleSpecification[] actualRequires = getBundle().getRequiredBundles();
 		requires = new BundleConstraint[actualRequires.length];
 		for (int i = 0; i < requires.length; i++)
 			requires[i] = new BundleConstraint(this, actualRequires[i]);
 
+		GenericSpecification[] actualGenericRequires = getBundle().getGenericRequires();
+		genericReqiures = new GenericConstraint[actualGenericRequires.length];
+		for (int i = 0; i < genericReqiures.length; i++)
+			genericReqiures[i] = new GenericConstraint(this, actualGenericRequires[i]);
+
 		fragments = null;
 		fragmentExports = null;
 		fragmentImports = null;
 		fragmentRequires = null;
-	}
-
-	boolean isExported(ResolverExport exp) {
-		ResolverExport[] exports = getExportPackages();
-		for (int i = 0; i < exports.length; i++) {
-			if (exp == exports[i]) {
-				return true;
-			}
-		}
-		return false;
-	}
-
-	ResolverImport getImport(ResolverExport exp) {
-		ResolverImport[] imports = getImportPackages();
-		for (int i = 0; i < imports.length; i++) {
-			if (exp.getName().equals(imports[i].getName())) {
-				return imports[i];
-			}
-		}
-		return null;
-	}
-
-	ResolverExport getExport(ResolverImport imp) {
-		ResolverExport[] exports = getExportPackages();
-		for (int i = 0; i < exports.length; i++) {
-			if (imp.getName().equals(exports[i].getName()) && exports[i].getExportPackageDescription().isRoot()) {
-				return exports[i];
-			}
-		}
-		return null;
+		fragmentGenericRequires = null;
 	}
 
 	ResolverExport getExport(String name) {
-		ResolverExport[] exports = getExportPackages();
-		for (int i = 0; i < exports.length; i++) {
-			if (name.equals(exports[i].getName())) {
-				return exports[i];
-			}
-		}
-		return null;
+		ResolverExport[] allExports = getExports(name);
+		return allExports.length == 0 ? null : allExports[0];
 	}
 
-	// Iterate thru the imports making sure they are wired
-	boolean isFullyWired() {
-		if (host != null && host.foundMatchingBundles())
-			return false;
-
-		ResolverImport[] imports = getImportPackages();
-		for (int i = 0; i < imports.length; i++) {
-			if (imports[i].getMatchingExport() == null && !imports[i].isOptional() && !imports[i].isDynamic()) {
-				return false;
-			}
-		}
-
-		BundleConstraint[] requires = getRequires();
-		for (int i = 0; i < requires.length; i++)
-			if (requires[i].getMatchingBundle() == null && !requires[i].isOptional())
-				return false;
-		return true;
+	ResolverExport[] getExports(String name) {
+		ArrayList results = new ArrayList(1); // rare to have more than one
+		// it is faster to ask the VersionHashMap for this package name and then compare the exporter to this
+		Object[] resolverExports = resolver.getResolverExports().get(name);
+		for (int i = 0; i < resolverExports.length; i++)
+			if (((ResolverExport)resolverExports[i]).getExporter() == this)
+				results.add(resolverExports[i]);
+		return (ResolverExport[]) results.toArray(new ResolverExport[results.size()]);
 	}
 
-	void clearWires() {
+	void clearWires(boolean clearUnresolvable) {
 		ResolverImport[] allImports = getImportPackages();
 		for (int i = 0; i < allImports.length; i++) {
 			allImports[i].setMatchingExport(null);
+			if (clearUnresolvable)
+				allImports[i].clearUnresolvableWirings();
 		}
 
 		if (host != null)
@@ -161,6 +129,10 @@ public class ResolverBundle implements VersionSupplier {
 		BundleConstraint[] allRequires = getRequires();
 		for (int i = 0; i < allRequires.length; i++)
 			allRequires[i].setMatchingBundle(null);
+
+		GenericConstraint[] allGenericRequires = getGenericRequires();
+		for (int i = 0; i < allGenericRequires.length; i++)
+			allGenericRequires[i].setMatchingCapability(null);
 	}
 
 	boolean isResolved() {
@@ -214,33 +186,20 @@ public class ResolverBundle implements VersionSupplier {
 	}
 
 	ResolverExport[] getSelectedExports() {
-		ResolverExport[] exports = getExportPackages();
-		ArrayList removedList = null;
-		for (int i = 0; i < exports.length; i++) {
-			ResolverImport imp = getImport(exports[i].getName());
-			if (imp != null && imp.getMatchingExport() != exports[i] && exports[i].getExportPackageDescription().isRoot()) {
-				if (removedList == null)
-					removedList = new ArrayList(1);
-				removedList.add(exports[i]);
-			}
-		}
-		if (removedList == null)
-			return exports;
-		ResolverExport[] selectedExports = new ResolverExport[exports.length - removedList.size()];
-		ResolverExport[] removedExports = (ResolverExport[]) removedList.toArray(new ResolverExport[removedList.size()]);
+		ResolverExport[] allExports = getExportPackages();
+		int removedExports = 0;
+		for (int i = 0; i < allExports.length; i++)
+			if (allExports[i].isDropped())
+				removedExports++;
+		if (removedExports == 0)
+			return allExports;
+		ResolverExport[] selectedExports = new ResolverExport[allExports.length - removedExports];
 		int index = 0;
-		for (int i = 0; i < exports.length; i++) {
-			boolean removed = false;
-			for (int j = 0; j < removedExports.length; j++) {
-				if (exports[i] == removedExports[j]) {
-					removed = true;
-					break;
-				}
-			}
-			if (!removed) {
-				selectedExports[index] = exports[i];
-				index++;
-			}
+		for (int i = 0; i < allExports.length; i++) {
+			if (allExports[i].isDropped())
+				continue;
+			selectedExports[index] = allExports[i];
+			index++;
 		}
 		return selectedExports;
 	}
@@ -249,6 +208,10 @@ public class ResolverBundle implements VersionSupplier {
 		return host;
 	}
 
+	GenericCapability[] getGenericCapabilities() {
+		return capabilities;
+	}
+
 	BundleConstraint[] getRequires() {
 		if (isFragment())
 			return new BundleConstraint[0];
@@ -266,77 +229,45 @@ public class ResolverBundle implements VersionSupplier {
 		return (BundleConstraint[]) resultList.toArray(new BundleConstraint[resultList.size()]);
 	}
 
+	GenericConstraint[] getGenericRequires() {
+		if (isFragment() || fragments == null || fragments.size() == 0)
+			return genericReqiures;
+		ArrayList resultList = new ArrayList(genericReqiures.length);
+		for (int i = 0; i < genericReqiures.length; i++)
+			resultList.add(genericReqiures[i]);
+		for (Iterator iter = fragments.iterator(); iter.hasNext();) {
+			ResolverBundle fragment = (ResolverBundle) iter.next();
+			ArrayList fragGenericRegs = (ArrayList) fragmentGenericRequires.get(fragment.bundleID);
+			if (fragGenericRegs != null)
+				resultList.addAll(fragGenericRegs);
+		}
+		return (GenericConstraint[]) resultList.toArray(new GenericConstraint[resultList.size()]);
+	}
+
 	BundleConstraint getRequire(String name) {
-		BundleConstraint[] requires = getRequires();
-		for (int i = 0; i < requires.length; i++)
-			if (requires[i].getVersionConstraint().getName().equals(name))
-				return requires[i];
+		BundleConstraint[] allRequires = getRequires();
+		for (int i = 0; i < allRequires.length; i++)
+			if (allRequires[i].getVersionConstraint().getName().equals(name))
+				return allRequires[i];
 		return null;
 	}
 
-	// Returns true if any cyclic dependencies have been recorded
-	boolean isDependentOnCycle() {
-		return cyclicDependencies.size() > 0;
-	}
-
-	// Record a cyclic dependency (i.e. this module is dependent on the supplied module)
-	void recordCyclicDependency(ResolverBundle dependentOn) {
-		if (!cyclicDependencies.contains(dependentOn)) {
-			cyclicDependencies.add(dependentOn);
-		}
-	}
-
-	ArrayList getCyclicDependencies() {
-		return cyclicDependencies;
-	}
-
-	// This method is called by the resolver to tell this modules that a RESOLVING module that it
-	// was dependent on has now RESOLVED.  If this is the last/only cyclic dependency then the
-	// resolver will move this module into RESOLVED state
-	boolean cyclicDependencyResolved(ResolverBundle dependentOn) {
-		for (int i = 0; i < cyclicDependencies.size(); i++) {
-			if (dependentOn == cyclicDependencies.get(i)) {
-				cyclicDependencies.remove(i);
-			}
-		}
-		return !isDependentOnCycle();
-	}
-
-	// This method is called by the resolver to tell this modules that a RESOLVING module that it
-	// was dependent on is unresolvable. The resolver will move this module into UNRESOLVED state
-	void cyclicDependencyFailed(ResolverBundle dependentOn) {
-		cyclicDependencies = new ArrayList();
-		detachAllFragments();
-		ResolverImport[] imports = getImportPackages();
-		for (int i = 0; i < imports.length; i++) {
-			imports[i].clearUnresolvableWirings();
-		}
-	}
-
 	public BundleDescription getBundle() {
-		return bundle;
+		return (BundleDescription) getBaseDescription();
 	}
 
 	ResolverImport getImport(String name) {
-		ResolverImport[] imports = getImportPackages();
-		for (int i = 0; i < imports.length; i++) {
-			if (imports[i].getName().equals(name)) {
-				return imports[i];
+		ResolverImport[] allImports = getImportPackages();
+		for (int i = 0; i < allImports.length; i++) {
+			if (allImports[i].getName().equals(name)) {
+				return allImports[i];
 			}
 		}
 		return null;
 	}
 
 	public String toString() {
-		return "[" + bundle + "]"; //$NON-NLS-1$ //$NON-NLS-2$
-	}
-
-	public Version getVersion() {
-		return bundle.getVersion();
-	}
-
-	public String getName() {
-		return bundle.getName();
+		return "[" + getBundle() + "]"; //$NON-NLS-1$ //$NON-NLS-2$
 	}
 
 	private void initFragments() {
@@ -348,39 +279,35 @@ public class ResolverBundle implements VersionSupplier {
 			fragmentImports = new HashMap(1);
 		if (fragmentRequires == null)
 			fragmentRequires = new HashMap(1);
+		if (fragmentGenericRequires == null)
+			fragmentGenericRequires = new HashMap(1);
 	}
 
 	private boolean isImported(String packageName) {
-		ResolverImport[] imports = getImportPackages();
-		for (int i = 0; i < imports.length; i++)
-			if (packageName.equals(imports[i].getName()))
+		ResolverImport[] allImports = getImportPackages();
+		for (int i = 0; i < allImports.length; i++)
+			if (packageName.equals(allImports[i].getName()))
 				return true;
 
 		return false;
 	}
 
 	private boolean isExported(String packageName) {
-		ResolverExport[] exports = getExportPackages();
-		for (int i = 0; i < exports.length; i++)
-			if (packageName.equals(exports[i].getName()))
-				return true;
-
-		return false;
+		ResolverExport export = getExport(packageName);
+		if (export == null)
+			return false;
+		// let exports from a bundle manifest be exported in addition to the ones from the vm profile
+		return 0 > ((Integer) export.getExportPackageDescription().getDirective(ExportPackageDescriptionImpl.EQUINOX_EE)).intValue();
 	}
 
 	private boolean isRequired(String bundleName) {
-		BundleConstraint[] requires = getRequires();
-		for (int i = 0; i < requires.length; i++)
-			if (bundleName.equals(requires[i].getVersionConstraint().getName()))
-				return true;
-
-		return false;
+		return getRequire(bundleName) != null;
 	}
 
-	ResolverExport[] attachFragment(ResolverBundle fragment, boolean addExports) {
+	ResolverExport[] attachFragment(ResolverBundle fragment, boolean dynamicAttach) {
 		if (isFragment())
 			return new ResolverExport[0]; // cannot attach to fragments;
-		if (!bundle.attachFragments() || (isResolved() && !bundle.dynamicFragments()))
+		if (!getBundle().attachFragments() || (isResolved() && !getBundle().dynamicFragments()))
 			return new ResolverExport[0]; // host is restricting attachment
 		if (fragment.getHost().getMatchingBundles() != null && !((HostSpecification) fragment.getHost().getVersionConstraint()).isMultiHost())
 			return new ResolverExport[0]; // fragment is restricting attachment
@@ -388,8 +315,10 @@ public class ResolverBundle implements VersionSupplier {
 		ImportPackageSpecification[] newImports = fragment.getBundle().getImportPackages();
 		BundleSpecification[] newRequires = fragment.getBundle().getRequiredBundles();
 		ExportPackageDescription[] newExports = fragment.getBundle().getExportPackages();
+		GenericSpecification[] newGenericRequires = fragment.getBundle().getGenericRequires();
 
-		if (constraintsConflict(newImports, newRequires))
+		// if this is not during initialization then check if constraints conflict
+		if (dynamicAttach && constraintsConflict(fragment.getBundle(), newImports, newRequires, newGenericRequires))
 			return new ResolverExport[0]; // do not allow fragments with conflicting constraints
 		if (isResolved() && newExports.length > 0)
 			fragment.setNewFragmentExports(true);
@@ -416,12 +345,19 @@ public class ResolverBundle implements VersionSupplier {
 			fragmentRequires.put(fragment.bundleID, hostRequires);
 		}
 
+		if (newGenericRequires.length > 0) {
+			ArrayList hostGenericRequires = new ArrayList(newGenericRequires.length);
+			for (int i = 0; i < newGenericRequires.length; i++)
+				hostGenericRequires.add(new GenericConstraint(this, newGenericRequires[i]));
+			fragmentGenericRequires.put(fragment.bundleID, hostGenericRequires);
+		}
+
 		ArrayList hostExports = new ArrayList(newExports.length);
-		if (newExports.length > 0 && addExports) {
+		if (newExports.length > 0 && dynamicAttach) {
 			StateObjectFactory factory = resolver.getState().getFactory();
 			for (int i = 0; i < newExports.length; i++) {
 				if (!isExported(newExports[i].getName())) {
-					ExportPackageDescription hostExport = factory.createExportPackageDescription(newExports[i].getName(), newExports[i].getVersion(), newExports[i].getDirectives(), newExports[i].getAttributes(), newExports[i].isRoot(), bundle);
+					ExportPackageDescription hostExport = factory.createExportPackageDescription(newExports[i].getName(), newExports[i].getVersion(), newExports[i].getDirectives(), newExports[i].getAttributes(), newExports[i].isRoot(), getBundle());
 					hostExports.add(new ResolverExport(this, hostExport));
 				}
 			}
@@ -430,22 +366,31 @@ public class ResolverBundle implements VersionSupplier {
 		return (ResolverExport[]) hostExports.toArray(new ResolverExport[hostExports.size()]);
 	}
 
-	private boolean constraintsConflict(ImportPackageSpecification[] newImports, BundleSpecification[] newRequires) {
+	private boolean constraintsConflict(BundleDescription fragment, ImportPackageSpecification[] newImports, BundleSpecification[] newRequires, GenericSpecification[] newGenericRequires) {
 		for (int i = 0; i < newImports.length; i++) {
 			ResolverImport importPkg = getImport(newImports[i].getName());
-			if (importPkg == null && isResolved())
+			if ((importPkg == null && isResolved()) || (importPkg != null && !isIncluded(newImports[i].getVersionRange(), importPkg.getVersionConstraint().getVersionRange()))) {
+				resolver.getState().addResolverError(fragment, ResolverError.FRAGMENT_CONFLICT, newImports[i].toString(), newImports[i]);
 				return true; // do not allow additional constraints when host is already resolved
-			if (importPkg != null && !newImports[i].getVersionRange().equals(importPkg.getImportPackageSpecification().getVersionRange()))
-				return true; // do not allow conflicting constraints from fragment 
+			}
 		}
 		for (int i = 0; i < newRequires.length; i++) {
 			BundleConstraint constraint = getRequire(newRequires[i].getName());
-			if (constraint == null && isResolved())
+			if ((constraint == null && isResolved()) || (constraint != null && !isIncluded(newRequires[i].getVersionRange(), constraint.getVersionConstraint().getVersionRange()))) {
+				resolver.getState().addResolverError(fragment, ResolverError.FRAGMENT_CONFLICT, newRequires[i].toString(), newRequires[i]);
 				return true; // do not allow additional constraints when host is already resolved
-			if (constraint != null && !newRequires[i].getVersionRange().equals(constraint.getVersionConstraint().getVersionRange()))
-				return true; // do not allow conflictin constraints from fragment
+			}
 		}
-		return false;
+		return !isResolved() ? false : newGenericRequires != null && newGenericRequires.length > 0;
+	}
+
+	// checks that the inner VersionRange is included in the outer VersionRange
+	private static boolean isIncluded(VersionRange outer, VersionRange inner) {
+		if (!outer.isIncluded(inner.getMinimum()) && (!inner.getMinimum().equals(outer.getMinimum()) || inner.getIncludeMinimum() != outer.getIncludeMinimum()))
+			return false;
+		if (!outer.isIncluded(inner.getMaximum()) && (!inner.getMaximum().equals(outer.getMaximum()) || inner.getIncludeMaximum() != outer.getIncludeMaximum()))
+			return false;
+		return true;
 	}
 
 	private void setNewFragmentExports(boolean newFragmentExports) {
@@ -456,7 +401,7 @@ public class ResolverBundle implements VersionSupplier {
 		return newFragmentExports;
 	}
 
-	ResolverExport[] detachFragment(ResolverBundle fragment, Object reason) {
+	ResolverExport[] detachFragment(ResolverBundle fragment, ResolverConstraint reason) {
 		if (isFragment())
 			return new ResolverExport[0];
 		initFragments();
@@ -468,21 +413,18 @@ public class ResolverBundle implements VersionSupplier {
 		ArrayList fragImports = (ArrayList) fragmentImports.remove(fragment.bundleID);
 		ArrayList fragRequires = (ArrayList) fragmentRequires.remove(fragment.bundleID);
 		ArrayList removedExports = (ArrayList) fragmentExports.remove(fragment.bundleID);
+		fragmentGenericRequires.remove(fragment.bundleID);
 		if (reason != null) {
 			ResolverBundle[] remainingFrags = (ResolverBundle[]) fragments.toArray(new ResolverBundle[fragments.size()]);
 			for (int i = 0; i < remainingFrags.length; i++) {
 				resolver.getResolverExports().remove(detachFragment(remainingFrags[i], null));
 				VersionConstraint[] constraints;
-				String reasonName;
-				if (reason instanceof ResolverImport) {
+				if (reason instanceof ResolverImport)
 					constraints = remainingFrags[i].getBundle().getImportPackages();
-					reasonName = ((ResolverImport)reason).getName();
-				} else {
-					reasonName = ((BundleConstraint)reason).getVersionConstraint().getName();
+				else
 					constraints = remainingFrags[i].getBundle().getRequiredBundles();
-				}
 				for (int j = 0; j < constraints.length; j++)
-					if (reasonName.equals(constraints[j].getName()))
+					if (reason.getName().equals(constraints[j].getName()))
 						continue; // this fragment should remained unattached.
 				resolver.getResolverExports().put(attachFragment(remainingFrags[i], true));
 				ArrayList newImports = (ArrayList) fragmentImports.get(remainingFrags[i].bundleID);
@@ -501,7 +443,7 @@ public class ResolverBundle implements VersionSupplier {
 						BundleConstraint newRequire = (BundleConstraint) iNewRequires.next();
 						for (Iterator iOldRequires = fragRequires.iterator(); iOldRequires.hasNext();) {
 							BundleConstraint oldRequire = (BundleConstraint) iOldRequires.next();
-							if (newRequire.getVersionConstraint().getName().equals(oldRequire.getVersionConstraint().getName()))
+							if (newRequire.getName().equals(oldRequire.getName()))
 								newRequire.setMatchingBundle(oldRequire.getMatchingBundle());
 						}
 					}
@@ -518,19 +460,6 @@ public class ResolverBundle implements VersionSupplier {
 			detachFragment(allFragments[i], null);
 	}
 
-	boolean isDependentOnUnresolvedFragment(ResolverBundle dependent) {
-		ResolverImport[] imports = dependent.getImportPackages();
-		for (int i = 0; i < imports.length; i++) {
-			ResolverExport exp = imports[i].getMatchingExport();
-			if (exp == null || exp.getExporter() != this)
-				continue;
-
-			if (!isExported(exp))
-				return true;
-		}
-		return false;
-	}
-
 	boolean isResolvable() {
 		return resolvable;
 	}
@@ -541,9 +470,8 @@ public class ResolverBundle implements VersionSupplier {
 
 	void addExport(ResolverExport re) {
 		ResolverExport[] newExports = new ResolverExport[exports.length + 1];
-		for (int i = 0; i < exports.length; i++) {
+		for (int i = 0; i < exports.length; i++)
 			newExports[i] = exports[i];
-		}
 		newExports[exports.length] = re;
 		exports = newExports;
 	}
@@ -551,4 +479,18 @@ public class ResolverBundle implements VersionSupplier {
 	ResolverImpl getResolver() {
 		return resolver;
 	}
+
+	void clearRefs() {
+		if (refs != null)
+			refs.clear();
+	}
+
+	void addRef(ResolverBundle ref) {
+		if (refs != null && !refs.contains(ref))
+			refs.add(ref);
+	}
+
+	int getRefs() {
+		return refs == null ? 0 : refs.size();
+	}
 }
