@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -12,13 +12,9 @@ package org.eclipse.pde.internal.core.isite;
 
 import org.eclipse.core.runtime.CoreException;
 
-/**
- * @author dejan
- *
- */
 public interface ISiteDescription extends ISiteObject {
-	String P_URL = "url";
-	String P_TEXT = "text";
+	String P_URL = "url"; //$NON-NLS-1$
+	String P_TEXT = "text"; //$NON-NLS-1$
 	
 	String getURL();
 	String getText();
