@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,9 +10,11 @@
  *******************************************************************************/
 package org.eclipse.pde.internal.ui;
 
-import org.eclipse.ui.*;
-import org.eclipse.jdt.ui.JavaUI;
 import org.eclipse.debug.ui.IDebugUIConstants;
+import org.eclipse.jdt.ui.JavaUI;
+import org.eclipse.ui.IFolderLayout;
+import org.eclipse.ui.IPageLayout;
+import org.eclipse.ui.IPerspectiveFactory;
 
 public class PDEPerspective implements IPerspectiveFactory {
 	
@@ -47,7 +49,7 @@ public class PDEPerspective implements IPerspectiveFactory {
 		topLeft.addPlaceholder(IPageLayout.ID_RES_NAV);
 		topLeft.addView(JavaUI.ID_PACKAGES);
 		topLeft.addPlaceholder(JavaUI.ID_TYPE_HIERARCHY);
-		topLeft.addView(PDEPlugin.PLUGINS_VIEW_ID);
+		topLeft.addView(IPDEUIConstants.PLUGINS_VIEW_ID);
 
 		IFolderLayout bottom =
 			factory.createFolder(
@@ -58,7 +60,6 @@ public class PDEPerspective implements IPerspectiveFactory {
 		bottom.addView("org.eclipse.pde.runtime.LogView"); //$NON-NLS-1$
 		bottom.addView(IPageLayout.ID_TASK_LIST);
 		bottom.addView(IPageLayout.ID_PROBLEM_VIEW);
-		bottom.addView(IPageLayout.ID_PROP_SHEET);
 		
 		factory.addView(
 			IPageLayout.ID_OUTLINE,
@@ -71,11 +72,13 @@ public class PDEPerspective implements IPerspectiveFactory {
 		factory.addActionSet(IDebugUIConstants.LAUNCH_ACTION_SET);
 		factory.addActionSet(IDebugUIConstants.DEBUG_ACTION_SET);
 		factory.addActionSet(JavaUI.ID_ACTION_SET);
-		factory.addActionSet(JavaUI.ID_ELEMENT_CREATION_ACTION_SET);		
+		factory.addActionSet(JavaUI.ID_ELEMENT_CREATION_ACTION_SET);	
+		factory.addActionSet("org.eclipse.debug.ui.breakpointActionSet"); //$NON-NLS-1$
 	}
 	
 	private void addNewWizardShortcuts() {
 		factory.addNewWizardShortcut("org.eclipse.pde.ui.NewProductConfigurationWizard"); //$NON-NLS-1$
+		factory.addNewWizardShortcut("org.eclipse.pde.ui.NewProfileWizard"); //$NON-NLS-1$
 		factory.addNewWizardShortcut("org.eclipse.jdt.ui.wizards.NewPackageCreationWizard"); //$NON-NLS-1$
 		factory.addNewWizardShortcut("org.eclipse.jdt.ui.wizards.NewClassCreationWizard"); //$NON-NLS-1$
 		factory.addNewWizardShortcut("org.eclipse.jdt.ui.wizards.NewInterfaceCreationWizard"); //$NON-NLS-1$
@@ -87,7 +90,7 @@ public class PDEPerspective implements IPerspectiveFactory {
 	private void addViewShortcuts() {
 		factory.addShowViewShortcut(JavaUI.ID_PACKAGES);
 		factory.addShowViewShortcut("org.eclipse.pde.runtime.LogView"); //$NON-NLS-1$
-		factory.addShowViewShortcut(PDEPlugin.PLUGINS_VIEW_ID);
+		factory.addShowViewShortcut(IPDEUIConstants.PLUGINS_VIEW_ID);
 		factory.addShowViewShortcut(IPageLayout.ID_RES_NAV);
 		factory.addShowViewShortcut(IPageLayout.ID_PROBLEM_VIEW);
 		factory.addShowViewShortcut(IPageLayout.ID_TASK_LIST);
