@@ -1,61 +1,112 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.pde.internal.ui.view;
 
-import java.io.*;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileOutputStream;
+import java.io.IOException;
 import java.lang.reflect.InvocationTargetException;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.Locale;
+import java.util.Set;
 
-import org.eclipse.core.resources.*;
-import org.eclipse.core.runtime.*;
-import org.eclipse.jdt.core.*;
+import org.eclipse.core.resources.IStorage;
+import org.eclipse.core.runtime.CoreException;
+import org.eclipse.core.runtime.IConfigurationElement;
+import org.eclipse.core.runtime.IProgressMonitor;
+import org.eclipse.jdt.core.IClassFile;
+import org.eclipse.jdt.core.IPackageFragment;
+import org.eclipse.jdt.core.JavaModelException;
 import org.eclipse.jdt.ui.JavaUI;
 import org.eclipse.jdt.ui.actions.OpenAction;
-import org.eclipse.jface.action.*;
-import org.eclipse.jface.dialogs.*;
+import org.eclipse.jface.action.Action;
+import org.eclipse.jface.action.IMenuListener;
+import org.eclipse.jface.action.IMenuManager;
+import org.eclipse.jface.action.IToolBarManager;
+import org.eclipse.jface.action.MenuManager;
+import org.eclipse.jface.action.Separator;
+import org.eclipse.jface.dialogs.IDialogSettings;
 import org.eclipse.jface.operation.IRunnableWithProgress;
 import org.eclipse.jface.resource.ImageDescriptor;
-import org.eclipse.jface.util.*;
-import org.eclipse.jface.viewers.*;
-import org.eclipse.pde.core.plugin.*;
-import org.eclipse.pde.internal.core.*;
+import org.eclipse.jface.util.IPropertyChangeListener;
+import org.eclipse.jface.util.PropertyChangeEvent;
+import org.eclipse.jface.viewers.DoubleClickEvent;
+import org.eclipse.jface.viewers.IDoubleClickListener;
+import org.eclipse.jface.viewers.ISelection;
+import org.eclipse.jface.viewers.ISelectionChangedListener;
+import org.eclipse.jface.viewers.IStructuredSelection;
+import org.eclipse.jface.viewers.LabelProvider;
+import org.eclipse.jface.viewers.SelectionChangedEvent;
+import org.eclipse.jface.viewers.StructuredSelection;
+import org.eclipse.jface.viewers.TreeViewer;
+import org.eclipse.jface.viewers.Viewer;
+import org.eclipse.jface.viewers.ViewerFilter;
+import org.eclipse.osgi.util.NLS;
+import org.eclipse.pde.core.plugin.IPluginBase;
+import org.eclipse.pde.core.plugin.IPluginImport;
+import org.eclipse.pde.core.plugin.IPluginModelBase;
+import org.eclipse.pde.internal.core.EntryFileAdapter;
+import org.eclipse.pde.internal.core.FileAdapter;
+import org.eclipse.pde.internal.core.ModelEntry;
+import org.eclipse.pde.internal.core.PDECore;
+import org.eclipse.pde.internal.core.PluginModelManager;
 import org.eclipse.pde.internal.core.plugin.WorkspacePluginModelBase;
 import org.eclipse.pde.internal.ui.IHelpContextIds;
+import org.eclipse.pde.internal.ui.IPDEUIConstants;
+import org.eclipse.pde.internal.ui.IPreferenceConstants;
 import org.eclipse.pde.internal.ui.PDEPlugin;
+import org.eclipse.pde.internal.ui.PDEPluginImages;
+import org.eclipse.pde.internal.ui.PDEUIMessages;
+import org.eclipse.pde.internal.ui.editor.JarEntryEditorInput;
 import org.eclipse.pde.internal.ui.editor.SystemFileEditorInput;
-import org.eclipse.pde.internal.ui.preferences.*;
+import org.eclipse.pde.internal.ui.editor.plugin.ManifestEditor;
 import org.eclipse.pde.internal.ui.search.PluginSearchActionGroup;
 import org.eclipse.pde.internal.ui.wizards.ListUtil;
-import org.eclipse.pde.internal.ui.wizards.imports.*;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.custom.BusyIndicator;
-import org.eclipse.swt.dnd.*;
+import org.eclipse.swt.dnd.Clipboard;
+import org.eclipse.swt.dnd.DND;
+import org.eclipse.swt.dnd.FileTransfer;
+import org.eclipse.swt.dnd.Transfer;
 import org.eclipse.swt.program.Program;
-import org.eclipse.swt.widgets.*;
-import org.eclipse.ui.*;
+import org.eclipse.swt.widgets.Composite;
+import org.eclipse.swt.widgets.Menu;
+import org.eclipse.ui.IActionBars;
+import org.eclipse.ui.IEditorInput;
+import org.eclipse.ui.IPageLayout;
+import org.eclipse.ui.ISharedImages;
+import org.eclipse.ui.IWorkbenchActionConstants;
+import org.eclipse.ui.IWorkbenchPage;
+import org.eclipse.ui.PartInitException;
+import org.eclipse.ui.PlatformUI;
 import org.eclipse.ui.actions.ActionContext;
-import org.eclipse.ui.help.WorkbenchHelp;
-import org.eclipse.ui.part.*;
+import org.eclipse.ui.actions.ActionFactory;
+import org.eclipse.ui.part.DrillDownAdapter;
+import org.eclipse.ui.part.ViewPart;
 
 public class PluginsView extends ViewPart {
-	private static final int TEMP_FILE_LIMIT = 10;
-	private static final String DEFAULT_EDITOR_ID = "org.eclipse.ui.DefaultTextEditor"; //$NON-NLS-1$
+	private static final String DEFAULT_EDITOR_ID = "org.eclipse.ui.DefaultTextEditor";  //$NON-NLS-1$
+	private static final String HIDE_WRKSPC = "hideWorkspace"; //$NON-NLS-1$
+	private static final String HIDE_EXENABLED = "hideEnabledExternal"; //$NON-NLS-1$
+	private static final String SHOW_EXDISABLED = "showDisabledExternal"; //$NON-NLS-1$
 	private TreeViewer treeViewer;
 	private DrillDownAdapter drillDownAdapter;
 	private IPropertyChangeListener propertyListener;
 	private Action openAction;
-	private Action importBinaryAction;
-	private Action importSourceAction;
-	private Action disabledFilterAction;
-	private Action workspaceFilterAction;
+	private Action hideExtDisabledFilterAction;
+	private Action hideExtEnabledFilterAction;
+	private Action hideWorkspaceFilterAction;
 	private Action openManifestAction;
 	private Action openSchemaAction;
 	private Action openSystemEditorAction;
@@ -67,23 +118,30 @@ public class PluginsView extends ViewPart {
 	private Action selectInJavaSearchAction;
 	private Action addToJavaSearchAction;
 	private Action removeFromJavaSearchAction;
+	private Action selectAllAction;
+    private CollapseAllAction collapseAllAction;
 	private ShowInWorkspaceAction showInNavigatorAction;
 	private ShowInWorkspaceAction showInPackagesAction;
-	private DisabledFilter disabledFilter = new DisabledFilter();
-	private WorkspaceFilter workspaceFilter = new WorkspaceFilter();
+	private DisabledFilter hideExtEnabledFilter = new DisabledFilter(true);
+	private DisabledFilter hideExtDisabledFilter = new DisabledFilter(false);
+	private WorkspaceFilter hideWorkspaceFilter = new WorkspaceFilter();
 	private JavaFilter javaFilter = new JavaFilter();
 	private CopyToClipboardAction copyAction;
-	private ArrayList tempFiles;
 	private Clipboard clipboard;
 
 	class DisabledFilter extends ViewerFilter {
+		
+		boolean fEnabled;
+		DisabledFilter(boolean enabled) {
+			fEnabled = enabled;
+		}
 		public boolean select(Viewer v, Object parent, Object element) {
 			if (element instanceof ModelEntry) {
 				ModelEntry entry = (ModelEntry) element;
 				if (entry.getWorkspaceModel() == null) {
 					IPluginModelBase externalModel = entry.getExternalModel();
 					if (externalModel != null)
-						return externalModel.isEnabled();
+						return externalModel.isEnabled() != fEnabled;
 				}
 			}
 			return true;
@@ -114,6 +172,26 @@ public class PluginsView extends ViewPart {
 		}
 	}
 
+    class CollapseAllAction extends Action {
+        public CollapseAllAction() {
+            super();
+            setText(PDEUIMessages.PluginsView_CollapseAllAction_label); 
+            setDescription(PDEUIMessages.PluginsView_CollapseAllAction_description); 
+            setToolTipText(PDEUIMessages.PluginsView_CollapseAllAction_tooltip); 
+            setImageDescriptor(PDEPluginImages.DESC_COLLAPSE_ALL);
+        }
+
+        /*
+         * (non-Javadoc)
+         * 
+         * @see org.eclipse.jface.action.Action#run()
+         */
+        public void run() {
+            super.run();
+            treeViewer.collapseAll();
+        }
+    }
+    
 	/**
 	 * Constructor for PluginsView.
 	 */
@@ -121,7 +199,7 @@ public class PluginsView extends ViewPart {
 		propertyListener = new IPropertyChangeListener() {
 			public void propertyChange(PropertyChangeEvent event) {
 				String property = event.getProperty();
-				if (property.equals(MainPreferencePage.PROP_SHOW_OBJECTS)) {
+				if (property.equals(IPreferenceConstants.PROP_SHOW_OBJECTS)) {
 					treeViewer.refresh();
 				}
 			}
@@ -129,16 +207,12 @@ public class PluginsView extends ViewPart {
 	}
 
 	public void dispose() {
-		PDEPlugin
-			.getDefault()
-			.getPreferenceStore()
-			.removePropertyChangeListener(
-			propertyListener);
-		PDECore.getDefault().getTempFileManager().disconnect(this);
+		PDEPlugin.getDefault().getPreferenceStore().removePropertyChangeListener(
+				propertyListener);
 		openDependenciesAction.dispose();
-		if (clipboard!=null) {
+		if (clipboard != null) {
 			clipboard.dispose();
-			clipboard=null;
+			clipboard = null;
 		}
 		super.dispose();
 	}
@@ -147,34 +221,32 @@ public class PluginsView extends ViewPart {
 	 * @see IWorkbenchPart#createPartControl(Composite)
 	 */
 	public void createPartControl(Composite parent) {
-		treeViewer =
-			new TreeViewer(parent, SWT.MULTI | SWT.V_SCROLL | SWT.H_SCROLL);
+		treeViewer = new TreeViewer(parent, SWT.MULTI | SWT.V_SCROLL | SWT.H_SCROLL);
 		drillDownAdapter = new DrillDownAdapter(treeViewer);
 		PluginModelManager manager = PDECore.getDefault().getModelManager();
-		treeViewer.setContentProvider(
-			new PluginsContentProvider(this, manager));
+		treeViewer.setContentProvider(new PluginsContentProvider(this, manager));
 		treeViewer.setLabelProvider(new PluginsLabelProvider());
 		treeViewer.setSorter(ListUtil.PLUGIN_SORTER);
-		;
 		initDragAndDrop();
 		makeActions();
 		initFilters();
 		IActionBars actionBars = getViewSite().getActionBars();
 		contributeToActionBars(actionBars);
+		registerGlobalActions(actionBars);
 		hookContextMenu();
 		hookDoubleClickAction();
-		treeViewer
-			.addSelectionChangedListener(new ISelectionChangedListener() {
+		treeViewer.addSelectionChangedListener(new ISelectionChangedListener() {
 			public void selectionChanged(SelectionChangedEvent e) {
 				handleSelectionChanged(e.getSelection());
 			}
 		});
 		treeViewer.setInput(manager);
+		updateContentDescription();
 		PDEPlugin.getDefault().getPreferenceStore().addPropertyChangeListener(
 			propertyListener);
 		getViewSite().setSelectionProvider(treeViewer);
 
-		WorkbenchHelp.setHelp(
+		PlatformUI.getWorkbench().getHelpSystem().setHelp(
 			treeViewer.getControl(),
 			IHelpContextIds.PLUGINS_VIEW);
 	}
@@ -185,12 +257,20 @@ public class PluginsView extends ViewPart {
 	}
 
 	private void contributeToDropDownMenu(IMenuManager manager) {
-		manager.add(workspaceFilterAction);
-		manager.add(disabledFilterAction);
+		manager.add(hideWorkspaceFilterAction);
+		manager.add(hideExtEnabledFilterAction);
+		manager.add(hideExtDisabledFilterAction);
 	}
 
 	private void contributeToLocalToolBar(IToolBarManager manager) {
-		drillDownAdapter.addNavigationActions(manager);
+ 		drillDownAdapter.addNavigationActions(manager);
+        manager.add(new Separator());
+        manager.add(collapseAllAction);
+	}
+	
+	private void registerGlobalActions(IActionBars actionBars) {
+		actionBars.setGlobalActionHandler(ActionFactory.SELECT_ALL.getId(),
+            selectAllAction);
 	}
 	private void makeActions() {
 		clipboard = new Clipboard(treeViewer.getTree().getDisplay());
@@ -199,7 +279,7 @@ public class PluginsView extends ViewPart {
 				handleDoubleClick();
 			}
 		};
-		openAction.setText(PDEPlugin.getResourceString("PluginsView.open")); //$NON-NLS-1$
+		openAction.setText(PDEUIMessages.PluginsView_open); 
 
 		openDependenciesAction = new OpenDependenciesAction();
 		openDependenciesAction.init(PDEPlugin.getActiveWorkbenchWindow());
@@ -208,56 +288,54 @@ public class PluginsView extends ViewPart {
 				ModelEntry entry = getEnclosingEntry();
 				IPluginModelBase model = entry.getActiveModel();
 				openDependenciesAction.selectionChanged(
-					this,
-					new StructuredSelection(model));
+					this, new StructuredSelection(model));
 				openDependenciesAction.run(this);
 			}
 		};
-		openDependenciesAdapter.setText(PDEPlugin.getResourceString("PluginsView.openDependencies")); //$NON-NLS-1$
-
-		importBinaryAction = new Action() {
+		openDependenciesAdapter.setText(PDEUIMessages.PluginsView_openDependencies); 
+		hideExtDisabledFilterAction = new Action() {
 			public void run() {
-				handleImport(false);
-			}
-		};
-		importBinaryAction.setText(PDEPlugin.getResourceString("PluginsView.asBinaryProject")); //$NON-NLS-1$
-		importSourceAction = new Action() {
-			public void run() {
-				handleImport(true);
-			}
-		};
-		importSourceAction.setText(PDEPlugin.getResourceString("PluginsView.asSourceProject")); //$NON-NLS-1$
-		disabledFilterAction = new Action() {
-			public void run() {
-				boolean checked = disabledFilterAction.isChecked();
+				boolean checked = hideExtDisabledFilterAction.isChecked();
 				if (checked)
-					treeViewer.removeFilter(disabledFilter);
+					treeViewer.removeFilter(hideExtDisabledFilter);
 				else
-					treeViewer.addFilter(disabledFilter);
-				getSettings().put("disabledFilter", !checked); //$NON-NLS-1$
+					treeViewer.addFilter(hideExtDisabledFilter);
+				getSettings().put(SHOW_EXDISABLED, checked); 
+				updateContentDescription();
 			}
 		};
-		disabledFilterAction.setText(PDEPlugin.getResourceString("PluginsView.showDisabled")); //$NON-NLS-1$
-		disabledFilterAction.setChecked(false);
-		workspaceFilterAction = new Action() {
+		hideExtDisabledFilterAction.setText(PDEUIMessages.PluginsView_showDisabled); 
+		hideExtEnabledFilterAction = new Action() {
 			public void run() {
-				boolean checked = workspaceFilterAction.isChecked();
+				boolean checked = hideExtEnabledFilterAction.isChecked();
 				if (checked)
-					treeViewer.removeFilter(workspaceFilter);
+					treeViewer.removeFilter(hideExtEnabledFilter);
 				else
-					treeViewer.addFilter(workspaceFilter);
-				getSettings().put("workspaceFilter", !checked); //$NON-NLS-1$
+					treeViewer.addFilter(hideExtEnabledFilter);
+				getSettings().put(HIDE_EXENABLED, !checked); 
+				updateContentDescription();
 			}
 		};
-		workspaceFilterAction.setText(PDEPlugin.getResourceString("PluginsView.showWorkspace")); //$NON-NLS-1$
-		workspaceFilterAction.setChecked(true);
+		hideExtEnabledFilterAction.setText(PDEUIMessages.PluginsView_showEnabled); 
+		hideWorkspaceFilterAction = new Action() {
+			public void run() {
+				boolean checked = hideWorkspaceFilterAction.isChecked();
+				if (checked)
+					treeViewer.removeFilter(hideWorkspaceFilter);
+				else
+					treeViewer.addFilter(hideWorkspaceFilter);
+				getSettings().put(HIDE_WRKSPC, !checked); 
+				updateContentDescription();
+			}
+		};
+		hideWorkspaceFilterAction.setText(PDEUIMessages.PluginsView_showWorkspace); 
 
 		openTextEditorAction = new Action() {
 			public void run() {
 				handleOpenTextEditor(getSelectedFile(), null);
 			}
 		};
-		openTextEditorAction.setText(PDEPlugin.getResourceString("PluginsView.textEditor")); //$NON-NLS-1$
+		openTextEditorAction.setText(PDEUIMessages.PluginsView_textEditor); 
 		openTextEditorAction.setImageDescriptor(
 			PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(
 				ISharedImages.IMG_OBJ_FILE));
@@ -267,60 +345,72 @@ public class PluginsView extends ViewPart {
 				handleOpenSystemEditor(getSelectedFile());
 			}
 		};
-		openSystemEditorAction.setText(PDEPlugin.getResourceString("PluginsView.systemEditor")); //$NON-NLS-1$
+		openSystemEditorAction.setText(PDEUIMessages.PluginsView_systemEditor); 
 		openManifestAction = new Action() {
 			public void run() {
 				handleOpenManifestEditor(getSelectedFile());
 			}
 		};
-		openManifestAction.setText(PDEPlugin.getResourceString("PluginsView.manifestEditor")); //$NON-NLS-1$
+		openManifestAction.setText(PDEUIMessages.PluginsView_manifestEditor); 
 		
 		openSchemaAction = new Action() {
 			public void run() {
 				handleOpenSchemaEditor(getSelectedFile());
 			}
 		};
-		openSchemaAction.setText(PDEPlugin.getResourceString("PluginsView.schemaEditor")); //$NON-NLS-1$
+		openSchemaAction.setText(PDEUIMessages.PluginsView_schemaEditor); 
 
 		copyAction = new CopyToClipboardAction(clipboard);
-		copyAction.setText(PDEPlugin.getResourceString("PluginsView.copy")); //$NON-NLS-1$
+		copyAction.setText(PDEUIMessages.PluginsView_copy); 
 
 		selectDependentAction = new Action() {
 			public void run() {
 				handleSelectDependent();
 			}
 		};
-		selectDependentAction.setText(PDEPlugin.getResourceString("PluginsView.dependentPlugins")); //$NON-NLS-1$
+		selectDependentAction.setText(PDEUIMessages.PluginsView_dependentPlugins); 
+		
 		selectInJavaSearchAction = new Action() {
 			public void run() {
 				handleSelectInJavaSearch();
 			}
 		};
-		selectInJavaSearchAction.setText(PDEPlugin.getResourceString("PluginsView.pluginsInJavaSearch")); //$NON-NLS-1$
-
+		selectInJavaSearchAction.setText(PDEUIMessages.PluginsView_pluginsInJavaSearch); 
+		
+		selectAllAction = new Action() {
+	        public void run() {
+	            super.run();
+	            treeViewer.getTree().selectAll();
+	        }
+		};
+		selectAllAction.setText(PDEUIMessages.PluginsView_SelectAllAction_label); 
+		
 		addToJavaSearchAction = new Action() {
 			public void run() {
 				handleJavaSearch(true);
 			}
 		};
-		addToJavaSearchAction.setText(PDEPlugin.getResourceString("PluginsView.addToJavaSearch")); //$NON-NLS-1$
+		addToJavaSearchAction.setText(PDEUIMessages.PluginsView_addToJavaSearch); 
 
 		removeFromJavaSearchAction = new Action() {
 			public void run() {
 				handleJavaSearch(false);
 			}
 		};
-		removeFromJavaSearchAction.setText(PDEPlugin.getResourceString("PluginsView.removeFromJavaSearch")); //$NON-NLS-1$
+		removeFromJavaSearchAction.setText(PDEUIMessages.PluginsView_removeFromJavaSearch); 
 
 		showInNavigatorAction =
 			new ShowInWorkspaceAction(IPageLayout.ID_RES_NAV, treeViewer);
-		showInNavigatorAction.setText(PDEPlugin.getResourceString("PluginsView.showInNavigator")); //$NON-NLS-1$
+		showInNavigatorAction.setText(PDEUIMessages.PluginsView_showInNavigator); 
 		showInPackagesAction =
 			new ShowInWorkspaceAction(JavaUI.ID_PACKAGES, treeViewer);
-		showInPackagesAction.setText(PDEPlugin.getResourceString("PluginsView.showInPackageExplorer")); //$NON-NLS-1$
+		showInPackagesAction.setText(PDEUIMessages.PluginsView_showInPackageExplorer); 
+        
+        collapseAllAction = new CollapseAllAction();
 
 		openClassFileAction = new OpenAction(getViewSite());
 	}
+
 	private FileAdapter getSelectedFile() {
 		Object obj = getSelectedObject();
 		if (obj instanceof FileAdapter)
@@ -359,10 +449,18 @@ public class PluginsView extends ViewPart {
 		if (selection.size() == 1) {
 			Object sobj = selection.getFirstElement();
 			boolean addSeparator = false;
+            if (sobj instanceof ModelEntry) {
+                ModelEntry entry = (ModelEntry) sobj;
+                IPluginModelBase model = entry.getActiveModel();
+                File file = new File(model.getInstallLocation());
+                if (file.isFile() || model.getUnderlyingResource() != null) {
+                    manager.add(openAction);
+                }
+            }
 			if (sobj instanceof FileAdapter
 				&& ((FileAdapter) sobj).isDirectory() == false) {
 				manager.add(openAction);
-				MenuManager openWithMenu = new MenuManager(PDEPlugin.getResourceString("PluginsView.openWith")); //$NON-NLS-1$
+				MenuManager openWithMenu = new MenuManager(PDEUIMessages.PluginsView_openWith); 
 				fillOpenWithMenu(openWithMenu, sobj);
 				manager.add(openWithMenu);
 				addSeparator = true;
@@ -401,11 +499,10 @@ public class PluginsView extends ViewPart {
 			if (addSeparator) {
 				manager.add(new Separator());
 			}
-			if (canImport(selection)) {
-				MenuManager importMenu = new MenuManager(PDEPlugin.getResourceString("PluginsView.import")); //$NON-NLS-1$
-				importMenu.add(importBinaryAction);
-				importMenu.add(importSourceAction);
-				manager.add(importMenu);
+			if (ImportActionGroup.canImport(selection)) {
+				ImportActionGroup actionGroup = new ImportActionGroup();
+				actionGroup.setContext(new ActionContext(selection));
+				actionGroup.fillContextMenu(manager);
 				manager.add(new Separator());
 			}
 			addSeparator = false;
@@ -423,63 +520,51 @@ public class PluginsView extends ViewPart {
 		}
 		copyAction.setSelection(selection);
 		manager.add(copyAction);
-		IMenuManager selectionMenu = new MenuManager(PDEPlugin.getResourceString("PluginsView.select")); //$NON-NLS-1$
+		IMenuManager selectionMenu = new MenuManager(PDEUIMessages.PluginsView_select); 
 		manager.add(selectionMenu);
 		if (selection.size() > 0)
 			selectionMenu.add(selectDependentAction);
 		selectionMenu.add(selectInJavaSearchAction);
+		selectionMenu.add(selectAllAction);
 		manager.add(new Separator());
 		drillDownAdapter.addNavigationActions(manager);
-		manager.add(new Separator("Additions")); //$NON-NLS-1$
+		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
 	}
 	private void fillOpenWithMenu(IMenuManager manager, Object obj) {
 		FileAdapter adapter = (FileAdapter) obj;
 		String editorId = adapter.getEditorId();
 
 		String fileName = adapter.getFile().getName();
-		String lcFileName = fileName.toLowerCase();
+		String lcFileName = fileName.toLowerCase(Locale.ENGLISH);
 		ImageDescriptor desc =
 			PlatformUI.getWorkbench().getEditorRegistry().getImageDescriptor(
 				fileName);
-		if (lcFileName.equals("plugin.xml") //$NON-NLS-1$
-		|| lcFileName.equals("fragment.xml")) { //$NON-NLS-1$
+		if (lcFileName.equals("plugin.xml")  //$NON-NLS-1$
+		|| lcFileName.equals("fragment.xml")  //$NON-NLS-1$
+		|| lcFileName.equals("manifest.mf")) {  //$NON-NLS-1$
 			openManifestAction.setImageDescriptor(desc);
 			manager.add(openManifestAction);
 			manager.add(new Separator());
 			openManifestAction.setChecked(
 				editorId != null
-					&& editorId.equals(PDEPlugin.MANIFEST_EDITOR_ID));
+					&& editorId.equals(IPDEUIConstants.MANIFEST_EDITOR_ID));
 		}
-		if (lcFileName.endsWith(".mxsd") || lcFileName.endsWith(".exsd")) {
+		if (lcFileName.endsWith(".mxsd") || lcFileName.endsWith(".exsd")) { //$NON-NLS-1$ //$NON-NLS-2$
 			openSchemaAction.setImageDescriptor(desc);
 			manager.add(openSchemaAction);
 			manager.add(new Separator());
 			openSchemaAction.setChecked(
 				editorId != null 
-					&& editorId.equals(PDEPlugin.SCHEMA_EDITOR_ID));
+					&& editorId.equals(IPDEUIConstants.SCHEMA_EDITOR_ID));
 		}
 		manager.add(openTextEditorAction);
 		openTextEditorAction.setChecked(
 			editorId == null || editorId.equals(DEFAULT_EDITOR_ID));
 		openSystemEditorAction.setImageDescriptor(desc);
-		openSystemEditorAction.setChecked(editorId != null && editorId.equals("@system")); //$NON-NLS-1$
+		openSystemEditorAction.setChecked(editorId != null && editorId.equals("@system"));  //$NON-NLS-1$
 		manager.add(openSystemEditorAction);
 	}
 
-	private boolean canImport(IStructuredSelection selection) {
-		int nexternal = 0;
-		for (Iterator iter = selection.iterator(); iter.hasNext();) {
-			Object obj = iter.next();
-			if (obj instanceof ModelEntry) {
-				ModelEntry entry = (ModelEntry) obj;
-				if (entry.getWorkspaceModel() == null)
-					nexternal++;
-			} else
-				return false;
-		}
-		return nexternal > 0;
-	}
-
 	private boolean canDoJavaSearchOperation(
 		IStructuredSelection selection,
 		boolean add) {
@@ -505,35 +590,37 @@ public class PluginsView extends ViewPart {
 		treeViewer.addDragSupport(
 			ops,
 			transfers,
-			new PluginsDragAdapter((ISelectionProvider) treeViewer));
+			new PluginsDragAdapter(treeViewer));
 	}
 
 	private IDialogSettings getSettings() {
 		IDialogSettings master = PDEPlugin.getDefault().getDialogSettings();
-		IDialogSettings section = master.getSection("pluginsView"); //$NON-NLS-1$
+		IDialogSettings section = master.getSection("pluginsView");  //$NON-NLS-1$
 		if (section == null) {
-			section = master.addNewSection("pluginsView"); //$NON-NLS-1$
+			section = master.addNewSection("pluginsView");  //$NON-NLS-1$
 		}
 		return section;
 	}
 
 	private void initFilters() {
-		boolean workspace = false;
-		boolean disabled = true;
 		IDialogSettings settings = getSettings();
-		workspace = settings.getBoolean("workspaceFilter"); //$NON-NLS-1$
-		disabled = !settings.getBoolean("disabledFilter"); //$NON-NLS-1$
-		if (workspace)
-			treeViewer.addFilter(workspaceFilter);
-		if (disabled)
-			treeViewer.addFilter(disabledFilter);
 		treeViewer.addFilter(javaFilter);
-		workspaceFilterAction.setChecked(!workspace);
-		disabledFilterAction.setChecked(!disabled);
+		boolean hideWorkspace = settings.getBoolean(HIDE_WRKSPC);
+		boolean hideEnabledExternal = settings.getBoolean(HIDE_EXENABLED);
+		boolean hideDisabledExternal = !settings.getBoolean(SHOW_EXDISABLED);
+		if (hideWorkspace)
+			treeViewer.addFilter(hideWorkspaceFilter);
+		if (hideEnabledExternal)
+			treeViewer.addFilter(hideExtEnabledFilter);
+		if (hideDisabledExternal)
+			treeViewer.addFilter(hideExtDisabledFilter);
+		hideWorkspaceFilterAction.setChecked(!hideWorkspace);
+		hideExtEnabledFilterAction.setChecked(!hideEnabledExternal);
+		hideExtDisabledFilterAction.setChecked(!hideDisabledExternal);
 	}
 
 	private void hookContextMenu() {
-		MenuManager menuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
+		MenuManager menuMgr = new MenuManager("#PopupMenu");  //$NON-NLS-1$
 		menuMgr.setRemoveAllWhenShown(true);
 		menuMgr.addMenuListener(new IMenuListener() {
 			public void menuAboutToShow(IMenuManager manager) {
@@ -548,7 +635,13 @@ public class PluginsView extends ViewPart {
 	private void handleDoubleClick() {
 		Object obj = getSelectedObject();
 		if (obj instanceof ModelEntry) {
-			treeViewer.setExpandedState(obj, !treeViewer.getExpandedState(obj));
+            boolean expanded = treeViewer.getExpandedState(obj);
+            treeViewer.setExpandedState(obj, !expanded);
+            if (treeViewer.getExpandedState(obj) == expanded) {
+                // not expandable, open editor
+                ModelEntry modelEntry = (ModelEntry) obj;
+                ManifestEditor.openPluginEditor(modelEntry.getId());
+            }
 		}
 		if (obj instanceof FileAdapter) {
 			FileAdapter adapter = (FileAdapter) obj;
@@ -559,7 +652,7 @@ public class PluginsView extends ViewPart {
 				return;
 			}
 			String editorId = adapter.getEditorId();
-			if (editorId != null && editorId.equals("@system")) //$NON-NLS-1$
+			if (editorId != null && editorId.equals("@system"))  //$NON-NLS-1$
 				handleOpenSystemEditor(adapter);
 			else
 				handleOpenTextEditor(adapter, editorId);
@@ -574,7 +667,7 @@ public class PluginsView extends ViewPart {
 
 	private void handleOpenStorage(IStorage obj) {
 		IWorkbenchPage page = PDEPlugin.getActivePage();
-		IEditorInput input = new JarEntryEditorInput((IStorage) obj);
+		IEditorInput input = new JarEntryEditorInput(obj);
 		try {
 			page.openEditor(input, DEFAULT_EDITOR_ID);
 		} catch (PartInitException e) {
@@ -582,37 +675,6 @@ public class PluginsView extends ViewPart {
 		}
 	}
 
-	private void handleImport(boolean extractSource) {
-		IStructuredSelection selection =
-			(IStructuredSelection) treeViewer.getSelection();
-		ArrayList externalModels = new ArrayList();
-		for (Iterator iter = selection.iterator(); iter.hasNext();) {
-			ModelEntry entry = (ModelEntry) iter.next();
-			if (entry.getWorkspaceModel() != null)
-				continue;
-			externalModels.add(entry.getExternalModel());
-		}
-		IPluginModelBase[] models =
-			(IPluginModelBase[]) externalModels.toArray(
-				new IPluginModelBase[externalModels.size()]);
-		try {
-			Shell shell = treeViewer.getTree().getShell();
-			IRunnableWithProgress op =
-				PluginImportWizard.getImportOperation(
-					shell,
-					true,
-					extractSource,
-					models,
-					new ArrayList());
-			ProgressMonitorDialog pmd = new ProgressMonitorDialog(shell);
-			pmd.run(true, true, op);
-		} catch (InterruptedException e) {
-		} catch (InvocationTargetException e) {
-			PDEPlugin.logException(e);
-		} catch (Exception e) {
-		}
-	}
-
 	private void handleJavaSearch(final boolean add) {
 		IStructuredSelection selection =
 			(IStructuredSelection) treeViewer.getSelection();
@@ -639,9 +701,7 @@ public class PluginsView extends ViewPart {
 				PluginModelManager manager =
 					PDECore.getDefault().getModelManager();
 				try {
-					boolean useContainers =
-						BuildpathPreferencePage.getUseClasspathContainers();
-					manager.setInJavaSearch(array, add, useContainers, monitor);
+					manager.setInJavaSearch(array, add, monitor);
 				} catch (CoreException e) {
 					throw new InvocationTargetException(e);
 				} finally {
@@ -650,10 +710,8 @@ public class PluginsView extends ViewPart {
 			}
 		};
 
-		ProgressMonitorDialog pmd =
-			new ProgressMonitorDialog(treeViewer.getTree().getShell());
 		try {
-			pmd.run(false, false, op);
+			PlatformUI.getWorkbench().getProgressService().busyCursorWhile(op);
 		} catch (InterruptedException e) {
 		} catch (InvocationTargetException e) {
 			PDEPlugin.logException(e);
@@ -704,10 +762,7 @@ public class PluginsView extends ViewPart {
 		for (int i = 0; i < iimports.length; i++) {
 			IPluginImport iimport = iimports[i];
 			ModelEntry ientry =
-				manager.findEntry(
-					iimport.getId(),
-					iimport.getVersion(),
-					iimport.getMatch());
+				manager.findEntry(iimport.getId());
 			if (ientry != null) {
 				set.add(ientry);
 				addDependentEntries(ientry, set);
@@ -721,12 +776,12 @@ public class PluginsView extends ViewPart {
 		IWorkbenchPage page = PDEPlugin.getActivePage();
 		if (editorId == null) {
 			if (adapter.isManifest())
-				editorId = PDEPlugin.MANIFEST_EDITOR_ID;
+				editorId = IPDEUIConstants.MANIFEST_EDITOR_ID;
 			else if (adapter.isSchema())
-				editorId = PDEPlugin.SCHEMA_EDITOR_ID;
+				editorId = IPDEUIConstants.SCHEMA_EDITOR_ID;
 		}
 		try {
-			if (editorId == null || editorId.equals("@system")) //$NON-NLS-1$
+			if (editorId == null || editorId.equals("@system"))  //$NON-NLS-1$
 				editorId = DEFAULT_EDITOR_ID;
 			page.openEditor(
 				new SystemFileEditorInput(adapter.getFile()),
@@ -738,11 +793,11 @@ public class PluginsView extends ViewPart {
 	}
 
 	private void handleOpenManifestEditor(FileAdapter adapter) {
-		handleOpenTextEditor(adapter, PDEPlugin.MANIFEST_EDITOR_ID);
+		handleOpenTextEditor(adapter, IPDEUIConstants.MANIFEST_EDITOR_ID);
 	}
 	
 	private void handleOpenSchemaEditor(FileAdapter adapter) {
-		handleOpenTextEditor(adapter, PDEPlugin.SCHEMA_EDITOR_ID);
+		handleOpenTextEditor(adapter, IPDEUIConstants.SCHEMA_EDITOR_ID);
 	}
 
 	private void handleOpenSystemEditor(FileAdapter adapter) {
@@ -774,11 +829,9 @@ public class PluginsView extends ViewPart {
 		if (!result[0]) {
 			PDEPlugin.logException(
 				new PartInitException(
-					PDEPlugin.getFormattedMessage(
-						"PluginsView.unableToOpen",
-						file.getName())));
+					NLS.bind(PDEUIMessages.PluginsView_unableToOpen, file.getName())));
 		} else {
-			adapter.setEditorId("@system"); //$NON-NLS-1$
+			adapter.setEditorId("@system");  //$NON-NLS-1$
 		}
 	}
 
@@ -798,11 +851,8 @@ public class PluginsView extends ViewPart {
 			prefix = fileName;
 		}
 
-		File tmpFile =
-			PDECore.getDefault().getTempFileManager().createTempFile(
-				this,
-				prefix,
-				suffix);
+		File tmpFile = File.createTempFile(prefix, suffix);
+		tmpFile.deleteOnExit();
 		FileOutputStream fos = new FileOutputStream(tmpFile);
 		FileInputStream fis = new FileInputStream(file);
 		byte[] cbuffer = new byte[1024];
@@ -821,7 +871,7 @@ public class PluginsView extends ViewPart {
 	}
 
 	private void handleSelectionChanged(ISelection selection) {
-		String text = ""; //$NON-NLS-1$
+		String text = "";  //$NON-NLS-1$
 		Object obj = getSelectedObject();
 		if (obj instanceof ModelEntry) {
 			IPluginModelBase model = ((ModelEntry) obj).getActiveModel();
@@ -832,7 +882,7 @@ public class PluginsView extends ViewPart {
 		}
 		getViewSite().getActionBars().getStatusLineManager().setMessage(text);
 	}
-
+	
 	private void hookDoubleClickAction() {
 		treeViewer.addDoubleClickListener(new IDoubleClickListener() {
 			public void doubleClick(DoubleClickEvent event) {
@@ -851,29 +901,33 @@ public class PluginsView extends ViewPart {
 		IConfigurationElement config = getConfigurationElement();
 		if (config == null)
 			return;
-		String viewName = config.getAttribute("name"); //$NON-NLS-1$
+		
 		if (newInput == null
-			|| newInput.equals(PDECore.getDefault().getModelManager())) {
-			// restore old
-			setTitle(viewName);
+				|| newInput.equals(PDECore.getDefault().getModelManager())) {
+			setContentDescription("");  //$NON-NLS-1$
 			setTitleToolTip(getTitle());
 		} else {
-			String name =
-				((LabelProvider) treeViewer.getLabelProvider()).getText(
-					newInput);
-			setTitle(viewName + ": " + name); //$NON-NLS-1$
+			String viewName = config.getAttribute("name");  //$NON-NLS-1$
+			String name = ((LabelProvider) treeViewer.getLabelProvider()).getText(newInput);
+			setContentDescription(viewName + ": " + name);  //$NON-NLS-1$
 			setTitleToolTip(getInputPath(newInput));
 		}
 	}
 	private String getInputPath(Object input) {
 		if (input instanceof FileAdapter) {
-			return "file: " + ((FileAdapter) input).getFile().getAbsolutePath(); //$NON-NLS-1$
+			return "file: " + ((FileAdapter) input).getFile().getAbsolutePath();  //$NON-NLS-1$
 		}
 		if (input instanceof ModelEntry) {
 			IPluginModelBase model = ((ModelEntry) input).getActiveModel();
-			return "plugin: " + model.getInstallLocation(); //$NON-NLS-1$
+			return "plugin: " + model.getInstallLocation();  //$NON-NLS-1$
 		}
-		return ""; //$NON-NLS-1$
+		return "";  //$NON-NLS-1$
+	}
+	
+	private void updateContentDescription() {
+		String total = Integer.toString(((PluginModelManager)treeViewer.getInput()).getEntries().length);
+		String visible = Integer.toString(treeViewer.getTree().getItemCount());
+		setContentDescription(NLS.bind(PDEUIMessages.PluginsView_description, visible, total)); 
 	}
 
-}
\ No newline at end of file
+}
