@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,16 +10,17 @@
  *******************************************************************************/
 package org.eclipse.pde.internal.ui.wizards.extension;
 
+import java.net.URL;
 import java.util.ArrayList;
 import java.util.HashSet;
+import java.util.Locale;
+import java.util.regex.Pattern;
+import java.util.regex.PatternSyntaxException;
 
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IConfigurationElement;
-import org.eclipse.jface.action.Action;
-import org.eclipse.jface.action.IMenuListener;
-import org.eclipse.jface.action.IMenuManager;
-import org.eclipse.jface.action.MenuManager;
+import org.eclipse.core.runtime.NullProgressMonitor;
 import org.eclipse.jface.dialogs.Dialog;
 import org.eclipse.jface.dialogs.MessageDialog;
 import org.eclipse.jface.viewers.DoubleClickEvent;
@@ -34,14 +35,22 @@ import org.eclipse.jface.viewers.SelectionChangedEvent;
 import org.eclipse.jface.viewers.TableViewer;
 import org.eclipse.jface.viewers.Viewer;
 import org.eclipse.jface.viewers.ViewerFilter;
+import org.eclipse.jface.wizard.IWizard;
 import org.eclipse.jface.wizard.IWizardNode;
 import org.eclipse.osgi.util.NLS;
+import org.eclipse.pde.core.plugin.IFragment;
+import org.eclipse.pde.core.plugin.IFragmentModel;
 import org.eclipse.pde.core.plugin.IPluginExtension;
 import org.eclipse.pde.core.plugin.IPluginExtensionPoint;
 import org.eclipse.pde.core.plugin.IPluginImport;
 import org.eclipse.pde.core.plugin.IPluginModelBase;
 import org.eclipse.pde.internal.core.PDECore;
 import org.eclipse.pde.internal.core.PluginModelManager;
+import org.eclipse.pde.internal.core.ischema.ISchema;
+import org.eclipse.pde.internal.core.schema.SchemaDescriptor;
+import org.eclipse.pde.internal.core.schema.SchemaRegistry;
+import org.eclipse.pde.internal.core.text.plugin.PluginExtensionPointNode;
+import org.eclipse.pde.internal.core.util.PatternConstructor;
 import org.eclipse.pde.internal.ui.IHelpContextIds;
 import org.eclipse.pde.internal.ui.PDELabelProvider;
 import org.eclipse.pde.internal.ui.PDEPlugin;
@@ -49,7 +58,6 @@ import org.eclipse.pde.internal.ui.PDEPluginImages;
 import org.eclipse.pde.internal.ui.PDEUIMessages;
 import org.eclipse.pde.internal.ui.elements.DefaultContentProvider;
 import org.eclipse.pde.internal.ui.elements.ElementLabelProvider;
-import org.eclipse.pde.internal.ui.model.plugin.PluginExtensionPointNode;
 import org.eclipse.pde.internal.ui.search.ShowDescriptionAction;
 import org.eclipse.pde.internal.ui.util.SharedLabelProvider;
 import org.eclipse.pde.internal.ui.wizards.BaseWizardSelectionPage;
@@ -62,8 +70,11 @@ import org.eclipse.pde.ui.IBasePluginWizard;
 import org.eclipse.pde.ui.IExtensionWizard;
 import org.eclipse.pde.ui.templates.ITemplateSection;
 import org.eclipse.swt.SWT;
-import org.eclipse.swt.custom.BusyIndicator;
 import org.eclipse.swt.custom.SashForm;
+import org.eclipse.swt.events.KeyEvent;
+import org.eclipse.swt.events.KeyListener;
+import org.eclipse.swt.events.ModifyEvent;
+import org.eclipse.swt.events.ModifyListener;
 import org.eclipse.swt.events.SelectionAdapter;
 import org.eclipse.swt.events.SelectionEvent;
 import org.eclipse.swt.graphics.Image;
@@ -73,10 +84,11 @@ import org.eclipse.swt.widgets.Button;
 import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Control;
 import org.eclipse.swt.widgets.Label;
-import org.eclipse.swt.widgets.Menu;
+import org.eclipse.swt.widgets.Link;
 import org.eclipse.swt.widgets.TabFolder;
 import org.eclipse.swt.widgets.TabItem;
 import org.eclipse.swt.widgets.TableItem;
+import org.eclipse.swt.widgets.Text;
 import org.eclipse.ui.PlatformUI;
 
 public class PointSelectionPage
@@ -88,16 +100,18 @@ public class PointSelectionPage
 	private Button fFilterCheck;
 	private IPluginExtensionPoint fCurrentPoint;
 	private HashSet fAvailableImports;
-	private Action fShowDetailsSection;
 	private IProject fProject;
 	private Label fTemplateLabel;
 	private ExtensionTreeSelectionPage fWizardsPage;
 	
 	private IPluginExtension fNewExtension;
-	private ShowDescriptionAction fShowDescriptionAction;
 	private WizardCollectionElement fTemplateCollection;
 	private WizardCollectionElement fWizardCollection;
 	private NewExtensionWizard fWizard;
+	private Text fFilterText;
+	private WildcardFilter fWildCardFilter;
+	private Text fPointDescription;
+	private Link fDescLink;
 	
 	class PointFilter extends ViewerFilter {
 		public boolean select(Viewer viewer, Object parentElement, Object element) {
@@ -111,6 +125,22 @@ public class PointSelectionPage
 			return fAvailableImports.contains(point.getPluginBase().getId());
 		}
 	}
+	class WildcardFilter extends ViewerFilter {
+		private String wMatch = "*"; //$NON-NLS-1$
+		protected void setMatchText(String match) {
+			wMatch = match + "*"; //$NON-NLS-1$
+		}
+		public boolean select(Viewer viewer, Object parentElement, Object element) {
+			String text = ((PointLabelProvider)fPointListViewer.getLabelProvider()).getColumnText(element, 0);
+			Pattern pattern = null;
+			try {
+				pattern = PatternConstructor.createPattern(wMatch, false);
+			} catch (PatternSyntaxException e) {
+				return false;
+			}
+			return pattern != null && pattern.matcher(text.subSequence(0, text.length())).matches();
+		}
+	}
 	
 	class TemplateContentProvider extends DefaultContentProvider implements IStructuredContentProvider{
 		public Object[] getElements(Object inputElement) {
@@ -195,26 +225,26 @@ public class PointSelectionPage
 	}
 
 	public PointSelectionPage(IProject project, IPluginModelBase model, WizardCollectionElement element, WizardCollectionElement templates, NewExtensionWizard wizard) {
-		super("pointSelectionPage", PDEUIMessages.NewExtensionWizard_PointSelectionPage_title); //$NON-NLS-1$ //$NON-NLS-2$
+		super("pointSelectionPage", PDEUIMessages.NewExtensionWizard_PointSelectionPage_title); //$NON-NLS-1$ 
 		this.fModel = model;
 		this.fWizardCollection = element;
 		this.fTemplateCollection = templates;
 		this.fWizard= wizard;
 		this.fProject=project;
+		fWildCardFilter = new WildcardFilter();
 		fAvailableImports = PluginSelectionDialog.getExistingImports(model.getPluginBase());
-		setTitle(PDEUIMessages.NewExtensionWizard_PointSelectionPage_title); //$NON-NLS-1$
-		setDescription(PDEUIMessages.NewExtensionWizard_PointSelectionPage_desc); //$NON-NLS-1$
+		setTitle(PDEUIMessages.NewExtensionWizard_PointSelectionPage_title); 
+		setDescription(PDEUIMessages.NewExtensionWizard_PointSelectionPage_desc); 
 		PDEPlugin.getDefault().getLabelProvider().connect(this);
-		makeActions();
 	}
 	
 	public void createControl(Composite parent) {
 		// tab folder
 		final TabFolder tabFolder = new TabFolder(parent, SWT.FLAT);
 		TabItem firstTab = new TabItem(tabFolder, SWT.NULL);
-		firstTab.setText(PDEUIMessages.PointSelectionPage_tab1); //$NON-NLS-1$
+		firstTab.setText(PDEUIMessages.PointSelectionPage_tab1); 
 		TabItem secondTab = new TabItem(tabFolder, SWT.NULL);
-		secondTab.setText(PDEUIMessages.PointSelectionPage_tab2); //$NON-NLS-1$
+		secondTab.setText(PDEUIMessages.PointSelectionPage_tab2); 
 		secondTab.setControl(createWizardsPage(tabFolder));
 		tabFolder.addSelectionListener(new SelectionAdapter() {
 			public void widgetSelected(SelectionEvent e) {
@@ -236,8 +266,42 @@ public class PointSelectionPage
 		gd = new GridData(GridData.FILL_BOTH);
 		pointContainer.setLayoutData(gd);
 
-		Label pointLabel = new Label(pointContainer, SWT.NONE);
-		pointLabel.setText(PDEUIMessages.NewExtensionWizard_PointSelectionPage_availExtPoints_label); //$NON-NLS-1$
+		Composite labelContainer = new Composite(pointContainer, SWT.NONE);
+		layout = new GridLayout(2, false);
+		layout.marginHeight = layout.marginWidth = 0;
+		labelContainer.setLayout(layout);
+		labelContainer.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
+		
+		Label filterLabel = new Label(labelContainer, SWT.NONE);
+		filterLabel.setText(PDEUIMessages.NewExtensionWizard_PointSelectionPage_availExtPoints_label);
+		gd = new GridData();
+		gd.verticalAlignment = GridData.CENTER;
+		filterLabel.setLayoutData(gd);
+		fFilterText = new Text(labelContainer, SWT.BORDER);
+		fFilterText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
+        fFilterText.addModifyListener(new ModifyListener() {
+			public void modifyText(ModifyEvent e) {
+				fWildCardFilter.setMatchText(fFilterText.getText());
+            	fPointListViewer.refresh();
+			}
+        });
+        fFilterText.addKeyListener(new KeyListener() {
+            public void keyPressed(KeyEvent e) {
+                if (e.keyCode == SWT.ARROW_DOWN) fPointListViewer.getControl().setFocus();
+            }
+            public void keyReleased(KeyEvent e) {}
+        });
+		
+		fFilterCheck = new Button(outerContainer, SWT.CHECK);
+		fFilterCheck.setText(PDEUIMessages.NewExtensionWizard_PointSelectionPage_filterCheck); 
+		gd = new GridData(GridData.FILL_HORIZONTAL);
+		fFilterCheck.setLayoutData(gd);
+		fFilterCheck.setSelection(true);
+		fFilterCheck.addSelectionListener(new SelectionAdapter() {
+			public void widgetSelected(SelectionEvent e) {
+				fPointListViewer.refresh();
+			}
+		});
 		
 		fPointListViewer =
 			new TableViewer(
@@ -256,23 +320,39 @@ public class PointSelectionPage
 				}
 			}
 		});
-
+		fPointListViewer.addFilter(fWildCardFilter);
 		fPointListViewer.setSorter(ListUtil.NAME_SORTER);
 		gd = new GridData(GridData.FILL_BOTH);
 		gd.heightHint = 150;
 		fPointListViewer.getTable().setLayoutData(gd);
 
-		Composite templateComposite =
-			new Composite(outerContainer, SWT.NONE);
+		Composite templateComposite = new Composite(outerContainer, SWT.NONE);
 		layout = new GridLayout();
-		layout.marginHeight = 9;
+		layout.marginHeight = 4;
 		layout.marginWidth = 0;
 		templateComposite.setLayout(layout);
 		gd = new GridData(GridData.FILL_HORIZONTAL);
 		templateComposite.setLayoutData(gd);
 		
-		fTemplateLabel = new Label(templateComposite, SWT.NONE);
-		fTemplateLabel.setText(PDEUIMessages.NewExtensionWizard_PointSelectionPage_contributedTemplates_title); //$NON-NLS-1$
+		fDescLink = new Link(templateComposite, SWT.NONE);
+		fDescLink.setText(NLS.bind(PDEUIMessages.PointSelectionPage_extPointDesc, ""));  //$NON-NLS-1$
+		fDescLink.addSelectionListener(new SelectionAdapter() {
+			public void widgetSelected(SelectionEvent e) {
+				if (fCurrentPoint != null)
+					new ShowDescriptionAction(fCurrentPoint, true).run();	
+			}
+		});
+		fDescLink.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
+		
+		
+		fPointDescription = new Text(templateComposite, SWT.NONE | SWT.WRAP | SWT.MULTI | SWT.BORDER | SWT.V_SCROLL | SWT.READ_ONLY);
+		fPointDescription.setText(PDEUIMessages.NewExtensionWizard_PointSelectionPage_extPointDescription); 
+		gd = new GridData(GridData.FILL_HORIZONTAL);
+		gd.heightHint = 55;
+		fPointDescription.setLayoutData(gd);
+		
+		fTemplateLabel = new Label(templateComposite, SWT.NONE | SWT.WRAP);
+		fTemplateLabel.setText(PDEUIMessages.NewExtensionWizard_PointSelectionPage_contributedTemplates_title); 
 		gd = new GridData(GridData.FILL_HORIZONTAL);
 		fTemplateLabel.setLayoutData(gd);
 		
@@ -320,21 +400,8 @@ public class PointSelectionPage
 		gd = new GridData(GridData.FILL_BOTH | GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL);
 		descriptionComposite.setLayoutData(gd);
 		createDescriptionIn(descriptionComposite);
-
-		
-		fFilterCheck = new Button(outerContainer, SWT.CHECK);
-		fFilterCheck.setText(PDEUIMessages.NewExtensionWizard_PointSelectionPage_filterCheck); //$NON-NLS-1$
-		gd = new GridData(GridData.FILL_HORIZONTAL);
-		fFilterCheck.setLayoutData(gd);
-		fFilterCheck.setSelection(true);
-		fFilterCheck.addSelectionListener(new SelectionAdapter() {
-			public void widgetSelected(SelectionEvent e) {
-				fPointListViewer.refresh();
-			}
-		});
-		
+	
 		getContainer().getShell().setSize(500, 500);
-		createMenuManager();
 		initialize();
 		setControl(tabFolder);
 		Dialog.applyDialogFont(outerContainer);
@@ -343,34 +410,13 @@ public class PointSelectionPage
 			IHelpContextIds.ADD_EXTENSIONS_SCHEMA_BASED);
 	}
 	private Control createWizardsPage(Composite parent) {
-		fWizardsPage = new ExtensionTreeSelectionPage(fWizardCollection, null, PDEUIMessages.PointSelectionPage_categories); //$NON-NLS-1$
+		fWizardsPage = new ExtensionTreeSelectionPage(fWizardCollection, null, PDEUIMessages.PointSelectionPage_categories); 
 		fWizardsPage.createControl(parent);
 		fWizardsPage.setWizard(fWizard);
 		fWizardsPage.getSelectionProvider().addSelectionChangedListener(this);
 		fWizardsPage.init(fProject, fModel.getPluginBase());
 		return fWizardsPage.getControl();
 	}
-	private void createMenuManager(){
-		MenuManager mgr = new MenuManager();
-		mgr.addMenuListener(new IMenuListener(){
-
-			public void menuAboutToShow(IMenuManager manager) {
-				fillContextMenu(manager);
-			}
-			
-		});
-		mgr.setRemoveAllWhenShown(true);
-		Control control = fPointListViewer.getControl();
-		Menu menu = mgr.createContextMenu(control);
-		control.setMenu(menu);
-	}
-	private void fillContextMenu(IMenuManager mgr){
-		mgr.add(fShowDetailsSection);
-		ISelection selection = fPointListViewer.getSelection();
-		IPluginExtensionPoint point = (IPluginExtensionPoint)((IStructuredSelection)selection).getFirstElement();
-		fShowDetailsSection.setEnabled(point != null);
-		
-	}
 	
 	public void advanceToNextPage() {
 		getContainer().showPage(getNextPage());
@@ -417,18 +463,14 @@ public class PointSelectionPage
 			
 			String pluginID = fCurrentPoint.getPluginBase().getId();
 			if (!(fCurrentPoint instanceof PluginExtensionPointNode)
-					&& !fAvailableImports.contains(pluginID)) {
-				if (MessageDialog
-						.openQuestion(
-								getShell(),
-								PDEUIMessages.NewExtensionWizard_PointSelectionPage_dependencyTitle,
-								NLS
-										.bind(
-												PDEUIMessages.NewExtensionWizard_PointSelectionPage_dependencyMessage,
-												new String[] { pluginID,
-														fCurrentPoint.getId() }))) {
-					IPluginImport importNode = fModel.getPluginFactory()
-							.createImport();
+					&& !fAvailableImports.contains(pluginID)
+					&& !(fCurrentPoint.getPluginBase() instanceof IFragment)) {
+				if (MessageDialog.openQuestion(
+						getShell(), PDEUIMessages.NewExtensionWizard_PointSelectionPage_dependencyTitle,
+						NLS.bind(
+								PDEUIMessages.NewExtensionWizard_PointSelectionPage_dependencyMessage,
+								new String[] { pluginID, fCurrentPoint.getId() }))) {
+					IPluginImport importNode = fModel.getPluginFactory().createImport();
 					importNode.setId(pluginID);
 					fModel.getPluginBase().add(importNode);
 				}
@@ -438,18 +480,6 @@ public class PointSelectionPage
 		}
 		return true;
 	}
-
-	private void doShowDescription() {
-		if (fShowDescriptionAction == null)
-			fShowDescriptionAction = new ShowDescriptionAction(fCurrentPoint);
-		else
-			fShowDescriptionAction.setExtensionPoint(fCurrentPoint);
-		BusyIndicator.showWhile(fPointListViewer.getControl().getDisplay(), new Runnable() {
-			public void run() {
-				fShowDescriptionAction.run();
-			}
-		});
-	}
 	
 	public IPluginExtension getNewExtension() {
 		return fNewExtension;
@@ -461,56 +491,65 @@ public class PointSelectionPage
 		fPointListViewer.getTable().setFocus();
 	}
 	
-	private void makeActions(){
-		fShowDetailsSection = new Action(){
-			public void run(){
-				doShowDescription();
-			}
-		};
-		fShowDetailsSection.setText(PDEUIMessages.NewExtensionWizard_PointSelectionPage_showDetails); //$NON-NLS-1$
-	}
-	
 	public void selectionChanged(SelectionChangedEvent event) {
-		
 		ISelection selection = event.getSelection();
-		setDescription(""); //$NON-NLS-1$
 		if (selection instanceof IStructuredSelection) {
 			IStructuredSelection ssel = (IStructuredSelection) selection;
 			if (ssel != null && !ssel.isEmpty()) {
-				if (ssel.getFirstElement() instanceof IPluginExtensionPoint){
-					fCurrentPoint = (IPluginExtensionPoint) ssel.getFirstElement();
-					fTemplateViewer.setInput(fCurrentPoint);
-					setDescription(NLS.bind(PDEUIMessages.NewExtensionWizard_PointSelectionPage_pluginDescription, fCurrentPoint.getFullId())); //$NON-NLS-1$
-					setDescriptionText(""); //$NON-NLS-1$
-					fTemplateLabel.setText(NLS.bind(PDEUIMessages.NewExtensionWizard_PointSelectionPage_contributedTemplates_label, fCurrentPoint.getFullId())); //$NON-NLS-1$
-					setSelectedNode(null);
-					setPageComplete(true);
-				} else if (ssel.getFirstElement() instanceof WizardElement) {
-					WizardElement wizardSelection = (WizardElement)ssel.getFirstElement();
-					setSelectedNode(createWizardNode(wizardSelection));
-					setDescriptionText(wizardSelection.getDescription());
-					setDescription(NLS.bind(PDEUIMessages.NewExtensionWizard_PointSelectionPage_templateDescription, wizardSelection.getLabel())); //$NON-NLS-1$
-					setPageComplete(false);
-				}
-			}
-			else {
+				Object element = ssel.getFirstElement();
+				if (element instanceof WizardElement)
+					handleTemplateSelection((WizardElement)element);
+				else if (element instanceof IPluginExtensionPoint)
+					handlePointSelection((IPluginExtensionPoint)element);
+			} else {
+				setDescription(""); //$NON-NLS-1$
+				setDescriptionText(""); //$NON-NLS-1$
+				fTemplateLabel.setText(PDEUIMessages.NewExtensionWizard_PointSelectionPage_contributedTemplates_title);
+				fPointDescription.setText(PDEUIMessages.PointSelectionPage_noDescAvailable);
+				fDescLink.setText(NLS.bind(PDEUIMessages.PointSelectionPage_extPointDesc, "")); //$NON-NLS-1$
 				setSelectedNode(null);
 				setPageComplete(false);
 			}
+			getContainer().updateButtons();
 		}
-		getContainer().updateButtons();
+	}
+	
+	private void handleTemplateSelection(WizardElement element) {
+		setSelectedNode(createWizardNode(element));
+		setDescriptionText(element.getDescription());
+		setDescription(NLS.bind(PDEUIMessages.NewExtensionWizard_PointSelectionPage_templateDescription, element.getLabel())); 
+		setPageComplete(false);
+	}
+	
+	private void handlePointSelection(IPluginExtensionPoint element) {
+		fCurrentPoint = element;
+		fTemplateViewer.setInput(fCurrentPoint);
+		URL url = SchemaRegistry.getSchemaURL(fCurrentPoint);
+		String fullID = fCurrentPoint.getFullId();
+		ISchema desc = new SchemaDescriptor(fullID, url).getSchema(false);
+		String schemaName = desc != null ? desc.getName() : fullID;
+		setDescription(NLS.bind(PDEUIMessages.NewExtensionWizard_PointSelectionPage_pluginDescription, schemaName));
+		setDescriptionText(""); //$NON-NLS-1$
+		fTemplateLabel.setText(NLS.bind(PDEUIMessages.NewExtensionWizard_PointSelectionPage_contributedTemplates_label, schemaName.toLowerCase(Locale.ENGLISH)));
+		fDescLink.setText(NLS.bind(PDEUIMessages.PointSelectionPage_extPointDesc, schemaName));
+		if (desc != null)
+			fPointDescription.setText(stripTags(desc.getDescription()));
+		else
+			fPointDescription.setText(PDEUIMessages.PointSelectionPage_noDescAvailable);
+		setSelectedNode(null);
+		setPageComplete(true);
 	}
 	
 	private void updateTabSelection(int index) {
-		if (index==0) {
+		if (index == 0) {
 			// extension point page
 			ISelection selection = fTemplateViewer.getSelection();
 			if (selection.isEmpty()==false)
 				selectionChanged(new SelectionChangedEvent(fTemplateViewer, selection));
 			else
 				selectionChanged(new SelectionChangedEvent(fPointListViewer, fPointListViewer.getSelection()));
-		}
-		else {
+			fFilterText.setFocus();
+		} else {
 			// wizard page
 			ISelectionProvider provider = fWizardsPage.getSelectionProvider();
 			selectionChanged(new SelectionChangedEvent(provider, provider.getSelection()));
@@ -543,8 +582,61 @@ public class PointSelectionPage
 	
 	private String getFullId(IPluginExtensionPoint point) {
 		if (point instanceof PluginExtensionPointNode) {
-			return fModel.getPluginBase().getId() + "." + point.getId(); //$NON-NLS-1$
+			String pointId = point.getId();
+			if ("3.2".equals(fModel.getPluginBase().getSchemaVersion()) && pointId.indexOf('.') > 0) //$NON-NLS-1$
+				return pointId;
+			String id = null;
+			if (fModel instanceof IFragmentModel) {
+				IFragment fragment = ((IFragmentModel)fModel).getFragment();
+				if (fragment != null)
+					id = fragment.getPluginId();
+			}
+			if (id == null)
+				id = fModel.getPluginBase().getId();
+			return id + '.' + pointId;
 		}
 		return point.getFullId();
 	}
+	
+	private String stripTags(String html) {
+		int length = html.length();
+		boolean write = true;
+		char oldChar = ' ';
+		StringBuffer sb = new StringBuffer(length);
+		for (int i = 0; i < length; i++) {
+			char curr = html.charAt(i);
+			if (curr == '<')
+				write = false;
+			else if (curr == '>')
+				write = true;
+			else if (write && curr != '\r' && curr != '\n' && curr != '\t')
+				if (!(curr == ' ') || !(oldChar == curr)) { // skip multiple spaces
+					sb.append(curr);
+					oldChar = curr;
+				}
+		}
+		return sb.toString();
+	}
+	
+	public void checkModel() {
+		IWizardNode node = getSelectedNode();
+		if (node == null)
+			return;
+		IWizard wizard = node.getWizard();
+		if (wizard instanceof NewExtensionTemplateWizard) {
+			if (((NewExtensionTemplateWizard) wizard).updatedDependencies()) {
+				if (MessageDialog.openQuestion(getShell(),
+								PDEUIMessages.PointSelectionPage_newDepFound,
+								PDEUIMessages.PointSelectionPage_newDepMessage)) {
+					fWizard.getEditor().doSave(new NullProgressMonitor());
+				}
+			}
+		}
+	}
+	
+	public void setVisible(boolean visible) {
+		if (visible)
+			fFilterText.setFocus();
+		super.setVisible(visible);
+	}
 }
