@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,6 +12,9 @@ package org.eclipse.search.internal.ui;
 
 import org.eclipse.jface.action.Action;
 
+/**
+ * @deprecated old search
+ */
 class ShowSearchAction extends Action {
 	private Search fSearch;
 	
@@ -27,8 +30,6 @@ class ShowSearchAction extends Action {
 	}
 	/**
 	 *	Invoke the resource wizard selection wizard
-	 *
-	 *	@param browser org.eclipse.jface.parts.Window
 	 */
 	public void run() {
 		if (fSearch != SearchManager.getDefault().getCurrentSearch())
