@@ -1815,10 +1815,19 @@ public boolean setText(String html) {
 		nsIWebBrowserStream stream = new nsIWebBrowserStream(result[0]);
 		rc = stream.OpenStream(uri.getAddress(), aContentType);
 		if (rc != XPCOM.NS_OK) error(rc);
-		int /*long*/ ptr = XPCOM.PR_Malloc(data.length);
+		int ptr = XPCOM.PR_Malloc(data.length);
 		XPCOM.memmove(ptr, data, data.length);
-		rc = stream.AppendToStream(ptr, data.length);
-		if (rc != XPCOM.NS_OK) error(rc);
+		int pageSize = 8192;
+		int pageCount = data.length / pageSize + 1;
+		int current = ptr;
+		for (int i = 0; i < pageCount; i++) {
+			int length = i == pageCount - 1 ? data.length % pageSize : pageSize;
+			if (length > 0) {
+				rc = stream.AppendToStream(current, length);
+				if (rc != XPCOM.NS_OK) error(rc);
+			}
+			current += pageSize;
+		}
 		rc = stream.CloseStream();
 		if (rc != XPCOM.NS_OK) error(rc);
 		XPCOM.PR_Free(ptr);
