@@ -1,24 +1,25 @@
 /*******************************************************************************
-* Copyright (c) 2000, 2005 IBM Corporation and others.
-* All rights reserved. This program and the accompanying materials
-* are made available under the terms of the Eclipse Public License v1.0
-* which accompanies this distribution, and is available at
-* http://www.eclipse.org/legal/epl-v10.html
-*
-* Contributors:
-*     IBM Corporation - initial API and implementation
-*******************************************************************************/
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors:
+ *    IBM Corporation - initial API and implementation
+ *******************************************************************************/
 
 #include "swt.h"
 #include "os_stats.h"
 
 #ifdef NATIVE_STATS
 
-int OS_nativeFunctionCount = 846;
-int OS_nativeFunctionCallCount[846];
+int OS_nativeFunctionCount = 921;
+int OS_nativeFunctionCallCount[921];
 char * OS_nativeFunctionNames[] = {
 	"AECountItems",
 	"AEGetNthPtr",
+	"AEInstallEventHandler",
 	"AEProcessAppleEvent",
 	"ATSFontGetPostScriptName",
 	"ATSUBatchBreakLines",
@@ -58,6 +59,10 @@ char * OS_nativeFunctionNames[] = {
 	"ATSUSetTransientFontMatching",
 	"ATSUTextDeleted",
 	"ATSUTextInserted",
+	"AXNotificationHIObjectNotify",
+	"AXUIElementCreateWithHIObjectAndIdentifier",
+	"AXUIElementGetIdentifier",
+	"AXValueCreate",
 	"ActiveNonFloatingWindow",
 	"AddDataBrowserItems",
 	"AddDataBrowserListViewColumn",
@@ -72,12 +77,14 @@ char * OS_nativeFunctionNames[] = {
 	"CFArrayGetValueAtIndex",
 	"CFDataGetBytes",
 	"CFDataGetLength",
+	"CFEqual",
 	"CFLocaleCopyCurrent",
 	"CFNumberFormatterCopyProperty",
 	"CFNumberFormatterCreate",
 	"CFRelease",
 	"CFStringCreateWithBytes",
-	"CFStringCreateWithCharacters",
+	"CFStringCreateWithCharacters__III",
+	"CFStringCreateWithCharacters__I_3CI",
 	"CFStringGetBytes",
 	"CFStringGetCharacters",
 	"CFStringGetLength",
@@ -175,6 +182,7 @@ char * OS_nativeFunctionNames[] = {
 	"CGFunctionRelease",
 	"CGGetDisplaysWithRect",
 	"CGImageCreate",
+	"CGImageCreateWithImageInRect",
 	"CGImageCreateWithJPEGDataProvider",
 	"CGImageCreateWithPNGDataProvider",
 	"CGImageGetAlphaInfo",
@@ -204,6 +212,7 @@ char * OS_nativeFunctionNames[] = {
 	"CGPointApplyAffineTransform",
 	"CGPostKeyboardEvent",
 	"CGPostMouseEvent",
+	"CGRectContainsPoint",
 	"CGShadingCreateAxial",
 	"CGShadingCreateRadial",
 	"CGShadingRelease",
@@ -211,8 +220,10 @@ char * OS_nativeFunctionNames[] = {
 	"CPSEnableForegroundOperation",
 	"CPSSetProcessName",
 	"CalcMenuSize",
+	"Call",
 	"CallNextEventHandler",
 	"CancelMenuTracking",
+	"ChangeWindowAttributes",
 	"CharWidth",
 	"ClearCurrentScrap",
 	"ClearKeyboardFocus",
@@ -264,6 +275,10 @@ char * OS_nativeFunctionNames[] = {
 	"CreateWindowGroup",
 	"DMGetFirstScreenDevice",
 	"DMGetNextScreenDevice",
+	"DataBrowserChangeAttributes",
+	"DataBrowserGetAttributes",
+	"DataBrowserGetMetric",
+	"DataBrowserSetMetric",
 	"DeleteGlobalRef",
 	"DeleteMenu",
 	"DeleteMenuItem",
@@ -337,7 +352,9 @@ char * OS_nativeFunctionNames[] = {
 	"GetControl32BitMaximum",
 	"GetControl32BitMinimum",
 	"GetControl32BitValue",
+	"GetControlAction",
 	"GetControlBounds",
+	"GetControlData__ISIILorg_eclipse_swt_internal_carbon_ControlEditTextSelectionRec_2_3I",
 	"GetControlData__ISIILorg_eclipse_swt_internal_carbon_ControlFontStyleRec_2_3I",
 	"GetControlData__ISIILorg_eclipse_swt_internal_carbon_Rect_2_3I",
 	"GetControlData__ISII_3B_3I",
@@ -345,6 +362,7 @@ char * OS_nativeFunctionNames[] = {
 	"GetControlData__ISII_3S_3I",
 	"GetControlEventTarget",
 	"GetControlFeatures",
+	"GetControlKind",
 	"GetControlOwner",
 	"GetControlProperty",
 	"GetControlReference",
@@ -398,6 +416,7 @@ char * OS_nativeFunctionNames[] = {
 	"GetEventParameter__III_3II_3I_3C",
 	"GetEventParameter__III_3II_3I_3I",
 	"GetEventParameter__III_3II_3I_3S",
+	"GetEventParameter__III_3II_3I_3Z",
 	"GetEventTime",
 	"GetFlavorData",
 	"GetFlavorDataSize",
@@ -430,6 +449,7 @@ char * OS_nativeFunctionNames[] = {
 	"GetMenuWidth",
 	"GetMouse",
 	"GetNextDevice",
+	"GetNextWindow",
 	"GetPixBounds",
 	"GetPixDepth",
 	"GetPort",
@@ -451,6 +471,7 @@ char * OS_nativeFunctionNames[] = {
 	"GetTabContentRect",
 	"GetThemeBrushAsColor",
 	"GetThemeButtonContentBounds",
+	"GetThemeButtonRegion",
 	"GetThemeDrawingState",
 	"GetThemeFont",
 	"GetThemeMenuItemExtra",
@@ -462,10 +483,12 @@ char * OS_nativeFunctionNames[] = {
 	"GetWRefCon",
 	"GetWindowActivationScope",
 	"GetWindowBounds",
+	"GetWindowClass",
 	"GetWindowDefaultButton",
 	"GetWindowEventTarget",
 	"GetWindowFromPort",
 	"GetWindowGroupOfClass",
+	"GetWindowList",
 	"GetWindowModality",
 	"GetWindowPort",
 	"GetWindowRegion",
@@ -477,14 +500,50 @@ char * OS_nativeFunctionNames[] = {
 	"HIComboBoxGetItemCount",
 	"HIComboBoxInsertTextItemAtIndex",
 	"HIComboBoxRemoveItemAtIndex",
+	"HICopyAccessibilityRoleDescription",
+	"HICreateTransformedCGImage",
 	"HIObjectCopyClassID",
 	"HIObjectCreate",
 	"HIObjectRegisterSubclass",
+	"HIScrollViewCreate",
+	"HITextViewCreate",
+	"HITextViewGetTXNObject",
+	"HIThemeDrawBackground",
+	"HIThemeDrawButton",
+	"HIThemeDrawFocusRect",
+	"HIThemeDrawFrame",
+	"HIThemeDrawGenericWell",
+	"HIThemeDrawGroupBox",
+	"HIThemeDrawGrowBox",
+	"HIThemeDrawPopupArrow",
+	"HIThemeDrawSeparator",
+	"HIThemeDrawTab",
+	"HIThemeDrawTabPane",
+	"HIThemeDrawTextBox",
+	"HIThemeDrawTrack",
+	"HIThemeGetButtonBackgroundBounds",
+	"HIThemeGetButtonContentBounds",
+	"HIThemeGetScrollBarTrackRect",
+	"HIThemeGetTextDimensions",
+	"HIThemeGetTrackBounds",
+	"HIThemeGetTrackLiveValue",
+	"HIThemeGetTrackPartBounds",
+	"HIThemeGetTrackThumbPositionFromBounds",
+	"HIThemeGetTrackThumbPositionFromOffset",
+	"HIThemeHitTestScrollBarArrows",
+	"HIThemeHitTestTrack",
+	"HIThemeSetFill",
 	"HIViewAddSubview",
+	"HIViewChangeAttributes",
+	"HIViewChangeFeatures",
 	"HIViewClick",
 	"HIViewConvertPoint",
+	"HIViewConvertRect",
+	"HIViewConvertRegion",
 	"HIViewCreateOffscreenImage",
 	"HIViewFindByID",
+	"HIViewGetBounds",
+	"HIViewGetFeatures",
 	"HIViewGetFirstSubview",
 	"HIViewGetFrame",
 	"HIViewGetLastSubview",
@@ -492,9 +551,13 @@ char * OS_nativeFunctionNames[] = {
 	"HIViewGetRoot",
 	"HIViewGetSizeConstraints",
 	"HIViewGetSubviewHit",
+	"HIViewGetSuperview",
 	"HIViewGetViewForMouseEvent",
+	"HIViewIsDrawingEnabled",
 	"HIViewIsVisible",
 	"HIViewRemoveFromSuperview",
+	"HIViewRender",
+	"HIViewScrollRect",
 	"HIViewSetBoundsOrigin",
 	"HIViewSetDrawingEnabled",
 	"HIViewSetFrame",
@@ -504,6 +567,7 @@ char * OS_nativeFunctionNames[] = {
 	"HIViewSetZOrder",
 	"HIViewSimulateClick",
 	"HLock",
+	"HMDisplayTag",
 	"HMGetTagDelay",
 	"HMHideTag",
 	"HMInstallControlContentCallback",
@@ -561,6 +625,7 @@ char * OS_nativeFunctionNames[] = {
 	"NavDialogGetSaveFileName",
 	"NavDialogGetUserAction",
 	"NavDialogRun",
+	"NavDialogSetFilterTypeIdentifiers",
 	"NavDialogSetSaveFileName",
 	"NavGetDefaultDialogCreationOptions",
 	"NewControl",
@@ -687,6 +752,7 @@ char * OS_nativeFunctionNames[] = {
 	"SetControlColorProc",
 	"SetControlData__IIIII",
 	"SetControlData__IIIILorg_eclipse_swt_internal_carbon_ControlButtonContentInfo_2",
+	"SetControlData__IIIILorg_eclipse_swt_internal_carbon_ControlEditTextSelectionRec_2",
 	"SetControlData__IIIILorg_eclipse_swt_internal_carbon_ControlTabInfoRecV1_2",
 	"SetControlData__IIIILorg_eclipse_swt_internal_carbon_Rect_2",
 	"SetControlData__IIII_3B",
@@ -725,13 +791,17 @@ char * OS_nativeFunctionNames[] = {
 	"SetDataBrowserTarget",
 	"SetDragAllowableActions",
 	"SetDragDropAction",
+	"SetDragImageWithCGImage",
 	"SetDragInputProc",
 	"SetDragItemFlavorData",
 	"SetDragSendProc",
 	"SetEventLoopTimerNextFireTime",
+	"SetEventParameter__IIIILorg_eclipse_swt_internal_carbon_CGPoint_2",
+	"SetEventParameter__IIIILorg_eclipse_swt_internal_carbon_HICommand_2",
 	"SetEventParameter__IIII_3C",
 	"SetEventParameter__IIII_3I",
 	"SetEventParameter__IIII_3S",
+	"SetEventParameter__IIII_3Z",
 	"SetFontInfoForSelection",
 	"SetFrontProcess",
 	"SetFrontProcessWithOptions",
@@ -790,6 +860,7 @@ char * OS_nativeFunctionNames[] = {
 	"TXNEchoMode",
 	"TXNFocus",
 	"TXNGetData",
+	"TXNGetHIRect",
 	"TXNGetLineCount",
 	"TXNGetLineMetrics",
 	"TXNGetRectBounds",
@@ -818,6 +889,7 @@ char * OS_nativeFunctionNames[] = {
 	"TextWidth",
 	"TrackDrag",
 	"TrackMouseLocationWithOptions",
+	"UTTypeCreatePreferredIdentifierForTag",
 	"UnionRect",
 	"UnionRgn",
 	"UnlockPortBits",
@@ -827,12 +899,14 @@ char * OS_nativeFunctionNames[] = {
 	"WaitMouseMoved",
 	"X2Fix",
 	"ZoomWindowIdeal",
+	"_1_1BIG_1ENDIAN_1_1",
 	"getenv",
 	"getpid",
 	"kCFNumberFormatterDecimalSeparator",
 	"kHIViewWindowContentID",
 	"kPMDocumentFormatPDF",
 	"kPMGraphicsContextCoreGraphics",
+	"kUTTagClassFilenameExtension",
 	"memcpy__III",
 	"memcpy__ILorg_eclipse_swt_internal_carbon_ATSUTab_2I",
 	"memcpy__ILorg_eclipse_swt_internal_carbon_BitMap_2I",
@@ -861,6 +935,7 @@ char * OS_nativeFunctionNames[] = {
 	"memcpy___3C_3BI",
 	"memcpy___3FII",
 	"memcpy___3III",
+	"memcpy___3ILorg_eclipse_swt_internal_carbon_TXNTab_2I",
 	"memset",
 	"strlen",
 };
