@@ -1,13 +1,13 @@
 /*******************************************************************************
-* Copyright (c) 2000, 2005 IBM Corporation and others.
-* All rights reserved. This program and the accompanying materials
-* are made available under the terms of the Eclipse Public License v1.0
-* which accompanies this distribution, and is available at
-* http://www.eclipse.org/legal/epl-v10.html
-*
-* Contributors:
-*     IBM Corporation - initial API and implementation
-*******************************************************************************/
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors:
+ *    IBM Corporation - initial API and implementation
+ *******************************************************************************/
 
 #ifdef NATIVE_STATS
 extern int OS_nativeFunctionCount;
@@ -23,6 +23,7 @@ extern char* OS_nativeFunctionNames[];
 typedef enum {
 	AECountItems_FUNC,
 	AEGetNthPtr_FUNC,
+	AEInstallEventHandler_FUNC,
 	AEProcessAppleEvent_FUNC,
 	ATSFontGetPostScriptName_FUNC,
 	ATSUBatchBreakLines_FUNC,
@@ -62,6 +63,10 @@ typedef enum {
 	ATSUSetTransientFontMatching_FUNC,
 	ATSUTextDeleted_FUNC,
 	ATSUTextInserted_FUNC,
+	AXNotificationHIObjectNotify_FUNC,
+	AXUIElementCreateWithHIObjectAndIdentifier_FUNC,
+	AXUIElementGetIdentifier_FUNC,
+	AXValueCreate_FUNC,
 	ActiveNonFloatingWindow_FUNC,
 	AddDataBrowserItems_FUNC,
 	AddDataBrowserListViewColumn_FUNC,
@@ -76,12 +81,14 @@ typedef enum {
 	CFArrayGetValueAtIndex_FUNC,
 	CFDataGetBytes_FUNC,
 	CFDataGetLength_FUNC,
+	CFEqual_FUNC,
 	CFLocaleCopyCurrent_FUNC,
 	CFNumberFormatterCopyProperty_FUNC,
 	CFNumberFormatterCreate_FUNC,
 	CFRelease_FUNC,
 	CFStringCreateWithBytes_FUNC,
-	CFStringCreateWithCharacters_FUNC,
+	CFStringCreateWithCharacters__III_FUNC,
+	CFStringCreateWithCharacters__I_3CI_FUNC,
 	CFStringGetBytes_FUNC,
 	CFStringGetCharacters_FUNC,
 	CFStringGetLength_FUNC,
@@ -179,6 +186,7 @@ typedef enum {
 	CGFunctionRelease_FUNC,
 	CGGetDisplaysWithRect_FUNC,
 	CGImageCreate_FUNC,
+	CGImageCreateWithImageInRect_FUNC,
 	CGImageCreateWithJPEGDataProvider_FUNC,
 	CGImageCreateWithPNGDataProvider_FUNC,
 	CGImageGetAlphaInfo_FUNC,
@@ -208,6 +216,7 @@ typedef enum {
 	CGPointApplyAffineTransform_FUNC,
 	CGPostKeyboardEvent_FUNC,
 	CGPostMouseEvent_FUNC,
+	CGRectContainsPoint_FUNC,
 	CGShadingCreateAxial_FUNC,
 	CGShadingCreateRadial_FUNC,
 	CGShadingRelease_FUNC,
@@ -215,8 +224,10 @@ typedef enum {
 	CPSEnableForegroundOperation_FUNC,
 	CPSSetProcessName_FUNC,
 	CalcMenuSize_FUNC,
+	Call_FUNC,
 	CallNextEventHandler_FUNC,
 	CancelMenuTracking_FUNC,
+	ChangeWindowAttributes_FUNC,
 	CharWidth_FUNC,
 	ClearCurrentScrap_FUNC,
 	ClearKeyboardFocus_FUNC,
@@ -268,6 +279,10 @@ typedef enum {
 	CreateWindowGroup_FUNC,
 	DMGetFirstScreenDevice_FUNC,
 	DMGetNextScreenDevice_FUNC,
+	DataBrowserChangeAttributes_FUNC,
+	DataBrowserGetAttributes_FUNC,
+	DataBrowserGetMetric_FUNC,
+	DataBrowserSetMetric_FUNC,
 	DeleteGlobalRef_FUNC,
 	DeleteMenu_FUNC,
 	DeleteMenuItem_FUNC,
@@ -341,7 +356,9 @@ typedef enum {
 	GetControl32BitMaximum_FUNC,
 	GetControl32BitMinimum_FUNC,
 	GetControl32BitValue_FUNC,
+	GetControlAction_FUNC,
 	GetControlBounds_FUNC,
+	GetControlData__ISIILorg_eclipse_swt_internal_carbon_ControlEditTextSelectionRec_2_3I_FUNC,
 	GetControlData__ISIILorg_eclipse_swt_internal_carbon_ControlFontStyleRec_2_3I_FUNC,
 	GetControlData__ISIILorg_eclipse_swt_internal_carbon_Rect_2_3I_FUNC,
 	GetControlData__ISII_3B_3I_FUNC,
@@ -349,6 +366,7 @@ typedef enum {
 	GetControlData__ISII_3S_3I_FUNC,
 	GetControlEventTarget_FUNC,
 	GetControlFeatures_FUNC,
+	GetControlKind_FUNC,
 	GetControlOwner_FUNC,
 	GetControlProperty_FUNC,
 	GetControlReference_FUNC,
@@ -402,6 +420,7 @@ typedef enum {
 	GetEventParameter__III_3II_3I_3C_FUNC,
 	GetEventParameter__III_3II_3I_3I_FUNC,
 	GetEventParameter__III_3II_3I_3S_FUNC,
+	GetEventParameter__III_3II_3I_3Z_FUNC,
 	GetEventTime_FUNC,
 	GetFlavorData_FUNC,
 	GetFlavorDataSize_FUNC,
@@ -434,6 +453,7 @@ typedef enum {
 	GetMenuWidth_FUNC,
 	GetMouse_FUNC,
 	GetNextDevice_FUNC,
+	GetNextWindow_FUNC,
 	GetPixBounds_FUNC,
 	GetPixDepth_FUNC,
 	GetPort_FUNC,
@@ -455,6 +475,7 @@ typedef enum {
 	GetTabContentRect_FUNC,
 	GetThemeBrushAsColor_FUNC,
 	GetThemeButtonContentBounds_FUNC,
+	GetThemeButtonRegion_FUNC,
 	GetThemeDrawingState_FUNC,
 	GetThemeFont_FUNC,
 	GetThemeMenuItemExtra_FUNC,
@@ -466,10 +487,12 @@ typedef enum {
 	GetWRefCon_FUNC,
 	GetWindowActivationScope_FUNC,
 	GetWindowBounds_FUNC,
+	GetWindowClass_FUNC,
 	GetWindowDefaultButton_FUNC,
 	GetWindowEventTarget_FUNC,
 	GetWindowFromPort_FUNC,
 	GetWindowGroupOfClass_FUNC,
+	GetWindowList_FUNC,
 	GetWindowModality_FUNC,
 	GetWindowPort_FUNC,
 	GetWindowRegion_FUNC,
@@ -481,14 +504,50 @@ typedef enum {
 	HIComboBoxGetItemCount_FUNC,
 	HIComboBoxInsertTextItemAtIndex_FUNC,
 	HIComboBoxRemoveItemAtIndex_FUNC,
+	HICopyAccessibilityRoleDescription_FUNC,
+	HICreateTransformedCGImage_FUNC,
 	HIObjectCopyClassID_FUNC,
 	HIObjectCreate_FUNC,
 	HIObjectRegisterSubclass_FUNC,
+	HIScrollViewCreate_FUNC,
+	HITextViewCreate_FUNC,
+	HITextViewGetTXNObject_FUNC,
+	HIThemeDrawBackground_FUNC,
+	HIThemeDrawButton_FUNC,
+	HIThemeDrawFocusRect_FUNC,
+	HIThemeDrawFrame_FUNC,
+	HIThemeDrawGenericWell_FUNC,
+	HIThemeDrawGroupBox_FUNC,
+	HIThemeDrawGrowBox_FUNC,
+	HIThemeDrawPopupArrow_FUNC,
+	HIThemeDrawSeparator_FUNC,
+	HIThemeDrawTab_FUNC,
+	HIThemeDrawTabPane_FUNC,
+	HIThemeDrawTextBox_FUNC,
+	HIThemeDrawTrack_FUNC,
+	HIThemeGetButtonBackgroundBounds_FUNC,
+	HIThemeGetButtonContentBounds_FUNC,
+	HIThemeGetScrollBarTrackRect_FUNC,
+	HIThemeGetTextDimensions_FUNC,
+	HIThemeGetTrackBounds_FUNC,
+	HIThemeGetTrackLiveValue_FUNC,
+	HIThemeGetTrackPartBounds_FUNC,
+	HIThemeGetTrackThumbPositionFromBounds_FUNC,
+	HIThemeGetTrackThumbPositionFromOffset_FUNC,
+	HIThemeHitTestScrollBarArrows_FUNC,
+	HIThemeHitTestTrack_FUNC,
+	HIThemeSetFill_FUNC,
 	HIViewAddSubview_FUNC,
+	HIViewChangeAttributes_FUNC,
+	HIViewChangeFeatures_FUNC,
 	HIViewClick_FUNC,
 	HIViewConvertPoint_FUNC,
+	HIViewConvertRect_FUNC,
+	HIViewConvertRegion_FUNC,
 	HIViewCreateOffscreenImage_FUNC,
 	HIViewFindByID_FUNC,
+	HIViewGetBounds_FUNC,
+	HIViewGetFeatures_FUNC,
 	HIViewGetFirstSubview_FUNC,
 	HIViewGetFrame_FUNC,
 	HIViewGetLastSubview_FUNC,
@@ -496,9 +555,13 @@ typedef enum {
 	HIViewGetRoot_FUNC,
 	HIViewGetSizeConstraints_FUNC,
 	HIViewGetSubviewHit_FUNC,
+	HIViewGetSuperview_FUNC,
 	HIViewGetViewForMouseEvent_FUNC,
+	HIViewIsDrawingEnabled_FUNC,
 	HIViewIsVisible_FUNC,
 	HIViewRemoveFromSuperview_FUNC,
+	HIViewRender_FUNC,
+	HIViewScrollRect_FUNC,
 	HIViewSetBoundsOrigin_FUNC,
 	HIViewSetDrawingEnabled_FUNC,
 	HIViewSetFrame_FUNC,
@@ -508,6 +571,7 @@ typedef enum {
 	HIViewSetZOrder_FUNC,
 	HIViewSimulateClick_FUNC,
 	HLock_FUNC,
+	HMDisplayTag_FUNC,
 	HMGetTagDelay_FUNC,
 	HMHideTag_FUNC,
 	HMInstallControlContentCallback_FUNC,
@@ -565,6 +629,7 @@ typedef enum {
 	NavDialogGetSaveFileName_FUNC,
 	NavDialogGetUserAction_FUNC,
 	NavDialogRun_FUNC,
+	NavDialogSetFilterTypeIdentifiers_FUNC,
 	NavDialogSetSaveFileName_FUNC,
 	NavGetDefaultDialogCreationOptions_FUNC,
 	NewControl_FUNC,
@@ -691,6 +756,7 @@ typedef enum {
 	SetControlColorProc_FUNC,
 	SetControlData__IIIII_FUNC,
 	SetControlData__IIIILorg_eclipse_swt_internal_carbon_ControlButtonContentInfo_2_FUNC,
+	SetControlData__IIIILorg_eclipse_swt_internal_carbon_ControlEditTextSelectionRec_2_FUNC,
 	SetControlData__IIIILorg_eclipse_swt_internal_carbon_ControlTabInfoRecV1_2_FUNC,
 	SetControlData__IIIILorg_eclipse_swt_internal_carbon_Rect_2_FUNC,
 	SetControlData__IIII_3B_FUNC,
@@ -729,13 +795,17 @@ typedef enum {
 	SetDataBrowserTarget_FUNC,
 	SetDragAllowableActions_FUNC,
 	SetDragDropAction_FUNC,
+	SetDragImageWithCGImage_FUNC,
 	SetDragInputProc_FUNC,
 	SetDragItemFlavorData_FUNC,
 	SetDragSendProc_FUNC,
 	SetEventLoopTimerNextFireTime_FUNC,
+	SetEventParameter__IIIILorg_eclipse_swt_internal_carbon_CGPoint_2_FUNC,
+	SetEventParameter__IIIILorg_eclipse_swt_internal_carbon_HICommand_2_FUNC,
 	SetEventParameter__IIII_3C_FUNC,
 	SetEventParameter__IIII_3I_FUNC,
 	SetEventParameter__IIII_3S_FUNC,
+	SetEventParameter__IIII_3Z_FUNC,
 	SetFontInfoForSelection_FUNC,
 	SetFrontProcess_FUNC,
 	SetFrontProcessWithOptions_FUNC,
@@ -794,6 +864,7 @@ typedef enum {
 	TXNEchoMode_FUNC,
 	TXNFocus_FUNC,
 	TXNGetData_FUNC,
+	TXNGetHIRect_FUNC,
 	TXNGetLineCount_FUNC,
 	TXNGetLineMetrics_FUNC,
 	TXNGetRectBounds_FUNC,
@@ -822,6 +893,7 @@ typedef enum {
 	TextWidth_FUNC,
 	TrackDrag_FUNC,
 	TrackMouseLocationWithOptions_FUNC,
+	UTTypeCreatePreferredIdentifierForTag_FUNC,
 	UnionRect_FUNC,
 	UnionRgn_FUNC,
 	UnlockPortBits_FUNC,
@@ -831,12 +903,14 @@ typedef enum {
 	WaitMouseMoved_FUNC,
 	X2Fix_FUNC,
 	ZoomWindowIdeal_FUNC,
+	_1_1BIG_1ENDIAN_1_1_FUNC,
 	getenv_FUNC,
 	getpid_FUNC,
 	kCFNumberFormatterDecimalSeparator_FUNC,
 	kHIViewWindowContentID_FUNC,
 	kPMDocumentFormatPDF_FUNC,
 	kPMGraphicsContextCoreGraphics_FUNC,
+	kUTTagClassFilenameExtension_FUNC,
 	memcpy__III_FUNC,
 	memcpy__ILorg_eclipse_swt_internal_carbon_ATSUTab_2I_FUNC,
 	memcpy__ILorg_eclipse_swt_internal_carbon_BitMap_2I_FUNC,
@@ -865,6 +939,7 @@ typedef enum {
 	memcpy___3C_3BI_FUNC,
 	memcpy___3FII_FUNC,
 	memcpy___3III_FUNC,
+	memcpy___3ILorg_eclipse_swt_internal_carbon_TXNTab_2I_FUNC,
 	memset_FUNC,
 	strlen_FUNC,
 } OS_FUNCS;
