@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others. All rights reserved.
+ * Copyright (c) 2000, 2006 IBM Corporation and others. All rights reserved.
  * The contents of this file are made available under the terms
  * of the GNU Lesser General Public License (LGPL) Version 2.1 that
  * accompanies this distribution (lgpl-v21.txt).  The LGPL is also
@@ -8,14 +8,17 @@
  * the LGPL accompanying this distribution and there is any conflict
  * between the two license versions, the terms of the LGPL accompanying
  * this distribution shall govern.
+ * 
+ * Contributors:
+ *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.swt.internal.gtk;
 
 
 public class GdkEventCrossing extends GdkEvent {
-	public int window;
+	public int /*long*/ window;
 	public byte send_event;
-	public int subwindow;
+	public int /*long*/ subwindow;
 	public int time;
 	public double x;
 	public double y;
@@ -25,5 +28,5 @@ public class GdkEventCrossing extends GdkEvent {
 	public int detail;
 	public boolean focus;
 	public int state;
-	public static final int sizeof = 68;
+	public static final int sizeof = OS.GdkEventCrossing_sizeof();
 }
