@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others. All rights reserved.
+ * Copyright (c) 2000, 2006 IBM Corporation and others. All rights reserved.
  * The contents of this file are made available under the terms
  * of the GNU Lesser General Public License (LGPL) Version 2.1 that
  * accompanies this distribution (lgpl-v21.txt).  The LGPL is also
@@ -8,19 +8,22 @@
  * the LGPL accompanying this distribution and there is any conflict
  * between the two license versions, the terms of the LGPL accompanying
  * this distribution shall govern.
+ * 
+ * Contributors:
+ *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.swt.internal.gtk;
 
 
 public class GdkEventKey extends GdkEvent {
-	public int window;
+	public int /*long*/ window;
 	public byte send_event;
 	public int time;
 	public int state;
 	public int keyval;
 	public int length;
-	public int string;
+	public int /*long*/ string;
 	public short hardware_keycode;
     public byte group;
-   	public static final int sizeof = 36;
+   	public static final int sizeof = OS.GdkEventKey_sizeof();
 }
