@@ -1,11 +1,11 @@
 #!/bin/sh
 #*******************************************************************************
-# Copyright (c) 2000, 2005 IBM Corporation and others.
+# Copyright (c) 2000, 2006 IBM Corporation and others.
 # All rights reserved. This program and the accompanying materials
 # are made available under the terms of the Eclipse Public License v1.0
 # which accompanies this distribution, and is available at
 # http://www.eclipse.org/legal/epl-v10.html
-# 
+#
 # Contributors:
 #     IBM Corporation - initial API and implementation
 #     Kevin Cornell (Rational Software Corporation)
@@ -51,9 +51,21 @@ case $OS in
 				MOTIF_HOME=/bluebird/teamswt/swt-builddir/motif21
 			fi
 			if [ "${GECKO_SDK}" = "" ]; then
-				GECKO_SDK=/mozilla/mozilla/1.4/linux_gtk2/mozilla/dist/sdk
-				GECKO_INCLUDES="-include ${GECKO_SDK}/mozilla-config.h -I${GECKO_SDK}/nspr/include -I${GECKO_SDK}/xpcom/include -I${GECKO_SDK}/string/include -I${GECKO_SDK}/embed_base/include -I${GECKO_SDK}/embedstring/include"
-				GECKO_LIBS="-L${GECKO_SDK}/embedstring/bin -lembedstring -L${GECKO_SDK}/embed_base/bin -lembed_base_s -L${GECKO_SDK}/xpcom/bin -lxpcomglue_s -lxpcom -L${GECKO_SDK}/nspr/bin -lnspr4 -lplds4 -lplc4"
+				GECKO_SDK=/bluebird/teamswt/swt-builddir/mozilla/1.4/linux_gtk2/mozilla/dist/sdk
+				GECKO_INCLUDES="-include ${GECKO_SDK}/mozilla-config.h -I${GECKO_SDK}/../include/xpcom -I${GECKO_SDK}/../include/nspr -I${GECKO_SDK}/../include/embed_base -I${GECKO_SDK}/../include/embedstring -I${GECKO_SDK}/../include/string -I${GECKO_SDK}/../include/profdirserviceprovider"
+				GECKO_LIBS="${GECKO_SDK}/../lib/libembed_base_s.a ${GECKO_SDK}/../lib/libprofdirserviceprovider_s.a ${GECKO_SDK}/../lib/libembedstring.a ${GECKO_SDK}/../lib/libxpcomglue_s.a -L${GECKO_SDK}/../bin -L${GECKO_SDK}/../lib/ -lxpcom -lnspr4 -lplds4 -lplc4"
+				PROFILE14_INCLUDES="-include ${GECKO_SDK}/mozilla-config.h -I${GECKO_SDK}/../include/xpcom -I${GECKO_SDK}/../include/nspr -I${GECKO_SDK}/../include/string -I${GECKO_SDK}/../include/profdirserviceprovider"
+				PROFILE14_LIBS="${GECKO_SDK}/../lib/libembed_base_s.a ${GECKO_SDK}/../lib/libprofdirserviceprovider_s.a -L${GECKO_SDK}/../lib/ -lxpcom"
+			fi
+			if [ "${GECKO17_SDK}" = "" ]; then
+				GECKO17_SDK=/bluebird/teamswt/swt-builddir/mozilla/1.7/linux_gtk2/mozilla/dist/sdk
+				PROFILE17_INCLUDES="-include ${GECKO17_SDK}/include/mozilla-config.h -I${GECKO17_SDK}/../include/xpcom -I${GECKO17_SDK}/../include/nspr -I${GECKO17_SDK}/../include/string -I${GECKO17_SDK}/../include/profdirserviceprovider"
+				PROFILE17_LIBS="${GECKO17_SDK}/../lib/libembed_base_s.a ${GECKO17_SDK}/../lib/libprofdirserviceprovider_s.a -L${GECKO17_SDK}/../lib/ -lxpcom"
+			fi
+			if [ "${GECKO18_SDK}" = "" ]; then
+				GECKO18_SDK=/bluebird/teamswt/swt-builddir/mozilla/1.8/linux_gtk2/mozilla/dist/sdk
+				PROFILE18_INCLUDES="-include ${GECKO18_SDK}/include/mozilla-config.h -I${GECKO18_SDK}/../include/xpcom -I${GECKO18_SDK}/../include/nspr -I${GECKO18_SDK}/../include/string -I${GECKO18_SDK}/../include/profdirserviceprovider"
+				PROFILE18_LIBS="${GECKO18_SDK}/../lib/libembed_base_s.a ${GECKO18_SDK}/../lib/libprofdirserviceprovider_s.a -L${GECKO18_SDK}/../lib/ -lxpcom"
 			fi
 			OUTPUT_DIR=../../../org.eclipse.swt.motif.linux.x86
 			makefile="make_linux.mak"
@@ -117,6 +129,6 @@ case $OS in
 		;;
 esac
 
-export JAVA_HOME MOTIF_HOME CDE_HOME GECKO_SDK GECKO_INCLUDES GECKO_LIBS OUTPUT_DIR
+export JAVA_HOME MOTIF_HOME CDE_HOME GECKO_SDK GECKO17_SDK GECKO18_SDK GECKO_INCLUDES GECKO_LIBS OUTPUT_DIR PROFILE14_INCLUDES PROFILE17_INCLUDES PROFILE18_INCLUDES PROFILE14_LIBS PROFILE17_LIBS PROFILE18_LIBS
 
-make -f $makefile $1 $2 $3 $4
\ No newline at end of file
+make -f $makefile $1 $2 $3 $4
