@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -23,6 +23,8 @@ public class Gdip extends Platform {
 	public static final int BrushTypeTextureFill = 2;
 	public static final int BrushTypePathGradient = 3;
 	public static final int BrushTypeLinearGradient = 4;
+	public static final int ColorAdjustTypeBitmap = 1;
+	public static final int ColorMatrixFlagsDefault = 0;
 	public static final int CombineModeReplace = 0;
 	public static final int CombineModeIntersect = 1;
 	public static final int CombineModeUnion = 2;
@@ -46,6 +48,7 @@ public class Gdip extends Platform {
     public static final int FontStyleBoldItalic = 3;
     public static final int FontStyleUnderline = 4;
     public static final int FontStyleStrikeout = 8;
+    public static final int PaletteFlagsHasAlpha = 0x0001;
     public static final int FlushIntentionFlush = 0;
     public static final int FlushIntentionSync = 1;
     public static final int HotkeyPrefixNone = 0;
@@ -117,7 +120,6 @@ public class Gdip extends Platform {
     public static final int StringFormatFlagsNoWrap = 0x00001000;
     public static final int StringFormatFlagsLineLimit = 0x00002000;
     public static final int StringFormatFlagsNoClip = 0x00004000;
-
     public static final int TextRenderingHintSystemDefault = 0;
     public static final int TextRenderingHintSingleBitPerPixelGridFit = 1;
     public static final int TextRenderingHintSingleBitPerPixel = 2;
@@ -138,7 +140,14 @@ public static final native void GdiplusShutdown(int[] token);
 public static final native int Bitmap_new(int hbm, int hpal);
 public static final native int Bitmap_new(int hicon);
 public static final native int Bitmap_new(int width, int height, int stride, int format, int scan0);
+public static final native int Bitmap_new(char[] filename, boolean useIcm);
 public static final native void Bitmap_delete(int bitmap);
+public static final native int Bitmap_GetHBITMAP(int bitmap, int colorBackground, int[] hbmReturn);
+public static final native int Bitmap_GetHICON(int bitmap, int[] hicon);
+public static final native int BitmapData_new();
+public static final native void BitmapData_delete(int bitmapData);
+public static final native int Bitmap_LockBits(int bitmap, int rect, int flags, int pixelFormat, int lockedBitmapData);
+public static final native int Bitmap_UnlockBits(int bitmap, int lockedBitmapData);
 public static final native int Brush_Clone(int brush);
 public static final native int Brush_GetType(int brush);
 public static final native int Color_new(int argb);
@@ -181,18 +190,24 @@ public static final native int Graphics_GetInterpolationMode(int graphics);
 public static final native int Graphics_GetSmoothingMode(int graphics);
 public static final native int Graphics_GetTextRenderingHint(int graphics);
 public static final native int Graphics_GetTransform(int graphics, int matrix);
+public static final native int Graphics_GetVisibleClipBounds(int graphics, Rect rect);
 public static final native int Graphics_MeasureString(int graphics, char[] string, int length, int font, PointF origin, RectF boundingBox);
 public static final native int Graphics_MeasureString(int graphics, char[] string, int length, int font, PointF origin, int format, RectF boundingBox);
 public static final native int Graphics_ResetClip(int graphics);
+public static final native int Graphics_Restore(int graphics, int gstate);
+public static final native int Graphics_Save(int graphics);
+public static final native int Graphics_ScaleTransform(int graphics, float sx, float sy, int order);
 public static final native int Graphics_SetClip(int graphics, int hrgn, int combineMode);
 public static final native int Graphics_SetClip(int graphics, int path);
 public static final native int Graphics_SetClip(int graphics, RectF rect);
 public static final native int Graphics_SetCompositingQuality(int graphics, int compositingQuality);
+public static final native int Graphics_SetPageUnit(int graphics, int unit);
 public static final native int Graphics_SetPixelOffsetMode(int graphics, int pixelOffsetMode);
 public static final native int Graphics_SetSmoothingMode(int graphics, int smoothingMode);
 public static final native int Graphics_SetTransform(int graphics, int matrix);
 public static final native int Graphics_SetInterpolationMode(int graphics, int mode);
 public static final native int Graphics_SetTextRenderingHint(int graphics, int mode);
+public static final native int Graphics_TranslateTransform(int graphics, float dx, float dy, int order);
 public static final native int GraphicsPath_new(int brushMode);
 public static final native void GraphicsPath_delete(int path);
 public static final native int GraphicsPath_AddArc(int path, float x, float y, float width, float height, float startAngle, float sweepAngle);
@@ -202,6 +217,7 @@ public static final native int GraphicsPath_AddPath(int path, int addingPath, bo
 public static final native int GraphicsPath_AddRectangle(int path, RectF rect);
 public static final native int GraphicsPath_AddString(int path, char[] string, int length, int family, int style, float emSize, PointF origin, int format);
 public static final native int GraphicsPath_CloseFigure(int path);
+public static final native int GraphicsPath_Flatten(int path, int matrix, float flatness);
 public static final native int GraphicsPath_GetBounds(int path, RectF bounds, int matrix, int pen);
 public static final native int GraphicsPath_GetLastPoint(int path, PointF lastPoint);
 public static final native int GraphicsPath_GetPathPoints(int path, float[] points, int count);
@@ -210,12 +226,23 @@ public static final native int GraphicsPath_GetPointCount(int path);
 public static final native boolean GraphicsPath_IsOutlineVisible(int path, float x, float y, int pen, int g);
 public static final native boolean GraphicsPath_IsVisible(int path, float x, float y, int g);
 public static final native int GraphicsPath_SetFillMode(int path, int fillmode);
+public static final native int GraphicsPath_StartFigure(int path);
+public static final native int GraphicsPath_Transform(int path, int matrix);
 public static final native int HatchBrush_new(int hatchStyle, int foreColor, int backColor);
+public static final native int Image_GetLastStatus(int image);
+public static final native int Image_GetPixelFormat(int bitmap);
 public static final native int Image_GetWidth(int image);
 public static final native int Image_GetHeight(int image);
+public static final native int Image_GetPalette(int image, int palette, int size);
+public static final native int Image_GetPaletteSize(int image);
+public static final native int ImageAttributes_new();
+public static final native void ImageAttributes_delete(int attrib);
+public static final native int ImageAttributes_SetWrapMode(int attrib, int wrap);
+public static final native int ImageAttributes_SetColorMatrix(int attrib, float[] matrix, int mode, int type);
 public static final native void HatchBrush_delete(int brush);
 public static final native int LinearGradientBrush_new(PointF point1, PointF point2, int color1, int color2);
 public static final native void LinearGradientBrush_delete(int brush);
+public static final native int LinearGradientBrush_SetInterpolationColors(int brush, int[] presetColors, float[] blendPositions, int count);
 public static final native int LinearGradientBrush_SetWrapMode(int brush, int wrapMode);
 public static final native int Matrix_new(float m11, float m12, float m21, float m22, float dx, float dy);
 public static final native void Matrix_delete(int matrix);
@@ -230,6 +257,8 @@ public static final native int Matrix_TransformPoints(int matrix, PointF pts, in
 public static final native int Matrix_TransformPoints(int matrix, float[] pts, int count);
 public static final native int Matrix_Translate(int matrix, float offsetX, float offsetY, int order);
 public static final native int Matrix_SetElements(int matrix, float m11, float m12, float m21, float m22, float dx, float dy);
+public static final native void MoveMemory(ColorPalette Destination, int SourcePtr, int Length);
+public static final native void MoveMemory(BitmapData Destination, int SourcePtr, int Length);
 public static final native int PathGradientBrush_new(int path);
 public static final native void PathGradientBrush_delete(int brush);
 public static final native int PathGradientBrush_SetCenterColor(int brush, int color);
@@ -250,6 +279,7 @@ public static final native int Region_new(int hRgn);
 public static final native int Region_new();
 public static final native void Region_delete(int region);
 public static final native int Region_GetHRGN(int region, int graphics);
+public static final native boolean Region_IsInfinite(int region, int graphics);
 public static final native int SolidBrush_new(int color);
 public static final native void SolidBrush_delete(int pen);
 public static final native void StringFormat_delete(int format);
