@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -94,10 +94,11 @@ public MenuItem (Menu parent, int style) {
  *
  * @param parent a menu control which will be the parent of the new instance (cannot be null)
  * @param style the style of control to construct
- * @param index the index to store the receiver in its parent
+ * @param index the zero-relative index to store the receiver in its parent
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
+ *    <li>ERROR_INVALID_RANGE - if the index is not between 0 and the number of elements in the parent (inclusive)</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
@@ -226,18 +227,9 @@ int createEmptyMenu () {
 	return 0;
 }
 
-void destroyEmptyMenu (int index) {
-	if ((parent.style & SWT.BAR) != 0) {
-		if (index == -1) index = parent.indexOf (this);
-		if (index != -1) {
-			short menuIndex = (short) (index + 1);
-			int [] outMenuRef = new int [1];
-			OS.GetMenuItemHierarchicalMenu (parent.handle, menuIndex, outMenuRef);
-			if (outMenuRef [0] != 0) {
-				OS.DisposeMenu (outMenuRef [0]);
-			}
-		}
-	}
+void destroyWidget () {
+	parent.destroyItem (this);
+	releaseHandle ();
 }
 
 /**
@@ -270,7 +262,7 @@ public int getAccelerator () {
 		int outMenuRef [] = new int [1];
 		if (OS.GetMenuItemHierarchicalMenu (menu.handle, (short)(index + 1), outMenuRef) != OS.noErr) {
 			return new Rectangle (0 ,0, 0, 0);
-		};
+		}
 		Rect rect = new Rect ();
 		//TODO - get the bounds of the menu item from the menu title
 //		if (... code needed to do this ... != OS.noErr) {
@@ -446,23 +438,23 @@ int keyGlyph (int key) {
 	return OS.kMenuNullGlyph;
 }
 
-void releaseChild () {
-	super.releaseChild ();
-	parent.destroyItem (this);
+void releaseHandle () {
+	super.releaseHandle ();
+	parent = null;
+}
+
+void releaseChildren (boolean destroy) {
+	if (menu != null) {
+		menu.release (false);
+		menu = null;
+	}
+	super.releaseChildren (destroy);
 }
 
 void releaseWidget () {
-	if (menu == null) {
-		destroyEmptyMenu (-1);
-	} else {
-		menu.releaseWidget ();
-		menu.destroyWidget ();
-	}
-	menu = null;
 	super.releaseWidget ();
 	accelerator = 0;
 	if (this == parent.defaultItem) parent.defaultItem = null;
-	parent = null;
 }
 
 /**
@@ -538,6 +530,29 @@ public void removeSelectionListener (SelectionListener listener) {
 	eventTable.unhook (SWT.DefaultSelection,listener);	
 }
 
+/*public*/ void select () {
+	checkWidget ();
+	Menu menu = parent, menuParent;
+	while (menu.cascade != null && (menuParent = menu.cascade.parent) != null) {
+		if ((menuParent.style & SWT.BAR) != 0) break;
+		menu = menuParent;
+	}
+	if (menu == null) return;
+	OS.HiliteMenu (OS.GetMenuID (menu.handle));
+	int[] event = new int[1];
+	OS.CreateEvent (0, OS.kEventClassCommand, OS.kEventProcessCommand, 0.0, 0, event);
+	if (event [0] != 0) {
+		int parentHandle = parent.handle;
+		HICommand command = new HICommand ();
+		command.attributes = OS.kHICommandFromMenu;
+		command.menu_menuRef = parentHandle;
+		command.menu_menuItemIndex = (short) (parent.indexOf (this) + 1);
+		OS.SetEventParameter (event [0], OS.kEventParamDirectObject, OS.typeHICommand, HICommand.sizeof, command);
+		OS.SendEventToEventTarget (event [0], OS.GetApplicationEventTarget ());
+		OS.ReleaseEvent (event [0]);
+	}
+}
+
 void selectRadio () {
 	int index = 0;
 	MenuItem [] items = parent.getItems ();
@@ -659,6 +674,11 @@ public void setImage (Image image) {
  * pull down menu. The sequence of key strokes, button presses
  * and/or button releases that are used to request a pull down
  * menu is platform specific.
+ * <p>
+ * Note: Disposing of a menu item that has a pull down menu
+ * will dispose of the menu.  To avoid this behavior, set the
+ * menu to null before the menu item is disposed.
+ * </p>
  *
  * @param menu the new pull down menu
  *
@@ -704,7 +724,6 @@ public void setMenu (Menu menu) {
 		menuHandle = createEmptyMenu ();
 	} else {
 		menu.cascade = this;
-		if (oldMenu == null) 	destroyEmptyMenu (index);
 		menuHandle = menu.handle;
 	}
 	short menuIndex = (short) (index + 1);
@@ -716,9 +735,11 @@ public void setMenu (Menu menu) {
 		OS.SetMenuTitleWithCFString (menuHandle, outString [0]);
 		OS.CFRelease (outString [0]);
 	}
+	if (oldMenu != null) OS.RetainMenu (oldMenu.handle);
 	if (OS.SetMenuItemHierarchicalMenu (parent.handle, menuIndex, menuHandle) != OS.noErr) {
 		error (SWT.ERROR_CANNOT_SET_MENU);
 	}
+	if (menuHandle != 0) OS.ReleaseMenu (menuHandle);
 }
 
 boolean setRadioSelection (boolean value) {
@@ -756,14 +777,14 @@ public void setSelection (boolean selected) {
  * Sets the receiver's text. The string may include
  * the mnemonic character and accelerator text.
  * <p>
- * Mnemonics are indicated by an '&amp' that causes the next
+ * Mnemonics are indicated by an '&amp;' that causes the next
  * character to be the mnemonic.  When the user presses a
  * key sequence that matches the mnemonic, a selection
  * event occurs. On most platforms, the mnemonic appears
  * underlined but may be emphasised in a platform specific
- * manner.  The mnemonic indicator character '&amp' can be
+ * manner.  The mnemonic indicator character '&amp;' can be
  * escaped by doubling it in the string, causing a single
- *'&amp' to be displayed.
+ * '&amp;' to be displayed.
  * </p>
  * <p>
  * Accelerator text is indicated by the '\t' character.
