@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -12,7 +12,6 @@ package org.eclipse.swt.widgets;
 
  
 import org.eclipse.swt.internal.carbon.OS;
-import org.eclipse.swt.internal.carbon.Rect;
 
 import org.eclipse.swt.*;
 import org.eclipse.swt.events.*;
@@ -23,7 +22,7 @@ import org.eclipse.swt.graphics.*;
  * objects that represent a range of positive, numeric values. 
  * <p>
  * At any given moment, a given scroll bar will have a 
- * single <em>selection</em> that is considered to be its
+ * single 'selection' that is considered to be its
  * value, which is constrained to be within the range of
  * values the scroll bar represents (that is, between its
  * <em>minimum</em> and <em>maximum</em> values).
@@ -83,7 +82,7 @@ import org.eclipse.swt.graphics.*;
  */
 public class ScrollBar extends Widget {
 	int handle;
-	int visibleRgn;
+	int visibleRgn, oldActionProc;
 	Scrollable parent;
 	boolean dragging;
 	int increment = 1;
@@ -106,7 +105,7 @@ ScrollBar (Scrollable parent, int style) {
  * interface.
  * <p>
  * When <code>widgetSelected</code> is called, the event object detail field contains one of the following values:
- * <code>0</code> - for the end of a drag.
+ * <code>SWT.NONE</code> - for the end of a drag.
  * <code>SWT.DRAG</code>.
  * <code>SWT.HOME</code>.
  * <code>SWT.END</code>.
@@ -143,24 +142,24 @@ static int checkStyle (int style) {
 	return checkBits (style, SWT.HORIZONTAL, SWT.VERTICAL, 0, 0, 0, 0);
 }
 
-int actionProc (int theControl, int partCode) {
+int actionProc (int theControl, int partCode) {	
 	Event event = new Event ();
-	int value = OS.GetControl32BitValue (handle);
-    switch (partCode) {
+	int inc = 0;
+	switch (partCode) {
 	    case OS.kControlUpButtonPart:
-			value -= increment;
+			inc -= increment;
 	        event.detail = SWT.ARROW_UP;
 	        break;
 	    case OS.kControlPageUpPart:
-			value -= pageIncrement;
+			inc -= pageIncrement;
 	        event.detail = SWT.PAGE_UP;
 	        break;
 	    case OS.kControlPageDownPart:
-			value += pageIncrement;
+			inc += pageIncrement;
 	        event.detail = SWT.PAGE_DOWN;
 	        break;
 	    case OS.kControlDownButtonPart:
-			value += increment;
+			inc += increment;
 	        event.detail = SWT.ARROW_DOWN;
 	        break;
 	    case OS.kControlIndicatorPart:
@@ -170,18 +169,23 @@ int actionProc (int theControl, int partCode) {
 		default:
 			return 0;
 	}
-	OS.SetControl32BitValue (handle, value);
+	if (oldActionProc != 0) {
+		OS.Call (oldActionProc, theControl, partCode);
+		parent.redrawBackgroundImage ();
+	} else {
+		int value = OS.GetControl32BitValue (handle) + inc;	    
+		OS.SetControl32BitValue (handle, value);
+	}
 	sendEvent (SWT.Selection, event);
-	parent.update (true);
+	if (!OS.HIVIEW) parent.update (true);
 	return 0;
 }
 
 void destroyWidget () {
-	Display display = this.display;
 	int theControl = handle;
 	releaseHandle ();
 	if (theControl != 0) {
-		display.addDisposeControl (theControl);
+		OS.DisposeControl (theControl);
 	}
 }
 
@@ -209,7 +213,7 @@ void createWidget () {
 
 void deregister () {
 	super.deregister ();
-	WidgetTable.remove (handle);
+	display.removeWidget (handle);
 }
 
 int getDrawCount (int control) {
@@ -303,7 +307,7 @@ public int getPageIncrement () {
 }
 
 /**
- * Returns the receiver's parent, which must be scrollable.
+ * Returns the receiver's parent, which must be a Scrollable.
  *
  * @return the receiver's parent
  *
@@ -318,7 +322,7 @@ public Scrollable getParent () {
 }
 
 /**
- * Returns the single <em>selection</em> that is the receiver's value.
+ * Returns the single 'selection' that is the receiver's value.
  *
  * @return the selection
  *
@@ -347,12 +351,11 @@ public int getSelection () {
  */
 public Point getSize () {
 	checkWidget();
-	Rect rect = getControlSize (handle);
-	return new Point (rect.right - rect.left, rect.bottom - rect.top);
+	return getControlSize (handle);
 }
 
 /**
- * Answers the size of the receiver's thumb relative to the
+ * Returns the size of the receiver's thumb relative to the
  * difference between its maximum and minimum values.
  *
  * @return the thumb value
@@ -403,12 +406,18 @@ int getVisibleRegion (int control, boolean clipChildren) {
 
 void hookEvents () {
 	super.hookEvents ();
-	int controlProc = display.controlProc;
-	int [] mask = new int [] {
-		OS.kEventClassControl, OS.kEventControlDraw,
-	};
-	int controlTarget = OS.GetControlEventTarget (handle);
-	OS.InstallEventHandler (controlTarget, controlProc, mask.length / 2, mask, handle, null);
+	if (!OS.HIVIEW) {
+		int controlProc = display.controlProc;
+		int [] mask = new int [] {
+			OS.kEventClassControl, OS.kEventControlDraw,
+		};
+		int controlTarget = OS.GetControlEventTarget (handle);
+		OS.InstallEventHandler (controlTarget, controlProc, mask.length / 2, mask, handle, null);
+	}
+	if ((parent.state & CANVAS) == 0) {
+		oldActionProc = OS.GetControlAction (handle);
+		OS.SetControlAction (handle, display.actionProc);
+	}
 }
 
 
@@ -417,6 +426,23 @@ void invalidateVisibleRegion (int control) {
 	parent.resetVisibleRegion (control);
 }
 
+void invalWindowRgn (int window, int rgn) {
+	parent.invalWindowRgn (window, rgn);
+}
+
+boolean isDrawing (int control) {
+	/*
+	* Feature in the Macintosh.  The scroll bars in a DataBrowser are
+	* always invisible according to IsControlVisible(), despite the fact
+	* that they are drawn.  The fix is to check our visibility flag
+	* instead of calling IsControlVisible().
+	* 
+	* Note: During resize IsControlVisible() returns true allowing the
+	* clipping to be properly calculated.
+	*/
+	return isVisible () && getDrawCount (control) == 0;
+}
+
 /**
  * Returns <code>true</code> if the receiver is enabled and all
  * of the receiver's ancestors are enabled, and <code>false</code>
@@ -457,7 +483,7 @@ boolean isTrimHandle (int trimHandle) {
  */
 public boolean isVisible () {
 	checkWidget();
-	return OS.IsControlVisible (handle);
+	return getVisible () && parent.isVisible ();
 }
 
 int kEventMouseDown (int nextHandler, int theEvent, int userData) {
@@ -473,6 +499,10 @@ int kEventMouseDown (int nextHandler, int theEvent, int userData) {
 	return status;
 }
 
+void redraw () {
+	redrawWidget (handle, false);
+}
+
 /**
  * Removes the listener from the collection of listeners who will
  * be notified when the receiver's value changes.
@@ -500,14 +530,7 @@ public void removeSelectionListener(SelectionListener listener) {
 
 void register () {
 	super.register ();
-	WidgetTable.put (handle, this);
-}
-
-void releaseChild () {
-	super.releaseChild ();
-	if (parent.horizontalBar == this) parent.horizontalBar = null;
-	if (parent.verticalBar == this) parent.verticalBar = null;
-	parent.layoutControl (true);
+	display.addWidget (handle, this);
 }
 
 void releaseHandle () {
@@ -515,6 +538,13 @@ void releaseHandle () {
 	handle = 0;
 }
 
+void releaseParent () {
+	super.releaseParent ();
+	if (parent.horizontalBar == this) parent.horizontalBar = null;
+	if (parent.verticalBar == this) parent.verticalBar = null;
+	parent.resizeClientArea ();
+}
+
 void releaseWidget () {
 	super.releaseWidget ();
 	if (visibleRgn != 0) OS.DisposeRgn (visibleRgn);
@@ -575,11 +605,12 @@ public void setEnabled (boolean enabled) {
 }
 
 /**
- * Sets the maximum value which the receiver will allow
- * to be the argument which must be greater than or
- * equal to zero.
+ * Sets the maximum. If this value is negative or less than or
+ * equal to the minimum, the value is ignored. If necessary, first
+ * the thumb and then the selection are adjusted to fit within the
+ * new range.
  *
- * @param value the new maximum (must be zero or greater)
+ * @param value the new maximum
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -590,17 +621,22 @@ public void setMaximum (int value) {
 	checkWidget();
 	if (value < 0) return;
 	int minimum = OS.GetControl32BitMinimum (handle);
+	if (value <= minimum) return;
 	int viewSize = OS.GetControlViewSize (handle);
-	if (value - minimum - viewSize < 0) return;
+	if (value - minimum < viewSize) {
+		viewSize = value - minimum;
+		OS.SetControlViewSize (handle, viewSize);
+	}
 	OS.SetControl32BitMaximum (handle, value - viewSize);
 }
 
 /**
- * Sets the minimum value which the receiver will allow
- * to be the argument which must be greater than or
- * equal to zero.
+ * Sets the minimum value. If this value is negative or greater
+ * than or equal to the maximum, the value is ignored. If necessary,
+ * first the thumb and then the selection are adjusted to fit within
+ * the new range.
  *
- * @param value the new minimum (must be zero or greater)
+ * @param value the new minimum
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -610,9 +646,14 @@ public void setMaximum (int value) {
 public void setMinimum (int value) {
 	checkWidget();
 	if (value < 0) return;
-	int maximum = OS.GetControl32BitMinimum (handle);
 	int viewSize = OS.GetControlViewSize (handle);
-	if (maximum - value - viewSize < 0) return;
+	int maximum = OS.GetControl32BitMaximum (handle) + viewSize;
+	if (value >= maximum) return;
+	if (maximum - value < viewSize) {
+		viewSize = maximum - value;
+		OS.SetControl32BitMaximum (handle, maximum - viewSize);
+		OS.SetControlViewSize (handle, viewSize);
+	}
 	OS.SetControl32BitMinimum (handle, value);
 }
 
@@ -622,7 +663,7 @@ public void setMinimum (int value) {
  * are selected to the argument, which must be at least
  * one.
  *
- * @return the page increment (must be greater than zero)
+ * @param value the page increment (must be greater than zero)
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -640,7 +681,7 @@ public void setPageIncrement (int value) {
  * value to the argument which must be greater than or equal
  * to zero.
  *
- * @param value the new selection (must be zero or greater)
+ * @param selection the new selection (must be zero or greater)
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -654,24 +695,26 @@ public void setSelection (int value) {
 
 /**
  * Sets the size of the receiver's thumb relative to the
- * difference between its maximum and minimum values to the
- * argument which must be at least one.
+ * difference between its maximum and minimum values.  This new
+ * value will be ignored if it is less than one, and will be
+ * clamped if it exceeds the receiver's current range.
  *
- * @param value the new thumb value (must be at least one)
+ * @param value the new thumb value, which must be at least one and not
+ * larger than the size of the current range
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- *
- * @see ScrollBar
  */
 public void setThumb (int value) {
 	checkWidget();
 	if (value < 1) return;
-	int maximum = OS.GetControl32BitMaximum (handle);
+	int minimum = OS.GetControl32BitMinimum (handle);
 	int viewSize = OS.GetControlViewSize (handle);
-	OS.SetControl32BitMaximum (handle, maximum + viewSize - value);
+	int maximum = OS.GetControl32BitMaximum (handle) + viewSize;
+	value = Math.min (value, maximum - minimum);
+	OS.SetControl32BitMaximum (handle, maximum - value);
     OS.SetControlViewSize (handle, value);
 }
 
@@ -679,7 +722,7 @@ public void setThumb (int value) {
  * Sets the receiver's selection, minimum value, maximum
  * value, thumb, increment and page increment all at once.
  * <p>
- * Note: This is equivalent to setting the values individually
+ * Note: This is similar to setting the values individually
  * using the appropriate methods, but may be implemented in a 
  * more efficient fashion on some platforms.
  * </p>
@@ -701,9 +744,9 @@ public void setValues (int selection, int minimum, int maximum, int thumb, int i
 	if (minimum < 0) return;
 	if (maximum < 0) return;
 	if (thumb < 1) return;
-	if (maximum - minimum - thumb < 0) return;
 	if (increment < 1) return;
 	if (pageIncrement < 1) return;
+	thumb = Math.min (thumb, maximum - minimum);
 	OS.SetControl32BitMinimum (handle, minimum);
 	OS.SetControl32BitMaximum (handle, maximum - thumb);
 	OS.SetControlViewSize (handle, thumb);
@@ -730,16 +773,7 @@ public void setValues (int selection, int minimum, int maximum, int thumb, int i
  */
 public void setVisible (boolean visible) {
 	checkWidget();
-	if (visible) {
-		if ((state & HIDDEN) == 0) return;
-		state &= ~HIDDEN;
-	} else {
-		if ((state & HIDDEN) != 0) return;
-		state |= HIDDEN;
-	}
-	setVisible (handle, visible);
-	parent.layoutControl (true);
-	sendEvent (visible ? SWT.Show : SWT.Hide);
+	parent.setScrollBarVisible (this, visible);
 }
 
 void setZOrder () {
