@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -39,7 +39,8 @@ import org.eclipse.swt.graphics.*;
  * </p>
  */
 public class Label extends Control {
-	int frameHandle, labelHandle, imageHandle;
+	int /*long*/ frameHandle, labelHandle, imageHandle;
+	ImageList imageList;
 	Image image;
 	String text;
 
@@ -85,6 +86,7 @@ public Label (Composite parent, int style) {
 }
 
 static int checkStyle (int style) {
+	style |= SWT.NO_FOCUS;
 	if ((style & SWT.SEPARATOR) != 0) {
 		style = checkBits (style, SWT.VERTICAL, SWT.HORIZONTAL, 0, 0, 0, 0);
 		return checkBits (style, SWT.SHADOW_OUT, SWT.SHADOW_IN, SWT.SHADOW_NONE, 0, 0, 0);
@@ -92,8 +94,20 @@ static int checkStyle (int style) {
 	return checkBits (style, SWT.LEFT, SWT.CENTER, SWT.RIGHT, 0, 0, 0);
 }
 
+void addRelation (Control control) {
+	if (!control.isDescribedByLabel ()) return;
+	if (labelHandle == 0) return;
+	int /*long*/ accessible = OS.gtk_widget_get_accessible (labelHandle);
+	int /*long*/ controlAccessible = OS.gtk_widget_get_accessible (control.handle);
+	if (accessible != 0 && controlAccessible != 0) {
+		OS.atk_object_add_relationship (controlAccessible, OS.ATK_RELATION_LABELLED_BY, accessible);
+	}
+}
+
 public Point computeSize (int wHint, int hHint, boolean changed) {
 	checkWidget ();
+	if (wHint != SWT.DEFAULT && wHint < 0) wHint = 0;
+	if (hHint != SWT.DEFAULT && hHint < 0) hHint = 0;
 	if ((style & SWT.SEPARATOR) != 0) {
 		if ((style & SWT.HORIZONTAL) != 0) {
 			if (wHint == SWT.DEFAULT) wHint = DEFAULT_WIDTH;
@@ -101,36 +115,61 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 			if (hHint == SWT.DEFAULT) hHint = DEFAULT_HEIGHT;
 		}
 	}
-	int width = OS.GTK_WIDGET_WIDTH (handle);
-	int height = OS.GTK_WIDGET_HEIGHT (handle);
-	int labelWidth = 0, labelHeight = 0;
-	if (labelHandle != 0) {
-		labelWidth = OS.GTK_WIDGET_WIDTH (labelHandle);
-		labelHeight = OS.GTK_WIDGET_HEIGHT (labelHandle);
+	boolean fixWrap = labelHandle != 0 && (style & SWT.WRAP) != 0;
+	if (fixWrap || frameHandle != 0) forceResize ();
+	int [] labelWidth = new int [1], labelHeight = new int [1];
+	if (fixWrap) {
+		OS.gtk_widget_get_size_request (labelHandle, labelWidth, labelHeight);
 		OS.gtk_widget_set_size_request (labelHandle, wHint, hHint);
-	}
-	GtkRequisition requisition = new GtkRequisition ();
+	}	
+	Point size; 
 	if (frameHandle != 0) {
-		int frameWidth = OS.GTK_WIDGET_WIDTH (frameHandle);
-		int frameHeight = OS.GTK_WIDGET_HEIGHT (frameHandle);
-		OS.gtk_widget_set_size_request (frameHandle, -1, -1);
+		int [] reqWidth = new int [1], reqHeight = new int [1];
+		OS.gtk_widget_get_size_request (handle, reqWidth, reqHeight);
 		OS.gtk_widget_set_size_request (handle, wHint, hHint);
-		OS.gtk_widget_size_request (frameHandle, requisition);
-		OS.gtk_widget_set_size_request (frameHandle, frameWidth, frameHeight);
+		size = computeNativeSize (frameHandle, -1, -1, changed);
+		OS.gtk_widget_set_size_request (handle, reqWidth [0], reqHeight [0]);
 	} else {
-		OS.gtk_widget_set_size_request (handle, wHint, hHint);
-		OS.gtk_widget_size_request (handle, requisition);
+		size = computeNativeSize (handle, wHint, hHint, changed);
 	}
-	if (labelHandle != 0) {
-		OS.gtk_widget_set_size_request (labelHandle, labelWidth, labelHeight);
+	if (fixWrap) {
+		OS.gtk_widget_set_size_request (labelHandle, labelWidth [0], labelHeight [0]);
 	}
-	OS.gtk_widget_set_size_request (handle, width, height);
-	return new Point (requisition.width, requisition.height);	
+	/*
+	* Feature in GTK.  Instead of using the font height to determine
+	* the preferred height of the widget, GTK uses the text metrics.
+	* The fix is to ensure that the preferred height is at least as
+	* tall as the font height.
+	* 
+	* NOTE: This work around does not fix the case when there are
+	* muliple lines of text.
+	*/
+	if (hHint == SWT.DEFAULT && labelHandle != 0) {
+		int /*long*/ layout = OS.gtk_label_get_layout (labelHandle);
+		int /*long*/ context = OS.pango_layout_get_context (layout);
+		int /*long*/ lang = OS.pango_context_get_language (context);
+		int /*long*/ font = getFontDescription ();
+		int /*long*/ metrics = OS.pango_context_get_metrics (context, font, lang);
+		int ascent = OS.PANGO_PIXELS (OS.pango_font_metrics_get_ascent (metrics));
+		int descent = OS.PANGO_PIXELS (OS.pango_font_metrics_get_descent (metrics));
+		OS.pango_font_metrics_unref (metrics);
+		int fontHeight = ascent + descent;
+		int [] buffer = new int [1];
+		OS.g_object_get (labelHandle, OS.ypad, buffer, 0);
+		fontHeight += 2 * buffer [0];
+		if (frameHandle != 0) {
+			int /*long*/ style = OS.gtk_widget_get_style (frameHandle);
+			fontHeight += 2 * OS.gtk_style_get_ythickness (style);
+			fontHeight += 2 * OS.gtk_container_get_border_width (frameHandle);
+		}
+		size.y = Math.max (size.y, fontHeight);
+	}
+	return size;
 }
 
 void createHandle (int index) {
-	state |= HANDLE;
-	fixedHandle = OS.gtk_fixed_new ();
+	state |= HANDLE | THEME_BACKGROUND;
+	fixedHandle = OS.g_object_new (display.gtk_fixed_get_type (), 0);
 	if (fixedHandle == 0) error (SWT.ERROR_NO_HANDLES);
 	OS.gtk_fixed_set_has_window (fixedHandle, true);
 	if ((style & SWT.SEPARATOR) != 0) {
@@ -139,7 +178,7 @@ void createHandle (int index) {
 		} else {
 			handle = OS.gtk_vseparator_new ();
 		}
-		OS.gtk_widget_show (handle);
+		if (handle == 0) error (SWT.ERROR_NO_HANDLES);
 	} else {
 		handle = OS.gtk_hbox_new (false, 0);
 		if (handle == 0) error (SWT.ERROR_NO_HANDLES);
@@ -149,24 +188,16 @@ void createHandle (int index) {
 		if (imageHandle == 0) error (SWT.ERROR_NO_HANDLES);
 		OS.gtk_container_add (handle, labelHandle);
 		OS.gtk_container_add (handle, imageHandle);
-		OS.gtk_widget_show (handle);
-		OS.gtk_widget_show (labelHandle);		
 	}
-	if (handle == 0) error (SWT.ERROR_NO_HANDLES);
 	if ((style & SWT.BORDER) != 0) {
 		frameHandle = OS.gtk_frame_new (null);
 		if (frameHandle == 0) error (SWT.ERROR_NO_HANDLES);
 		OS.gtk_container_add (fixedHandle, frameHandle);
 		OS.gtk_container_add (frameHandle, handle);
-		OS.gtk_widget_show (frameHandle);
-		// CHECK THEME
 		OS.gtk_frame_set_shadow_type (frameHandle, OS.GTK_SHADOW_ETCHED_IN);
 	} else {
 		OS.gtk_container_add (fixedHandle, handle);
 	}
-	int parentHandle = parent.parentingHandle ();
-	OS.gtk_container_add (parentHandle, fixedHandle);
-	OS.gtk_widget_show (fixedHandle);
 	if ((style & SWT.SEPARATOR) != 0) return;
 	if ((style & SWT.WRAP) != 0) OS.gtk_label_set_line_wrap (labelHandle, true);
 	if ((style & SWT.LEFT) != 0) {
@@ -196,12 +227,12 @@ void createWidget (int index) {
 
 void deregister () {
 	super.deregister ();
-	if (frameHandle != 0) WidgetTable.remove (frameHandle);
-	if (labelHandle != 0) WidgetTable.remove (labelHandle);
-	if (imageHandle != 0) WidgetTable.remove (imageHandle);
+	if (frameHandle != 0) display.removeWidget (frameHandle);
+	if (labelHandle != 0) display.removeWidget (labelHandle);
+	if (imageHandle != 0) display.removeWidget (imageHandle);
 }
 
-int eventHandle () {
+int /*long*/ eventHandle () {
 	return fixedHandle;
 }
 
@@ -228,6 +259,14 @@ public int getAlignment () {
 	return SWT.LEFT;
 }
 
+public int getBorderWidth () {
+	checkWidget();
+	if (frameHandle != 0) {
+		return OS.gtk_style_get_xthickness (OS.gtk_widget_get_style (frameHandle));
+	}
+	return 0;
+}
+
 /**
  * Returns the receiver's image if it has one, or null
  * if it does not.
@@ -266,37 +305,49 @@ public String getText () {
 	return text;
 }
 
-int gtk_mnemonic_activate (int widget, int arg1) {
-	Composite control = this.parent;
-	while (control != null) {
-		Control [] children = control._getChildren ();
-		int index = 0;
-		while (index < children.length) {
-			if (children [index] == this) break;
-			index++;
-		}
-		index++;
-		if (index < children.length) {
-			if (children [index].setFocus ()) return 1;
-		}
-		control = control.parent;
-	}
-	return 1;
-}
-
 void hookEvents () {
 	super.hookEvents();
 	if (labelHandle != 0) {
-		Display display = getDisplay ();
-		OS.g_signal_connect (labelHandle, OS.mnemonic_activate, display.windowProc3, MNEMONIC_ACTIVATE);
+		OS.g_signal_connect_closure_by_id (labelHandle, display.signalIds [MNEMONIC_ACTIVATE], 0, display.closures [MNEMONIC_ACTIVATE], false);
 	}
 }
 
+boolean isDescribedByLabel () {
+	return false;
+}
+
+boolean mnemonicHit (char key) {
+	if (labelHandle == 0) return false;
+	boolean result = super.mnemonicHit (labelHandle, key);
+	if (result) {
+		Composite control = this.parent;
+		while (control != null) {
+			Control [] children = control._getChildren ();
+			int index = 0;
+			while (index < children.length) {
+				if (children [index] == this) break;
+				index++;
+			}
+			index++;
+			if (index < children.length) {
+				if (children [index].setFocus ()) return result;
+			}
+			control = control.parent;
+		}
+	}
+	return result;
+}
+
+boolean mnemonicMatch (char key) {
+	if (labelHandle == 0) return false;
+	return mnemonicMatch (labelHandle, key);
+}
+
 void register () {
 	super.register ();
-	if (frameHandle != 0) WidgetTable.put (frameHandle, this);
-	if (labelHandle != 0) WidgetTable.put (labelHandle, this);
-	if (imageHandle != 0) WidgetTable.put (imageHandle, this);
+	if (frameHandle != 0) display.addWidget (frameHandle, this);
+	if (labelHandle != 0) display.addWidget (labelHandle, this);
+	if (imageHandle != 0) display.addWidget (imageHandle, this);
 }
 
 void releaseHandle () {
@@ -306,31 +357,15 @@ void releaseHandle () {
 
 void releaseWidget () {
 	super.releaseWidget ();
+	if (imageList != null) imageList.dispose ();
+	imageList = null;
 	image = null;
 	text = null;
 }
 
 void resizeHandle (int width, int height) {
 	OS.gtk_widget_set_size_request (fixedHandle, width, height);
-	int widgetHandle = frameHandle != 0 ? frameHandle : handle;
-	OS.gtk_widget_set_size_request (widgetHandle, width, height);
-
-	/*
-	* Feature in GTK.  Some widgets do not allocate the size
-	* of their internal children in gtk_widget_size_allocate().
-	* Instead this is done in gtk_widget_size_request().  This
-	* means that the client area of the widget is not correct.
-	* The fix is to call gtk_widget_size_request() (and throw
-	* the results away).
-	*
-	* Note: The following widgets rely on this feature:
-	* 	GtkScrolledWindow
-	* 	GtkNotebook
-	* 	GtkFrame
-	* 	GtkCombo
-	*/
-	GtkRequisition requisition = new GtkRequisition ();
-	OS.gtk_widget_size_request (widgetHandle, requisition);
+	OS.gtk_widget_set_size_request (frameHandle != 0 ? frameHandle : handle, width, height);
 }
 
 /**
@@ -352,9 +387,10 @@ public void setAlignment (int alignment) {
 	if ((alignment & (SWT.LEFT | SWT.RIGHT | SWT.CENTER)) == 0) return;
 	style &= ~(SWT.LEFT | SWT.RIGHT | SWT.CENTER);
 	style |= alignment & (SWT.LEFT | SWT.RIGHT | SWT.CENTER);
+	boolean isRTL = (style & SWT.RIGHT_TO_LEFT) != 0;
 	if ((style & SWT.LEFT) != 0) {
 		OS.gtk_misc_set_alignment (labelHandle, 0.0f, 0.0f);
-		OS.gtk_label_set_justify (labelHandle, OS.GTK_JUSTIFY_LEFT);
+		OS.gtk_label_set_justify (labelHandle, isRTL ? OS.GTK_JUSTIFY_RIGHT : OS.GTK_JUSTIFY_LEFT);
 		OS.gtk_misc_set_alignment (imageHandle, 0.0f, 0.5f);
 		return;
 	}
@@ -366,7 +402,7 @@ public void setAlignment (int alignment) {
 	}
 	if ((style & SWT.RIGHT) != 0) {
 		OS.gtk_misc_set_alignment (labelHandle, 1.0f, 0.0f);
-		OS.gtk_label_set_justify (labelHandle, OS.GTK_JUSTIFY_RIGHT);
+		OS.gtk_label_set_justify (labelHandle, isRTL ? OS.GTK_JUSTIFY_LEFT : OS.GTK_JUSTIFY_RIGHT);
 		OS.gtk_misc_set_alignment (imageHandle, 1.0f, 0.5f);
 		return;
 	}
@@ -374,12 +410,12 @@ public void setAlignment (int alignment) {
 
 void setBackgroundColor (GdkColor color) {
 	super.setBackgroundColor (color);
-	OS.gtk_widget_modify_bg (fixedHandle, 0, color);
-	if (labelHandle != 0) OS.gtk_widget_modify_bg (labelHandle, 0, color);
-	if (imageHandle != 0) OS.gtk_widget_modify_bg (imageHandle, 0, color);
+	setBackgroundColor(fixedHandle, color);
+	if (labelHandle != 0) setBackgroundColor(labelHandle, color);
+	if (imageHandle != 0) setBackgroundColor(imageHandle, color);
 }
 
-boolean setBounds (int x, int y, int width, int height, boolean move, boolean resize) {
+int setBounds (int x, int y, int width, int height, boolean move, boolean resize) {
 	/*
 	* Bug in GTK.  For some reason, when the label is
 	* wrappable and its container is resized, it does not
@@ -394,7 +430,7 @@ boolean setBounds (int x, int y, int width, int height, boolean move, boolean re
 	*/
 	boolean fixWrap = resize && labelHandle != 0 && (style & SWT.WRAP) != 0;
 	if (fixWrap) OS.gtk_widget_set_size_request (labelHandle, -1, -1);
-	boolean changed = super.setBounds (x, y, width, height, move, resize);
+	int result = super.setBounds (x, y, width, height, move, resize);
 	/*
 	* Bug in GTK.  For some reason, when the label is
 	* wrappable and its container is resized, it does not
@@ -410,11 +446,23 @@ boolean setBounds (int x, int y, int width, int height, boolean move, boolean re
 		int labelWidth = OS.GTK_WIDGET_WIDTH (handle);
 		int labelHeight = OS.GTK_WIDGET_HEIGHT (handle);
 		OS.gtk_widget_set_size_request (labelHandle, labelWidth, labelHeight);
+		/*
+		* Bug in GTK.  Setting the size request should invalidate the label's
+		* layout, but it does not.  The fix is to resize the label directly. 
+		*/
+		GtkRequisition requisition = new GtkRequisition ();
+		OS.gtk_widget_size_request (labelHandle, requisition);
+		GtkAllocation allocation = new GtkAllocation ();
+		allocation.x = OS.GTK_WIDGET_X (labelHandle);
+		allocation.y = OS.GTK_WIDGET_Y (labelHandle);
+		allocation.width = labelWidth;
+		allocation.height = labelHeight;
+		OS.gtk_widget_size_allocate (labelHandle, allocation);
 	}
-	return changed;
+	return result;
 }
 
-void setFontDescription (int font) {
+void setFontDescription (int /*long*/ font) {
 	super.setFontDescription (font);
 	if (labelHandle != 0) OS.gtk_widget_modify_font (labelHandle, font);
 	if (imageHandle != 0) OS.gtk_widget_modify_font (imageHandle, font);
@@ -422,9 +470,23 @@ void setFontDescription (int font) {
 
 void setForegroundColor (GdkColor color) {
 	super.setForegroundColor (color);
-	OS.gtk_widget_modify_fg (fixedHandle, 0, color);
-	if (labelHandle != 0) OS.gtk_widget_modify_fg (labelHandle, 0, color);
-	if (imageHandle != 0) OS.gtk_widget_modify_fg (imageHandle, 0, color);
+	OS.gtk_widget_modify_fg (fixedHandle,  OS.GTK_STATE_NORMAL, color);
+	if (labelHandle != 0) OS.gtk_widget_modify_fg (labelHandle,  OS.GTK_STATE_NORMAL, color);
+	if (imageHandle != 0) OS.gtk_widget_modify_fg (imageHandle,  OS.GTK_STATE_NORMAL, color);
+}
+
+void setOrientation () {
+	super.setOrientation ();
+	if ((style & SWT.RIGHT_TO_LEFT) != 0) {
+		if (labelHandle != 0) OS.gtk_widget_set_direction (labelHandle, OS.GTK_TEXT_DIR_RTL);
+		if (imageHandle != 0) OS.gtk_widget_set_direction (imageHandle, OS.GTK_TEXT_DIR_RTL);
+		if ((style & SWT.LEAD) != 0) {
+			if (labelHandle != 0) OS.gtk_label_set_justify (labelHandle, OS.GTK_JUSTIFY_RIGHT);
+		}
+		if ((style & SWT.TRAIL) != 0) {
+			if (labelHandle != 0) OS.gtk_label_set_justify (labelHandle, OS.GTK_JUSTIFY_LEFT);
+		}
+	}
 }
 
 /**
@@ -445,12 +507,17 @@ public void setImage (Image image) {
 	checkWidget ();
 	if ((style & SWT.SEPARATOR) != 0) return;
 	this.image = image;
+	if (imageList != null) imageList.dispose ();
+	imageList = null;
 	if (image != null) {
-		OS.gtk_image_set_from_pixmap (imageHandle, image.pixmap, image.mask);
+		imageList = new ImageList ();
+		int imageIndex = imageList.add (image);
+		int /*long*/ pixbuf = imageList.getPixbuf (imageIndex);
+		OS.gtk_image_set_from_pixbuf (imageHandle, pixbuf);
 		OS.gtk_widget_hide (labelHandle);
 		OS.gtk_widget_show (imageHandle);
 	} else {
-		OS.gtk_image_set_from_pixmap (imageHandle, 0, 0);
+		OS.gtk_image_set_from_pixbuf (imageHandle, 0);
 		OS.gtk_widget_show (labelHandle);
 		OS.gtk_widget_hide (imageHandle);
 	}
@@ -463,14 +530,14 @@ public void setImage (Image image) {
  * the mnemonic character and line delimiters.
  * </p>
  * <p>
- * Mnemonics are indicated by an '&amp' that causes the next
+ * Mnemonics are indicated by an '&amp;' that causes the next
  * character to be the mnemonic.  When the user presses a
  * key sequence that matches the mnemonic, focus is assigned
  * to the control that follows the label. On most platforms,
  * the mnemonic appears underlined but may be emphasised in a
  * platform specific manner.  The mnemonic indicator character
- *'&amp' can be escaped by doubling it in the string, causing
- * a single '&amp' to be displayed.
+ * '&amp;' can be escaped by doubling it in the string, causing
+ * a single '&amp;' to be displayed.
  * </p>
  * 
  * @param string the new text
@@ -484,15 +551,20 @@ public void setImage (Image image) {
  * </ul>
  */
 public void setText (String string) {
-	checkWidget();
+	checkWidget ();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
-	text = string;
 	if ((style & SWT.SEPARATOR) != 0) return;
+	text = string;
 	char [] chars = fixMnemonic (string);
-	byte [] buffer = Converter.wcsToMbcs (null, chars, false);
+	byte [] buffer = Converter.wcsToMbcs (null, chars, true);
 	OS.gtk_label_set_text_with_mnemonic (labelHandle, buffer);
 	OS.gtk_widget_hide (imageHandle);
 	OS.gtk_widget_show (labelHandle);
 }
 
+void showWidget () {
+	super.showWidget ();
+	if (frameHandle != 0) OS.gtk_widget_show (frameHandle);
+	if (labelHandle != 0) OS.gtk_widget_show (labelHandle);
+}
 }
