@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -14,6 +14,7 @@ package org.eclipse.swt.widgets;
 import org.eclipse.swt.internal.*;
 import org.eclipse.swt.internal.motif.*;
 import org.eclipse.swt.*;
+import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.events.*;
 
 /**
@@ -93,10 +94,11 @@ public MenuItem (Menu parent, int style) {
  *
  * @param parent a menu control which will be the parent of the new instance (cannot be null)
  * @param style the style of control to construct
- * @param index the index to store the receiver in its parent
+ * @param index the zero-relative index to store the receiver in its parent
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
+ *    <li>ERROR_INVALID_RANGE - if the index is not between 0 and the number of elements in the parent (inclusive)</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
@@ -118,7 +120,7 @@ public MenuItem (Menu parent, int style, int index) {
 	createWidget (index);
 }
 void addAccelerator () {
-	if (accelerator == 0) return;
+	if (accelerator == 0 || !getEnabled ()) return;
 	/*
 	* Bug in Solaris.  When accelerators are set more
 	* than once in the same menu bar, the time it takes
@@ -137,16 +139,21 @@ void addAccelerator () {
 	if (newKey != 0) {
 		keysym = newKey;
 	} else {
-		keysym = wcsToMbcs ((char) keysym);
+		keysym = Display.wcsToMbcs ((char) keysym);
 	}
 	/*
 	* Feature in Motif.  Motif does not activate an accelerator
-	* when the CapsLoc, NumLoc and NumLock+CapsLoc keys are pressed.
+	* when the CapsLock, NumLock and NumLock+CapsLock keys are pressed.
 	* In order to activate accelerators when these keys are pressed,
-	* it is necessary to look for all of these key sequences.
+	* it is necessary to look for all of these key sequences.  The fix
+	* is to add these modifiers to the accelerator.
 	*/
 	String key = ctrl + alt + shift + "<Key>" + keysymName (keysym);
-	String allKeys = key + ",Lock " + key + ",Mod2 " + key + ",Lock Mod2 " + key;
+	String allKeys = key + ",Lock " + key;
+	String numLock = Display.numLock;
+	if (numLock != null) {
+		allKeys += "," + numLock + " " + key + ",Lock " + numLock + " " + key;
+	}
 	/* Use the character encoding for the default locale */
 	byte [] buffer = Converter.wcsToMbcs (null, allKeys, true);		
 	int ptr = OS.XtMalloc (buffer.length);
@@ -212,7 +219,7 @@ public void addHelpListener (HelpListener listener) {
 }
 /**
  * Adds the listener to the collection of listeners who will
- * be notified when the control is selected, by sending
+ * be notified when the menu item is selected, by sending
  * it one of the messages defined in the <code>SelectionListener</code>
  * interface.
  * <p>
@@ -248,7 +255,6 @@ protected void checkSubclass () {
 	if (!isValidSubclass ()) error (SWT.ERROR_INVALID_SUBCLASS);
 }
 void createHandle (int index) {
-	state |= HANDLE;
 	int parentHandle = parent.handle;
 	int [] argList1 = {OS.XmNchildren, 0, OS.XmNnumChildren, 0};
 	OS.XtGetValues (parentHandle, argList1, argList1.length / 2);
@@ -312,11 +318,13 @@ void destroyWidget () {
 	super.destroyWidget ();
 }
 /**
- * Return the widget accelerator.  An accelerator is the bit-wise
+ * Returns the widget accelerator.  An accelerator is the bit-wise
  * OR of zero or more modifier masks and a key. Examples:
  * <code>SWT.CONTROL | SWT.SHIFT | 'T', SWT.ALT | SWT.F2</code>.
+ * The default value is zero, indicating that the menu item does
+ * not have an accelerator.
  *
- * @return the accelerator
+ * @return the accelerator or 0
  *
  * </ul>
  * @exception SWTException <ul>
@@ -328,14 +336,16 @@ public int getAccelerator () {
 	checkWidget();
 	return accelerator;
 }
-public Display getDisplay () {
-	Menu parent = this.parent;
-	if (parent == null) error (SWT.ERROR_WIDGET_DISPOSED);
-	return parent.getDisplay ();
+/*public*/ Rectangle getBounds () {
+	checkWidget();
+	if (!OS.XtIsManaged (handle)) return new Rectangle (0, 0, 0, 0);
+	int [] argList = {OS.XmNx, 0, OS.XmNy, 0, OS.XmNwidth, 0, OS.XmNheight, 0};
+	OS.XtGetValues (handle, argList, argList.length / 2);
+	return new Rectangle ((short) argList [1], (short) argList [3], argList [5], argList [7]);
 }
 /**
  * Returns <code>true</code> if the receiver is enabled, and
- * <code>false</code> otherwise. A disabled control is typically
+ * <code>false</code> otherwise. A disabled menu item is typically
  * not selectable from the user interface and draws with an
  * inactive or "grayed" look.
  *
@@ -413,7 +423,7 @@ public boolean getSelection () {
 }
 void hookEvents () {
 	if ((style & SWT.SEPARATOR) != 0) return;
-	int windowProc = getDisplay ().windowProc;
+	int windowProc = display.windowProc;
 	OS.XtAddCallback (handle, OS.XmNhelpCallback, windowProc, HELP_CALLBACK);
 	if ((style & SWT.CASCADE) != 0) {
 		OS.XtAddCallback (handle, OS.XmNactivateCallback, windowProc, ACTIVATE_CALLBACK);
@@ -432,13 +442,14 @@ boolean isAccelActive () {
 	while (menu != null && menu.cascade != null) {
 		menu = menu.cascade.parent;
 	}
+	if (menu == null) return false;
 	Decorations shell = menu.parent;
 	return shell.menuBar == menu;
 }
 /**
  * Returns <code>true</code> if the receiver is enabled and all
  * of the receiver's ancestors are enabled, and <code>false</code>
- * otherwise. A disabled control is typically not selectable from the
+ * otherwise. A disabled menu item is typically not selectable from the
  * user interface and draws with an inactive or "grayed" look.
  *
  * @return the receiver's enabled state
@@ -455,20 +466,20 @@ public boolean isEnabled () {
 }
 String keysymName (int keysym) {
 	switch (keysym) {
-		case 8: return "BackSpace";
-		case 9: return "Tab";
+		case SWT.BS: return "BackSpace";
+		case SWT.TAB: return "Tab";
 		/*
 		* Bug in Motif. For some reason, the XmNaccelerator
 		* resource will not accept XK_Linefeed and prints Xt
 		* warnings.  The fix is to use XK_Return instead.
 		*/
-//		case 10:
+//		case SWT.LF:
 //		case OS.XK_Linefeed: return "Linefeed";
-		case 10:
+		case SWT.LF:
 		case OS.XK_Linefeed:
-		case 13: return "Return";
-		case 27: return "Escape";
-		case 127: return "Delete";
+		case SWT.CR: return "Return";
+		case SWT.ESC: return "Escape";
+		case SWT.DEL: return "Delete";
 	}
 	if (('0' <= keysym && keysym <= '9') ||
 		('a' <= keysym && keysym <= 'z') ||
@@ -490,14 +501,19 @@ String keysymName (int keysym) {
 void manageChildren () {
 	OS.XtManageChild (handle);
 }
-void releaseChild () {
-	super.releaseChild ();
+void releaseChildren (boolean destroy) {
+	if (menu != null) {
+		menu.release (false);
+		menu = null;
+	}
+	super.releaseChildren (destroy);
+}
+void releaseParent () {
+	super.releaseParent ();
 	if (menu != null) menu.dispose ();
 	menu = null;
 }
 void releaseWidget () {
-	if (menu != null && !menu.isDisposed ()) menu.releaseResources ();
-	menu = null;
 	super.releaseWidget ();
 	accelerator = 0;
 	if (this == parent.defaultItem) {
@@ -526,7 +542,7 @@ void removeAccelerators () {
  * Removes the listener from the collection of listeners who will
  * be notified when the arm events are generated for the control.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -549,7 +565,7 @@ public void removeArmListener (ArmListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the help events are generated for the control.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -572,7 +588,7 @@ public void removeHelpListener (HelpListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is selected.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -607,6 +623,8 @@ void selectRadio () {
  * OR of zero or more modifier masks and a key. Examples:
  * <code>SWT.MOD1 | SWT.MOD2 | 'T', SWT.MOD3 | SWT.F2</code>.
  * <code>SWT.CONTROL | SWT.SHIFT | 'T', SWT.ALT | SWT.F2</code>.
+ * The default value is zero, indicating that the menu item does
+ * not have an accelerator.
  *
  * @param accelerator an integer that is the bit-wise OR of masks and a key
  *
@@ -630,7 +648,7 @@ public void setAccelerator (int accelerator) {
 }
 /**
  * Enables the receiver if the argument is <code>true</code>,
- * and disables it otherwise. A disabled control is typically
+ * and disables it otherwise. A disabled menu item is typically
  * not selectable from the user interface and draws with an
  * inactive or "grayed" look.
  *
@@ -643,8 +661,16 @@ public void setAccelerator (int accelerator) {
  */
 public void setEnabled (boolean enabled) {
 	checkWidget();
+	if (getEnabled () == enabled) return;
 	int [] argList = {OS.XmNsensitive, enabled ? 1 : 0};
 	OS.XtSetValues (handle, argList, argList.length / 2);
+	if (isAccelActive ()) {
+		if (enabled) {
+			addAccelerator ();
+		} else {
+			removeAccelerator ();
+		}
+	}
 }
 /**
  * Sets the receiver's pull down menu to the argument.
@@ -652,6 +678,11 @@ public void setEnabled (boolean enabled) {
  * pull down menu. The sequence of key strokes, button presses
  * and/or button releases that are used to request a pull down
  * menu is platform specific.
+ * <p>
+ * Note: Disposing of a menu item that has a pull down menu
+ * will dispose of the menu.  To avoid this behavior, set the
+ * menu to null before the menu item is disposed.
+ * </p>
  *
  * @param menu the new pull down menu
  *
@@ -741,14 +772,14 @@ public void setSelection (boolean selected) {
  * Sets the receiver's text. The string may include
  * the mnemonic character and accelerator text.
  * <p>
- * Mnemonics are indicated by an '&amp' that causes the next
+ * Mnemonics are indicated by an '&amp;' that causes the next
  * character to be the mnemonic.  When the user presses a
  * key sequence that matches the mnemonic, a selection
  * event occurs. On most platforms, the mnemonic appears
  * underlined but may be emphasised in a platform specific
- * manner.  The mnemonic indicator character '&amp' can be
+ * manner.  The mnemonic indicator character '&amp;' can be
  * escaped by doubling it in the string, causing a single
- *'&amp' to be displayed.
+ * '&amp;' to be displayed.
  * </p>
  * <p>
  * Accelerator text is indicated by the '\t' character.
@@ -801,32 +832,38 @@ public void setText (String string) {
 	boolean accel = false;
 	int i=0, j=0, mnemonic=0;
 	while (i < text.length) {
-		if (text [i] == '\t') {accel = true; break;};
-		if ((text [j++] = text [i++]) == Mnemonic) {
+		if (text [i] == '\t') {accel = true; break;}
+		if ((text [j++] = text [i++]) == '&') {
 			if (i == text.length) {continue;}
-			if (text [i] == Mnemonic) {i++; continue;}
+			if (text [i] == '&') {i++; continue;}
 			if (mnemonic == 0) mnemonic = text [i];
 			j--;
 		}
 	}
-	byte [] buffer2;
+	int xmString2 = 0;
 	if (accel && ++i < text.length) {
 		char [] accelText = new char [text.length - i];
 		System.arraycopy (text, i, accelText, 0, accelText.length);
 		/* Use the character encoding for the default locale */
-		buffer2 = Converter.wcsToMbcs (null, accelText, true);
+		byte [] buffer2 = Converter.wcsToMbcs (null, accelText, true);
+		xmString2 = OS.XmStringParseText (
+			buffer2,
+			0,
+			OS.XmFONTLIST_DEFAULT_TAG, 
+			OS.XmCHARSET_TEXT, 
+			null,
+			0,
+			0);
+		if (xmString2 == 0) error (SWT.ERROR_CANNOT_SET_TEXT);
 	} else {
-		buffer2 = new byte [1];
+		/*
+		* Bug in linux.  In some versions of linux motif setting a menu item's
+		* accelerator to NULL will cause a GP.  The workaround is to instead
+		* set these accelerators to a functionally equivalent non-null value.
+		*/
+		xmString2 = OS.XmStringGenerate (new byte[1], null, OS.XmCHARSET_TEXT, null);
+		if (xmString2 == 0) error (SWT.ERROR_CANNOT_SET_TEXT);
 	}
-	int xmString2 = OS.XmStringParseText (
-		buffer2,
-		0,
-		OS.XmFONTLIST_DEFAULT_TAG, 
-		OS.XmCHARSET_TEXT, 
-		null,
-		0,
-		0);
-	if (xmString2 == 0) error (SWT.ERROR_CANNOT_SET_TEXT);
 	while (j < text.length) text [j++] = 0;
 	/* Use the character encoding for the default locale */
 	byte [] buffer1 = Converter.wcsToMbcs (null, text, true);
@@ -841,7 +878,6 @@ public void setText (String string) {
 	if (xmString1 == 0) error (SWT.ERROR_CANNOT_SET_TEXT);
 	if (mnemonic == 0) mnemonic = OS.XK_VoidSymbol;
 	int [] argList = {
-		OS.XmNlabelType, OS.XmSTRING,
 		OS.XmNlabelString, xmString1,
 		OS.XmNmnemonic, mnemonic,
 		OS.XmNacceleratorText, xmString2,
@@ -850,9 +886,9 @@ public void setText (String string) {
 	if (xmString1 != 0) OS.XmStringFree (xmString1);
 	if (xmString2 != 0) OS.XmStringFree (xmString2);
 }
-boolean translateAccelerator (int accel) {
+boolean translateAccelerator (int accel, boolean doit) {
 	if (!getEnabled ()) return false;
-	if (menu != null) return menu.translateAccelerator (accel);
+	if (menu != null) return menu.translateAccelerator (accel, doit);
 	int accelerator = this.accelerator;
 	if ((accelerator & SWT.KEYCODE_BIT) == 0) {
 		int key = accelerator & SWT.KEY_MASK;
@@ -863,14 +899,14 @@ boolean translateAccelerator (int accel) {
 		accelerator = mods | key;
 	}
 	if (accelerator == accel) {
-		postEvent (SWT.Selection);
+		if (doit) postEvent (SWT.Selection);
 		return true;
 	}
 	return false;
 }
 int XmNactivateCallback (int w, int client_data, int call_data) {
 	if ((style & SWT.CASCADE) != 0) {
-		postEvent (SWT.Arm);
+		sendEvent (SWT.Arm);
 	}
 	if (!isEnabled ()) return 0;
 	XmAnyCallbackStruct struct = new XmAnyCallbackStruct ();
@@ -878,14 +914,14 @@ int XmNactivateCallback (int w, int client_data, int call_data) {
 	Event event = new Event ();
 	if (struct.event != 0) {
 		XButtonEvent xEvent = new XButtonEvent ();
-		OS.memmove (xEvent, struct.event, XAnyEvent.sizeof);
+		OS.memmove (xEvent, struct.event, XButtonEvent.sizeof);
 		event.time = xEvent.time;
 		switch (xEvent.type) {
 			case OS.ButtonPress:
 			case OS.ButtonRelease:
 			case OS.KeyPress:
 			case OS.KeyRelease:
-				setInputState (event, xEvent);
+				setInputState (event, xEvent.state);
 				break;
 		}
 	}
@@ -893,11 +929,17 @@ int XmNactivateCallback (int w, int client_data, int call_data) {
 	return 0;
 }
 int XmNarmCallback (int w, int client_data, int call_data) {
-	postEvent (SWT.Arm);
+	sendEvent (SWT.Arm);
 	return 0;
 }
 int XmNcascadingCallback (int w, int client_data, int call_data) {
-	postEvent (SWT.Arm);
+	/*
+	* Bug in Motif.  When XmNlabelString is set as a result of
+	* an XmNcascadingCallback after the callback has returned,
+	* Motif measures the new string properly but does not draw
+	* it.  The fix is to send rather than post the SWT.Arm event.
+	*/
+	sendEvent (SWT.Arm);
 	return 0;
 }
 int XmNhelpCallback (int w, int client_data, int call_data) {
@@ -915,14 +957,14 @@ int XmNvalueChangedCallback (int w, int client_data, int call_data) {
 	Event event = new Event ();
 	if (struct.event != 0) {
 		XButtonEvent xEvent = new XButtonEvent ();
-		OS.memmove (xEvent, struct.event, XAnyEvent.sizeof);
+		OS.memmove (xEvent, struct.event, XButtonEvent.sizeof);
 		event.time = xEvent.time;
 		switch (xEvent.type) {
 			case OS.ButtonPress:
 			case OS.ButtonRelease:
 			case OS.KeyPress:
 			case OS.KeyRelease:
-				setInputState (event, xEvent);
+				setInputState (event, xEvent.state);
 				break;
 		}
 	}
