@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -357,18 +357,27 @@ Decorations menuShell () {
 	return this;
 }
 
-void releaseWidget () {
+void releaseChildren (boolean destroy) {
+	if (menuBar != null) {
+		menuBar.release (false);
+		menuBar = null;
+	}
+	super.releaseChildren (destroy);
 	if (menus != null) {
 		for (int i=0; i<menus.length; i++) {
 			Menu menu = menus [i];
-			if (menu != null && !menu.isDisposed ()) menu.releaseResources ();
+			if (menu != null && !menu.isDisposed ()) {
+				menu.release (false);
+			} 
 		}
+		menus = null;
 	}
-	menuBar = null;
-	menus = null;
+}
+
+void releaseWidget () {
+	super.releaseWidget ();
 	image = null;
 	images = null;
-	super.releaseWidget ();
 	defaultButton = saveDefault = null;
 	text = null;
 }
@@ -425,6 +434,7 @@ void resizeBounds (int width, int height) {
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_INVALID_ARGUMENT - if the button has been disposed</li> 
+ *    <li>ERROR_INVALID_PARENT - if the control is not in the same widget tree</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -433,6 +443,10 @@ void resizeBounds (int width, int height) {
  */
 public void setDefaultButton (Button button) {
 	checkWidget();
+	if (button != null) {
+		if (button.isDisposed()) error (SWT.ERROR_INVALID_ARGUMENT);
+		if (button.menuShell () != this) error (SWT.ERROR_INVALID_PARENT);
+	}
 	setDefaultButton (button, true);
 }
 void setDefaultButton (Button button, boolean save) {
@@ -442,7 +456,6 @@ void setDefaultButton (Button button, boolean save) {
 			return;
 		}
 	} else {
-		if (button.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
 		if ((button.style & SWT.PUSH) == 0) return;
 		if (button == defaultButton) return;
 	}
