@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -21,7 +21,7 @@ import org.eclipse.swt.events.*;
  * objects that represent a range of positive, numeric values. 
  * <p>
  * At any given moment, a given scroll bar will have a 
- * single <em>selection</em> that is considered to be its
+ * single 'selection' that is considered to be its
  * value, which is constrained to be within the range of
  * values the scroll bar represents (that is, between its
  * <em>minimum</em> and <em>maximum</em> values).
@@ -107,7 +107,7 @@ static int checkStyle (int style) {
  * interface.
  * <p>
  * When <code>widgetSelected</code> is called, the event object detail field contains one of the following values:
- * <code>0</code> - for the end of a drag.
+ * <code>SWT.NONE</code> - for the end of a drag.
  * <code>SWT.DRAG</code>.
  * <code>SWT.HOME</code>.
  * <code>SWT.END</code>.
@@ -143,7 +143,6 @@ public void addSelectionListener (SelectionListener listener) {
 void createHandle (int index) {
 	state |= HANDLE;
 	if (handle != 0) return;
-	Display display = getDisplay ();
 	int parentHandle = parent.scrolledHandle;
 	int orientation, sizeArg, size, basicFlags;
 	if ((style & SWT.HORIZONTAL) != 0) {
@@ -159,7 +158,7 @@ void createHandle (int index) {
 	}
 	int [] args = {
 		sizeArg, size, 0,
-		OS.Pt_ARG_MAXIMUM, 100, 0,
+		OS.Pt_ARG_MAXIMUM, 99, 0,
 		OS.Pt_ARG_PAGE_INCREMENT, 10, 0,
 		OS.Pt_ARG_SLIDER_SIZE, 10, 0,
 		OS.Pt_ARG_BASIC_FLAGS, basicFlags, ~0,
@@ -171,12 +170,6 @@ void createHandle (int index) {
 	if (handle == 0) error (SWT.ERROR_NO_HANDLES);
 }
 
-public Display getDisplay () {
-	Scrollable parent = this.parent;
-	if (parent == null) error (SWT.ERROR_WIDGET_DISPOSED);
-	return parent.getDisplay ();
-}
-
 /**
  * Returns <code>true</code> if the receiver is enabled, and
  * <code>false</code> otherwise. A disabled control is typically
@@ -248,7 +241,7 @@ public int getMaximum () {
 	checkWidget();
 	int [] args = {OS.Pt_ARG_MAXIMUM, 0, 0};
 	OS.PtGetResources (handle, args.length / 3, args);
-	return args [1];
+	return args [1] + 1;
 }
 
 /**
@@ -271,7 +264,7 @@ public int getPageIncrement () {
 }
 
 /**
- * Returns the receiver's parent, which must be scrollable.
+ * Returns the receiver's parent, which must be a Scrollable.
  *
  * @return the receiver's parent
  *
@@ -286,7 +279,7 @@ public Scrollable getParent () {
 }
 
 /**
- * Returns the single <em>selection</em> that is the receiver's value.
+ * Returns the single 'selection' that is the receiver's value.
  *
  * @return the selection
  *
@@ -323,7 +316,7 @@ public Point getSize () {
 }
 
 /**
- * Answers the size of the receiver's thumb relative to the
+ * Returns the size of the receiver's thumb relative to the
  * difference between its maximum and minimum values.
  *
  * @return the thumb value
@@ -366,7 +359,7 @@ public boolean getVisible () {
 }
 
 void hookEvents () {
-	int windowProc = getDisplay ().windowProc;
+	int windowProc = display.windowProc;
 	OS.PtAddCallback (handle, OS.Pt_CB_SCROLL_MOVE, windowProc, OS.Pt_CB_SCROLL_MOVE);
 }
 
@@ -469,6 +462,12 @@ public void removeSelectionListener (SelectionListener listener) {
 	eventTable.unhook (SWT.DefaultSelection,listener);	
 }
 
+void releaseParent () {
+	super.releaseParent ();
+	if (parent.horizontalBar == this) parent.horizontalBar = null;
+	if (parent.verticalBar == this) parent.verticalBar = null;
+}
+
 void setBounds (int x, int y, int width, int height) {
 	PhArea_t area = new PhArea_t ();
 	area.pos_x = (short) x;
@@ -519,11 +518,12 @@ public void setIncrement (int value) {
 }
 
 /**
- * Sets the maximum value which the receiver will allow
- * to be the argument which must be greater than or
- * equal to zero.
+ * Sets the maximum. If this value is negative or less than or
+ * equal to the minimum, the value is ignored. If necessary, first
+ * the thumb and then the selection are adjusted to fit within the
+ * new range.
  *
- * @param value the new maximum (must be zero or greater)
+ * @param value the new maximum
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -532,15 +532,32 @@ public void setIncrement (int value) {
  */
 public void setMaximum (int value) {
 	checkWidget();
-	OS.PtSetResource (handle, OS.Pt_ARG_MAXIMUM, value - 1, 0);
+	int [] args = {
+		OS.Pt_ARG_MAXIMUM, 0, 0,
+		OS.Pt_ARG_MINIMUM, 0, 0,
+		OS.Pt_ARG_SLIDER_SIZE, 0, 0,
+		OS.Pt_ARG_GAUGE_VALUE, 0, 0,
+	};
+	OS.PtGetResources (handle, args.length / 3, args);
+	int minimum = args [4];
+	if (value <= minimum) return;
+	int thumb = args [7];
+	thumb = Math.min (thumb, value - minimum);
+	int selection = args [10];
+	selection = Math.min (selection, value - thumb);
+	args [1] = value - 1;
+	args [7] = thumb;
+	args [10] = selection;
+	OS.PtSetResources (handle, args.length / 3, args);
 }
 
 /**
- * Sets the minimum value which the receiver will allow
- * to be the argument which must be greater than or
- * equal to zero.
+ * Sets the minimum value. If this value is negative or greater
+ * than or equal to the maximum, the value is ignored. If necessary,
+ * first the thumb and then the selection are adjusted to fit within
+ * the new range.
  *
- * @param value the new minimum (must be zero or greater)
+ * @param value the new minimum
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -549,7 +566,24 @@ public void setMaximum (int value) {
  */
 public void setMinimum (int value) {
 	checkWidget();
-	OS.PtSetResource (handle, OS.Pt_ARG_MINIMUM, value, 0);
+	if (value < 0) return;
+	int [] args = {
+		OS.Pt_ARG_MAXIMUM, 0, 0,
+		OS.Pt_ARG_MINIMUM, 0, 0,
+		OS.Pt_ARG_SLIDER_SIZE, 0, 0,
+		OS.Pt_ARG_GAUGE_VALUE, 0, 0,
+	};
+	OS.PtGetResources (handle, args.length / 3, args);
+	int maximum = args [1] + 1;
+	if (value >= maximum) return;
+	int thumb = args [7];
+	thumb = Math.min (thumb, maximum - value);
+	int selection = args [10];
+	selection = Math.max (selection, value);
+	args [4] = value;
+	args [7] = thumb;
+	args [10] = selection;
+	OS.PtSetResources (handle, args.length / 3, args);
 }
 
 /**
@@ -558,7 +592,7 @@ public void setMinimum (int value) {
  * are selected to the argument, which must be at least
  * one.
  *
- * @return the page increment (must be greater than zero)
+ * @param value the page increment (must be greater than zero)
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -575,7 +609,7 @@ public void setPageIncrement (int value) {
  * value to the argument which must be greater than or equal
  * to zero.
  *
- * @param value the new selection (must be zero or greater)
+ * @param selection the new selection (must be zero or greater)
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -589,20 +623,26 @@ public void setSelection (int value) {
 
 /**
  * Sets the size of the receiver's thumb relative to the
- * difference between its maximum and minimum values to the
- * argument which must be at least one.
+ * difference between its maximum and minimum values.  This new
+ * value will be ignored if it is less than one, and will be
+ * clamped if it exceeds the receiver's current range.
  *
- * @param value the new thumb value (must be at least one)
+ * @param value the new thumb value, which must be at least one and not
+ * larger than the size of the current range
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- *
- * @see ScrollBar
  */
 public void setThumb (int value) {
 	checkWidget();
+	if (value < 1) return;
+	int [] args = {OS.Pt_ARG_MINIMUM, 0, 0, OS.Pt_ARG_MAXIMUM, 0, 0}; 
+	OS.PtGetResources (handle, args.length / 3, args);
+	int minimum = args [1];
+	int maximum = args [4] + 1;
+	value = Math.min (value, maximum - minimum);
 	OS.PtSetResource (handle, OS.Pt_ARG_SLIDER_SIZE, value, 0);
 }
 
@@ -610,7 +650,7 @@ public void setThumb (int value) {
  * Sets the receiver's selection, minimum value, maximum
  * value, thumb, increment and page increment all at once.
  * <p>
- * Note: This is equivalent to setting the values individually
+ * Note: This is similar to setting the values individually
  * using the appropriate methods, but may be implemented in a 
  * more efficient fashion on some platforms.
  * </p>
@@ -632,16 +672,16 @@ public void setValues (int selection, int minimum, int maximum, int thumb, int i
 	if (minimum < 0) return;
 	if (maximum < 0) return;
 	if (thumb < 1) return;
-	if (maximum - minimum - thumb < 0) return;
 	if (increment < 1) return;
 	if (pageIncrement < 1) return;
+	thumb = Math.min (thumb, maximum - minimum);
 	int [] args = {
+		OS.Pt_ARG_MAXIMUM, maximum - 1, 0,
+		OS.Pt_ARG_MINIMUM, minimum, 0,
+		OS.Pt_ARG_SLIDER_SIZE, thumb, 0,
 		OS.Pt_ARG_GAUGE_VALUE, selection, 0,
 		OS.Pt_ARG_INCREMENT, increment, 0,
 		OS.Pt_ARG_PAGE_INCREMENT, pageIncrement, 0,
-		OS.Pt_ARG_SLIDER_SIZE, thumb, 0,
-		OS.Pt_ARG_MINIMUM, minimum, 0,
-		OS.Pt_ARG_MAXIMUM, maximum - 1, 0,
 	};
 	OS.PtSetResources (handle, args.length / 3, args);
 }
@@ -665,20 +705,18 @@ public void setValues (int selection, int minimum, int maximum, int thumb, int i
 public void setVisible (boolean visible) {
 	checkWidget ();
 	int topHandle = topHandle ();
+	int oldFlags = OS.PtWidgetFlags (topHandle);
 	int flags = visible ? 0 : OS.Pt_DELAY_REALIZE;
 	OS.PtSetResource (topHandle, OS.Pt_ARG_FLAGS, flags, OS.Pt_DELAY_REALIZE);
-	if (OS.PtWidgetIsRealized (parent.handle)) {
-		if (visible != OS.PtWidgetIsRealized (topHandle)) {
-			if (visible) {
-				sendEvent (SWT.Show);
-				OS.PtRealizeWidget (topHandle);
-			} else {
-				OS.PtUnrealizeWidget (topHandle);
-				sendEvent(SWT.Hide);
-			}
-		}
-	}
+	if ((oldFlags & OS.Pt_DELAY_REALIZE) == flags) return;
 	parent.resizeClientArea ();
+	if (visible) {
+		sendEvent (SWT.Show);
+		OS.PtRealizeWidget (topHandle);
+	} else {
+		OS.PtUnrealizeWidget (topHandle);
+		sendEvent(SWT.Hide);
+	}
 }
 
 }
