@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -23,7 +23,7 @@ import org.eclipse.swt.events.*;
  * <p>
  * <dl>
  * <dt><b>Styles:</b></dt>
- * <dd>CENTER, LEFT, MULTI, SINGLE, RIGHT, READ_ONLY, WRAP</dd>
+ * <dd>CENTER, LEFT, MULTI, PASSWORD, SINGLE, RIGHT, READ_ONLY, WRAP</dd>
  * <dt><b>Events:</b></dt>
  * <dd>DefaultSelection, Modify, Verify</dd>
  * </dl>
@@ -40,8 +40,22 @@ public class Text extends Scrollable {
 	int tabs, lastModifiedText;
 	PtTextCallback_t textVerify;
 	
+	/**
+	* The maximum number of characters that can be entered
+	* into a text widget.
+	* <p>
+	* Note that this value is platform dependent, based upon
+	* the native widget implementation.
+	* </p>
+	*/
 	public static final int LIMIT;
+	/**
+	* The delimiter used by multi-line text widgets.  When text
+	* is queried and from the widget, it will be delimited using
+	* this delimiter.
+	*/
 	public static final String DELIMITER;
+	static final char PASSWORD = '*';
 	/*
 	* These values can be different on different platforms.
 	* Therefore they are not initialized in the declaration
@@ -88,13 +102,18 @@ public Text (Composite parent, int style) {
 }
 
 static int checkStyle (int style) {
+	if ((style & SWT.SINGLE) != 0 && (style & SWT.MULTI) != 0) {
+		style &= ~SWT.MULTI;
+	}
 	style = checkBits (style, SWT.LEFT, SWT.CENTER, SWT.RIGHT, 0, 0, 0);
 	if ((style & SWT.SINGLE) != 0) style &= ~(SWT.H_SCROLL | SWT.V_SCROLL | SWT.WRAP);
-	if ((style & SWT.WRAP) != 0) style |= SWT.MULTI;
-	if ((style & (SWT.SINGLE | SWT.MULTI)) != 0) return style;
-	if ((style & (SWT.H_SCROLL | SWT.V_SCROLL)) != 0) {
-		return style | SWT.MULTI;
+	if ((style & SWT.WRAP) != 0) {
+		style |= SWT.MULTI;
+		style &= ~SWT.H_SCROLL;
 	}
+	if ((style & SWT.MULTI) != 0) style &= ~SWT.PASSWORD;
+	if ((style & (SWT.SINGLE | SWT.MULTI)) != 0) return style;
+	if ((style & (SWT.H_SCROLL | SWT.V_SCROLL)) != 0) return style | SWT.MULTI;
 	return style | SWT.SINGLE;
 }
 public Point computeSize (int wHint, int hHint, boolean changed) {
@@ -162,7 +181,6 @@ public void clearSelection () {
 }
 void createHandle (int index) {
 	state |= HANDLE;
-	Display display = getDisplay ();
 	int parentHandle = parent.parentingHandle ();
 	boolean hasBorder = (style & SWT.BORDER) != 0;
 	int textFlags = (style & SWT.READ_ONLY) != 0 ? 0 : OS.Pt_EDITABLE;
@@ -198,6 +216,8 @@ void createWidget (int index) {
 	super.createWidget (index);
 //	doubleClick = true;
 	setTabStops (tabs = 8);
+	hiddenText = "";
+	if ((style & SWT.PASSWORD) != 0) setEchoChar (PASSWORD);
 }
 
 /**
@@ -294,7 +314,7 @@ public void addVerifyListener (VerifyListener listener) {
  * @param string the string to be appended
  *
  * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
+ *    <li>ERROR_NULL_ARGUMENT - if the string is null</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -339,9 +359,6 @@ public void copy () {
  * clipboard and then deleted from the widget.
  * </p>
  *
- * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
- * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -378,26 +395,21 @@ void deregister () {
 }
 
 int defaultBackground () {
-	Display display = getDisplay ();
 	return display.TEXT_BACKGROUND;
 }
 
 int defaultForeground () {
-	Display display = getDisplay ();
 	return display.TEXT_FOREGROUND;
 }
 
 /**
- * Gets the line number of the caret.
+ * Returns the line number of the caret.
  * <p>
  * The line number of the caret is returned.
  * </p>
  *
  * @return the line number
  *
- * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
- * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -410,7 +422,8 @@ public int getCaretLineNumber () {
 }
 
 /**
- * Gets the location the caret.
+ * Returns a point describing the receiver's location relative
+ * to its parent (or its display if its parent is null).
  * <p>
  * The location of the caret is returned.
  * </p>
@@ -429,9 +442,9 @@ public Point getCaretLocation () {
 }
 
 /**
- * Gets the position of the caret.
+ * Returns the character position of the caret.
  * <p>
- * The character position of the caret is returned.
+ * Indexing is zero based.
  * </p>
  *
  * @return the position of the caret
@@ -449,7 +462,7 @@ public int getCaretPosition () {
 }
 
 /**
- * Gets the number of characters.
+ * Returns the number of characters.
  *
  * @return number of characters in the widget
  *
@@ -467,12 +480,14 @@ public int getCharCount () {
 }
 
 /**
- * Gets the double click enabled flag.
+ * Returns the double click enabled flag.
  * <p>
  * The double click flag enables or disables the
  * default action of the text widget when the user
  * double clicks.
  * </p>
+ * 
+ * @return whether or not double click is enabled
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -486,17 +501,21 @@ public boolean getDoubleClickEnabled () {
 }
 
 /**
- * Gets the echo character.
+ * Returns the echo character.
  * <p>
  * The echo character is the character that is
  * displayed when the user enters text or the
  * text is changed by the programmer.
  * </p>
+ * 
+ * @return the echo character
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ * 
+ * @see #setEchoChar
  */
 public char getEchoChar () {
 	checkWidget();
@@ -504,8 +523,10 @@ public char getEchoChar () {
 }
 
 /**
- * Gets the editable state.
+ * Returns the editable state.
  *
+ * @return whether or not the receiver is editable
+ * 
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -519,7 +540,7 @@ public boolean getEditable () {
 }
 
 /**
- * Gets the number of lines.
+ * Returns the number of lines.
  *
  * @return the number of lines in the widget
  *
@@ -537,7 +558,7 @@ public int getLineCount () {
 }
 
 /**
- * Gets the line delimiter.
+ * Returns the line delimiter.
  *
  * @return a string that is the line delimiter
  *
@@ -545,6 +566,8 @@ public int getLineCount () {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ * 
+ * @see #DELIMITER
  */
 public String getLineDelimiter () {
 	checkWidget();
@@ -552,7 +575,7 @@ public String getLineDelimiter () {
 }
 
 /**
- * Gets the height of a line.
+ * Returns the height of a line.
  *
  * @return the height of a row of text
  *
@@ -595,9 +618,10 @@ String getNameText () {
 }
 
 /**
- * Returns the orientation of the receiver.
+ * Returns the orientation of the receiver, which will be one of the
+ * constants <code>SWT.LEFT_TO_RIGHT</code> or <code>SWT.RIGHT_TO_LEFT</code>.
  *
- * @return the orientation bit.
+ * @return the orientation style
  * 
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -612,14 +636,17 @@ public int getOrientation () {
 }
 
 /**
- * Gets the position of the selected text.
+ * Returns a <code>Point</code> whose x coordinate is the
+ * character position representing the start of the selected
+ * text, and whose y coordinate is the character position
+ * representing the end of the selection. An "empty" selection
+ * is indicated by the x and y coordinates having the same value.
  * <p>
- * Indexing is zero based.  The range of
- * a selection is from 0..N where N is
- * the number of characters in the widget.
+ * Indexing is zero based.  The range of a selection is from
+ * 0..N where N is the number of characters in the widget.
  * </p>
- * 
- * @return the start and end of the selection
+ *
+ * @return a point representing the selection start and end
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -642,7 +669,7 @@ public Point getSelection () {
 }
 
 /**
- * Gets the number of selected characters.
+ * Returns the number of selected characters.
  *
  * @return the number of selected characters.
  *
@@ -658,7 +685,7 @@ public int getSelectionCount () {
 }
 
 /**
- * Gets the selected text.
+ * Gets the selected text, or an empty string if there is no current selection.
  *
  * @return the selected text
  * 
@@ -679,7 +706,7 @@ public String getSelectionText () {
 }
 
 /**
- * Gets the number of tabs.
+ * Returns the number of tabs.
  * <p>
  * Tab stop spacing is specified in terms of the
  * space (' ') character.  The width of a single
@@ -711,7 +738,8 @@ int getTabWidth (int tabs) {
 }
 
 /**
- * Gets a range of text.
+ * Returns a range of text.  Returns an empty string if the
+ * start of the range is greater than the end.
  * <p>
  * Indexing is zero based.  The range of
  * a selection is from 0..N-1 where N is
@@ -728,20 +756,26 @@ int getTabWidth (int tabs) {
  * </ul>
  */
 public String getText (int start, int end) {
-	checkWidget();
+	checkWidget ();
+	if (!(start <= end && 0 <= end)) return "";
+	String text = getText ();
+	int length = text.length ();
+	start = Math.max (0, start);
+	end = Math.min (end, length - 1);
 	/*
 	* NOTE: The current implementation uses substring ()
 	* which can reference a potentially large character
 	* array.
 	*/
 	//NOT DONE - use OS in SINGLE text
-	return getText ().substring (start, end + 1);
+	return text.substring (start, end + 1);
 }
 
 /**
- * Gets the widget text.
+ * Returns the widget text.
  * <p>
- * The text for a text widget is the characters in the widget.
+ * The text for a text widget is the characters in the widget, or
+ * an empty string if this has never been set.
  * </p>
  *
  * @return the widget text
@@ -777,6 +811,8 @@ public String getText () {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ * 
+ * @see #LIMIT
  */
 public int getTextLimit () {
 	checkWidget();
@@ -808,7 +844,7 @@ public int getTopIndex () {
 }
 
 /**
- * Gets the top pixel.
+ * Returns the top pixel.
  * <p>
  * The top pixel is the pixel position of the line
  * that is currently at the top of the widget.  On
@@ -835,7 +871,7 @@ public int getTopPixel () {
 
 void hookEvents () {
 	super.hookEvents ();
-	int windowProc = getDisplay ().windowProc;
+	int windowProc = display.windowProc;
 	OS.PtAddCallback (handle, OS.Pt_CB_MODIFY_VERIFY, windowProc, OS.Pt_CB_MODIFY_VERIFY); 
 	OS.PtAddCallback (handle, OS.Pt_CB_TEXT_CHANGED, windowProc, OS.Pt_CB_TEXT_CHANGED);
 }
@@ -848,6 +884,9 @@ void hookEvents () {
  *
  * @param string the string
  *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the string is <code>null</code></li>
+ * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -896,6 +935,29 @@ public void paste () {
 	OS.free(ptr);
 }
 
+int Ph_EV_BOUNDARY (int widget, int info) {
+	/*
+	* Bug in Photon.  PtMultiText reports boundary events for
+	* the internal (hidden) child widget.  This makes it appear as
+	* though the pointer leaves as soon as it enters the widget
+	* area.  The fix is to filter out these theser events.
+	*/
+	if ((style & SWT.MULTI) != 0) {
+		if (info == 0) return OS.Pt_END;
+		PtCallbackInfo_t cbinfo = new PtCallbackInfo_t ();
+		OS.memmove (cbinfo, info, PtCallbackInfo_t.sizeof);
+		if (cbinfo.event == 0) return OS.Pt_END;
+		PhEvent_t ev = new PhEvent_t ();
+		OS.memmove (ev, cbinfo.event, PhEvent_t.sizeof);
+		switch ((int) ev.subtype) {
+			case OS.Ph_EV_PTR_ENTER_FROM_CHILD:
+			case OS.Ph_EV_PTR_LEAVE_TO_CHILD:
+				return OS.Pt_CONTINUE;
+		}
+	}
+	return super.Ph_EV_BOUNDARY (widget, info);
+}
+
 int Pt_CB_GOT_FOCUS (int widget, int info) {
 	
 	/*
@@ -938,7 +1000,7 @@ int Pt_CB_MODIFY_VERIFY (int widget, int info) {
 				PhKeyEvent_t ke = new PhKeyEvent_t ();
 				OS.memmove (ke, data, PhKeyEvent_t.sizeof);
 				if ((ke.key_flags & (OS.Pk_KF_Key_Down | OS.Pk_KF_Key_Repeat)) != 0) {
-					setKeyState (event, ke);
+					setKeyState (event, SWT.Verify, ke);
 				}
 			}
 		}
@@ -964,7 +1026,7 @@ int Pt_CB_MODIFY_VERIFY (int widget, int info) {
 				OS.memmove(textVerify.text, buffer2, length);
 			} else {
 				int ptr = OS.malloc (length);
-				OS.memmove (ptr, buffer2, buffer2.length);
+				OS.memmove (ptr, buffer2, length);
 				textVerify.new_insert += length - textVerify.length;
 				textVerify.text = ptr;
 				textVerify.length = length;
@@ -1037,7 +1099,7 @@ public void removeModifyListener (ModifyListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is selected.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -1062,7 +1124,7 @@ public void removeSelectionListener (SelectionListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is verified.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -1104,8 +1166,9 @@ public void selectAll () {
  * the echo character to '\0' clears the echo
  * character and redraws the original text.
  * If for any reason the echo character is invalid,
- * the default echo character for the platform
- * is used.
+ * or if the platform does not allow modification
+ * of the echo character, the default echo character
+ * for the platform is used.
  * </p>
  *
  * @param echo the new echo character
@@ -1117,11 +1180,12 @@ public void selectAll () {
  */
 public void setEchoChar (char echo) {
 	checkWidget();
+	if ((style & SWT.MULTI) != 0) return;
 	if (echoCharacter == echo) return;
 	String newText;
 	if (echo == 0) {
 		newText = hiddenText;
-		hiddenText = null;
+		hiddenText = "";
 	} else {
 		newText = hiddenText = getText();
 	}
@@ -1140,6 +1204,9 @@ public void setEchoChar (char echo) {
  * The double click flag enables or disables the
  * default action of the text widget when the user
  * double clicks.
+ * </p><p>
+ * Note: This operation is a hint and is not supported on
+ * platforms that do not have this concept.
  * </p>
  * 
  * @param doubleClick the new double click flag
@@ -1179,10 +1246,13 @@ public void setFont (Font font) {
 
 /**
  * Sets the orientation of the receiver, which must be one
- * of the constants <code>SWT.LEFT_TO_RIGHT</code> or <code>SWT.LEFT_TO_RIGHT</code>.
+ * of the constants <code>SWT.LEFT_TO_RIGHT</code> or <code>SWT.RIGHT_TO_LEFT</code>.
  * <p>
+ * Note: This operation is a hint and is not supported on
+ * platforms that do not have this concept.
+ * </p>
  *
- * @param orientation new orientation bit
+ * @param orientation new orientation style
  * 
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -1235,7 +1305,10 @@ public void setSelection (int position) {
 }
 
 /**
- * Sets the selection.
+ * Sets the selection to the range specified
+ * by the given point, where the x coordinate
+ * represents the start index and the y coordinate
+ * represents the end index.
  * <p>
  * Indexing is zero based.  The range of
  * a selection is from 0..N where N is
@@ -1267,7 +1340,8 @@ public void setSelection (Point selection) {
 }
 
 /**
- * Sets the selection.
+ * Sets the selection to the range specified
+ * by the given start and end indices.
  * <p>
  * Indexing is zero based.  The range of
  * a selection is from 0..N where N is
@@ -1341,7 +1415,7 @@ void setTabStops (int tabs) {
  * SINGLE and the argument contains multiple lines of text, the result of this
  * operation is undefined and may vary from platform to platform.
  *
- * @param text the new text
+ * @param string the new text
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the string is null</li>
@@ -1369,6 +1443,8 @@ public void setText (String string) {
  * creating a read-only text widget.
  * </p><p>
  * To reset this value to the default, use <code>setTextLimit(Text.LIMIT)</code>.
+ * Specifying a limit value larger than <code>Text.LIMIT</code> sets the
+ * receiver's limit to <code>Text.LIMIT</code>.
  * </p>
  *
  * @param limit new text limit
@@ -1380,6 +1456,8 @@ public void setText (String string) {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ * 
+ * @see #LIMIT
  */
 public void setTextLimit (int limit) {
 	checkWidget();
@@ -1413,9 +1491,6 @@ public void setTopIndex (int index) {
  * lines are scrolled until the selection is visible.
  * </p>
  * 
- * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
- * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
