@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -68,11 +68,23 @@ public abstract class Widget {
 	static final int MOVED			= 1 << 5;
 	static final int RESIZED		= 1 << 6;
 	static final int GRAB			= 1 << 7;
-	static final int LAYOUT_CHANGED	= 1 << 8;
-	static final int LAYOUT_NEEDED  = 1 << 9;
 	
+	/* A layout was requested on this widget */
+	static final int LAYOUT_NEEDED	= 1 << 8;
+	
+	/* The preferred size of a child has changed */
+	static final int LAYOUT_CHANGED	= 1 << 9;
+	
+	/* A layout was requested in this widget hierachy */
+	static final int LAYOUT_CHILD	= 1 << 10;
+
+	/* More global state flags */
+	static final int RELEASED		= 1<<11;
+	static final int DISPOSE_SENT	= 1<<12;
+	
+	/* Default size for widgets */
 	static final int DEFAULT_WIDTH	= 64;
-	static final int DEFAULT_HEIGHT = 64;
+	static final int DEFAULT_HEIGHT	= 64;
 
 Widget () {
 	/* Do nothing */
@@ -243,9 +255,10 @@ int copyPhImage(int image) {
 
 /**
  * Adds the listener to the collection of listeners who will
- * be notifed when an event of the given type occurs. When the
+ * be notified when an event of the given type occurs. When the
  * event does occur in the widget, the listener is notified by
- * sending it the <code>handleEvent()</code> message.
+ * sending it the <code>handleEvent()</code> message. The event
+ * type is one of the event constants defined in class <code>SWT</code>.
  *
  * @param eventType the type of event to listen for
  * @param listener the listener which should be notified when the event occurs
@@ -259,7 +272,9 @@ int copyPhImage(int image) {
  * </ul>
  *
  * @see Listener
+ * @see SWT
  * @see #removeListener
+ * @see #notifyListeners
  */
 public void addListener (int eventType, Listener handler) {
 	checkWidget();
@@ -270,7 +285,7 @@ public void addListener (int eventType, Listener handler) {
 
 /**
  * Adds the listener to the collection of listeners who will
- * be notifed when the widget is disposed. When the widget is
+ * be notified when the widget is disposed. When the widget is
  * disposed, the listener is notified by sending it the
  * <code>widgetDisposed()</code> message.
  *
@@ -420,11 +435,9 @@ public void dispose () {
 	* Note:  It is valid to attempt to dispose a widget
 	* more than once.  If this happens, fail silently.
 	*/
-	if (isDisposed()) return;
+	if (isDisposed ()) return;
 	if (!isValidThread ()) error (SWT.ERROR_THREAD_INVALID_ACCESS);
-	releaseChild ();
-	releaseWidget ();
-	destroyWidget ();
+	release (true);
 }
 
 int drawProc (int widget, int damage) {
@@ -541,7 +554,7 @@ public Display getDisplay () {
 String getName () {
 	String string = getClass ().getName ();
 	int index = string.length ();
-	while (--index > 0 && string.charAt (index) != '.');
+	while (--index > 0 && string.charAt (index) != '.') {/* empty */}
 	return string.substring (index + 1, string.length ());
 }
 
@@ -611,15 +624,18 @@ boolean isValidSubclass () {
 /**
  * Returns <code>true</code> if there are any listeners
  * for the specified event type associated with the receiver,
- * and <code>false</code> otherwise.
+ * and <code>false</code> otherwise. The event type is one of
+ * the event constants defined in class <code>SWT</code>.
  *
- * @param	eventType the type of event
+ * @param eventType the type of event
  * @return true if the event is hooked
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ *
+ * @see SWT
  */
 public boolean isListening (int eventType) {
 	checkWidget();
@@ -633,7 +649,9 @@ boolean isValidThread () {
 /**
  * Notifies all of the receiver's listeners for events
  * of the given type that one such event has occurred by
- * invoking their <code>handleEvent()</code> method.
+ * invoking their <code>handleEvent()</code> method.  The
+ * event type is one of the event constants defined in class
+ * <code>SWT</code>.
  *
  * @param eventType the type of event which has occurred
  * @param event the event data
@@ -642,6 +660,10 @@ boolean isValidThread () {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ * 
+ * @see SWT
+ * @see #addListener
+ * @see #removeListener
  */
 public void notifyListeners (int eventType, Event event) {
 	checkWidget();
@@ -749,28 +771,47 @@ int Pt_CB_WINDOW (int widget, int info) {
 	return OS.Pt_CONTINUE;
 }
 
-void releaseChild () {
-	/* Do nothing */
-}
 
 void register () {
 	if (handle == 0) return;
 	WidgetTable.put (handle, this);
 }
 
+void release (boolean destroy) {
+	if ((state & DISPOSE_SENT) == 0) {
+		state |= DISPOSE_SENT; 
+		sendEvent (SWT.Dispose);
+	}
+	if ((state & DISPOSED) == 0) {
+		releaseChildren (destroy);
+	}
+	if ((state & RELEASED) == 0) {
+		state |= RELEASED;
+		if (destroy) {
+			releaseParent ();
+			releaseWidget ();
+			destroyWidget ();
+		} else {
+			releaseWidget ();
+			releaseHandle ();
+		}
+	}
+}
+
+void releaseChildren (boolean destroy) {
+}
+
 void releaseHandle () {
 	handle = 0;
 	state |= DISPOSED;
 	display = null;
 }
 
-void releaseResources () {
-	releaseWidget ();
-	releaseHandle ();
+void releaseParent () {
+	/* Do nothing */
 }
 
 void releaseWidget () {
-	sendEvent (SWT.Dispose);
 	deregister ();
 	eventTable = null;
 	data = null;
@@ -778,7 +819,8 @@ void releaseWidget () {
 
 /**
  * Removes the listener from the collection of listeners who will
- * be notifed when an event of the given type occurs.
+ * be notified when an event of the given type occurs. The event
+ * type is one of the event constants defined in class <code>SWT</code>.
  *
  * @param eventType the type of event to listen for
  * @param listener the listener which should no longer be notified when the event occurs
@@ -792,7 +834,9 @@ void releaseWidget () {
  * </ul>
  *
  * @see Listener
+ * @see SWT
  * @see #addListener
+ * @see #notifyListeners
  */
 public void removeListener (int eventType, Listener handler) {
 	checkWidget();
@@ -803,7 +847,7 @@ public void removeListener (int eventType, Listener handler) {
 
 /**
  * Removes the listener from the collection of listeners who will
- * be notifed when an event of the given type occurs.
+ * be notified when an event of the given type occurs.
  * <p>
  * <b>IMPORTANT:</b> This method is <em>not</em> part of the SWT
  * public API. It is marked public only so that it can be shared
@@ -834,7 +878,7 @@ protected void removeListener (int eventType, SWTEventListener handler) {
 
 /**
  * Removes the listener from the collection of listeners who will
- * be notifed when the widget is disposed.
+ * be notified when the widget is disposed.
  *
  * @param listener the listener which should no longer be notified when the receiver is disposed
  *
