@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -38,6 +38,12 @@ public class Menu extends Widget {
 	/**
 	 * the handle to the OS resource 
 	 * (Warning: This field is platform dependent)
+	 * <p>
+	 * <b>IMPORTANT:</b> This field is <em>not</em> part of the SWT
+	 * public API. It is marked public only so that it can be shared
+	 * within the packages provided by SWT. It is not available on all
+	 * platforms and should never be accessed from application code.
+	 * </p>
 	 */
 	public int handle;
 	
@@ -45,6 +51,9 @@ public class Menu extends Widget {
 	boolean hasLocation;
 	MenuItem cascade;
 	Decorations parent;
+	ImageList imageList;
+	
+	/* Resource ID for SHMENUBARINFO */
 	static final int ID_PPC = 100;
 	
 	/* SmartPhone SoftKeyBar resource ids */
@@ -75,7 +84,7 @@ public class Menu extends Widget {
  * @see Widget#getStyle
  */
 public Menu (Control parent) {
-	this (checkNull (parent).getShell (), SWT.POP_UP);
+	this (checkNull (parent).menuShell (), SWT.POP_UP);
 }
 
 /**
@@ -119,7 +128,7 @@ public Menu (Decorations parent, int style) {
  * for the instance so that the instance will be a drop-down
  * menu on the given parent's parent.
  *
- * @param parent a menu which will be the parent of the new instance (cannot be null)
+ * @param parentMenu a menu which will be the parent of the new instance (cannot be null)
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
@@ -143,7 +152,7 @@ public Menu (Menu parentMenu) {
  * for the instance so that the instance will be a drop-down
  * menu on the given parent's parent menu.
  *
- * @param parent a menu item which will be the parent of the new instance (cannot be null)
+ * @param parentItem a menu item which will be the parent of the new instance (cannot be null)
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
@@ -186,38 +195,39 @@ Menu (Decorations parent, int style, int handle) {
 void _setVisible (boolean visible) {
 	if ((style & (SWT.BAR | SWT.DROP_DOWN)) != 0) return;
 	int hwndParent = parent.handle;
-	if (!visible) {
+	if (visible) {
+		int flags = OS.TPM_LEFTBUTTON;
+		if (OS.GetKeyState (OS.VK_LBUTTON) >= 0) flags |= OS.TPM_RIGHTBUTTON;
+		if ((style & SWT.RIGHT_TO_LEFT) != 0) flags |= OS.TPM_RIGHTALIGN;
+		if ((parent.style & SWT.MIRRORED) != 0) {
+			flags &= ~OS.TPM_RIGHTALIGN;
+			if ((style & SWT.LEFT_TO_RIGHT) != 0) flags |= OS.TPM_RIGHTALIGN;
+		}
+		int nX = x, nY = y;
+		if (!hasLocation) {
+			int pos = OS.GetMessagePos ();
+			nX = (short) (pos & 0xFFFF);
+			nY = (short) (pos >> 16);
+		}
+		/*
+		* Feature in Windows.  It is legal use TrackPopupMenu()
+		* to display an empty menu as long as menu items are added
+		* inside of WM_INITPOPUPMENU.  If no items are added, then
+		* TrackPopupMenu() fails and does not send an indication
+		* that the menu has been closed.  This is not strictly a
+		* bug but leads to unwanted behavior when application code
+		* assumes that every WM_INITPOPUPMENU will eventually result
+		* in a WM_MENUSELECT, wParam=0xFFFF0000, lParam=0 to indicate
+		* that the menu has been closed.  The fix is to detect the
+		* case when TrackPopupMenu() fails and the number of items in
+		* the menu is zero and issue a fake WM_MENUSELECT.
+		*/
+		boolean success = OS.TrackPopupMenu (handle, flags, nX, nY, 0, hwndParent, null);
+		if (!success && GetMenuItemCount (handle) == 0) {
+			OS.SendMessage (hwndParent, OS.WM_MENUSELECT, 0xFFFF0000, 0);
+		}
+	} else {
 		OS.SendMessage (hwndParent, OS.WM_CANCELMODE, 0, 0);
-		return;
-	}
-	int flags = OS.TPM_LEFTBUTTON | OS.TPM_RIGHTBUTTON;
-	if ((style & SWT.RIGHT_TO_LEFT) != 0) flags |= OS.TPM_RIGHTALIGN;
-	if ((parent.style & SWT.MIRRORED) != 0) {
-		flags &= ~OS.TPM_RIGHTALIGN;
-		if ((style & SWT.LEFT_TO_RIGHT) != 0) flags |= OS.TPM_RIGHTALIGN;
-	}
-	int nX = x, nY = y;
-	if (!hasLocation) {
-		int pos = OS.GetMessagePos ();
-		nX = (short) (pos & 0xFFFF);
-		nY = (short) (pos >> 16);
-	}
-	/*
-	* Feature in Windows.  It is legal use TrackPopupMenu ()
-	* to display an empty menu as long as menu items are added
-	* inside of WM_INITPOPUPMENU.  If no items are added, then
-	* TrackPopupMenu () fails and does not send an indication
-	* that the menu has been closed.  This is not strictly a
-	* bug but leads to unwanted behavior when application code
-	* assumes that every WM_INITPOPUPMENU will eventually result
-	* in a WM_MENUSELECT, wParam=0xFFFF0000, lParam=0 to indicate
-	* that the menu has been closed.  The fix is to detect the
-	* case when TrackPopupMenu fails and the number of items in
-	* the menu is zero and issue a fake WM_MENUSELECT.
-	*/
-	boolean success = OS.TrackPopupMenu (handle, flags, nX, nY, 0, hwndParent, null);
-	if (!success && GetMenuItemCount (handle) == 0) {
-		OS.SendMessage (hwndParent, OS.WM_MENUSELECT, 0xFFFF0000, 0);
 	}
 }
 
@@ -412,16 +422,17 @@ void createHandle () {
 void createItem (MenuItem item, int index) {
 	int count = GetMenuItemCount (handle);
 	if (!(0 <= index && index <= count)) error (SWT.ERROR_INVALID_RANGE);
-	parent.add (item);
+	display.addMenuItem (item);
 	boolean success = false;
 	if ((OS.IsPPC || OS.IsSP) && hwndCB != 0) {
 		if (OS.IsSP) return;
 		TBBUTTON lpButton = new TBBUTTON ();
 		lpButton.idCommand = item.id;
-		lpButton.fsStyle = (byte) (OS.TBSTYLE_DROPDOWN | OS.TBSTYLE_AUTOSIZE | 0x80);
+		lpButton.fsStyle = (byte) OS.TBSTYLE_AUTOSIZE;
+		if ((item.style & SWT.CASCADE) != 0) lpButton.fsStyle |= OS.TBSTYLE_DROPDOWN | 0x80;
+		if ((item.style & SWT.SEPARATOR) != 0) lpButton.fsStyle = (byte) OS.BTNS_SEP;
 		lpButton.fsState = (byte) OS.TBSTATE_ENABLED;
 		lpButton.iBitmap = OS.I_IMAGENONE;
-		if ((item.style & SWT.SEPARATOR) != 0) lpButton.fsStyle = (byte) OS.BTNS_SEP;
 		success = OS.SendMessage (hwndCB, OS.TB_INSERTBUTTON, index, lpButton) != 0;
 	} else {
 		if (OS.IsWinCE) {
@@ -430,7 +441,7 @@ void createItem (MenuItem item, int index) {
 			if ((item.style & SWT.SEPARATOR) != 0) {
 				uFlags |= OS.MF_SEPARATOR;
 			} else {
-				lpNewItem = new TCHAR (0, "", true);
+				lpNewItem = new TCHAR (0, " ", true);
 			}
 			success = OS.InsertMenu (handle, index, uFlags, item.id, lpNewItem);
 			if (success) {
@@ -442,13 +453,22 @@ void createItem (MenuItem item, int index) {
 			}
 		} else {
 			/*
-			* Bug in Windows.  For some reason, when InsertMenuItem ()
+			* Bug in Windows.  For some reason, when InsertMenuItem()
 			* is used to insert an item without text, it is not possible
-			* to use SetMenuItemInfo () to set the text at a later time.
-			* The fix is to insert the item with an empty string.
+			* to use SetMenuItemInfo() to set the text at a later time.
+			* The fix is to insert the item with some text.
+			* 
+			* Feature in Windows.  When an empty string is used instead
+			* of a space and InsertMenuItem() is used to set a submenu
+			* before setting text to a non-empty string, the menu item
+			* becomes unexpectedly disabled.  The fix is to insert a
+			* space.
 			*/
 			int hHeap = OS.GetProcessHeap ();
-			int pszText = OS.HeapAlloc (hHeap, OS.HEAP_ZERO_MEMORY, TCHAR.sizeof);
+			TCHAR buffer = new TCHAR (0, " ", true);
+			int byteCount = buffer.length () * TCHAR.sizeof;
+			int pszText = OS.HeapAlloc (hHeap, OS.HEAP_ZERO_MEMORY, byteCount);
+			OS.MoveMemory (pszText, buffer, byteCount);	
 			MENUITEMINFO info = new MENUITEMINFO ();
 			info.cbSize = MENUITEMINFO.sizeof;
 			info.fMask = OS.MIIM_ID | OS.MIIM_TYPE | OS.MIIM_DATA;
@@ -460,7 +480,7 @@ void createItem (MenuItem item, int index) {
 		}
 	}
 	if (!success) {
-		parent.remove (item);
+		display.removeMenuItem (item);
 		error (SWT.ERROR_ITEM_NOT_ADDED);
 	}
 	redraw ();
@@ -483,7 +503,7 @@ void createWidget () {
 	*/
 //	checkOrientation (parent);
 	createHandle ();
-	parent.add (this);
+	parent.addMenu (this);
 }
 
 /*
@@ -515,6 +535,14 @@ void destroyItem (MenuItem item) {
 			if (OS.SendMessage (hwndCB, OS.TB_DELETEBUTTON, index, 0) == 0) {
 				error (SWT.ERROR_ITEM_NOT_REMOVED);
 			}
+			int count = OS.SendMessage (hwndCB, OS.TB_BUTTONCOUNT, 0, 0);
+			if (count == 0) {
+				if (imageList != null) {
+					OS.SendMessage (handle, OS.TB_SETIMAGELIST, 0, 0);
+					display.releaseImageList (imageList);
+					imageList = null;
+				}
+			}
 		} else {
 			int index = 0;
 			MENUITEMINFO info = new MENUITEMINFO ();
@@ -527,12 +555,12 @@ void destroyItem (MenuItem item) {
 			if (info.dwItemData != item.id) {
 				error (SWT.ERROR_ITEM_NOT_REMOVED);
 			}	
-			if (!OS.RemoveMenu (handle, index, OS.MF_BYPOSITION)) {
+			if (!OS.DeleteMenu (handle, index, OS.MF_BYPOSITION)) {
 				error (SWT.ERROR_ITEM_NOT_REMOVED);
 			}
 		}
 	} else {
-		if (!OS.RemoveMenu (handle, item.id, OS.MF_BYCOMMAND)) {
+		if (!OS.DeleteMenu (handle, item.id, OS.MF_BYCOMMAND)) {
 			error (SWT.ERROR_ITEM_NOT_REMOVED);
 		}
 	}
@@ -549,6 +577,69 @@ void destroyWidget () {
 	}
 }
 
+void fixMenus (Decorations newParent) {
+	MenuItem [] items = getItems ();
+	for (int i=0; i<items.length; i++) {
+		items [i].fixMenus (newParent);
+	}
+	parent.removeMenu (this);
+	newParent.addMenu (this);
+	this.parent = newParent;
+}
+
+/**
+ * Returns a rectangle describing the receiver's size and location
+ * relative to its parent (or its display if its parent is null),
+ * unless the receiver is a menu or a shell. In this case, the
+ * location is relative to the display.
+ * <p>
+ * Note that the bounds of a menu or menu item are undefined when
+ * the menu is not visible.  This is because most platforms compute
+ * the bounds of a menu dynamically just before it is displayed.
+ * </p>
+ *
+ * @return the receiver's bounding rectangle
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.1
+ */
+/*public*/ Rectangle getBounds () {
+	checkWidget ();
+	if (OS.IsWinCE) return new Rectangle (0, 0, 0, 0);
+	if ((style & SWT.BAR) != 0) {
+		if (parent.menuBar != this) {
+			return new Rectangle (0, 0, 0, 0);
+		}
+		int hwndShell = parent.handle;
+		MENUBARINFO info = new MENUBARINFO ();
+		info.cbSize = MENUBARINFO.sizeof;
+		if (OS.GetMenuBarInfo (hwndShell, OS.OBJID_MENU, 0, info)) {
+			int width = info.right - info.left;
+			int height = info.bottom - info.top;
+			return new Rectangle (info.left, info.top, width, height);
+		}
+	} else {
+		int count = GetMenuItemCount (handle);
+		if (count != 0) {
+			RECT rect1 = new RECT ();
+			if (OS.GetMenuItemRect (0, handle, 0, rect1)) {
+				RECT rect2 = new RECT ();
+				if (OS.GetMenuItemRect (0, handle, count - 1, rect2)) {
+					int x = rect1.left - 2, y = rect1.top - 2;
+					int width = (rect2.right - rect2.left) + 4;
+					int height = (rect2.bottom - rect1.top) + 4;
+					return new Rectangle (x, y, width, height);
+				}
+			}
+		}
+	}
+	return new Rectangle (0, 0, 0, 0);
+}
+
 /**
  * Returns the default menu item or null if none has
  * been previously set.
@@ -570,20 +661,14 @@ public MenuItem getDefaultItem () {
 	info.cbSize = MENUITEMINFO.sizeof;
 	info.fMask = OS.MIIM_ID;
 	if (OS.GetMenuItemInfo (handle, id, false, info)) {
-		return parent.findMenuItem (info.wID);
+		return display.getMenuItem (info.wID);
 	}
 	return null;
 }
 
-public Display getDisplay () {
-	Decorations parent = this.parent;
-	if (parent == null) error (SWT.ERROR_WIDGET_DISPOSED);
-	return parent.getDisplay ();
-}
-
 /**
  * Returns <code>true</code> if the receiver is enabled, and
- * <code>false</code> otherwise. A disabled control is typically
+ * <code>false</code> otherwise. A disabled menu is typically
  * not selectable from the user interface and draws with an
  * inactive or "grayed" look.
  *
@@ -639,7 +724,7 @@ public MenuItem getItem (int index) {
 		}
 		id = info.dwItemData;
 	}
-	return parent.findMenuItem (id);
+	return display.getMenuItem (id);
 }
 
 /**
@@ -658,8 +743,8 @@ public int getItemCount () {
 }
 
 /**
- * Returns an array of <code>MenuItem</code>s which are the items
- * in the receiver. 
+ * Returns a (possibly empty) array of <code>MenuItem</code>s which
+ * are the items in the receiver. 
  * <p>
  * Note: This is not the actual structure used by the receiver
  * to maintain its list of items, so modifying the array will
@@ -678,8 +763,8 @@ public MenuItem [] getItems () {
 	if ((OS.IsPPC || OS.IsSP) && hwndCB != 0) {
 		if (OS.IsSP) {
 			MenuItem [] result = new MenuItem [2];
-			result[0] = parent.findMenuItem (id0);
-			result[1] = parent.findMenuItem (id1);		
+			result[0] = display.getMenuItem (id0);
+			result[1] = display.getMenuItem (id1);
 			return result;
 		}
 		int count = OS.SendMessage (hwndCB, OS.TB_BUTTONCOUNT, 0, 0);
@@ -687,32 +772,33 @@ public MenuItem [] getItems () {
 		MenuItem [] result = new MenuItem [count];
 		for (int i=0; i<count; i++) {
 			OS.SendMessage (hwndCB, OS.TB_GETBUTTON, i, lpButton);
-			result [i] = parent.findMenuItem (lpButton.idCommand);
+			result [i] = display.getMenuItem (lpButton.idCommand);
 		}
 		return result;
 	}
-	int index = 0;
+	int index = 0, count = 0;
 	int length = OS.IsWinCE ? 4 : OS.GetMenuItemCount (handle);
 	MenuItem [] items = new MenuItem [length];
 	MENUITEMINFO info = new MENUITEMINFO ();
 	info.cbSize = MENUITEMINFO.sizeof;
 	info.fMask = OS.MIIM_DATA;
 	while (OS.GetMenuItemInfo (handle, index, true, info)) {
-		if (index == items.length) {
-			MenuItem [] newItems = new MenuItem [index + 4];
-			System.arraycopy (items, 0, newItems, 0, index);
+		if (count == items.length) {
+			MenuItem [] newItems = new MenuItem [count + 4];
+			System.arraycopy (items, 0, newItems, 0, count);
 			items = newItems;
 		}
-		items [index++] = parent.findMenuItem (info.dwItemData);
+		MenuItem item = display.getMenuItem (info.dwItemData);
+		if (item != null) items [count++] = item;
+		index++;
 	}
-	if (index == items.length) return items;
-	MenuItem [] result = new MenuItem [index];
-	System.arraycopy (items, 0, result, 0, index);
+	if (count == items.length) return items;
+	MenuItem [] result = new MenuItem [count];
+	System.arraycopy (items, 0, result, 0, count);
 	return result;
 }
 
 int GetMenuItemCount (int handle) {
-	checkWidget ();
 	if (OS.IsWinCE) {
 		if ((OS.IsPPC || OS.IsSP) && hwndCB != 0) {
 			return OS.IsSP ? 2 : OS.SendMessage (hwndCB, OS.TB_BUTTONCOUNT, 0, 0);
@@ -832,14 +918,37 @@ public boolean getVisible () {
 		return this == parent.menuShell ().menuBar;
 	}
 	if ((style & SWT.POP_UP) != 0) {
-		Display display = getDisplay ();
 		Menu [] popups = display.popups;
 		if (popups == null) return false;
 		for (int i=0; i<popups.length; i++) {
 			if (popups [i] == this) return true;
 		}
 	}
-	return this == getShell ().activeMenu;
+	Shell shell = getShell ();
+	Menu menu = shell.activeMenu;
+	while (menu != null && menu != this) {
+		menu = menu.getParentMenu ();
+	}
+	return this == menu;
+}
+
+int imageIndex (Image image) {
+	if (hwndCB == 0 || image == null) return OS.I_IMAGENONE;
+	if (imageList == null) {
+		Rectangle bounds = image.getBounds ();
+		imageList = display.getImageList (style & SWT.RIGHT_TO_LEFT, bounds.width, bounds.height);
+		int index = imageList.add (image);
+		int hImageList = imageList.getHandle ();
+		OS.SendMessage (hwndCB, OS.TB_SETIMAGELIST, 0, hImageList);
+		return index;
+	}
+	int index = imageList.indexOf (image);
+	if (index == -1) {
+		index = imageList.add (image);
+	} else {
+		imageList.put (index, image);
+	}
+	return index;
 }
 
 /**
@@ -888,7 +997,7 @@ public int indexOf (MenuItem item) {
 /**
  * Returns <code>true</code> if the receiver is enabled and all
  * of the receiver's ancestors are enabled, and <code>false</code>
- * otherwise. A disabled control is typically not selectable from the
+ * otherwise. A disabled menu is typically not selectable from the
  * user interface and draws with an inactive or "grayed" look.
  *
  * @return the receiver's enabled state
@@ -927,50 +1036,59 @@ public boolean isVisible () {
 }
 
 void redraw () {
+	if (!isVisible ()) return;
 	if ((style & SWT.BAR) != 0) {
-		Display display = getDisplay ();
 		display.addBar (this);
 	} else {
 		update ();
 	}
 }
 
-void releaseChild () {
-	super.releaseChild ();
-	if (cascade != null) cascade.releaseMenu ();
-	if ((style & SWT.BAR) != 0) {
-		Display display = getDisplay ();
-		display.removeBar (this);
-		if (this == parent.menuBar) {
-			parent.setMenuBar (null);
-		}
-	} else {
-		if ((style & SWT.POP_UP) != 0) {
-			Display display = getDisplay ();
-			display.removePopup (this);
-		}
-	}
-}
-
 void releaseHandle () {
 	super.releaseHandle ();
 	handle = hwndCB = 0;
 }
 
-void releaseWidget () {
+void releaseChildren (boolean destroy) {
 	MenuItem [] items = getItems ();
 	for (int i=0; i<items.length; i++) {
 		MenuItem item = items [i];
-		if (!item.isDisposed ()) {
+		if (item != null && !item.isDisposed ()) {
 			if (OS.IsPPC && hwndCB != 0) {
 				item.dispose ();
 			} else {
-				item.releaseResources ();
+				item.release (false);
 			}
 		}
 	}
+	super.releaseChildren (destroy);
+}
+
+void releaseParent () {
+	super.releaseParent ();
+	if (cascade != null) cascade.releaseMenu ();
+	if ((style & SWT.BAR) != 0) {
+		display.removeBar (this);
+		if (this == parent.menuBar) {
+			parent.setMenuBar (null);
+		}
+	} else {
+		if ((style & SWT.POP_UP) != 0) {
+			display.removePopup (this);
+		}
+	}
+}
+
+void releaseWidget () {
 	super.releaseWidget ();
-	if (parent != null) parent.remove (this);
+	if (OS.IsPPC && hwndCB != 0) {
+		if (imageList != null) {
+			OS.SendMessage (hwndCB, OS.TB_SETIMAGELIST, 0, 0);
+			display.releaseToolImageList (imageList);
+			imageList = null;
+		}
+	}
+	if (parent != null) parent.removeMenu (this);
 	parent = null;
 	cascade = null;
 }
@@ -979,7 +1097,7 @@ void releaseWidget () {
  * Removes the listener from the collection of listeners who will
  * be notified when the help events are generated for the control.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -1003,7 +1121,7 @@ public void removeHelpListener (HelpListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the menu events are generated for the control.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -1055,7 +1173,7 @@ public void setDefaultItem (MenuItem item) {
 
 /**
  * Enables the receiver if the argument is <code>true</code>,
- * and disables it otherwise. A disabled control is typically
+ * and disables it otherwise. A disabled menu is typically
  * not selectable from the user interface and draws with an
  * inactive or "grayed" look.
  *
@@ -1073,11 +1191,15 @@ public void setEnabled (boolean enabled) {
 }
 
 /**
- * Sets the receiver's location to the point specified by
- * the arguments which are relative to the display.
+ * Sets the location of the receiver, which must be a popup,
+ * to the point specified by the arguments which are relative
+ * to the display.
  * <p>
- * Note:  This is different from most widgets where the
+ * Note that this is different from most widgets where the
  * location of the widget is relative to the parent.
+ * </p><p>
+ * Note that the platform window manager ultimately has control
+ * over the location of popup menus.
  * </p>
  *
  * @param x the new x coordinate for the receiver
@@ -1097,11 +1219,15 @@ public void setLocation (int x, int y) {
 }
 
 /**
- * Sets the receiver's location to the point specified by
- * the arguments which are relative to the display.
+ * Sets the location of the receiver, which must be a popup,
+ * to the point specified by the argument which is relative
+ * to the display.
  * <p>
- * Note:  This is different from most widgets where the
+ * Note that this is different from most widgets where the
  * location of the widget is relative to the parent.
+ * </p><p>
+ * Note that the platform window manager ultimately has control
+ * over the location of popup menus.
  * </p>
  *
  * @param location the new location for the receiver
@@ -1141,7 +1267,6 @@ public void setLocation (Point location) {
 public void setVisible (boolean visible) {
 	checkWidget ();
 	if ((style & (SWT.BAR | SWT.DROP_DOWN)) != 0) return;
-	Display display = getDisplay ();
 	if (visible) {
 		display.addPopup (this);
 	} else {
@@ -1171,24 +1296,57 @@ void update () {
 		}
 		return;
 	}
+	if (OS.IsWinCE) return;
 	if ((style & SWT.BAR) != 0) {
 		if (this == parent.menuBar) OS.DrawMenuBar (parent.handle);
 		return;
 	}
-	if ((OS.WIN32_MAJOR << 16 | OS.WIN32_MINOR) < (4 << 16 | 10)) {
+	if (OS.WIN32_VERSION < OS.VERSION (4, 10)) {
 		return;
 	}
 	boolean hasCheck = false, hasImage = false;
 	MenuItem [] items = getItems ();
 	for (int i=0; i<items.length; i++) {
 		MenuItem item = items [i];
-		if (item.getImage () != null) {
+		if (item.image != null) {
 			if ((hasImage = true) && hasCheck) break;
 		}
-		if ((item.getStyle () & (SWT.CHECK | SWT.RADIO)) != 0) {
+		if ((item.style & (SWT.CHECK | SWT.RADIO)) != 0) {
 			if ((hasCheck = true) && hasImage) break;
 		}
 	}
+	
+	/*
+	* Bug in Windows.  If a menu contains items that have
+	* images and can be checked, Windows does not include
+	* the width of the image and the width of the check when
+	* computing the width of the menu.  When the longest item
+	* does not have an image, the label and the accelerator
+	* text can overlap.  The fix is to use SetMenuItemInfo()
+	* to indicate that all items have a bitmap and then include
+	* the width of the widest bitmap in WM_MEASURECHILD.
+	* 
+	* NOTE:  This work around causes problems on Windows 98.
+	* Under certain circumstances that have yet to be isolated,
+	* some menus can become huge and blank.  For now, do not
+	* run the code on Windows 98.
+	*/	
+	if (!OS.IsWin95) {
+		MENUITEMINFO info = new MENUITEMINFO ();
+		info.cbSize = MENUITEMINFO.sizeof;
+		info.fMask = OS.MIIM_BITMAP;
+		for (int i=0; i<items.length; i++) {
+			MenuItem item = items [i];
+			if ((style & SWT.SEPARATOR) == 0) {
+				if (item.image == null) {
+					info.hbmpItem = hasImage ? OS.HBMMENU_CALLBACK : 0;
+					OS.SetMenuItemInfo (handle, item.id, false, info);
+				}
+			}
+		}
+	}
+
+	/* Update the menu to hide or show the space for bitmaps */
 	MENUINFO lpcmi = new MENUINFO ();
 	lpcmi.cbSize = MENUINFO.sizeof;
 	lpcmi.fMask = OS.MIM_STYLE;
