@@ -1,25 +1,19 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Sebastian Davids <sdavids@gmx.de> - bug 74959
+ *     Maik Schreiber - bug 102461
  *******************************************************************************/
 package org.eclipse.team.internal.ccvs.ui.repo;
 
 
-import java.io.BufferedInputStream;
-import java.io.BufferedOutputStream;
-import java.io.DataInputStream;
-import java.io.EOFException;
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileOutputStream;
-import java.io.IOException;
-import java.io.InputStream;
+import java.io.*;
 import java.lang.reflect.InvocationTargetException;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -31,38 +25,22 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
-import org.apache.xerces.parsers.SAXParser;
+import javax.xml.parsers.*;
+
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.IResource;
-import org.eclipse.core.runtime.IAdaptable;
-import org.eclipse.core.runtime.IPath;
-import org.eclipse.core.runtime.IProgressMonitor;
-import org.eclipse.core.runtime.Status;
-import org.eclipse.core.runtime.SubProgressMonitor;
+import org.eclipse.core.runtime.*;
 import org.eclipse.jface.dialogs.IDialogConstants;
 import org.eclipse.jface.operation.IRunnableWithProgress;
+import org.eclipse.jface.window.Window;
+import org.eclipse.osgi.util.NLS;
 import org.eclipse.swt.widgets.Shell;
-import org.eclipse.team.core.RepositoryProvider;
 import org.eclipse.team.core.TeamException;
-import org.eclipse.team.core.sync.IRemoteSyncElement;
-import org.eclipse.team.internal.ccvs.core.CVSException;
-import org.eclipse.team.internal.ccvs.core.CVSProviderPlugin;
-import org.eclipse.team.internal.ccvs.core.CVSTag;
-import org.eclipse.team.internal.ccvs.core.CVSTeamProvider;
-import org.eclipse.team.internal.ccvs.core.ICVSFolder;
-import org.eclipse.team.internal.ccvs.core.ICVSListener;
-import org.eclipse.team.internal.ccvs.core.ICVSRemoteFolder;
-import org.eclipse.team.internal.ccvs.core.ICVSRemoteResource;
-import org.eclipse.team.internal.ccvs.core.ICVSRepositoryLocation;
-import org.eclipse.team.internal.ccvs.core.ICVSResource;
-import org.eclipse.team.internal.ccvs.core.client.Command;
+import org.eclipse.team.internal.ccvs.core.*;
 import org.eclipse.team.internal.ccvs.core.connection.CVSRepositoryLocation;
-import org.eclipse.team.internal.ccvs.ui.AddToVersionControlDialog;
-import org.eclipse.team.internal.ccvs.ui.CVSUIPlugin;
-import org.eclipse.team.internal.ccvs.ui.IRepositoryListener;
+import org.eclipse.team.internal.ccvs.core.util.KnownRepositories;
+import org.eclipse.team.internal.ccvs.ui.*;
 import org.eclipse.team.internal.ccvs.ui.Policy;
-import org.eclipse.team.internal.ccvs.ui.ReleaseCommentDialog;
-import org.eclipse.team.internal.ccvs.ui.XMLWriter;
 import org.eclipse.ui.IWorkingSet;
 import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
@@ -80,8 +58,10 @@ public class RepositoryManager {
 	// new state file
 	private static final String REPOSITORIES_VIEW_FILE = "repositoriesView.xml"; //$NON-NLS-1$
 	private static final String COMMENT_HIST_FILE = "commitCommentHistory.xml"; //$NON-NLS-1$
+    private static final String COMMENT_TEMPLATES_FILE = "commentTemplates.xml"; //$NON-NLS-1$
 	static final String ELEMENT_COMMIT_COMMENT = "CommitComment"; //$NON-NLS-1$
 	static final String ELEMENT_COMMIT_HISTORY = "CommitComments"; //$NON-NLS-1$
+    static final String ELEMENT_COMMENT_TEMPLATES = "CommitCommentTemplates"; //$NON-NLS-1$
 
 	private Map repositoryRoots = new HashMap();
 	
@@ -89,6 +69,7 @@ public class RepositoryManager {
 
 	// The previously remembered comment
 	static String[] previousComments = new String[0];
+    static String[] commentTemplates = new String[0];
 	
 	public static boolean notifyRepoView = true;
 	
@@ -102,7 +83,7 @@ public class RepositoryManager {
 	 * Answer an array of all known remote roots.
 	 */
 	public ICVSRepositoryLocation[] getKnownRepositoryLocations() {
-		return CVSProviderPlugin.getPlugin().getKnownRepositories();
+		return KnownRepositories.getInstance().getRepositories();
 	}
 	
 	/**
@@ -140,7 +121,7 @@ public class RepositoryManager {
 
 			return (CVSTag[])result.toArray(new CVSTag[result.size()]);
 		} catch(CVSException e) {
-			CVSUIPlugin.log(e.getStatus());
+			CVSUIPlugin.log(e);
 			return new CVSTag[0];
 		}
 	}
@@ -150,12 +131,12 @@ public class RepositoryManager {
 	 */
 	public CVSTag[] getKnownTags(ICVSRepositoryLocation location, int tagType) {
 		Set result = new HashSet();
-		RepositoryRoot root = (RepositoryRoot)repositoryRoots.get(location.getLocation());
+		RepositoryRoot root = (RepositoryRoot)repositoryRoots.get(location.getLocation(false));
 		if (root != null) {
 			String[] paths = root.getKnownRemotePaths();
 			for (int i = 0; i < paths.length; i++) {
 				String path = paths[i];
-				CVSTag[] tags = root.getKnownTags(path);
+				CVSTag[] tags = root.getAllKnownTags(path);
 				for (int j = 0; j < tags.length; j++) {
 					CVSTag tag = tags[j];
 					if (tag.getType() == tagType)
@@ -191,7 +172,7 @@ public class RepositoryManager {
 	public CVSTag[] getKnownTags(ICVSFolder project) throws CVSException {
 		RepositoryRoot root = getRepositoryRootFor(project);
 		String remotePath = RepositoryRoot.getRemotePathFor(project);
-		return root.getKnownTags(remotePath);
+		return root.getAllKnownTags(remotePath);
 	}
 	
 	/*
@@ -204,47 +185,62 @@ public class RepositoryManager {
 		for (int i = 0; i < paths.length; i++) {
 			String path = paths[i];
 			Set result = new HashSet();
-			result.addAll(Arrays.asList(root.getKnownTags(path)));
+			result.addAll(Arrays.asList(root.getAllKnownTags(path)));
 			knownTags.put(path, result);
 		}
 		return knownTags;
 	}
 	
-	public ICVSRemoteResource[] getFoldersForTag(ICVSRepositoryLocation location, CVSTag tag, IProgressMonitor monitor) throws CVSException {
-		if (tag.getType() == CVSTag.HEAD) {
-			ICVSRemoteResource[] resources = location.members(tag, false, monitor);
-			RepositoryRoot root = getRepositoryRootFor(location);
-			ICVSRemoteResource[] modules = root.getDefinedModules(tag, monitor);
-			ICVSRemoteResource[] result = new ICVSRemoteResource[resources.length + modules.length];
-			System.arraycopy(resources, 0, result, 0, resources.length);
-			System.arraycopy(modules, 0, result, resources.length, modules.length);
-			return result;
-		}
-		Set result = new HashSet();
-		// Get the tags for the location
-		RepositoryRoot root = getRepositoryRootFor(location);
-		String[] paths = root.getKnownRemotePaths();
-		for (int i = 0; i < paths.length; i++) {
-			String path = paths[i];
-			List tags = Arrays.asList(root.getKnownTags(path));
-			if (tags.contains(tag)) {
-				ICVSRemoteFolder remote = root.getRemoteFolder(path, tag, monitor);
-				result.add(remote);
+	public ICVSRemoteResource[] getFoldersForTag(ICVSRepositoryLocation location, CVSTag tag, IProgressMonitor monitor) throws CVSException {		
+		monitor = Policy.monitorFor(monitor);
+		try {
+			monitor.beginTask(NLS.bind(CVSUIMessages.RepositoryManager_fetchingRemoteFolders, new String[] { tag.getName() }), 100); 
+			if (tag.getType() == CVSTag.HEAD) {
+				ICVSRemoteResource[] resources = location.members(tag, false, Policy.subMonitorFor(monitor, 60));
+				RepositoryRoot root = getRepositoryRootFor(location);
+				ICVSRemoteResource[] modules = root.getDefinedModules(tag, Policy.subMonitorFor(monitor, 40));
+				ICVSRemoteResource[] result = new ICVSRemoteResource[resources.length + modules.length];
+				System.arraycopy(resources, 0, result, 0, resources.length);
+				System.arraycopy(modules, 0, result, resources.length, modules.length);
+				return result;
 			}
+			if (tag.getType() == CVSTag.DATE) {
+				ICVSRemoteResource[] resources = location.members(tag, false, Policy.subMonitorFor(monitor, 60));
+				RepositoryRoot root = getRepositoryRootFor(location);
+				ICVSRemoteResource[] modules = root.getDefinedModules(tag, Policy.subMonitorFor(monitor, 40));
+				ICVSRemoteResource[] result = new ICVSRemoteResource[resources.length + modules.length];
+				System.arraycopy(resources, 0, result, 0, resources.length);
+				System.arraycopy(modules, 0, result, resources.length, modules.length);
+				return result;
+			}
+			Set result = new HashSet();
+			// Get the tags for the location
+			RepositoryRoot root = getRepositoryRootFor(location);
+			String[] paths = root.getKnownRemotePaths();
+			for (int i = 0; i < paths.length; i++) {
+				String path = paths[i];
+				List tags = Arrays.asList(root.getAllKnownTags(path));
+				if (tags.contains(tag)) {
+					ICVSRemoteFolder remote = root.getRemoteFolder(path, tag, Policy.subMonitorFor(monitor, 100));
+					result.add(remote);
+				}
+			}
+			return (ICVSRemoteResource[])result.toArray(new ICVSRemoteResource[result.size()]);
+		} finally {
+			monitor.done();
 		}
-		return (ICVSRemoteResource[])result.toArray(new ICVSRemoteResource[result.size()]);
 	}
 		
 	/*
-	 * Fetches tags from .project and .vcm_meta if they exist. Then fetches tags from the user defined auto-refresh file
+	 * Fetches tags from auto-refresh files if they exist. Then fetches tags from the user defined auto-refresh file
 	 * list. The fetched tags are cached in the CVS ui plugin's tag cache.
 	 */
-	public void refreshDefinedTags(ICVSFolder project, boolean replace, boolean notify, IProgressMonitor monitor) throws TeamException {
-		RepositoryRoot root = getRepositoryRootFor(project);
-		String remotePath = RepositoryRoot.getRemotePathFor(project);
-		root.refreshDefinedTags(remotePath, replace, monitor);
-		if (notify)
+	public CVSTag[] refreshDefinedTags(ICVSFolder folder, boolean recurse, boolean notify, IProgressMonitor monitor) throws TeamException {
+		RepositoryRoot root = getRepositoryRootFor(folder);
+		CVSTag[] tags = root.refreshDefinedTags(folder, recurse, monitor);
+		if (tags.length > 0 && notify)
 			broadcastRepositoryChange(root);
+		return tags;
 	}
 	
 	/**
@@ -263,8 +259,8 @@ public class RepositoryManager {
 	 * Remove the tags defined for this root and notify any listeners
 	 */
 	public void rootRemoved(ICVSRepositoryLocation root) {
-		RepositoryRoot repoRoot = (RepositoryRoot)repositoryRoots.remove(root.getLocation());
-		if (root != null)
+		RepositoryRoot repoRoot = (RepositoryRoot)repositoryRoots.remove(root.getLocation(false));
+		if (repoRoot != null)
 			broadcastRepositoryChange(repoRoot);
 	}
 	
@@ -280,22 +276,27 @@ public class RepositoryManager {
 		root.addTags(remotePath, tags);
 		broadcastRepositoryChange(root);
 	}
-	
+	public void addDateTag(ICVSRepositoryLocation location, CVSTag tag) {
+		if(tag == null) return;
+		RepositoryRoot root = getRepositoryRootFor(location);
+		root.addDateTag(tag);
+		broadcastRepositoryChange(root);
+	}
+	public CVSTag[] getDateTags(ICVSRepositoryLocation location) {
+		RepositoryRoot root = getRepositoryRootFor(location);
+		return root.getDateTags();
+	}
+	public void removeDateTag(ICVSRepositoryLocation location, CVSTag tag){
+		RepositoryRoot root = getRepositoryRootFor(location);
+		root.removeDateTag(tag);
+		broadcastRepositoryChange(root);
+	}
 	public void setAutoRefreshFiles(ICVSFolder project, String[] filePaths) throws CVSException {
 		RepositoryRoot root = getRepositoryRootFor(project);
 		String remotePath = RepositoryRoot.getRemotePathFor(project);
 		root.setAutoRefreshFiles(remotePath, filePaths);
 	}
 	
-	public void removeAutoRefreshFiles(ICVSFolder project, String[] relativeFilePaths) throws CVSException {
-		RepositoryRoot root = getRepositoryRootFor(project);
-		String remotePath = RepositoryRoot.getRemotePathFor(project);
-		Set set = new HashSet();
-		set.addAll(Arrays.asList(root.getAutoRefreshFiles(remotePath)));
-		set.removeAll(Arrays.asList(relativeFilePaths));
-		root.setAutoRefreshFiles(remotePath, (String[]) set.toArray(new String[set.size()]));
-	}
-	
 	public String[] getAutoRefreshFiles(ICVSFolder project) throws CVSException {
 		RepositoryRoot root = getRepositoryRootFor(project);
 		String remotePath = RepositoryRoot.getRemotePathFor(project);
@@ -313,9 +314,10 @@ public class RepositoryManager {
 		broadcastRepositoryChange(root);
 	}
 	
-	public void startup() throws TeamException {
+	public void startup() {
 		loadState();
 		loadCommentHistory();
+        loadCommentTemplates();
 		CVSProviderPlugin.getPlugin().addRepositoryListener(new ICVSListener() {
 			public void repositoryAdded(ICVSRepositoryLocation root) {
 				rootAdded(root);
@@ -329,9 +331,10 @@ public class RepositoryManager {
 	public void shutdown() throws TeamException {
 		saveState();
 		saveCommentHistory();
+        saveCommentTemplates();
 	}
 	
-	private void loadState() throws TeamException {
+	private void loadState() {
 		IPath pluginStateLocation = CVSUIPlugin.getPlugin().getStateLocation().append(REPOSITORIES_VIEW_FILE);
 		File file = pluginStateLocation.toFile();
 		if (file.exists()) {
@@ -343,9 +346,9 @@ public class RepositoryManager {
 					is.close();
 				}
 			} catch (IOException e) {
-				CVSUIPlugin.log(new Status(Status.ERROR, CVSUIPlugin.ID, TeamException.UNABLE, Policy.bind("RepositoryManager.ioException"), e)); //$NON-NLS-1$
+				CVSUIPlugin.log(IStatus.ERROR, CVSUIMessages.RepositoryManager_ioException, e); 
 			} catch (TeamException e) {
-				CVSUIPlugin.log(e.getStatus());
+				CVSUIPlugin.log(e);
 			}
 		} else {
 			IPath oldPluginStateLocation = CVSUIPlugin.getPlugin().getStateLocation().append(STATE_FILE);
@@ -361,14 +364,14 @@ public class RepositoryManager {
 					saveState();
 					file.delete();
 				} catch (IOException e) {
-					CVSUIPlugin.log(new Status(Status.ERROR, CVSUIPlugin.ID, TeamException.UNABLE, Policy.bind("RepositoryManager.ioException"), e)); //$NON-NLS-1$
+					CVSUIPlugin.log(IStatus.ERROR, CVSUIMessages.RepositoryManager_ioException, e); 
 				} catch (TeamException e) {
-					CVSUIPlugin.log(e.getStatus());
+					CVSUIPlugin.log(e);
 				}
 			} 
 		}
 	}
-	private void loadCommentHistory() throws TeamException {
+	private void loadCommentHistory() {
 		IPath pluginStateLocation = CVSUIPlugin.getPlugin().getStateLocation().append(COMMENT_HIST_FILE);
 		File file = pluginStateLocation.toFile();
 		if (!file.exists()) return;
@@ -380,11 +383,28 @@ public class RepositoryManager {
 				is.close();
 			}
 		} catch (IOException e) {
-			CVSUIPlugin.log(new Status(Status.ERROR, CVSUIPlugin.ID, TeamException.UNABLE, Policy.bind("RepositoryManager.ioException"), e)); //$NON-NLS-1$
+			CVSUIPlugin.log(IStatus.ERROR, CVSUIMessages.RepositoryManager_ioException, e); 
 		} catch (TeamException e) {
-			CVSUIPlugin.log(e.getStatus());
+			CVSUIPlugin.log(e);
 		}
 	}
+    private void loadCommentTemplates() {
+        IPath pluginStateLocation = CVSUIPlugin.getPlugin().getStateLocation().append(COMMENT_TEMPLATES_FILE);
+        File file = pluginStateLocation.toFile();
+        if (!file.exists()) return;
+        try {
+            BufferedInputStream is = new BufferedInputStream(new FileInputStream(file));
+            try {
+                readCommentTemplates(is);
+            } finally {
+                is.close();
+            }
+        } catch (IOException e) {
+            CVSUIPlugin.log(IStatus.ERROR, CVSUIMessages.RepositoryManager_ioException, e);
+        } catch (TeamException e) {
+            CVSUIPlugin.log(e);
+        }
+    }
 	
 	protected void saveState() throws TeamException {
 		IPath pluginStateLocation = CVSUIPlugin.getPlugin().getStateLocation();
@@ -402,13 +422,13 @@ public class RepositoryManager {
 			}
 			boolean renamed = tempFile.renameTo(stateFile);
 			if (!renamed) {
-				throw new TeamException(new Status(Status.ERROR, CVSUIPlugin.ID, TeamException.UNABLE, Policy.bind("RepositoryManager.rename", tempFile.getAbsolutePath()), null)); //$NON-NLS-1$
+				throw new TeamException(new Status(IStatus.ERROR, CVSUIPlugin.ID, TeamException.UNABLE, NLS.bind(CVSUIMessages.RepositoryManager_rename, new String[] { tempFile.getAbsolutePath() }), null)); 
 			}
 		} catch (IOException e) {
-			throw new TeamException(new Status(Status.ERROR, CVSUIPlugin.ID, TeamException.UNABLE, Policy.bind("RepositoryManager.save",stateFile.getAbsolutePath()), e)); //$NON-NLS-1$
+			throw new TeamException(new Status(IStatus.ERROR, CVSUIPlugin.ID, TeamException.UNABLE, NLS.bind(CVSUIMessages.RepositoryManager_save, new String[] { stateFile.getAbsolutePath() }), e)); 
 		}
 	}
-	private void writeState(XMLWriter writer) throws IOException, CVSException {
+	private void writeState(XMLWriter writer) {
 		writer.startTag(RepositoriesViewContentHandler.REPOSITORIES_VIEW_TAG, null, true);
 		// Write the repositories
 		Collection repos = Arrays.asList(getKnownRepositoryLocations());
@@ -422,21 +442,26 @@ public class RepositoryManager {
 	}
 	
 	private void readState(InputStream stream) throws IOException, TeamException {
-		SAXParser parser = new SAXParser();
-		parser.setContentHandler(new RepositoriesViewContentHandler(this));
 		try {
-			parser.parse(new InputSource(stream));
+			SAXParserFactory factory = SAXParserFactory.newInstance();
+			SAXParser parser = factory.newSAXParser();
+			parser.parse(new InputSource(stream), new RepositoriesViewContentHandler(this));
 		} catch (SAXException ex) {
-			throw new CVSException(Policy.bind("RepositoryManager.parsingProblem", REPOSITORIES_VIEW_FILE), ex); //$NON-NLS-1$
+			throw new CVSException(NLS.bind(CVSUIMessages.RepositoryManager_parsingProblem, new String[] { REPOSITORIES_VIEW_FILE }), ex); 
+		} catch (ParserConfigurationException ex) {
+			throw new CVSException(NLS.bind(CVSUIMessages.RepositoryManager_parsingProblem, new String[] { REPOSITORIES_VIEW_FILE }), ex); 
 		}
 	}
+	
 	private void readCommentHistory(InputStream stream) throws IOException, TeamException {
-		SAXParser parser = new SAXParser();
-		parser.setContentHandler(new CommentHistoryContentHandler());
 		try {
-			parser.parse(new InputSource(stream));
+			SAXParserFactory factory = SAXParserFactory.newInstance();
+			SAXParser parser = factory.newSAXParser();
+			parser.parse(new InputSource(stream), new CommentHistoryContentHandler());
 		} catch (SAXException ex) {
-			throw new CVSException(Policy.bind("RepositoryManager.parsingProblem", COMMENT_HIST_FILE), ex); //$NON-NLS-1$
+			throw new CVSException(NLS.bind(CVSUIMessages.RepositoryManager_parsingProblem, new String[] { COMMENT_HIST_FILE }), ex); 
+		} catch (ParserConfigurationException ex) {
+			throw new CVSException(NLS.bind(CVSUIMessages.RepositoryManager_parsingProblem, new String[] { COMMENT_HIST_FILE }), ex); 
 		}
 	}
 	
@@ -448,7 +473,7 @@ public class RepositoryManager {
 			repoSize = dis.readInt();
 		}
 		for (int i = 0; i < repoSize; i++) {
-			ICVSRepositoryLocation root = CVSProviderPlugin.getPlugin().getRepository(dis.readUTF());
+			ICVSRepositoryLocation root = KnownRepositories.getInstance().getRepository(dis.readUTF());
 			RepositoryRoot repoRoot = getRepositoryRootFor(root);
 			
 			// read branch tags associated with this root
@@ -486,7 +511,7 @@ public class RepositoryManager {
 							Set filenames = new HashSet();
 							int numFilenames = dis.readInt();
 							for (int k = 0; k < numFilenames; k++) {
-								filenames.add(dis.readUTF());
+								filenames.add(name + "/" + dis.readUTF()); //$NON-NLS-1$
 							}
 							repoRoot.setAutoRefreshFiles(name, (String[]) filenames.toArray(new String[filenames.size()]));
 						}
@@ -515,13 +540,13 @@ public class RepositoryManager {
 		 		 }
 		 		 boolean renamed = tempFile.renameTo(histFile);
 		 		 if (!renamed) {
-		 		 		 throw new TeamException(new Status(Status.ERROR, CVSUIPlugin.ID, TeamException.UNABLE, Policy.bind("RepositoryManager.rename", tempFile.getAbsolutePath()), null)); //$NON-NLS-1$
+		 		 		 throw new TeamException(new Status(IStatus.ERROR, CVSUIPlugin.ID, TeamException.UNABLE, NLS.bind(CVSUIMessages.RepositoryManager_rename, new String[] { tempFile.getAbsolutePath() }), null)); 
 		 		 }
 		 } catch (IOException e) {
-		 		 throw new TeamException(new Status(Status.ERROR, CVSUIPlugin.ID, TeamException.UNABLE, Policy.bind("RepositoryManager.save",histFile.getAbsolutePath()), e)); //$NON-NLS-1$
+		 		 throw new TeamException(new Status(IStatus.ERROR, CVSUIPlugin.ID, TeamException.UNABLE, NLS.bind(CVSUIMessages.RepositoryManager_save, new String[] { histFile.getAbsolutePath() }), e)); 
 		 }
 	}
-	private void writeCommentHistory(XMLWriter writer) throws IOException, CVSException {
+	private void writeCommentHistory(XMLWriter writer) {
 		writer.startTag(ELEMENT_COMMIT_HISTORY, null, false);
 		for (int i=0; i<previousComments.length && i<MAX_COMMENTS; i++)
 			writer.printSimpleTag(ELEMENT_COMMIT_COMMENT, previousComments[i]);
@@ -536,100 +561,20 @@ public class RepositoryManager {
 		listeners.remove(listener);
 	}
 	
-	/**
-	 * Add the given resources to their associated providers.
-	 * This schedules the resources for addition; they still need to be committed.
-	 */
-	public void add(IResource[] resources, IProgressMonitor monitor) throws TeamException {
-		Map table = getProviderMapping(resources);
-		Set keySet = table.keySet();
-		monitor.beginTask("", keySet.size() * 1000); //$NON-NLS-1$
-		monitor.setTaskName(Policy.bind("RepositoryManager.adding")); //$NON-NLS-1$
-		Iterator iterator = keySet.iterator();
-		while (iterator.hasNext()) {
-			IProgressMonitor subMonitor = new SubProgressMonitor(monitor, 1000);
-			CVSTeamProvider provider = (CVSTeamProvider)iterator.next();
-			provider.setComment(getCurrentComment());
-			List list = (List)table.get(provider);
-			IResource[] providerResources = (IResource[])list.toArray(new IResource[list.size()]);
-			provider.add(providerResources, IResource.DEPTH_ZERO, subMonitor);
-		}		
-	}
-	
-	/**
-	 * Method getCurrentComment.
-	 * @return String
-	 */
-	private String getCurrentComment() {
-		if (previousComments.length == 0)
-			return ""; //$NON-NLS-1$
-		return (String)previousComments[0];
-	}
-
-	/**
-	 * Delete the given resources from their associated providers.
-	 * This schedules the resources for deletion; they still need to be committed.
-	 */
-	public void delete(IResource[] resources, IProgressMonitor monitor) throws TeamException {
-		Map table = getProviderMapping(resources);
-		Set keySet = table.keySet();
-		monitor.beginTask("", keySet.size() * 1000); //$NON-NLS-1$
-		monitor.setTaskName(Policy.bind("RepositoryManager.deleting")); //$NON-NLS-1$
-		Iterator iterator = keySet.iterator();
-		while (iterator.hasNext()) {
-			IProgressMonitor subMonitor = new SubProgressMonitor(monitor, 1000);
-			CVSTeamProvider provider = (CVSTeamProvider)iterator.next();
-			provider.setComment(getCurrentComment());
-			List list = (List)table.get(provider);
-			IResource[] providerResources = (IResource[])list.toArray(new IResource[list.size()]);
-			provider.delete(providerResources, subMonitor);
-		}		
-	}
-	
-	public void update(IResource[] resources, Command.LocalOption[] options, boolean createBackups, IProgressMonitor monitor) throws TeamException {
-		Map table = getProviderMapping(resources);
-		Set keySet = table.keySet();
-		monitor.beginTask("", keySet.size() * 1000); //$NON-NLS-1$
-		monitor.setTaskName(Policy.bind("RepositoryManager.updating")); //$NON-NLS-1$
-		Iterator iterator = keySet.iterator();
-		while (iterator.hasNext()) {
-			IProgressMonitor subMonitor = new SubProgressMonitor(monitor, 1000);
-			CVSTeamProvider provider = (CVSTeamProvider)iterator.next();
-			List list = (List)table.get(provider);
-			IResource[] providerResources = (IResource[])list.toArray(new IResource[list.size()]);
-			provider.update(providerResources, options, null, createBackups, subMonitor);
-		}		
-	}
-	
-	/**
-	 * Mark the files as merged.
-	 */
-	public void merged(IRemoteSyncElement[] elements) throws TeamException {
-		Map table = getProviderMapping(elements);
-		Set keySet = table.keySet();
-		Iterator iterator = keySet.iterator();
-		while (iterator.hasNext()) {
-			CVSTeamProvider provider = (CVSTeamProvider)iterator.next();
-			provider.setComment(getCurrentComment());
-			List list = (List)table.get(provider);
-			IRemoteSyncElement[] providerElements = (IRemoteSyncElement[])list.toArray(new IRemoteSyncElement[list.size()]);
-			provider.merged(providerElements);
-		}		
-	}
-	
 	/**
 	 * Return the entered comment or null if canceled.
+	 * @param proposedComment
 	 */
-	public String promptForComment(final Shell shell, IResource[] resourcesToCommit) {
+	public String promptForComment(final Shell shell, IResource[] resourcesToCommit, String proposedComment) {
 		final int[] result = new int[1];
-		final ReleaseCommentDialog dialog = new ReleaseCommentDialog(shell, resourcesToCommit); 
+		final ReleaseCommentDialog dialog = new ReleaseCommentDialog(shell, resourcesToCommit, proposedComment, IResource.DEPTH_INFINITE); 
 		shell.getDisplay().syncExec(new Runnable() {
 			public void run() {
 				result[0] = dialog.open();
-				if (result[0] != ReleaseCommentDialog.OK) return;
+				if (result[0] != Window.OK) return;
 			}
 		});
-		if (result[0] != ReleaseCommentDialog.OK) return null;
+		if (result[0] != Window.OK) return null;
 		return dialog.getComment();
 	}
 	
@@ -660,68 +605,12 @@ public class RepositoryManager {
 		});
 		return result[0];
 	}
-	/**
-	 * Commit the given resources to their associated providers.
-	 * 
-	 * @param resources  the resources to commit
-	 * @param monitor  the progress monitor
-	 */
-	public void commit(IResource[] resources, String comment, IProgressMonitor monitor) throws TeamException {
-		Map table = getProviderMapping(resources);
-		Set keySet = table.keySet();
-		monitor.beginTask("", keySet.size() * 1000); //$NON-NLS-1$
-		monitor.setTaskName(Policy.bind("RepositoryManager.committing")); //$NON-NLS-1$
-		Iterator iterator = keySet.iterator();
-		while (iterator.hasNext()) {
-			IProgressMonitor subMonitor = new SubProgressMonitor(monitor, 1000);
-			CVSTeamProvider provider = (CVSTeamProvider)iterator.next();
-			provider.setComment(comment);
-			List list = (List)table.get(provider);
-			IResource[] providerResources = (IResource[])list.toArray(new IResource[list.size()]);
-			provider.checkin(providerResources, IResource.DEPTH_INFINITE, subMonitor);
-		}
-	}
-	
-	/**
-	 * Helper method. Return a Map mapping provider to a list of resources
-	 * shared with that provider.
-	 */
-	private Map getProviderMapping(IResource[] resources) {
-		Map result = new HashMap();
-		for (int i = 0; i < resources.length; i++) {
-			RepositoryProvider provider = RepositoryProvider.getProvider(resources[i].getProject(), CVSProviderPlugin.getTypeId());
-			List list = (List)result.get(provider);
-			if (list == null) {
-				list = new ArrayList();
-				result.put(provider, list);
-			}
-			list.add(resources[i]);
-		}
-		return result;
-	}
-	/**
-	 * Helper method. Return a Map mapping provider to a list of IRemoteSyncElements
-	 * shared with that provider.
-	 */
-	private Map getProviderMapping(IRemoteSyncElement[] elements) {
-		Map result = new HashMap();
-		for (int i = 0; i < elements.length; i++) {
-			RepositoryProvider provider = RepositoryProvider.getProvider(elements[i].getLocal().getProject(), CVSProviderPlugin.getTypeId());
-			List list = (List)result.get(provider);
-			if (list == null) {
-				list = new ArrayList();
-				result.put(provider, list);
-			}
-			list.add(elements[i]);
-		}
-		return result;
-	}
 	
 	public ICVSRepositoryLocation getRepositoryLocationFor(ICVSResource resource) {
 		try {
 			return internalGetRepositoryLocationFor(resource);
 		} catch (CVSException e) {
-			CVSUIPlugin.log(e.getStatus());
+			CVSUIPlugin.log(e);
 			return null;
 		}
 	}
@@ -734,7 +623,7 @@ public class RepositoryManager {
 			folder = resource.getParent();
 		}
 		if (folder.isCVSFolder()) {
-			ICVSRepositoryLocation location = CVSProviderPlugin.getPlugin().getRepository(folder.getFolderSyncInfo().getRoot());
+			ICVSRepositoryLocation location = KnownRepositories.getInstance().getRepository(folder.getFolderSyncInfo().getRoot());
 			return location;
 		}
 		// XXX This is asking for trouble
@@ -748,7 +637,7 @@ public class RepositoryManager {
 	}
 	
 	public RepositoryRoot getRepositoryRootFor(ICVSRepositoryLocation location) {
-		RepositoryRoot root = (RepositoryRoot)repositoryRoots.get(location.getLocation());
+		RepositoryRoot root = (RepositoryRoot)repositoryRoots.get(location.getLocation(false));
 		if (root == null) {
 			root = new RepositoryRoot(location);
 			add(root);
@@ -763,7 +652,7 @@ public class RepositoryManager {
 	 * @param currentRepositoryRoot
 	 */
 	public void add(RepositoryRoot root) {
-		repositoryRoots.put(root.getRoot().getLocation(), root);
+		repositoryRoots.put(root.getRoot().getLocation(false), root);
 		broadcastRepositoryChange(root);
 	}
 	
@@ -771,7 +660,7 @@ public class RepositoryManager {
 		if (notificationLevel == 0) {
 			broadcastRepositoriesChanged(new ICVSRepositoryLocation[] {root.getRoot()});
 		} else {
-			changedRepositories.put(root.getRoot().getLocation(), root.getRoot());
+			changedRepositories.put(root.getRoot().getLocation(false), root.getRoot());
 		}
 	}
 	
@@ -852,22 +741,12 @@ public class RepositoryManager {
 		return (ICVSRemoteResource[]) result.toArray(new ICVSRemoteResource[result.size()]);
 	}
 	
-	/**
-	 * This method is invoked whenever the refresh button in the
-	 * RepositoriesView is pressed.
-	 */
-	void clearCaches() {
-		for (Iterator iter = repositoryRoots.values().iterator(); iter.hasNext();) {
-			RepositoryRoot root = (RepositoryRoot) iter.next();
-			root.clearCache();
-		}
-	}
 	/**
 	 * Method setLabel.
 	 * @param location
 	 * @param label
 	 */
-	public void setLabel(CVSRepositoryLocation location, String label) throws CVSException {
+	public void setLabel(CVSRepositoryLocation location, String label) {
 		RepositoryRoot root = getRepositoryRootFor(location);
 		String oldLabel = root.getName();
 		if (oldLabel == null) {
@@ -891,23 +770,22 @@ public class RepositoryManager {
 	 */
 	public void replaceRepositoryLocation(
 			final ICVSRepositoryLocation oldLocation,
-			final CVSRepositoryLocation newLocation) throws CVSException {
+			final CVSRepositoryLocation newLocation) {
 		
 		try {
 			run(new IRunnableWithProgress() {
 				public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
-					try {
-						RepositoryRoot root = getRepositoryRootFor(oldLocation);
-						// Disposing of the old location will result in the deletion of the
-						// cached root through a listener callback
-						CVSProviderPlugin.getPlugin().disposeRepository(oldLocation);
-						
-						newLocation.updateCache();
-						root.setRepositoryLocation(newLocation);
-						add(root);
-					} catch (CVSException e) {
-						throw new InvocationTargetException(e);
-					}
+					RepositoryRoot root = getRepositoryRootFor(oldLocation);
+					// Disposing of the old location will result in the deletion of the
+					// cached root through a listener callback
+					KnownRepositories.getInstance().disposeRepository(oldLocation);
+					
+					// Get the new location from the CVS plugin to ensure we use the
+					// instance that will be returned by future calls to getRepository()
+					boolean isNew = !KnownRepositories.getInstance().isKnownRepository(newLocation.getLocation());
+					root.setRepositoryLocation(
+							KnownRepositories.getInstance().addRepository(newLocation, isNew /* broadcast */));
+					add(root);
 				}
 			}, Policy.monitorFor(null));
 		} catch (InvocationTargetException e) {
@@ -939,8 +817,15 @@ public class RepositoryManager {
 	 * @param string
 	 */
 	public void addComment(String comment) {
-		// Only add the comment if the first entry isn't the same already
-		if (previousComments.length > 0 && previousComments[0].equals(comment)) return;
+		// Make comment first element if it's already there
+		int index = getCommentIndex(comment);
+		if (index != -1) {
+			makeFirstElement(index);
+			return;
+		}
+		if (containsCommentTemplate(comment))
+			return;
+		
 		// Insert the comment as the first element
 		String[] newComments = new String[Math.min(previousComments.length + 1, MAX_COMMENTS)];
 		newComments[0] = comment;
@@ -949,4 +834,105 @@ public class RepositoryManager {
 		}
 		previousComments = newComments;
 	}
+
+	private int getCommentIndex(String comment) {
+		for (int i = 0; i < previousComments.length; i++) {
+			if (previousComments[i].equals(comment)) {
+				return i;
+			}
+		}
+		return -1;
+	}
+	
+	private void makeFirstElement(int index) {
+		String[] newComments = new String[previousComments.length];
+		newComments[0] = previousComments[index];
+		System.arraycopy(previousComments, 0, newComments, 1, index);
+		int maxIndex = previousComments.length - 1;
+		if (index != maxIndex) {
+			int nextIndex = (index + 1);
+			System.arraycopy(previousComments, nextIndex, newComments,
+					nextIndex, (maxIndex - index));
+		}
+		previousComments = newComments;
+	}
+	
+	private void readCommentTemplates(InputStream stream) throws IOException, TeamException {
+		try {
+			SAXParserFactory factory = SAXParserFactory.newInstance();
+			SAXParser parser = factory.newSAXParser();
+			parser.parse(new InputSource(stream),
+					new CommentTemplatesContentHandler());
+		} catch (SAXException ex) {
+			throw new CVSException(NLS.bind(
+					CVSUIMessages.RepositoryManager_parsingProblem,
+					new String[] { COMMENT_TEMPLATES_FILE }), ex);
+		} catch (ParserConfigurationException ex) {
+			throw new CVSException(NLS.bind(
+					CVSUIMessages.RepositoryManager_parsingProblem,
+					new String[] { COMMENT_TEMPLATES_FILE }), ex);
+		}
+	}
+	
+	protected void saveCommentTemplates() throws TeamException {
+		IPath pluginStateLocation = CVSUIPlugin.getPlugin().getStateLocation();
+		File tempFile = pluginStateLocation.append(
+				COMMENT_TEMPLATES_FILE + ".tmp").toFile(); //$NON-NLS-1$
+		File histFile = pluginStateLocation.append(COMMENT_TEMPLATES_FILE)
+				.toFile();
+		try {
+			XMLWriter writer = new XMLWriter(new BufferedOutputStream(
+					new FileOutputStream(tempFile)));
+			try {
+				writeCommentTemplates(writer);
+			} finally {
+				writer.close();
+			}
+			if (histFile.exists()) {
+				histFile.delete();
+			}
+			boolean renamed = tempFile.renameTo(histFile);
+			if (!renamed) {
+				throw new TeamException(new Status(IStatus.ERROR,
+						CVSUIPlugin.ID, TeamException.UNABLE, NLS.bind(
+								CVSUIMessages.RepositoryManager_rename,
+								new String[] { tempFile.getAbsolutePath() }),
+						null));
+			}
+		} catch (IOException e) {
+			throw new TeamException(new Status(IStatus.ERROR, CVSUIPlugin.ID,
+					TeamException.UNABLE, NLS.bind(
+							CVSUIMessages.RepositoryManager_save,
+							new String[] { histFile.getAbsolutePath() }), e));
+		}
+	}
+	
+	private void writeCommentTemplates(XMLWriter writer) {
+		writer.startTag(ELEMENT_COMMENT_TEMPLATES, null, false);
+		for (int i = 0; i < commentTemplates.length; i++)
+			writer.printSimpleTag(ELEMENT_COMMIT_COMMENT, commentTemplates[i]);
+		writer.endTag(ELEMENT_COMMENT_TEMPLATES);
+	}
+	
+	private boolean containsCommentTemplate(String comment) {
+		for (int i = 0; i < commentTemplates.length; i++) {
+			if (commentTemplates[i].equals(comment)) {
+				return true;
+			}
+		}
+		return false;
+	}
+	
+	/**
+	 * Get list of comment templates.
+	 */
+	public String[] getCommentTemplates() {
+		return commentTemplates;
+	}
+	
+	public void replaceAndSaveCommentTemplates(String[] templates)
+			throws TeamException {
+		commentTemplates = templates;
+		saveCommentTemplates();
+	}
 }
