@@ -1,23 +1,27 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.team.internal.ui.synchronize;
 
 import java.io.*;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
 
 import org.eclipse.core.runtime.*;
 import org.eclipse.jface.dialogs.IDialogConstants;
 import org.eclipse.jface.dialogs.MessageDialogWithToggle;
 import org.eclipse.jface.preference.IPreferenceStore;
-import org.eclipse.jface.util.ListenerList;
 import org.eclipse.osgi.util.NLS;
 import org.eclipse.team.core.TeamException;
 import org.eclipse.team.internal.ui.*;
@@ -100,7 +104,7 @@ public class SynchronizeManager implements ISynchronizeManager {
 		private ISynchronizeParticipant[] fChanged;
 
 		public void handleException(Throwable exception) {
-			TeamUIPlugin.log(IStatus.ERROR, TeamUIMessages.SynchronizeManager_7, exception); //$NON-NLS-1$
+			TeamUIPlugin.log(IStatus.ERROR, TeamUIMessages.SynchronizeManager_7, exception); 
 		}
 
 		public void run() throws Exception {
@@ -223,7 +227,7 @@ public class SynchronizeManager implements ISynchronizeManager {
 			} catch (TeamException e) {
 				TeamUIPlugin.log(e);
 				participantReferences.remove(key);
-				throw new TeamException(TeamUIMessages.SynchronizeManager_8, e); //$NON-NLS-1$
+				throw new TeamException(TeamUIMessages.SynchronizeManager_8, e); 
 			}
 		}
 
@@ -247,11 +251,11 @@ public class SynchronizeManager implements ISynchronizeManager {
 					savedState = null;
 					return participant;
 				} catch (PartInitException e) {				
-					throw new TeamException(NLS.bind(TeamUIMessages.SynchronizeManager_11, new String[] { descriptor.getName() }), e);  //$NON-NLS-1$
+					throw new TeamException(NLS.bind(TeamUIMessages.SynchronizeManager_11, new String[] { descriptor.getName() }), e);  
 				} catch (CoreException e) {
 					throw TeamException.asTeamException(e);
 				} catch(Exception e) {
-					throw new TeamException(NLS.bind(TeamUIMessages.SynchronizeManager_11, new String[] { descriptor.getName() }), e);  //$NON-NLS-1$
+					throw new TeamException(NLS.bind(TeamUIMessages.SynchronizeManager_11, new String[] { descriptor.getName() }), e);  
 				}
 			}
 
@@ -282,7 +286,7 @@ public class SynchronizeManager implements ISynchronizeManager {
 	 */
 	public void addSynchronizeParticipantListener(ISynchronizeParticipantListener listener) {
 		if (fListeners == null) {
-			fListeners = new ListenerList(5);
+			fListeners = new ListenerList(ListenerList.IDENTITY);
 		}
 		fListeners.add(listener);
 	}
@@ -315,7 +319,7 @@ public class SynchronizeManager implements ISynchronizeManager {
 		SynchronizeParticipantDescriptor desc = participantRegistry.find(type);
 		// ensure that the view id is valid
 		if (desc == null)
-			throw new PartInitException(NLS.bind(TeamUIMessages.SynchronizeManager_19, new String[] { type })); //$NON-NLS-1$
+			throw new PartInitException(NLS.bind(TeamUIMessages.SynchronizeManager_19, new String[] { type })); 
 		// ensure that multiple instances are allowed if a secondary id is given
 		if (secondaryId != null) {
 //		    if (!desc.isMultipleInstances()) {
@@ -345,7 +349,7 @@ public class SynchronizeManager implements ISynchronizeManager {
 				try {
 					ParticipantInstance ref = createParticipantReference(participant.getId(), participant.getSecondaryId(), participant.getName());
 					ref.setParticipant(participant);
-					removeMatchingPinnedParticipant(participant.getId());
+					removeMatchingParticipant(participant.getId());
 					participantReferences.put(key, ref);
 					added.add(participant);
 				} catch (PartInitException e) {
@@ -360,7 +364,7 @@ public class SynchronizeManager implements ISynchronizeManager {
 		}
 	}
 	
-	private void removeMatchingPinnedParticipant(String id) {
+	private void removeMatchingParticipant(String id) {
 		ISynchronizeParticipantReference[] refs = get(id);
 		if (refs.length > 0) {
 			// Find an un-pinned participant and replace it
@@ -369,7 +373,7 @@ public class SynchronizeManager implements ISynchronizeManager {
 				ISynchronizeParticipant p;
 				try {
 					p = reference.getParticipant();
-					if (!p.isPinned()) {
+					if (!p.isPinned() && !isDirty(p)) {
 						removeSynchronizeParticipants(new ISynchronizeParticipant[]{p});
 						break;
 					}
@@ -380,6 +384,17 @@ public class SynchronizeManager implements ISynchronizeManager {
 		}
 	}
 
+	private boolean isDirty(ISynchronizeParticipant p) {
+		if (p instanceof ModelSynchronizeParticipant) {
+			ModelSynchronizeParticipant msp = (ModelSynchronizeParticipant) p;
+			Saveable s = msp.getActiveSaveable();
+			if (s != null && s.isDirty()) {
+				return true;
+			}
+		}
+		return false;
+	}
+
 	/*
 	 * (non-Javadoc)
 	 * 
@@ -450,7 +465,7 @@ public class SynchronizeManager implements ISynchronizeManager {
 				String pId = TeamUIPlugin.getPlugin().getPreferenceStore().getString(IPreferenceIds.SYNCVIEW_DEFAULT_PERSPECTIVE);
 				activePage = workbench.showPerspective(pId, window);
 			} catch (WorkbenchException e) {
-				Utils.handleError(window.getShell(), e, TeamUIMessages.SynchronizeView_14, e.getMessage()); //$NON-NLS-1$
+				Utils.handleError(window.getShell(), e, TeamUIMessages.SynchronizeView_14, e.getMessage()); 
 			}
 		}
 		try {
@@ -465,11 +480,11 @@ public class SynchronizeManager implements ISynchronizeManager {
 				return (ISynchronizeView) part;
 			} catch (ClassCastException e) {
 				// Strange that we cannot cast the part (see bug 53671)
-				TeamUIPlugin.log(IStatus.ERROR, NLS.bind(TeamUIMessages.SynchronizeManager_18, new String[] { part.getClass().getName() }), e); //$NON-NLS-1$
+				TeamUIPlugin.log(IStatus.ERROR, NLS.bind(TeamUIMessages.SynchronizeManager_18, new String[] { part.getClass().getName() }), e); 
 				return null;
 			}
 		} catch (PartInitException pe) {
-			Utils.handleError(window.getShell(), pe, TeamUIMessages.SynchronizeView_16, pe.getMessage()); //$NON-NLS-1$
+			Utils.handleError(window.getShell(), pe, TeamUIMessages.SynchronizeView_16, pe.getMessage()); 
 			return null;
 		}
 	}
@@ -503,10 +518,17 @@ public class SynchronizeManager implements ISynchronizeManager {
 		
 		if(perspectiveDescriptor != null) {
 			
+			String message;;
+			String desc = perspectiveDescriptor.getDescription();
+			if (desc == null) {
+				message = NLS.bind(TeamUIMessages.SynchronizeManager_30, new String[] { perspectiveDescriptor.getLabel() });
+			} else {
+				message = NLS.bind(TeamUIMessages.SynchronizeManager_32, new String[] { perspectiveDescriptor.getLabel(), desc });
+			}
 			MessageDialogWithToggle m = MessageDialogWithToggle.openYesNoQuestion(Utils.getShell(null),
-						TeamUIMessages.SynchronizeManager_27,  //$NON-NLS-1$
-						NLS.bind(TeamUIMessages.SynchronizeManager_30, new String[] { perspectiveDescriptor.getLabel() }), //$NON-NLS-1$
-						TeamUIMessages.SynchronizeManager_31,  //$NON-NLS-1$
+						TeamUIMessages.SynchronizeManager_27,  
+						message, 
+						TeamUIMessages.SynchronizeManager_31,  
 						false /* toggle state */,
 						store,
 						IPreferenceIds.SYNCHRONIZING_COMPLETE_PERSPECTIVE);
@@ -540,7 +562,7 @@ public class SynchronizeManager implements ISynchronizeManager {
 			// previous session.
 			restoreSavedParticipants();
 		} catch (CoreException e) {
-			TeamUIPlugin.log(new Status(IStatus.ERROR, TeamUIPlugin.ID, 1, TeamUIMessages.SynchronizeManager_8, e)); //$NON-NLS-1$
+			TeamUIPlugin.log(new Status(IStatus.ERROR, TeamUIPlugin.ID, 1, TeamUIMessages.SynchronizeManager_8, e)); 
 		}
 	}
 
@@ -587,7 +609,7 @@ public class SynchronizeManager implements ISynchronizeManager {
 					String key = Utils.getKey(id, secondayId);
 					participantReferences.put(key, new ParticipantInstance(desc, secondayId, displayName, memento2.getChild(CTX_PARTICIPANT_DATA)));
 				} else {
-					TeamUIPlugin.log(new Status(IStatus.ERROR, TeamUIPlugin.ID, 1, NLS.bind(TeamUIMessages.SynchronizeManager_9, new String[] { id }), null)); //$NON-NLS-1$
+					TeamUIPlugin.log(new Status(IStatus.ERROR, TeamUIPlugin.ID, 1, NLS.bind(TeamUIMessages.SynchronizeManager_9, new String[] { id }), null)); 
 				}
 			}
 		}
@@ -623,13 +645,13 @@ public class SynchronizeManager implements ISynchronizeManager {
 				writer.close();
 			}
 		} catch (IOException e) {
-			TeamUIPlugin.log(new Status(IStatus.ERROR, TeamUIPlugin.ID, 1, TeamUIMessages.SynchronizeManager_10, e)); //$NON-NLS-1$
+			TeamUIPlugin.log(new Status(IStatus.ERROR, TeamUIPlugin.ID, 1, TeamUIMessages.SynchronizeManager_10, e)); 
 		}
 	}
 
 	private File getStateFile() {
 		IPath pluginStateLocation = TeamUIPlugin.getPlugin().getStateLocation();
-		return pluginStateLocation.append(FILENAME).toFile(); //$NON-NLS-1$	
+		return pluginStateLocation.append(FILENAME).toFile(); //	
 	}
 	
 	/**
