@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -13,23 +13,19 @@ package org.eclipse.team.internal.ui.synchronize;
 import java.util.HashMap;
 import java.util.Map;
 
-import org.eclipse.core.runtime.ISafeRunnable;
-import org.eclipse.core.runtime.Platform;
+import org.eclipse.core.runtime.*;
 import org.eclipse.jface.action.IMenuManager;
 import org.eclipse.jface.operation.IRunnableContext;
 import org.eclipse.jface.util.IPropertyChangeListener;
-import org.eclipse.jface.util.ListenerList;
 import org.eclipse.jface.util.PropertyChangeEvent;
 import org.eclipse.jface.viewers.ILabelDecorator;
+import org.eclipse.team.core.diff.IDiff;
+import org.eclipse.team.core.diff.IThreeWayDiff;
+import org.eclipse.team.core.mapping.IResourceDiff;
 import org.eclipse.team.core.synchronize.SyncInfoSet;
 import org.eclipse.team.internal.ui.TeamUIPlugin;
-import org.eclipse.team.internal.ui.synchronize.actions.DefaultSynchronizePageActions;
-import org.eclipse.team.ui.synchronize.ISynchronizeModelElement;
-import org.eclipse.team.ui.synchronize.ISynchronizePage;
-import org.eclipse.team.ui.synchronize.ISynchronizePageConfiguration;
-import org.eclipse.team.ui.synchronize.ISynchronizePageSite;
-import org.eclipse.team.ui.synchronize.ISynchronizeParticipant;
-import org.eclipse.team.ui.synchronize.SynchronizePageActionGroup;
+import org.eclipse.team.internal.ui.mapping.CommonViewerAdvisor;
+import org.eclipse.team.ui.synchronize.*;
 import org.eclipse.ui.IActionBars;
 import org.eclipse.ui.actions.ActionContext;
 
@@ -77,14 +73,14 @@ public class SynchronizePageConfiguration extends SynchronizePageActionGroup imp
 
 	/**
 	 * Property that gives access to a set the
-	 * contains all out-of-sync resources for the particpant
+	 * contains all out-of-sync resources for the participant
 	 * in the selected working set.
 	 */
 	public static final String P_WORKING_SET_SYNC_INFO_SET = TeamUIPlugin.ID + ".P_WORKING_SET_SYNC_INFO_SET"; //$NON-NLS-1$
 
 	/**
 	 * Property that gives access to a set the
-	 * contains all out-of-sync resources for the particpant
+	 * contains all out-of-sync resources for the participant
 	 * before any filtering (working set or modes) is applied.
 	 */
 	public static final String P_PARTICIPANT_SYNC_INFO_SET = TeamUIPlugin.ID + ".P_PARTICIPANT_SYNC_INFO_SET"; //$NON-NLS-1$
@@ -110,16 +106,16 @@ public class SynchronizePageConfiguration extends SynchronizePageActionGroup imp
     
 	private ISynchronizeParticipant participant;
 	private ISynchronizePageSite site;
-	private ListenerList propertyChangeListeners = new ListenerList();
-	private ListenerList actionContributions = new ListenerList();
+	private ListenerList propertyChangeListeners = new ListenerList(ListenerList.IDENTITY);
+	private ListenerList actionContributions = new ListenerList(ListenerList.IDENTITY);
 	private Map properties = new HashMap();
 	private int actionState = UNINITIALIZED;
 	private ISynchronizePage page;
 	private IRunnableContext context;
 	
 	/**
-	 * Create a configuration for creating a page from the given particpant.
-	 * @param participant the particpant whose page is being configured
+	 * Create a configuration for creating a page from the given participant.
+	 * @param participant the participant whose page is being configured
 	 */
 	public SynchronizePageConfiguration(ISynchronizeParticipant participant) {
 		this.participant = participant;
@@ -127,7 +123,6 @@ public class SynchronizePageConfiguration extends SynchronizePageActionGroup imp
 		setProperty(P_TOOLBAR_MENU, DEFAULT_TOOLBAR_MENU);
 		setProperty(P_VIEW_MENU, DEFAULT_VIEW_MENU);
 		setProperty(P_COMPARISON_TYPE, THREE_WAY);
-		addActionContribution(new DefaultSynchronizePageActions());
 	}
 	
 	/* (non-Javadoc)
@@ -232,7 +227,7 @@ public class SynchronizePageConfiguration extends SynchronizePageActionGroup imp
 		final PropertyChangeEvent event = new PropertyChangeEvent(this, key, oldValue, newValue);
 		for (int i = 0; i < listeners.length; i++) {
 			final IPropertyChangeListener listener = (IPropertyChangeListener)listeners[i];
-			Platform.run(new ISafeRunnable() {
+			SafeRunner.run(new ISafeRunnable() {
 				public void handleException(Throwable exception) {
 					// Error is logged by platform
 				}
@@ -260,7 +255,7 @@ public class SynchronizePageConfiguration extends SynchronizePageActionGroup imp
         }
 		for (int i= 0; i < listeners.length; i++) {
 			final SynchronizePageActionGroup contribution = (SynchronizePageActionGroup)listeners[i];
-			Platform.run(new ISafeRunnable() {
+			SafeRunner.run(new ISafeRunnable() {
 				public void handleException(Throwable exception) {
 					// Logged by Platform
 				}
@@ -282,7 +277,7 @@ public class SynchronizePageConfiguration extends SynchronizePageActionGroup imp
         }
 		for (int i= 0; i < listeners.length; i++) {
 			final SynchronizePageActionGroup contribution = (SynchronizePageActionGroup)listeners[i];
-			Platform.run(new ISafeRunnable() {
+			SafeRunner.run(new ISafeRunnable() {
 				public void handleException(Throwable exception) {
 					// Logged by Platform
 				}
@@ -305,7 +300,7 @@ public class SynchronizePageConfiguration extends SynchronizePageActionGroup imp
         }
 		for (int i= 0; i < listeners.length; i++) {
 			final SynchronizePageActionGroup contribution = (SynchronizePageActionGroup)listeners[i];
-			Platform.run(new ISafeRunnable() {
+			SafeRunner.run(new ISafeRunnable() {
 				public void handleException(Throwable exception) {
 					// Logged by Platform
 				}
@@ -317,7 +312,7 @@ public class SynchronizePageConfiguration extends SynchronizePageActionGroup imp
 	}
 
 	/**
-	 * Callback invoked from the page to fil the action bars.
+	 * Callback invoked from the page to fill the action bars.
 	 * @param actionBars the action bars of the view
 	 */
 	public void fillActionBars(final IActionBars actionBars) {
@@ -330,7 +325,7 @@ public class SynchronizePageConfiguration extends SynchronizePageActionGroup imp
         }
 		for (int i= 0; i < listeners.length; i++) {
 			final SynchronizePageActionGroup contribution = (SynchronizePageActionGroup)listeners[i];
-			Platform.run(new ISafeRunnable() {
+			SafeRunner.run(new ISafeRunnable() {
 				public void handleException(Throwable exception) {
 					// Logged by Platform
 				}
@@ -351,7 +346,7 @@ public class SynchronizePageConfiguration extends SynchronizePageActionGroup imp
         }
 		for (int i= 0; i < listeners.length; i++) {
 			final SynchronizePageActionGroup contribution = (SynchronizePageActionGroup)listeners[i];
-			Platform.run(new ISafeRunnable() {
+			SafeRunner.run(new ISafeRunnable() {
 				public void handleException(Throwable exception) {
 					// Logged by Platform
 				}
@@ -372,7 +367,7 @@ public class SynchronizePageConfiguration extends SynchronizePageActionGroup imp
         }
 		for (int i= 0; i < listeners.length; i++) {
 			final SynchronizePageActionGroup contribution = (SynchronizePageActionGroup)listeners[i];
-			Platform.run(new ISafeRunnable() {
+			SafeRunner.run(new ISafeRunnable() {
 				public void handleException(Throwable exception) {
 					// Logged by Platform
 				}
@@ -395,7 +390,7 @@ public class SynchronizePageConfiguration extends SynchronizePageActionGroup imp
         }
 		for (int i= 0; i < listeners.length; i++) {
 			final SynchronizePageActionGroup contribution = (SynchronizePageActionGroup)listeners[i];
-			Platform.run(new ISafeRunnable() {
+			SafeRunner.run(new ISafeRunnable() {
 				public void handleException(Throwable exception) {
 					// Logged by Platform
 				}
@@ -477,7 +472,7 @@ public class SynchronizePageConfiguration extends SynchronizePageActionGroup imp
 
 	/**
 	 * @param group
-	 * @return
+	 * @return the group id
 	 */
 	public String getGroupId(String group) {
 		String id = getParticipant().getId();
@@ -540,7 +535,7 @@ public class SynchronizePageConfiguration extends SynchronizePageActionGroup imp
 	}
 
 	/**
-	 * @return
+	 * @return the viewer style
 	 */
 	public int getViewerStyle() {
 		Object o = getProperty(P_VIEWER_STYLE);
@@ -595,4 +590,60 @@ public class SynchronizePageConfiguration extends SynchronizePageActionGroup imp
 	public IRunnableContext getRunnableContext() {
 		return context;
 	}
+
+	public String getViewerId() {
+		String viewerId = (String)getProperty(P_VIEWER_ID);
+		if (viewerId != null)
+			return viewerId;
+		return CommonViewerAdvisor.TEAM_NAVIGATOR_CONTENT;
+	}
+	
+	/**
+	 * Return whether the given node is visible in the page based
+	 * on the mode in the configuration.
+	 * @param node a diff node
+	 * @return whether the given node is visible in the page
+	 */
+	public boolean isVisible(IDiff node) {
+		if (getComparisonType() == ISynchronizePageConfiguration.THREE_WAY 
+				&& node instanceof IThreeWayDiff) {
+			IThreeWayDiff twd = (IThreeWayDiff) node;
+			return includeDirection(twd.getDirection());
+		}
+		return getComparisonType() == ISynchronizePageConfiguration.TWO_WAY && node instanceof IResourceDiff;
+	}
+	
+	/**
+	 * Return whether elements with the given direction should be included in
+	 * the contents.
+	 * 
+	 * @param direction
+	 *            the synchronization direction
+	 * @return whether elements with the given synchronization kind should be
+	 *         included in the contents
+	 */
+	public boolean includeDirection(int direction) {
+		int mode = getMode();
+		switch (mode) {
+		case ISynchronizePageConfiguration.BOTH_MODE:
+			return true;
+		case ISynchronizePageConfiguration.CONFLICTING_MODE:
+			return direction == IThreeWayDiff.CONFLICTING;
+		case ISynchronizePageConfiguration.INCOMING_MODE:
+			return direction == IThreeWayDiff.CONFLICTING || direction == IThreeWayDiff.INCOMING;
+		case ISynchronizePageConfiguration.OUTGOING_MODE:
+			return direction == IThreeWayDiff.CONFLICTING || direction == IThreeWayDiff.OUTGOING;
+		default:
+			break;
+		}
+		return true;
+	}
+	
+	public ILabelDecorator getLabelDecorator() {
+		ILabelDecorator[] decorators = (ILabelDecorator[])getProperty(ISynchronizePageConfiguration.P_LABEL_DECORATORS);
+		if (decorators == null) {
+			return null;
+		}
+		return new MultiLabelDecorator(decorators);
+	}
 }
