@@ -1,5 +1,5 @@
-/**********************************************************************
- * Copyright (c) 2004, 2005 IBM Corporation and others.
+/*******************************************************************************
+ * Copyright (c) 2004, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,7 +7,7 @@
  *
  * Contributors:
  *    IBM Corporation - Initial API and implementation
- **********************************************************************/
+ *******************************************************************************/
 package org.eclipse.ui.internal.browser;
 
 import java.beans.PropertyChangeEvent;
@@ -20,6 +20,7 @@ import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.jface.action.IStatusLineManager;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.SWTError;
+import org.eclipse.swt.SWTException;
 import org.eclipse.swt.browser.Browser;
 import org.eclipse.swt.browser.CloseWindowListener;
 import org.eclipse.swt.browser.LocationEvent;
@@ -328,7 +329,7 @@ public class BrowserViewer extends Composite {
         // shell within the browser area thereby maintaining the session.
         browser.addOpenWindowListener(new OpenWindowListener() {
             public void open(WindowEvent event) {
-                Shell shell2 = new Shell(getDisplay());
+                Shell shell2 = new Shell(getShell());
                 shell2.setLayout(new FillLayout());
                 shell2.setText(Messages.viewWebBrowserTitle);
                 shell2.setImage(getShell().getImage());
@@ -831,13 +832,11 @@ public class BrowserViewer extends Composite {
     }
 
     public boolean setFocus() {
-        if (combo != null)
-            combo.setFocus();
-        else if (browser!=null)
+        if (browser!=null) {
             browser.setFocus();
-        else if (text!=null)
-            text.setFocus();
-        updateHistory();
+            updateHistory();
+            return true;
+        }
         return super.setFocus();
     }
 
@@ -864,6 +863,11 @@ public class BrowserViewer extends Composite {
     }
 
     public void setContainer(IBrowserViewerContainer container) {
+    	if (container==null && this.container!=null) {
+    		IStatusLineManager manager = this.container.getActionBars().getStatusLineManager();
+    		if (manager!=null) 
+    			manager.getProgressMonitor().done();
+    	}
         this.container = container;
     }
 
@@ -944,4 +948,4 @@ public class BrowserViewer extends Composite {
    	 browser.removeLocationListener(locationListener2);
    	 locationListener2 = null;
     }
-}
\ No newline at end of file
+}
