@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,7 +12,6 @@
 
 package org.eclipse.ui.texteditor;
 
-import java.text.MessageFormat;
 import java.util.Stack;
 
 import org.eclipse.swt.SWT;
@@ -26,6 +25,11 @@ import org.eclipse.swt.events.VerifyEvent;
 import org.eclipse.swt.graphics.Color;
 import org.eclipse.swt.graphics.Point;
 
+import org.eclipse.core.commands.ExecutionEvent;
+import org.eclipse.core.commands.ExecutionException;
+import org.eclipse.core.commands.IExecutionListener;
+import org.eclipse.core.commands.NotHandledException;
+
 import org.eclipse.jface.action.IStatusLineManager;
 import org.eclipse.jface.viewers.ISelection;
 import org.eclipse.jface.viewers.ISelectionChangedListener;
@@ -42,11 +46,15 @@ import org.eclipse.jface.text.ITextViewer;
 import org.eclipse.jface.text.ITextViewerExtension;
 import org.eclipse.jface.text.TextEvent;
 
+import org.eclipse.ui.PlatformUI;
+import org.eclipse.ui.commands.ICommandService;
+import org.eclipse.ui.internal.texteditor.NLSUtility;
+
 /**
  * An incremental find target. Replace is always disabled.
  * @since 2.0
  */
-class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExtension, VerifyKeyListener, MouseListener, FocusListener, ISelectionChangedListener, ITextListener {
+class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExtension, VerifyKeyListener, MouseListener, FocusListener, ISelectionChangedListener, ITextListener, IExecutionListener {
 
 	/** The string representing rendered tab */
 	private final static String TAB= EditorMessages.Editor_FindIncremental_render_tab;
@@ -362,6 +370,10 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 			((ITextViewerExtension) fTextViewer).prependVerifyKeyListener(this);
 		else
 			text.addVerifyKeyListener(this);
+		
+		ICommandService commandService= (ICommandService)PlatformUI.getWorkbench().getAdapter(ICommandService.class);
+		if (commandService != null)
+			commandService.addExecutionListener(this);
 
 		fInstalled= true;
 	}
@@ -391,6 +403,10 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 				text.removeVerifyKeyListener(this);
 		}
 
+		ICommandService commandService= (ICommandService)PlatformUI.getWorkbench().getAdapter(ICommandService.class);
+		if (commandService != null)
+			commandService.removeExecutionListener(this);
+
 		fInstalled= false;
 	}
 
@@ -409,7 +425,7 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 
 		if (!fFound) {
 			String pattern= EditorMessages.Editor_FindIncremental_not_found_pattern;
-			statusError(MessageFormat.format(pattern, new Object[] { reversePrefix, wrapPrefix, string }));
+			statusError(NLSUtility.format(pattern, new Object[] { reversePrefix, wrapPrefix, string }));
 
 		} else if (string.length() == 0) {
 			if (fForward)
@@ -418,7 +434,7 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 				statusMessage(REVERSE_FIELD_NAME);
 		} else if (!fForward || fWrapPosition > -1) {
 			String pattern= EditorMessages.Editor_FindIncremental_found_pattern;
-			statusMessage(MessageFormat.format(pattern, new Object[] { reversePrefix, wrapPrefix, string }));
+			statusMessage(NLSUtility.format(pattern, new Object[] { reversePrefix, wrapPrefix, string }));
 		} else {
 			statusMessage(string);
 		}
@@ -783,4 +799,36 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 		fStatusField= statusField;
 		fIsStatusFieldExtension= fStatusField instanceof IStatusFieldExtension;
 	}
+
+	/*
+	 * @see org.eclipse.core.commands.IExecutionListener#notHandled(java.lang.String, org.eclipse.core.commands.NotHandledException)
+	 * @since 3.2
+	 */
+	public void notHandled(String commandId, NotHandledException exception) {
+	}
+
+	/*
+	 * @see org.eclipse.core.commands.IExecutionListener#postExecuteFailure(java.lang.String, org.eclipse.core.commands.ExecutionException)
+	 * @since 3.2
+	 */
+	public void postExecuteFailure(String commandId, ExecutionException exception) {
+	}
+
+	/*
+	 * @see org.eclipse.core.commands.IExecutionListener#postExecuteSuccess(java.lang.String, java.lang.Object)
+	 * @since 3.2
+	 */
+	public void postExecuteSuccess(String commandId, Object returnValue) {
+	}
+
+	/*
+	 * @see org.eclipse.core.commands.IExecutionListener#preExecute(java.lang.String, org.eclipse.core.commands.ExecutionEvent)
+	 * @since 3.2
+	 */
+	public void preExecute(String commandId, ExecutionEvent event) {
+		if (IWorkbenchActionDefinitionIds.FIND_INCREMENTAL.equals(commandId)
+				|| IWorkbenchActionDefinitionIds.FIND_INCREMENTAL_REVERSE.equals(commandId))
+			return;
+		leave();
+	}
 }
