@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,10 +12,6 @@ package org.eclipse.ui.texteditor.templates;
 
 import java.io.BufferedInputStream;
 import java.io.BufferedOutputStream;
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileNotFoundException;
-import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
@@ -24,8 +20,7 @@ import java.util.Iterator;
 import java.util.List;
 
 import org.eclipse.swt.SWT;
-import org.eclipse.swt.events.ControlAdapter;
-import org.eclipse.swt.events.ControlEvent;
+import org.eclipse.swt.graphics.GC;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
@@ -39,23 +34,22 @@ import org.eclipse.swt.widgets.Listener;
 import org.eclipse.swt.widgets.Table;
 import org.eclipse.swt.widgets.TableColumn;
 
-import org.eclipse.jface.text.Document;
-import org.eclipse.jface.text.IDocument;
-import org.eclipse.jface.text.source.SourceViewer;
-import org.eclipse.jface.text.source.SourceViewerConfiguration;
-import org.eclipse.jface.text.templates.ContextTypeRegistry;
-import org.eclipse.jface.text.templates.Template;
-import org.eclipse.jface.text.templates.TemplateContextType;
-import org.eclipse.jface.text.templates.persistence.TemplatePersistenceData;
-import org.eclipse.jface.text.templates.persistence.TemplateReaderWriter;
-import org.eclipse.jface.text.templates.persistence.TemplateStore;
+import org.eclipse.core.filesystem.EFS;
+import org.eclipse.core.filesystem.IFileInfo;
+import org.eclipse.core.filesystem.IFileStore;
+
+import org.eclipse.core.runtime.CoreException;
+import org.eclipse.core.runtime.Path;
 
 import org.eclipse.jface.dialogs.Dialog;
 import org.eclipse.jface.dialogs.MessageDialog;
 import org.eclipse.jface.preference.IPreferenceStore;
 import org.eclipse.jface.preference.PreferencePage;
+import org.eclipse.jface.resource.JFaceResources;
 import org.eclipse.jface.viewers.CheckStateChangedEvent;
 import org.eclipse.jface.viewers.CheckboxTableViewer;
+import org.eclipse.jface.viewers.ColumnPixelData;
+import org.eclipse.jface.viewers.ColumnWeightData;
 import org.eclipse.jface.viewers.DoubleClickEvent;
 import org.eclipse.jface.viewers.ICheckStateListener;
 import org.eclipse.jface.viewers.IDoubleClickListener;
@@ -69,6 +63,18 @@ import org.eclipse.jface.viewers.TableViewer;
 import org.eclipse.jface.viewers.Viewer;
 import org.eclipse.jface.viewers.ViewerSorter;
 import org.eclipse.jface.window.Window;
+
+import org.eclipse.jface.text.Document;
+import org.eclipse.jface.text.IDocument;
+import org.eclipse.jface.text.source.SourceViewer;
+import org.eclipse.jface.text.source.SourceViewerConfiguration;
+import org.eclipse.jface.text.templates.ContextTypeRegistry;
+import org.eclipse.jface.text.templates.Template;
+import org.eclipse.jface.text.templates.TemplateContextType;
+import org.eclipse.jface.text.templates.persistence.TemplatePersistenceData;
+import org.eclipse.jface.text.templates.persistence.TemplateReaderWriter;
+import org.eclipse.jface.text.templates.persistence.TemplateStore;
+
 import org.eclipse.ui.IWorkbench;
 import org.eclipse.ui.IWorkbenchPreferencePage;
 import org.eclipse.ui.internal.texteditor.NLSUtility;
@@ -222,28 +228,45 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
         gd.horizontalSpan= 2;
         innerParent.setLayoutData(gd);
 
-		Table table= new Table(innerParent, SWT.CHECK | SWT.BORDER | SWT.MULTI | SWT.FULL_SELECTION);
-
-		GridData data= new GridData(GridData.FILL_BOTH);
-		data.widthHint= 360;
-		data.heightHint= convertHeightInCharsToPixels(10);
-		table.setLayoutData(data);
+        Composite tableComposite= new Composite(innerParent, SWT.NONE);
+        GridData data= new GridData(GridData.FILL_BOTH);
+        data.widthHint= 360;
+        data.heightHint= convertHeightInCharsToPixels(10);
+        tableComposite.setLayoutData(data);
+        
+        ColumnLayout columnLayout= new ColumnLayout();
+        tableComposite.setLayout(columnLayout);
+		Table table= new Table(tableComposite, SWT.CHECK | SWT.BORDER | SWT.MULTI | SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
 
 		table.setHeaderVisible(true);
 		table.setLinesVisible(true);
 
+		GC gc= new GC(getShell());
+		gc.setFont(JFaceResources.getDialogFont());
+		
 		TableColumn column1= new TableColumn(table, SWT.NONE);
 		column1.setText(TextEditorTemplateMessages.TemplatePreferencePage_column_name);
+		int minWidth= computeMinimumColumnWidth(gc, TextEditorTemplateMessages.TemplatePreferencePage_column_name);
+		columnLayout.addColumnData(new ColumnWeightData(2, minWidth, true));
 
 		TableColumn column2= new TableColumn(table, SWT.NONE);
 		column2.setText(TextEditorTemplateMessages.TemplatePreferencePage_column_context);
+		minWidth= computeMinimumColumnWidth(gc, TextEditorTemplateMessages.TemplatePreferencePage_column_context);
+		columnLayout.addColumnData(new ColumnWeightData(1, minWidth, true));
 
 		TableColumn column3= new TableColumn(table, SWT.NONE);
 		column3.setText(TextEditorTemplateMessages.TemplatePreferencePage_column_description);
+		minWidth= computeMinimumColumnWidth(gc, TextEditorTemplateMessages.TemplatePreferencePage_column_description);
+		columnLayout.addColumnData(new ColumnWeightData(3, minWidth, true));
 
 		TableColumn column4= new TableColumn(table, SWT.NONE);
 		column4.setAlignment(SWT.CENTER);
 		column4.setText(TextEditorTemplateMessages.TemplatePreferencePage_column_autoinsert);
+		minWidth= computeMinimumColumnWidth(gc, TextEditorTemplateMessages.TemplatePreferencePage_column_autoinsert);
+		minWidth= Math.max(minWidth, computeMinimumColumnWidth(gc, TextEditorTemplateMessages.TemplatePreferencePage_on));
+		columnLayout.addColumnData(new ColumnPixelData(minWidth, false, false));
+		
+		gc.dispose();
 
 		fTableViewer= new CheckboxTableViewer(table);
 		fTableViewer.setLabelProvider(new TemplateLabelProvider());
@@ -376,14 +399,19 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 		fTableViewer.setCheckedElements(getEnabledTemplates());
 
 		updateButtons();
-        configureTableResizing(table);
-
 		Dialog.applyDialogFont(parent);
-		// trigger the resize
-		table.getHorizontalBar().setVisible(true);
+		innerParent.layout();
+		
 		return parent;
 	}
 
+	/*
+	 * @since 3.2
+	 */
+	private int computeMinimumColumnWidth(GC gc, String string) {
+		return gc.stringExtent(string).x + 10; // pad 10 to accommodate table header trimmings
+	}
+
 	/**
 	 * Creates a separator between buttons
 	 * @param parent
@@ -410,90 +438,6 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 		return true;
 	}
 
-	/**
-	 * Correctly resizes the table so no phantom columns appear
-	 * 
-	 * @param table the table
-	 * @since 3.1
-	 */
-	private static void configureTableResizing(final Table table) {
-	        ControlAdapter resizer= new ControlAdapter() {
-	        	private boolean fIsResizing= false;
-	        	private final int[] fWidths= {80, 80, 160, 50};
-	        	private int fSum= 370;
-	            public void controlResized(ControlEvent e) {
-	            	if (fIsResizing)
-	            		return;
-	            	try {
-	            		fIsResizing= true;
-	            		int clientAreaWidth= table.getClientArea().width;
-	            		TableColumn[] columns= table.getColumns();
-	            		int tableWidth= 0;
-	            		
-	            		if (e.widget == table) {
-	            			int initial[]= {80, 80, 80, 50};
-	            			int minimums[]= new int[columns.length];
-	            			int minSum= 0;
-	            			for (int i= 0; i < columns.length; i++) {
-								// don't make a column narrower than the minimum, 
-								// or than what it is currently if less than the minimum
-								minimums[i]= Math.min(fWidths[i], initial[i]);
-								minSum+= minimums[i];
-							}
-	            			
-	            			int newWidth= fSum < clientAreaWidth ? clientAreaWidth : Math.max(clientAreaWidth, minSum);
-							final int toDistribute= newWidth - fSum;
-							int lastPart= toDistribute;
-							if (toDistribute != 0) {
-								int[] iteration= {0,1,3,2}; // give the description column all the rest
-								for (int i= 0; i < iteration.length; i++) {
-									int c= iteration[i];
-									int width;
-									if (fSum > 0) {
-										int part;
-										if (i == iteration.length - 1)
-											part= lastPart;
-										else
-											// current width is the weight for the distribution of the extra space
-											part= toDistribute * fWidths[c] / fSum;
-										lastPart-= part;
-										width= Math.max(minimums[c], fWidths[c] + part);
-									} else {
-										width= toDistribute * initial[c] / 280;
-									}
-									columns[c].setWidth(width);
-									fWidths[c]= width;
-									tableWidth+= width;
-								}
-								fSum= tableWidth;
-							}
-	            		} else {
-	            			// column being resized
-	            			// on GTK, the last column gets auto-adapted - ignore this
-	            			if (e.widget == columns[3])
-	            				return;
-	            			for (int i= 0; i < columns.length; i++) {
-	            				fWidths[i]= columns[i].getWidth();
-								tableWidth+= fWidths[i];
-	            			}
-	            			fSum= tableWidth;
-	            		}
-	            		
-	            		// set scroll bar visible
-	            		table.getHorizontalBar().setVisible(tableWidth > clientAreaWidth);
-	            	} finally {
-	            		fIsResizing= false;
-	            	}
-	            }
-	        };
-			table.addControlListener(resizer);
-			TableColumn[] columns= table.getColumns();
-			for (int i= 0; i < columns.length; i++) {
-				columns[i].addControlListener(resizer);
-			}
-	    }
-
-
 	private TemplatePersistenceData[] getEnabledTemplates() {
 		List enabled= new ArrayList();
 		TemplatePersistenceData[] datas= fTemplateStore.getTemplateData(false);
@@ -685,12 +629,13 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 
 		if (path == null)
 			return;
+		
+		IFileStore fileStore= EFS.getLocalFileSystem().getStore(new Path(path));
 
 		try {
 			TemplateReaderWriter reader= new TemplateReaderWriter();
-			File file= new File(path);
-			if (file.exists()) {
-				InputStream input= new BufferedInputStream(new FileInputStream(file));
+			if (fileStore.fetchInfo().exists()) {
+				InputStream input= new BufferedInputStream(fileStore.openInputStream(EFS.NONE, null));
 				try {
 					TemplatePersistenceData[] datas= reader.read(input, null);
 					for (int i= 0; i < datas.length; i++) {
@@ -701,6 +646,7 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 					try {
 						input.close();
 					} catch (IOException x) {
+						// ignore
 					}
 				}
 			}
@@ -709,10 +655,10 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 			fTableViewer.setAllChecked(false);
 			fTableViewer.setCheckedElements(getEnabledTemplates());
 
-		} catch (FileNotFoundException e) {
-			openReadErrorDialog(e);
+		} catch (CoreException e) {
+			openReadErrorDialog();
 		} catch (IOException e) {
-			openReadErrorDialog(e);
+			openReadErrorDialog();
 		}
 	}
 
@@ -729,7 +675,7 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 
 	private void export(TemplatePersistenceData[] templates) {
 		FileDialog dialog= new FileDialog(getShell(), SWT.SAVE);
-		dialog.setText(NLSUtility.format(TextEditorTemplateMessages.TemplatePreferencePage_export_title, new Integer(templates.length)));
+		dialog.setText(TextEditorTemplateMessages.TemplatePreferencePage_export_title);
 		dialog.setFilterExtensions(new String[] {TextEditorTemplateMessages.TemplatePreferencePage_export_extension});
 		dialog.setFileName(TextEditorTemplateMessages.TemplatePreferencePage_export_filename);
 		String path= dialog.open();
@@ -737,46 +683,49 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 		if (path == null)
 			return;
 
-		File file= new File(path);
+		IFileStore fileStore= EFS.getLocalFileSystem().getStore(new Path(path));
+		IFileInfo fileInfo= fileStore.fetchInfo();
 
-		if (file.isHidden()) {
+		if (fileInfo.getAttribute(EFS.ATTRIBUTE_HIDDEN)) {
 			String title= TextEditorTemplateMessages.TemplatePreferencePage_export_error_title;
-			String message= NLSUtility.format(TextEditorTemplateMessages.TemplatePreferencePage_export_error_hidden, file.getAbsolutePath());
+			String message= NLSUtility.format(TextEditorTemplateMessages.TemplatePreferencePage_export_error_hidden, fileStore.toString());
 			MessageDialog.openError(getShell(), title, message);
 			return;
 		}
 
-		if (file.exists() && !file.canWrite()) {
+		if (fileInfo.exists() && fileInfo.getAttribute(EFS.ATTRIBUTE_READ_ONLY)) {
 			String title= TextEditorTemplateMessages.TemplatePreferencePage_export_error_title;
-			String message= NLSUtility.format(TextEditorTemplateMessages.TemplatePreferencePage_export_error_canNotWrite, file.getAbsolutePath());
+			String message= NLSUtility.format(TextEditorTemplateMessages.TemplatePreferencePage_export_error_canNotWrite, fileStore.toString());
 			MessageDialog.openError(getShell(), title, message);
 			return;
 		}
 
-		if (!file.exists() || confirmOverwrite(file)) {
+		if (!fileInfo.exists() || confirmOverwrite(fileStore)) {
 			OutputStream output= null;
 			try {
-				output= new BufferedOutputStream(new FileOutputStream(file));
+				output= new BufferedOutputStream(fileStore.openOutputStream(EFS.NONE, null));
 				TemplateReaderWriter writer= new TemplateReaderWriter();
 				writer.save(templates, output);
-				output.close();
+			} catch (CoreException e) {
+				openWriteErrorDialog();
 			} catch (IOException e) {
+				openWriteErrorDialog();
+			} finally {
 				if (output != null) {
 					try {
 						output.close();
-					} catch (IOException e2) {
+					} catch (IOException e) {
 						// ignore
 					}
 				}
-				openWriteErrorDialog(e);
 			}
 		}
 	}
 
-	private boolean confirmOverwrite(File file) {
+	private boolean confirmOverwrite(IFileStore fileStore) {
 		return MessageDialog.openQuestion(getShell(),
 			TextEditorTemplateMessages.TemplatePreferencePage_export_exists_title,
-			NLSUtility.format(TextEditorTemplateMessages.TemplatePreferencePage_export_exists_message, file.getAbsolutePath()));
+			NLSUtility.format(TextEditorTemplateMessages.TemplatePreferencePage_export_exists_message, fileStore.toString()));
 	}
 
 	private void remove() {
@@ -850,7 +799,7 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 		try {
 			fTemplateStore.save();
 		} catch (IOException e) {
-			openWriteErrorDialog(e);
+			openWriteErrorDialog();
 		}
 
 		return super.performOk();
@@ -872,19 +821,25 @@ public abstract class TemplatePreferencePage extends PreferencePage implements I
 		try {
 			fTemplateStore.load();
 		} catch (IOException e) {
-			openReadErrorDialog(e);
+			openReadErrorDialog();
 			return false;
 		}
 		return super.performCancel();
 	}
 
-	private void openReadErrorDialog(Exception e) {
+	/*
+	 * @since 3.2
+	 */
+	private void openReadErrorDialog() {
 		String title= TextEditorTemplateMessages.TemplatePreferencePage_error_read_title;
 		String message= TextEditorTemplateMessages.TemplatePreferencePage_error_read_message;
 		MessageDialog.openError(getShell(), title, message);
 	}
 
-	private void openWriteErrorDialog(Exception e) {
+	/*
+	 * @since 3.2
+	 */
+	private void openWriteErrorDialog() {
 		String title= TextEditorTemplateMessages.TemplatePreferencePage_error_write_title;
 		String message= TextEditorTemplateMessages.TemplatePreferencePage_error_write_message;
 		MessageDialog.openError(getShell(), title, message);
