@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -82,4 +82,15 @@ public interface IWorkbenchPartReference {
      * @since 3.0  
      */
     public String getContentDescription();
+    
+
+    /**
+     * Returns whether the part is dirty (i.e. has unsaved changes).
+     * 
+     * @return <code>true</code> if the part is dirty, <code>false</code> otherwise
+     * 
+     * @since 3.2 (previously existed on IEditorReference)
+     */
+    public boolean isDirty();
+    
 }
