@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -94,11 +94,12 @@ public class ListDialog extends SelectionDialog {
     }
 
     protected void createButtonsForButtonBar(Composite parent) {
-        if (!fAddCancelButton)
-            createButton(parent, IDialogConstants.OK_ID,
+        if (!fAddCancelButton) {
+			createButton(parent, IDialogConstants.OK_ID,
                     IDialogConstants.OK_LABEL, true);
-        else
-            super.createButtonsForButtonBar(parent);
+		} else {
+			super.createButtonsForButtonBar(parent);
+		}
     }
 
     protected Control createDialogArea(Composite container) {
@@ -110,14 +111,16 @@ public class ListDialog extends SelectionDialog {
         fTableViewer.setInput(fInput);
         fTableViewer.addDoubleClickListener(new IDoubleClickListener() {
             public void doubleClick(DoubleClickEvent event) {
-                if (fAddCancelButton)
-                    okPressed();
+                if (fAddCancelButton) {
+					okPressed();
+				}
             }
         });
         List initialSelection = getInitialElementSelections();
-        if (initialSelection != null)
-            fTableViewer
+        if (initialSelection != null) {
+			fTableViewer
                     .setSelection(new StructuredSelection(initialSelection));
+		}
         GridData gd = new GridData(GridData.FILL_BOTH);
         gd.heightHint = convertHeightInCharsToPixels(heightInChars);
         gd.widthHint = convertWidthInCharsToPixels(widthInChars);
