@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -68,9 +68,10 @@ public class ActivateEditorAction extends PageEventAction {
             } else {
                 IWorkbenchPartReference ref = page.getActivePartReference();
                 if (ref instanceof IViewReference) {
-                    if (((WorkbenchPage) page).isFastView((IViewReference) ref))
-                        ((WorkbenchPage) page)
+                    if (((WorkbenchPage) page).isFastView((IViewReference) ref)) {
+						((WorkbenchPage) page)
                                 .toggleFastView((IViewReference) ref);
+					}
                 }
             }
         }
