@@ -9,9 +9,12 @@
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.update.internal.core;
+
 import java.io.*;
 import java.net.*;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.List;
 
 import org.eclipse.core.runtime.*;
 import org.eclipse.osgi.util.NLS;
@@ -206,8 +209,13 @@ public class SiteFileContentConsumer extends SiteContentConsumer {
 		if (contentConsumers != null) {
 			Iterator iter = contentConsumers.iterator();
 			while (iter.hasNext()) {
-				SiteFilePluginContentConsumer element = (SiteFilePluginContentConsumer) iter.next();
-				element.abort();
+				Object element = iter.next();
+				if (element instanceof  SiteFilePluginContentConsumer) {
+					((SiteFilePluginContentConsumer)element).abort();
+				} else if (element instanceof  SiteFilePackedPluginContentConsumer){
+					((SiteFilePackedPluginContentConsumer)element).abort();
+				}
+				
 			}
 		}
 		contentConsumers = null;
@@ -223,7 +231,7 @@ public class SiteFileContentConsumer extends SiteContentConsumer {
 		}
 
 		if (!sucess) {
-			String msg = NLS.bind(Messages.SiteFileContentConsumer_unableToDelete, (new String[] { oldPath })); //$NON-NLS-1$
+			String msg = NLS.bind(Messages.SiteFileContentConsumer_unableToDelete, (new String[] { oldPath })); 
 			UpdateCore.log(msg, null);
 		} else {
 			// remove the feature files;
@@ -274,7 +282,7 @@ public class SiteFileContentConsumer extends SiteContentConsumer {
 				URL url = null;
 				if (pluginEntries[i] instanceof PluginEntryModel
 						&& !((PluginEntryModel) pluginEntries[i]).isUnpack()) {
-					url = new URL(getSite().getURL(), Site.DEFAULT_PLUGIN_PATH	+ versionId + ".jar");
+					url = new URL(getSite().getURL(), Site.DEFAULT_PLUGIN_PATH	+ versionId + ".jar"); //$NON-NLS-1$
 				} else {
 					url = new URL(getSite().getURL(), Site.DEFAULT_PLUGIN_PATH	+ versionId + File.separator);
 				}
