@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,15 +10,26 @@
  *******************************************************************************/
 package org.eclipse.update.internal.core;
 
-import java.io.*;
-import java.net.*;
+import java.io.File;
+import java.io.IOException;
+import java.net.URL;
 
-import org.eclipse.core.runtime.*;
-import org.eclipse.update.configuration.*;
-import org.eclipse.update.core.*;
-import org.eclipse.update.configurator.*;
-import org.osgi.framework.*;
-import org.osgi.service.packageadmin.*;
+import org.eclipse.core.runtime.CoreException;
+import org.eclipse.core.runtime.IStatus;
+import org.eclipse.core.runtime.Platform;
+import org.eclipse.core.runtime.Plugin;
+import org.eclipse.osgi.internal.provisional.verifier.CertificateVerifierFactory;
+import org.eclipse.update.configuration.IInstallConfiguration;
+import org.eclipse.update.configurator.ConfiguratorUtils;
+import org.eclipse.update.configurator.IPlatformConfiguration;
+import org.eclipse.update.core.IFeature;
+import org.eclipse.update.core.IImport;
+import org.eclipse.update.core.JarContentReference;
+import org.eclipse.update.core.SiteManager;
+import org.eclipse.update.core.Utilities;
+import org.eclipse.update.internal.core.connection.ConnectionThreadManagerFactory;
+import org.osgi.framework.BundleContext;
+import org.osgi.service.packageadmin.PackageAdmin;
 import org.osgi.util.tracker.ServiceTracker;
 
 /**
@@ -50,6 +61,7 @@ public class UpdateCore extends Plugin {
 	private static final String PREFIX = "org.eclipse.update.core"; //$NON-NLS-1$
 	public static final String P_HISTORY_SIZE = PREFIX + ".historySize"; //$NON-NLS-1$
 	public static final String P_CHECK_SIGNATURE = PREFIX + ".checkSignature"; //$NON-NLS-1$
+	public static final String P_AUTOMATICALLY_CHOOSE_MIRROR =  PREFIX + ".automaticallyChooseMirror"; //$NON-NLS-1$
 	public static final String P_UPDATE_VERSIONS = PREFIX + ".updateVersions"; //$NON-NLS-1$
 	public static final String EQUIVALENT_VALUE = "equivalent"; //$NON-NLS-1$
 	public static final String COMPATIBLE_VALUE = "compatible"; //$NON-NLS-1$
@@ -62,13 +74,14 @@ public class UpdateCore extends Plugin {
 	//log
 	private static UpdateManagerLogWriter log;
 	private static final String LOG_FILE="install.log"; //$NON-NLS-1$
-	
-	//Connection manager
-	private ConnectionThreadManager connectionManager;
 
 	// bundle data
 	private BundleContext context;
 	private ServiceTracker pkgAdminTracker;
+	private ServiceTracker verifierFactoryTracker;
+	
+	// Session
+	private UpdateSession updateSession = null;
 	
 	/**
 	 * The constructor.
@@ -85,14 +98,6 @@ public class UpdateCore extends Plugin {
 		return plugin;
 	}
 	
-	/**
-	 * Returns the manager that manages URL connection threads.
-	 */
-	public ConnectionThreadManager getConnectionManager() {
-		if (connectionManager==null)
-			connectionManager = new ConnectionThreadManager();
-		return connectionManager;
-	}
 
 	private boolean getBooleanDebugOption(String flag, boolean dflt) {
 		String result = Platform.getDebugOption(flag);
@@ -212,34 +217,7 @@ public class UpdateCore extends Plugin {
 	 * <li>An I/O error occurs while communicating with the server
 	 * <ul>
 	 */
-	public Response get(URL url) throws IOException {
-		//Request request = null;
-		Response response = null;
-		
-		if ("file".equals(url.getProtocol())) { //$NON-NLS-1$
-			response = new FileResponse(url);
-		} else if (url != null && url.getProtocol().startsWith("http")) { //$NON-NLS-1$
-			response = new HttpResponse(url);
-		} else {
-			response = new OtherResponse(url);
-		}
-		
-		/*else {
-			try {
-				request = new Request("GET", url, null);
-				response = client.invoke(request);
-			} finally {
-				if (request != null) {
-					try {
-						request.close();
-					} catch (IOException e) {
-						e.printStackTrace();
-					}
-				}
-			}
-		}*/
-		return response;
-	}
+
 	
 	/*
 	 * Returns true if the feature is a patch
@@ -296,14 +274,19 @@ public class UpdateCore extends Plugin {
 		Utilities.shutdown(); // cleanup temp area
 		if (log!=null)
 			log.shutdown();
-		if (connectionManager!=null)
-			connectionManager.shutdown();
+		
+		ConnectionThreadManagerFactory.getConnectionManager().shutdown();
+
 		
 		this.context = null;
 		if (pkgAdminTracker != null) {
 			pkgAdminTracker.close();
 			pkgAdminTracker = null;
 		}
+		if (verifierFactoryTracker != null) {
+			verifierFactoryTracker.close();
+			verifierFactoryTracker = null;
+		}
 	}
 	
 	BundleContext getBundleContext() {
@@ -339,4 +322,22 @@ public class UpdateCore extends Plugin {
 		
 		SiteManager.setHttpProxyInfo(httpProxyEnable, httpProxyHost, httpProxyPort );
 	}
+
+
+	public CertificateVerifierFactory getVerifierFactory() {
+		if (verifierFactoryTracker == null) {
+			verifierFactoryTracker = new ServiceTracker(context, CertificateVerifierFactory.class.getName(), null);
+			verifierFactoryTracker.open();
+		}
+		return (CertificateVerifierFactory)verifierFactoryTracker.getService();
+	}
+	
+	public UpdateSession getUpdateSession() {
+		synchronized(UpdateSession.class) {
+			if (updateSession == null) {
+				updateSession = new UpdateSession();
+			}
+		}		
+		return updateSession;
+	}
 }
